from typing import overload
from enum import Enum
import QuantConnect.Data.Custom.AlphaStreams
import System


class PlaceHolder(System.Object):
    """
    Static class for place holder
    
    'QuantConnect.Data.Custom.Alphas' namespace is obsolete
    """


