from typing import overload
from enum import Enum
import abc
import typing

import System
import System.Numerics
import System.Runtime.Intrinsics
import System.Runtime.Intrinsics.Arm


class ArmBase(System.Object, metaclass=abc.ABCMeta):
    """Provides access to the ARM base hardware instructions via intrinsics."""

    class Arm64(System.Object, metaclass=abc.ABCMeta):
        """Provides access to the ARM base hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

        @staticmethod
        @overload
        def leading_sign_count(value: int) -> int:
            """A64: CLS Wd, Wn"""
            ...

        @staticmethod
        @overload
        def leading_sign_count(value: int) -> int:
            """A64: CLS Xd, Xn"""
            ...

        @staticmethod
        @overload
        def leading_zero_count(value: int) -> int:
            """A64: CLZ Xd, Xn"""
            ...

        @staticmethod
        @overload
        def leading_zero_count(value: int) -> int:
            """A64: CLZ Xd, Xn"""
            ...

        @staticmethod
        @overload
        def multiply_high(left: int, right: int) -> int:
            """A64: SMULH Xd, Xn, Xm"""
            ...

        @staticmethod
        @overload
        def multiply_high(left: int, right: int) -> int:
            """A64: UMULH Xd, Xn, Xm"""
            ...

        @staticmethod
        @overload
        def reverse_element_bits(value: int) -> int:
            """A64: RBIT Xd, Xn"""
            ...

        @staticmethod
        @overload
        def reverse_element_bits(value: int) -> int:
            """A64: RBIT Xd, Xn"""
            ...

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def leading_zero_count(value: int) -> int:
        """A32: CLZ Rd, Rm  A64: CLZ Wd, Wn"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: int) -> int:
        """A32: CLZ Rd, Rm  A64: CLZ Wd, Wn"""
        ...

    @staticmethod
    @overload
    def reverse_element_bits(value: int) -> int:
        """A32: RBIT Rd, Rm  A64: RBIT Wd, Wn"""
        ...

    @staticmethod
    @overload
    def reverse_element_bits(value: int) -> int:
        """A32: RBIT Rd, Rm  A64: RBIT Wd, Wn"""
        ...

    @staticmethod
    def Yield() -> None:
        """A32: YIELD  A64: YIELD"""
        ...


class Crc32(System.Runtime.Intrinsics.Arm.ArmBase, metaclass=abc.ABCMeta):
    """Provides access to the ARM Crc32 hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.ArmBase.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARM Crc32 hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

        @staticmethod
        @overload
        def compute_crc_32(crc: int, data: int) -> int:
            """uint32_t __crc32d (uint32_t a, uint64_t b)  A64: CRC32X Wd, Wn, Xm"""
            ...

        @staticmethod
        @overload
        def compute_crc_32(crc: int, data: int) -> int:
            """uint32_t __crc32d (uint32_t a, uint64_t b)  A64: CRC32X Wd, Wn, Xm"""
            ...

        @staticmethod
        @overload
        def compute_crc_32c(crc: int, data: int) -> int:
            """uint32_t __crc32cd (uint32_t a, uint64_t b)  A64: CRC32CX Wd, Wn, Xm"""
            ...

        @staticmethod
        @overload
        def compute_crc_32c(crc: int, data: int) -> int:
            """uint32_t __crc32cd (uint32_t a, uint64_t b)  A64: CRC32CX Wd, Wn, Xm"""
            ...

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def compute_crc_32(crc: int, data: int) -> int:
        """uint32_t __crc32b (uint32_t a, uint8_t b)  A32: CRC32B Rd, Rn, Rm  A64: CRC32B Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32(crc: int, data: int) -> int:
        """uint32_t __crc32h (uint32_t a, uint16_t b)  A32: CRC32H Rd, Rn, Rm  A64: CRC32H Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32(crc: int, data: int) -> int:
        """uint32_t __crc32w (uint32_t a, uint32_t b)  A32: CRC32W Rd, Rn, Rm  A64: CRC32W Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32(crc: int, data: int) -> int:
        """uint32_t __crc32b (uint32_t a, uint8_t b)  A32: CRC32B Rd, Rn, Rm  A64: CRC32B Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32(crc: int, data: int) -> int:
        """uint32_t __crc32h (uint32_t a, uint16_t b)  A32: CRC32H Rd, Rn, Rm  A64: CRC32H Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32(crc: int, data: int) -> int:
        """uint32_t __crc32w (uint32_t a, uint32_t b)  A32: CRC32W Rd, Rn, Rm  A64: CRC32W Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32c(crc: int, data: int) -> int:
        """uint32_t __crc32cb (uint32_t a, uint8_t b)  A32: CRC32CB Rd, Rn, Rm  A64: CRC32CB Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32c(crc: int, data: int) -> int:
        """uint32_t __crc32ch (uint32_t a, uint16_t b)  A32: CRC32CH Rd, Rn, Rm  A64: CRC32CH Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32c(crc: int, data: int) -> int:
        """uint32_t __crc32cw (uint32_t a, uint32_t b)  A32: CRC32CW Rd, Rn, Rm  A64: CRC32CW Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32c(crc: int, data: int) -> int:
        """uint32_t __crc32cb (uint32_t a, uint8_t b)  A32: CRC32CB Rd, Rn, Rm  A64: CRC32CB Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32c(crc: int, data: int) -> int:
        """uint32_t __crc32ch (uint32_t a, uint16_t b)  A32: CRC32CH Rd, Rn, Rm  A64: CRC32CH Wd, Wn, Wm"""
        ...

    @staticmethod
    @overload
    def compute_crc_32c(crc: int, data: int) -> int:
        """uint32_t __crc32cw (uint32_t a, uint32_t b)  A32: CRC32CW Rd, Rn, Rm  A64: CRC32CW Wd, Wn, Wm"""
        ...


class Aes(System.Runtime.Intrinsics.Arm.ArmBase, metaclass=abc.ABCMeta):
    """Provides access to the ARM AES hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.ArmBase.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARM AES hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def decrypt(value: System.Runtime.Intrinsics.Vector128[int], round_key: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaesdq_u8 (uint8x16_t data, uint8x16_t key)  A32: AESD.8 Qd, Qm  A64: AESD Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def decrypt(value: System.Runtime.Intrinsics.Vector128[int], round_key: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaesdq_u8 (uint8x16_t data, uint8x16_t key)  A32: AESD.8 Qd, Qm  A64: AESD Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def encrypt(value: System.Runtime.Intrinsics.Vector128[int], round_key: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaeseq_u8 (uint8x16_t data, uint8x16_t key)  A32: AESE.8 Qd, Qm  A64: AESE Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def encrypt(value: System.Runtime.Intrinsics.Vector128[int], round_key: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaeseq_u8 (uint8x16_t data, uint8x16_t key)  A32: AESE.8 Qd, Qm  A64: AESE Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def inverse_mix_columns(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaesimcq_u8 (uint8x16_t data)  A32: AESIMC.8 Qd, Qm  A64: AESIMC Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def inverse_mix_columns(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaesimcq_u8 (uint8x16_t data)  A32: AESIMC.8 Qd, Qm  A64: AESIMC Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def mix_columns(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaesmcq_u8 (uint8x16_t data)  A32: AESMC.8 Qd, Qm  A64: AESMC V>.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def mix_columns(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaesmcq_u8 (uint8x16_t data)  A32: AESMC.8 Qd, Qm  A64: AESMC V>.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_p64 (poly64_t a, poly64_t b)  A32: VMULL.P8 Qd, Dn, Dm  A64: PMULL Vd.1Q, Vn.1D, Vm.1D"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_p64 (poly64_t a, poly64_t b)  A32: VMULL.P8 Qd, Dn, Dm  A64: PMULL Vd.1Q, Vn.1D, Vm.1D"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_p64 (poly64_t a, poly64_t b)  A32: VMULL.P8 Qd, Dn, Dm  A64: PMULL Vd.1Q, Vn.1D, Vm.1D"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_p64 (poly64_t a, poly64_t b)  A32: VMULL.P8 Qd, Dn, Dm  A64: PMULL Vd.1Q, Vn.1D, Vm.1D"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_high_p64 (poly64x2_t a, poly64x2_t b)  A32: VMULL.P8 Qd, Dn+1, Dm+1  A64: PMULL2 Vd.1Q, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_high_p64 (poly64x2_t a, poly64x2_t b)  A32: VMULL.P8 Qd, Dn+1, Dm+1  A64: PMULL2 Vd.1Q, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_high_p64 (poly64x2_t a, poly64x2_t b)  A32: VMULL.P8 Qd, Dn+1, Dm+1  A64: PMULL2 Vd.1Q, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly128_t vmull_high_p64 (poly64x2_t a, poly64x2_t b)  A32: VMULL.P8 Qd, Dn+1, Dm+1  A64: PMULL2 Vd.1Q, Vn.2D, Vm.2D"""
        ...


class SveMaskPattern(Enum):
    """This class has no documentation."""

    LARGEST_POWER_OF_2 = 0
    """POW2"""

    VECTOR_COUNT_1 = 1
    """VL1"""

    VECTOR_COUNT_2 = 2
    """VL2"""

    VECTOR_COUNT_3 = 3
    """VL3"""

    VECTOR_COUNT_4 = 4
    """VL4"""

    VECTOR_COUNT_5 = 5
    """VL5"""

    VECTOR_COUNT_6 = 6
    """VL6"""

    VECTOR_COUNT_7 = 7
    """VL7"""

    VECTOR_COUNT_8 = 8
    """VL8"""

    VECTOR_COUNT_16 = 9
    """VL16"""

    VECTOR_COUNT_32 = 10
    """VL32"""

    VECTOR_COUNT_64 = 11
    """VL64"""

    VECTOR_COUNT_128 = 12
    """VL128"""

    VECTOR_COUNT_256 = 13
    """VL256"""

    LARGEST_MULTIPLE_OF_4 = 29
    """MUL4"""

    LARGEST_MULTIPLE_OF_3 = 30
    """MUL3"""

    ALL = 31
    """ALL"""


class SvePrefetchType(Enum):
    """This class has no documentation."""

    LOAD_L_1_TEMPORAL = 0
    """PLDL1KEEP"""

    LOAD_L_1_NON_TEMPORAL = 1
    """PLDL1STRM"""

    LOAD_L_2_TEMPORAL = 2
    """PLDL2KEEP"""

    LOAD_L_2_NON_TEMPORAL = 3
    """PLDL2STRM"""

    LOAD_L_3_TEMPORAL = 4
    """PLDL3KEEP"""

    LOAD_L_3_NON_TEMPORAL = 5
    """PLDL3STRM"""

    STORE_L_1_TEMPORAL = 8
    """PSTL1KEEP"""

    STORE_L_1_NON_TEMPORAL = 9
    """PSTL1STRM"""

    STORE_L_2_TEMPORAL = 10
    """PSTL2KEEP"""

    STORE_L_2_NON_TEMPORAL = 11
    """PSTL2STRM"""

    STORE_L_3_TEMPORAL = 12
    """PSTL3KEEP"""

    STORE_L_3_NON_TEMPORAL = 13
    """PSTL3STRM"""


class AdvSimd(System.Runtime.Intrinsics.Arm.ArmBase, metaclass=abc.ABCMeta):
    """Provides access to the ARM AdvSIMD hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.ArmBase.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARM AdvSIMD hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

        @staticmethod
        @overload
        def abs(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vabsq_f64 (float64x2_t a)  A64: FABS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def abs(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vabsq_s64 (int64x2_t a)  A64: ABS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def absolute_compare_greater_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcagtq_f64 (float64x2_t a, float64x2_t b)  A64: FACGT Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        def absolute_compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcageq_f64 (float64x2_t a, float64x2_t b)  A64: FACGE Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def absolute_compare_greater_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vcage_f64 (float64x1_t a, float64x1_t b)  A64: FACGE Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def absolute_compare_greater_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vcages_f32 (float32_t a, float32_t b)  A64: FACGE Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def absolute_compare_greater_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vcagt_f64 (float64x1_t a, float64x1_t b)  A64: FACGT Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def absolute_compare_greater_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vcagts_f32 (float32_t a, float32_t b)  A64: FACGT Sd, Sn, Sm"""
            ...

        @staticmethod
        def absolute_compare_less_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcaltq_f64 (float64x2_t a, float64x2_t b)  A64: FACGT Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        def absolute_compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcaleq_f64 (float64x2_t a, float64x2_t b)  A64: FACGE Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def absolute_compare_less_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vcale_f64 (float64x1_t a, float64x1_t b)  A64: FACGE Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def absolute_compare_less_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vcales_f32 (float32_t a, float32_t b)  A64: FACGE Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def absolute_compare_less_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vcalt_f64 (float64x1_t a, float64x1_t b)  A64: FACGT Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def absolute_compare_less_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vcalts_f32 (float32_t a, float32_t b)  A64: FACGT Sd, Sn, Sm"""
            ...

        @staticmethod
        def absolute_difference(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vabdq_f64 (float64x2_t a, float64x2_t b)  A64: FABD Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def absolute_difference_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vabd_f64 (float64x1_t a, float64x1_t b)  A64: FABD Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def absolute_difference_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vabds_f32 (float32_t a, float32_t b)  A64: FABD Sd, Sn, Sm"""
            ...

        @staticmethod
        def abs_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vqabsq_s64 (int64x2_t a)  A64: SQABS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def abs_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqabsh_s16 (int16_t a)  A64: SQABS Hd, Hn"""
            ...

        @staticmethod
        @overload
        def abs_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqabss_s32 (int32_t a)  A64: SQABS Sd, Sn"""
            ...

        @staticmethod
        @overload
        def abs_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqabsd_s64 (int64_t a)  A64: SQABS Dd, Dn"""
            ...

        @staticmethod
        @overload
        def abs_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqabsb_s8 (int8_t a)  A64: SQABS Bd, Bn"""
            ...

        @staticmethod
        def abs_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vabs_s64 (int64x1_t a)  A64: ABS Dd, Dn"""
            ...

        @staticmethod
        def add(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vaddq_f64 (float64x2_t a, float64x2_t b)  A64: FADD Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vaddv_u8 (uint8x8_t a)  A64: ADDV Bd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vaddv_s16 (int16x4_t a)  A64: ADDV Hd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vaddv_s8 (int8x8_t a)  A64: ADDV Bd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vaddv_u16 (uint16x4_t a)  A64: ADDV Hd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vaddvq_u8 (uint8x16_t a)  A64: ADDV Bd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vaddvq_s16 (int16x8_t a)  A64: ADDV Hd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vaddvq_s32 (int32x4_t a)  A64: ADDV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vaddvq_s8 (int8x16_t a)  A64: ADDV Bd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vaddvq_u16 (uint16x8_t a)  A64: ADDV Hd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def add_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vaddvq_u32 (uint32x4_t a)  A64: ADDV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vaddlv_u8 (uint8x8_t a)  A64: UADDLV Hd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vaddlv_s16 (int16x4_t a)  A64: SADDLV Sd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vaddlv_s8 (int8x8_t a)  A64: SADDLV Hd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vaddlv_u16 (uint16x4_t a)  A64: UADDLV Sd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vaddlvq_u8 (uint8x16_t a)  A64: UADDLV Hd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vaddlvq_s16 (int16x8_t a)  A64: SADDLV Sd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vaddlvq_s32 (int32x4_t a)  A64: SADDLV Dd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vaddlvq_s8 (int8x16_t a)  A64: SADDLV Hd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vaddlvq_u16 (uint16x8_t a)  A64: UADDLV Sd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def add_across_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64_t vaddlvq_u32 (uint32x4_t a)  A64: UADDLV Dd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vpaddq_u8 (uint8x16_t a, uint8x16_t b)  A64: ADDP Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vpaddq_f64 (float64x2_t a, float64x2_t b)  A64: FADDP Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vpaddq_s16 (int16x8_t a, int16x8_t b)  A64: ADDP Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vpaddq_s32 (int32x4_t a, int32x4_t b)  A64: ADDP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vpaddq_s64 (int64x2_t a, int64x2_t b)  A64: ADDP Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vpaddq_s8 (int8x16_t a, int8x16_t b)  A64: ADDP Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vpaddq_f32 (float32x4_t a, float32x4_t b)  A64: FADDP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vpaddq_u16 (uint16x8_t a, uint16x8_t b)  A64: ADDP Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vpaddq_u32 (uint32x4_t a, uint32x4_t b)  A64: ADDP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def add_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vpaddq_u64 (uint64x2_t a, uint64x2_t b)  A64: ADDP Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def add_pairwise_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vpadds_f32 (float32x2_t a)  A64: FADDP Sd, Vn.2S"""
            ...

        @staticmethod
        @overload
        def add_pairwise_scalar(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vpaddd_f64 (float64x2_t a)  A64: FADDP Dd, Vn.2D"""
            ...

        @staticmethod
        @overload
        def add_pairwise_scalar(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vpaddd_s64 (int64x2_t a)  A64: ADDP Dd, Vn.2D"""
            ...

        @staticmethod
        @overload
        def add_pairwise_scalar(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64_t vpaddd_u64 (uint64x2_t a)  A64: ADDP Dd, Vn.2D"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vsqadd_u8 (uint8x8_t a, int8x8_t b)  A64: USQADD Vd.8B, Vn.8B"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vuqadd_s16 (int16x4_t a, uint16x4_t b)  A64: SUQADD Vd.4H, Vn.4H"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vuqadd_s32 (int32x2_t a, uint32x2_t b)  A64: SUQADD Vd.2S, Vn.2S"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vuqadd_s8 (int8x8_t a, uint8x8_t b)  A64: SUQADD Vd.8B, Vn.8B"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vsqadd_u16 (uint16x4_t a, int16x4_t b)  A64: USQADD Vd.4H, Vn.4H"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vsqadd_u32 (uint32x2_t a, int32x2_t b)  A64: USQADD Vd.2S, Vn.2S"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vsqaddq_u8 (uint8x16_t a, int8x16_t b)  A64: USQADD Vd.16B, Vn.16B"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vuqaddq_s16 (int16x8_t a, uint16x8_t b)  A64: SUQADD Vd.8H, Vn.8H"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vuqaddq_s32 (int32x4_t a, uint32x4_t b)  A64: SUQADD Vd.4S, Vn.4S"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vuqaddq_s64 (int64x2_t a, uint64x2_t b)  A64: SUQADD Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vuqaddq_s8 (int8x16_t a, uint8x16_t b)  A64: SUQADD Vd.16B, Vn.16B"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vsqaddq_u16 (uint16x8_t a, int16x8_t b)  A64: USQADD Vd.8H, Vn.8H"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vsqaddq_u32 (uint32x4_t a, int32x4_t b)  A64: USQADD Vd.4S, Vn.4S"""
            ...

        @staticmethod
        @overload
        def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vsqaddq_u64 (uint64x2_t a, int64x2_t b)  A64: USQADD Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqaddb_u8 (uint8_t a, uint8_t b)  A64: UQADD Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vsqaddb_u8 (uint8_t a, int8_t b)  A64: USQADD Bd, Bn"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqaddh_s16 (int16_t a, int16_t b)  A64: SQADD Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vuqaddh_s16 (int16_t a, uint16_t b)  A64: SUQADD Hd, Hn"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqadds_s32 (int32_t a, int32_t b)  A64: SQADD Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vuqadds_s32 (int32_t a, uint32_t b)  A64: SUQADD Sd, Sn"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vuqadd_s64 (int64x1_t a, uint64x1_t b)  A64: SUQADD Dd, Dn"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqaddb_s8 (int8_t a, int8_t b)  A64: SQADD Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vuqaddb_s8 (int8_t a, uint8_t b)  A64: SUQADD Bd, Bn"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqaddh_u16 (uint16_t a, uint16_t b)  A64: UQADD Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vsqaddh_u16 (uint16_t a, int16_t b)  A64: USQADD Hd, Hn"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqadds_u32 (uint32_t a, uint32_t b)  A64: UQADD Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vsqadds_u32 (uint32_t a, int32_t b)  A64: USQADD Sd, Sn"""
            ...

        @staticmethod
        @overload
        def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vsqadd_u64 (uint64x1_t a, int64x1_t b)  A64: USQADD Dd, Dn"""
            ...

        @staticmethod
        def ceiling(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrndpq_f64 (float64x2_t a)  A64: FRINTP Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def compare_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vceqq_f64 (float64x2_t a, float64x2_t b)  A64: FCMEQ Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vceqq_s64 (int64x2_t a, int64x2_t b)  A64: CMEQ Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vceqq_u64 (uint64x2_t a, uint64x2_t b)  A64: CMEQ Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vceq_f64 (float64x1_t a, float64x1_t b)  A64: FCMEQ Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_equal_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vceq_s64 (int64x1_t a, int64x1_t b)  A64: CMEQ Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vceqs_f32 (float32_t a, float32_t b)  A64: FCMEQ Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def compare_equal_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vceq_u64 (uint64x1_t a, uint64x1_t b)  A64: CMEQ Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcgtq_f64 (float64x2_t a, float64x2_t b)  A64: FCMGT Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcgtq_s64 (int64x2_t a, int64x2_t b)  A64: CMGT Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcgtq_u64 (uint64x2_t a, uint64x2_t b)  A64: CMHI Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcgeq_f64 (float64x2_t a, float64x2_t b)  A64: FCMGE Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcgeq_s64 (int64x2_t a, int64x2_t b)  A64: CMGE Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcgeq_u64 (uint64x2_t a, uint64x2_t b)  A64: CMHS Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vcge_f64 (float64x1_t a, float64x1_t b)  A64: FCMGE Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcge_s64 (int64x1_t a, int64x1_t b)  A64: CMGE Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vcges_f32 (float32_t a, float32_t b)  A64: FCMGE Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcge_u64 (uint64x1_t a, uint64x1_t b)  A64: CMHS Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vcgt_f64 (float64x1_t a, float64x1_t b)  A64: FCMGT Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcgt_s64 (int64x1_t a, int64x1_t b)  A64: CMGT Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vcgts_f32 (float32_t a, float32_t b)  A64: FCMGT Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def compare_greater_than_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcgt_u64 (uint64x1_t a, uint64x1_t b)  A64: CMHI Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_less_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcltq_f64 (float64x2_t a, float64x2_t b)  A64: FCMGT Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcltq_s64 (int64x2_t a, int64x2_t b)  A64: CMGT Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcltq_u64 (uint64x2_t a, uint64x2_t b)  A64: CMHI Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vcleq_f64 (float64x2_t a, float64x2_t b)  A64: FCMGE Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcleq_s64 (int64x2_t a, int64x2_t b)  A64: CMGE Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcleq_u64 (uint64x2_t a, uint64x2_t b)  A64: CMHS Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_less_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vcle_f64 (float64x1_t a, float64x1_t b)  A64: FCMGE Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_less_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcle_s64 (int64x1_t a, int64x1_t b)  A64: CMGE Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_less_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vcles_f32 (float32_t a, float32_t b)  A64: FCMGE Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def compare_less_than_or_equal_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcle_u64 (uint64x1_t a, uint64x1_t b)  A64: CMHS Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_less_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vclt_f64 (float64x1_t a, float64x1_t b)  A64: FCMGT Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_less_than_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vclt_s64 (int64x1_t a, int64x1_t b)  A64: CMGT Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_less_than_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint32_t vclts_f32 (float32_t a, float32_t b)  A64: FCMGT Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def compare_less_than_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vclt_u64 (uint64x1_t a, uint64x1_t b)  A64: CMHI Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_test(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """uint64x2_t vtstq_f64 (float64x2_t a, float64x2_t b)  A64: CMTST Vd.2D, Vn.2D, Vm.2DThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
            ...

        @staticmethod
        @overload
        def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vtstq_s64 (int64x2_t a, int64x2_t b)  A64: CMTST Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vtstq_u64 (uint64x2_t a, uint64x2_t b)  A64: CMTST Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def compare_test_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """uint64x1_t vtst_f64 (float64x1_t a, float64x1_t b)  A64: CMTST Dd, Dn, DmThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
            ...

        @staticmethod
        @overload
        def compare_test_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vtst_s64 (int64x1_t a, int64x1_t b)  A64: CMTST Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def compare_test_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vtst_u64 (uint64x1_t a, uint64x1_t b)  A64: CMTST Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def convert_to_double(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vcvt_f64_f32 (float32x2_t a)  A64: FCVTL Vd.2D, Vn.2S"""
            ...

        @staticmethod
        @overload
        def convert_to_double(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vcvtq_f64_s64 (int64x2_t a)  A64: SCVTF Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def convert_to_double(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vcvtq_f64_u64 (uint64x2_t a)  A64: UCVTF Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def convert_to_double_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vcvt_f64_s64 (int64x1_t a)  A64: SCVTF Dd, Dn"""
            ...

        @staticmethod
        @overload
        def convert_to_double_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vcvt_f64_u64 (uint64x1_t a)  A64: UCVTF Dd, Dn"""
            ...

        @staticmethod
        def convert_to_double_upper(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vcvt_high_f64_f32 (float32x4_t a)  A64: FCVTL2 Vd.2D, Vn.4S"""
            ...

        @staticmethod
        def convert_to_int_64_round_away_from_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vcvtaq_s64_f64 (float64x2_t a)  A64: FCVTAS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_int_64_round_away_from_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vcvta_s64_f64 (float64x1_t a)  A64: FCVTAS Dd, Dn"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_even(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vcvtnq_s64_f64 (float64x2_t a)  A64: FCVTNS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_even_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vcvtn_s64_f64 (float64x1_t a)  A64: FCVTNS Dd, Dn"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vcvtmq_s64_f64 (float64x2_t a)  A64: FCVTMS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_negative_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vcvtm_s64_f64 (float64x1_t a)  A64: FCVTMS Dd, Dn"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vcvtpq_s64_f64 (float64x2_t a)  A64: FCVTPS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_positive_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vcvtp_s64_f64 (float64x1_t a)  A64: FCVTPS Dd, Dn"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vcvtq_s64_f64 (float64x2_t a)  A64: FCVTZS Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_int_64_round_to_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vcvt_s64_f64 (float64x1_t a)  A64: FCVTZS Dd, Dn"""
            ...

        @staticmethod
        def convert_to_single_lower(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vcvt_f32_f64 (float64x2_t a)  A64: FCVTN Vd.2S, Vn.2D"""
            ...

        @staticmethod
        def convert_to_single_round_to_odd_lower(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vcvtx_f32_f64 (float64x2_t a)  A64: FCVTXN Vd.2S, Vn.2D"""
            ...

        @staticmethod
        def convert_to_single_round_to_odd_upper(lower: System.Runtime.Intrinsics.Vector64[float], value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vcvtx_high_f32_f64 (float32x2_t r, float64x2_t a)  A64: FCVTXN2 Vd.4S, Vn.2D"""
            ...

        @staticmethod
        def convert_to_single_upper(lower: System.Runtime.Intrinsics.Vector64[float], value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vcvt_high_f32_f64 (float32x2_t r, float64x2_t a)  A64: FCVTN2 Vd.4S, Vn.2D"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_away_from_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcvtaq_u64_f64 (float64x2_t a)  A64: FCVTAU Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_away_from_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcvta_u64_f64 (float64x1_t a)  A64: FCVTAU Dd, Dn"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_even(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcvtnq_u64_f64 (float64x2_t a)  A64: FCVTNU Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_even_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcvtn_u64_f64 (float64x1_t a)  A64: FCVTNU Dd, Dn"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcvtmq_u64_f64 (float64x2_t a)  A64: FCVTMU Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_negative_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcvtm_u64_f64 (float64x1_t a)  A64: FCVTMU Dd, Dn"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcvtpq_u64_f64 (float64x2_t a)  A64: FCVTPU Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_positive_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcvtp_u64_f64 (float64x1_t a)  A64: FCVTPU Dd, Dn"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcvtq_u64_f64 (float64x2_t a)  A64: FCVTZU Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def convert_to_u_int_64_round_to_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint64x1_t vcvt_u64_f64 (float64x1_t a)  A64: FCVTZU Dd, Dn"""
            ...

        @staticmethod
        @overload
        def divide(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vdiv_f32 (float32x2_t a, float32x2_t b)  A64: FDIV Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def divide(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vdivq_f64 (float64x2_t a, float64x2_t b)  A64: FDIV Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def divide(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vdivq_f32 (float32x4_t a, float32x4_t b)  A64: FDIV Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[float], index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vdupq_laneq_f64 (float64x2_t vec, const int lane)  A64: DUP Vd.2D, Vn.D[index]"""
            ...

        @staticmethod
        @overload
        def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vdupq_laneq_s64 (int64x2_t vec, const int lane)  A64: DUP Vd.2D, Vn.D[index]"""
            ...

        @staticmethod
        @overload
        def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vdupq_laneq_u64 (uint64x2_t vec, const int lane)  A64: DUP Vd.2D, Vn.D[index]"""
            ...

        @staticmethod
        @overload
        def duplicate_to_vector_128(value: float) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vdupq_n_f64 (float64_t value)  A64: DUP Vd.2D, Vn.D[0]"""
            ...

        @staticmethod
        @overload
        def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vdupq_n_s64 (int64_t value)  A64: DUP Vd.2D, Rn"""
            ...

        @staticmethod
        @overload
        def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vdupq_n_s64 (uint64_t value)  A64: DUP Vd.2D, Rn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqmovnh_u16 (uint16_t a)  A64: UQXTN Bd, Hn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqmovns_s32 (int32_t a)  A64: SQXTN Hd, Sn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqmovnd_s64 (int64_t a)  A64: SQXTN Sd, Dn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqmovnh_s16 (int16_t a)  A64: SQXTN Bd, Hn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqmovns_u32 (uint32_t a)  A64: UQXTN Hd, Sn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqmovnd_u64 (uint64_t a)  A64: UQXTN Sd, Dn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqmovunh_s16 (int16_t a)  A64: SQXTUN Bd, Hn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqmovuns_s32 (int32_t a)  A64: SQXTUN Hd, Sn"""
            ...

        @staticmethod
        @overload
        def extract_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqmovund_s64 (int64_t a)  A64: SQXTUN Sd, Dn"""
            ...

        @staticmethod
        def floor(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrndmq_f64 (float64x2_t a)  A64: FRINTM Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def fused_multiply_add(addend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vfmaq_f64 (float64x2_t a, float64x2_t b, float64x2_t c)  A64: FMLA Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vfma_n_f32 (float32x2_t a, float32x2_t b, float32_t n)  A64: FMLA Vd.2S, Vn.2S, Vm.S[0]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vfmaq_n_f64 (float64x2_t a, float64x2_t b, float64_t n)  A64: FMLA Vd.2D, Vn.2D, Vm.D[0]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vfmaq_n_f32 (float32x4_t a, float32x4_t b, float32_t n)  A64: FMLA Vd.4S, Vn.4S, Vm.S[0]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vfma_lane_f32 (float32x2_t a, float32x2_t b, float32x2_t v, const int lane)  A64: FMLA Vd.2S, Vn.2S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vfma_laneq_f32 (float32x2_t a, float32x2_t b, float32x4_t v, const int lane)  A64: FMLA Vd.2S, Vn.2S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vfmaq_laneq_f64 (float64x2_t a, float64x2_t b, float64x2_t v, const int lane)  A64: FMLA Vd.2D, Vn.2D, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vfmaq_lane_f32 (float32x4_t a, float32x4_t b, float32x2_t v, const int lane)  A64: FMLA Vd.4S, Vn.4S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vfmaq_laneq_f32 (float32x4_t a, float32x4_t b, float32x4_t v, const int lane)  A64: FMLA Vd.4S, Vn.4S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_scalar_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vfmad_laneq_f64 (float64_t a, float64_t b, float64x2_t v, const int lane)  A64: FMLA Dd, Dn, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_scalar_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vfmas_lane_f32 (float32_t a, float32_t b, float32x2_t v, const int lane)  A64: FMLA Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_add_scalar_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vfmas_laneq_f32 (float32_t a, float32_t b, float32x4_t v, const int lane)  A64: FMLA Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        def fused_multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vfmsq_f64 (float64x2_t a, float64x2_t b, float64x2_t c)  A64: FMLS Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vfms_n_f32 (float32x2_t a, float32x2_t b, float32_t n)  A64: FMLS Vd.2S, Vn.2S, Vm.S[0]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vfmsq_n_f64 (float64x2_t a, float64x2_t b, float64_t n)  A64: FMLS Vd.2D, Vn.2D, Vm.D[0]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vfmsq_n_f32 (float32x4_t a, float32x4_t b, float32_t n)  A64: FMLS Vd.4S, Vn.4S, Vm.S[0]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vfms_lane_f32 (float32x2_t a, float32x2_t b, float32x2_t v, const int lane)  A64: FMLS Vd.2S, Vn.2S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vfms_laneq_f32 (float32x2_t a, float32x2_t b, float32x4_t v, const int lane)  A64: FMLS Vd.2S, Vn.2S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vfmsq_laneq_f64 (float64x2_t a, float64x2_t b, float64x2_t v, const int lane)  A64: FMLS Vd.2D, Vn.2D, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vfmsq_lane_f32 (float32x4_t a, float32x4_t b, float32x2_t v, const int lane)  A64: FMLS Vd.4S, Vn.4S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vfmsq_laneq_f32 (float32x4_t a, float32x4_t b, float32x4_t v, const int lane)  A64: FMLS Vd.4S, Vn.4S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_scalar_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vfmsd_laneq_f64 (float64_t a, float64_t b, float64x2_t v, const int lane)  A64: FMLS Dd, Dn, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_scalar_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vfmss_lane_f32 (float32_t a, float32_t b, float32x2_t v, const int lane)  A64: FMLS Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def fused_multiply_subtract_scalar_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vfmss_laneq_f32 (float32_t a, float32_t b, float32x4_t v, const int lane)  A64: FMLS Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vcopy_lane_u8 (uint8x8_t a, const int lane1, uint8x8_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vcopy_laneq_u8 (uint8x8_t a, const int lane1, uint8x16_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vcopy_lane_s16 (int16x4_t a, const int lane1, int16x4_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vcopy_laneq_s16 (int16x4_t a, const int lane1, int16x8_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vcopy_lane_s32 (int32x2_t a, const int lane1, int32x2_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vcopy_laneq_s32 (int32x2_t a, const int lane1, int32x4_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vcopy_lane_s8 (int8x8_t a, const int lane1, int8x8_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vcopy_laneq_s8 (int8x8_t a, const int lane1, int8x16_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[float], result_index: int, value: System.Runtime.Intrinsics.Vector64[float], value_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vcopy_lane_f32 (float32x2_t a, const int lane1, float32x2_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[float], result_index: int, value: System.Runtime.Intrinsics.Vector128[float], value_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vcopy_laneq_f32 (float32x2_t a, const int lane1, float32x4_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vcopy_lane_u16 (uint16x4_t a, const int lane1, uint16x4_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vcopy_laneq_u16 (uint16x4_t a, const int lane1, uint16x8_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vcopy_lane_u32 (uint32x2_t a, const int lane1, uint32x2_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector64[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vcopy_laneq_u32 (uint32x2_t a, const int lane1, uint32x4_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vcopyq_lane_u8 (uint8x16_t a, const int lane1, uint8x8_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vcopyq_laneq_u8 (uint8x16_t a, const int lane1, uint8x16_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[float], result_index: int, value: System.Runtime.Intrinsics.Vector128[float], value_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vcopyq_laneq_f64 (float64x2_t a, const int lane1, float64x2_t b, const int lane2)  A64: INS Vd.D[lane1], Vn.D[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vcopyq_lane_s16 (int16x8_t a, const int lane1, int16x4_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vcopyq_laneq_s16 (int16x8_t a, const int lane1, int16x8_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vcopyq_lane_s32 (int32x4_t a, const int lane1, int32x2_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vcopyq_laneq_s32 (int32x4_t a, const int lane1, int32x4_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vcopyq_laneq_s64 (int64x2_t a, const int lane1, int64x2_t b, const int lane2)  A64: INS Vd.D[lane1], Vn.D[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vcopyq_lane_s8 (int8x16_t a, const int lane1, int8x8_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vcopyq_laneq_s8 (int8x16_t a, const int lane1, int8x16_t b, const int lane2)  A64: INS Vd.B[lane1], Vn.B[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[float], result_index: int, value: System.Runtime.Intrinsics.Vector64[float], value_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vcopyq_lane_f32 (float32x4_t a, const int lane1, float32x2_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[float], result_index: int, value: System.Runtime.Intrinsics.Vector128[float], value_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vcopyq_laneq_f32 (float32x4_t a, const int lane1, float32x4_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vcopyq_lane_u16 (uint16x8_t a, const int lane1, uint16x4_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vcopyq_laneq_u16 (uint16x8_t a, const int lane1, uint16x8_t b, const int lane2)  A64: INS Vd.H[lane1], Vn.H[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vcopyq_lane_u32 (uint32x4_t a, const int lane1, uint32x2_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vcopyq_laneq_u32 (uint32x4_t a, const int lane1, uint32x4_t b, const int lane2)  A64: INS Vd.S[lane1], Vn.S[lane2]"""
            ...

        @staticmethod
        @overload
        def insert_selected_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector128[int], value_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vcopyq_laneq_u64 (uint64x2_t a, const int lane1, uint64x2_t b, const int lane2)  A64: INS Vd.D[lane1], Vn.D[lane2]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD1 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD1 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD2 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_2_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD2 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD1 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD1 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD3 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_3_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD3 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD1 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D}, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD1 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD1 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_4_x_vector_128_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.16B, Vn+1.16B }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.16B, Vn+1.16B }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.8H, Vn+1.8H }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.8H, Vn+1.8H }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.4S, Vn+1.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.4S, Vn+1.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.2D, Vn+1.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2 { Vn.2D, Vn+1.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD2 { Vn.4S, Vn+1.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD2 { Vn.2D, Vn+1.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.16B, Vn+1.16B, Vn+2.16B }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.16B, Vn+1.16B, Vn+2.16B }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.8H, Vn+1.8H, Vn+2.8H }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.8H, Vn+1.8H, Vn+2.8H }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.4S, Vn+1.4S, Vn+2.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.4S, Vn+1.4S, Vn+2.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.2D, Vn+1.2D, Vn+2.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3 { Vn.2D, Vn+1.2D, Vn+2.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD3 { Vn.4S, Vn+1.4S, Vn+2.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD3 { Vn.2D, Vn+1.2D, Vn+2.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }[Vm], [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vld1q_dup_f64 (float64_t const * ptr)  A64: LD1R { Vt.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vld1q_dup_s64 (int64_t const * ptr)  A64: LD1R { Vt.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vld1q_dup_u64 (uint64_t const * ptr)  A64: LD1R { Vt.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD2R { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD2R { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD2R { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD3R { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD3R { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD3R { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LD4R { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD4R { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_and_replicate_to_vector_128x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LD4R { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_scalar_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_scalar_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
            """A64: LDP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_scalar_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_scalar_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_scalar_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
            """A64: LDNP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_scalar_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_128_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]:
            """A64: LDNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def load_pair_vector_64_non_temporal(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
            """A64: LDNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        def max(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmaxq_f64 (float64x2_t a, float64x2_t b)  A64: FMAX Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vmaxv_u8 (uint8x8_t a)  A64: UMAXV Bd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vmaxv_s16 (int16x4_t a)  A64: SMAXV Hd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vmaxv_s8 (int8x8_t a)  A64: SMAXV Bd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vmaxv_u16 (uint16x4_t a)  A64: UMAXV Hd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vmaxvq_u8 (uint8x16_t a)  A64: UMAXV Bd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vmaxvq_s16 (int16x8_t a)  A64: SMAXV Hd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vmaxvq_s32 (int32x4_t a)  A64: SMAXV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vmaxvq_s8 (int8x16_t a)  A64: SMAXV Bd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vmaxvq_f32 (float32x4_t a)  A64: FMAXV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vmaxvq_u16 (uint16x8_t a)  A64: UMAXV Hd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def max_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vmaxvq_u32 (uint32x4_t a)  A64: UMAXV Sd, Vn.4S"""
            ...

        @staticmethod
        def max_number(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmaxnmq_f64 (float64x2_t a, float64x2_t b)  A64: FMAXNM Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        def max_number_across(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vmaxnmvq_f32 (float32x4_t a)  A64: FMAXNMV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def max_number_pairwise(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vpmaxnm_f32 (float32x2_t a, float32x2_t b)  A64: FMAXNMP Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def max_number_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vpmaxnmq_f64 (float64x2_t a, float64x2_t b)  A64: FMAXNMP Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def max_number_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vpmaxnmq_f32 (float32x4_t a, float32x4_t b)  A64: FMAXNMP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def max_number_pairwise_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vpmaxnms_f32 (float32x2_t a)  A64: FMAXNMP Sd, Vn.2S"""
            ...

        @staticmethod
        @overload
        def max_number_pairwise_scalar(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vpmaxnmqd_f64 (float64x2_t a)  A64: FMAXNMP Dd, Vn.2D"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vpmaxq_u8 (uint8x16_t a, uint8x16_t b)  A64: UMAXP Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vpmaxq_f64 (float64x2_t a, float64x2_t b)  A64: FMAXP Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vpmaxq_s16 (int16x8_t a, int16x8_t b)  A64: SMAXP Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vpmaxq_s32 (int32x4_t a, int32x4_t b)  A64: SMAXP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vpmaxq_s8 (int8x16_t a, int8x16_t b)  A64: SMAXP Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vpmaxq_f32 (float32x4_t a, float32x4_t b)  A64: FMAXP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vpmaxq_u16 (uint16x8_t a, uint16x8_t b)  A64: UMAXP Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def max_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vpmaxq_u32 (uint32x4_t a, uint32x4_t b)  A64: UMAXP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def max_pairwise_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vpmaxs_f32 (float32x2_t a)  A64: FMAXP Sd, Vn.2S"""
            ...

        @staticmethod
        @overload
        def max_pairwise_scalar(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vpmaxqd_f64 (float64x2_t a)  A64: FMAXP Dd, Vn.2D"""
            ...

        @staticmethod
        @overload
        def max_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vmax_f64 (float64x1_t a, float64x1_t b)  A64: FMAX Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def max_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vmaxs_f32 (float32_t a, float32_t b)  A64: FMAX Sd, Sn, SmThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
            ...

        @staticmethod
        def min(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vminq_f64 (float64x2_t a, float64x2_t b)  A64: FMIN Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vminv_u8 (uint8x8_t a)  A64: UMINV Bd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vminv_s16 (int16x4_t a)  A64: SMINV Hd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vminv_s8 (int8x8_t a)  A64: SMINV Bd, Vn.8B"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vminv_u16 (uint16x4_t a)  A64: UMINV Hd, Vn.4H"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vminvq_u8 (uint8x16_t a)  A64: UMINV Bd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vminvq_s16 (int16x8_t a)  A64: SMINV Hd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vaddvq_s32 (int32x4_t a)  A64: SMINV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vminvq_s8 (int8x16_t a)  A64: SMINV Bd, Vn.16B"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vminvq_f32 (float32x4_t a)  A64: FMINV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vminvq_u16 (uint16x8_t a)  A64: UMINV Hd, Vn.8H"""
            ...

        @staticmethod
        @overload
        def min_across(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vminvq_u32 (uint32x4_t a)  A64: UMINV Sd, Vn.4S"""
            ...

        @staticmethod
        def min_number(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vminnmq_f64 (float64x2_t a, float64x2_t b)  A64: FMINNM Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        def min_number_across(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vminnmvq_f32 (float32x4_t a)  A64: FMINNMV Sd, Vn.4S"""
            ...

        @staticmethod
        @overload
        def min_number_pairwise(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vpminnm_f32 (float32x2_t a, float32x2_t b)  A64: FMINNMP Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def min_number_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vpminnmq_f64 (float64x2_t a, float64x2_t b)  A64: FMINNMP Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def min_number_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vpminnmq_f32 (float32x4_t a, float32x4_t b)  A64: FMINNMP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def min_number_pairwise_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vpminnms_f32 (float32x2_t a)  A64: FMINNMP Sd, Vn.2S"""
            ...

        @staticmethod
        @overload
        def min_number_pairwise_scalar(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vpminnmqd_f64 (float64x2_t a)  A64: FMINNMP Dd, Vn.2D"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vpminq_u8 (uint8x16_t a, uint8x16_t b)  A64: UMINP Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vpminq_f64 (float64x2_t a, float64x2_t b)  A64: FMINP Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vpminq_s16 (int16x8_t a, int16x8_t b)  A64: SMINP Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vpminq_s32 (int32x4_t a, int32x4_t b)  A64: SMINP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vpminq_s8 (int8x16_t a, int8x16_t b)  A64: SMINP Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vpminq_f32 (float32x4_t a, float32x4_t b)  A64: FMINP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vpminq_u16 (uint16x8_t a, uint16x8_t b)  A64: UMINP Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def min_pairwise(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vpminq_u32 (uint32x4_t a, uint32x4_t b)  A64: UMINP Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def min_pairwise_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vpmins_f32 (float32x2_t a)  A64: FMINP Sd, Vn.2S"""
            ...

        @staticmethod
        @overload
        def min_pairwise_scalar(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vpminqd_f64 (float64x2_t a)  A64: FMINP Dd, Vn.2D"""
            ...

        @staticmethod
        @overload
        def min_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vmin_f64 (float64x1_t a, float64x1_t b)  A64: FMIN Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def min_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vmins_f32 (float32_t a, float32_t b)  A64: FMIN Sd, Sn, SmThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
            ...

        @staticmethod
        def multiply(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmulq_f64 (float64x2_t a, float64x2_t b)  A64: FMUL Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmulq_n_f64 (float64x2_t a, float64_t b)  A64: FMUL Vd.2D, Vn.2D, Vm.D[0]"""
            ...

        @staticmethod
        def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmulq_laneq_f64 (float64x2_t a, float64x2_t v, const int lane)  A64: FMUL Vd.2D, Vn.2D, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_saturate_high_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqdmulhh_s16 (int16_t a, int16_t b)  A64: SQDMULH Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_saturate_high_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmulhs_s32 (int32_t a, int32_t b)  A64: SQDMULH Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqdmulhh_lane_s16 (int16_t a, int16x4_t v, const int lane)  A64: SQDMULH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqdmulhh_laneq_s16 (int16_t a, int16x8_t v, const int lane)  A64: SQDMULH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmulhs_lane_s32 (int32_t a, int32x2_t v, const int lane)  A64: SQDMULH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmulhs_laneq_s32 (int32_t a, int32x4_t v, const int lane)  A64: SQDMULH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_and_add_saturate_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmlalh_s16 (int32_t a, int16_t b, int16_t c)  A64: SQDMLAL Sd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_and_add_saturate_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmlals_s32 (int64_t a, int32_t b, int32_t c)  A64: SQDMLAL Dd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_and_subtract_saturate_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmlslh_s16 (int32_t a, int16_t b, int16_t c)  A64: SQDMLSL Sd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_and_subtract_saturate_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmlsls_s32 (int64_t a, int32_t b, int32_t c)  A64: SQDMLSL Dd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmullh_s16 (int16_t a, int16_t b)  A64: SQDMULL Sd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmulls_s32 (int32_t a, int32_t b)  A64: SQDMULL Dd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_saturate_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmullh_lane_s16 (int16_t a, int16x4_t v, const int lane)  A64: SQDMULL Sd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_saturate_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmullh_laneq_s16 (int16_t a, int16x8_t v, const int lane)  A64: SQDMULL Sd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_saturate_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmulls_lane_s32 (int32_t a, int32x2_t v, const int lane)  A64: SQDMULL Dd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_saturate_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmulls_laneq_s32 (int32_t a, int32x4_t v, const int lane)  A64: SQDMULL Dd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmlalh_lane_s16 (int32_t a, int16_t b, int16x4_t v, const int lane)  A64: SQDMLAL Sd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmlalh_laneq_s16 (int32_t a, int16_t b, int16x8_t v, const int lane)  A64: SQDMLAL Sd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmlals_lane_s32 (int64_t a, int32_t b, int32x2_t v, const int lane)  A64: SQDMLAL Dd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmlals_laneq_s32 (int64_t a, int32_t b, int32x4_t v, const int lane)  A64: SQDMLAL Dd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmlslh_lane_s16 (int32_t a, int16_t b, int16x4_t v, const int lane)  A64: SQDMLSL Sd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqdmlslh_laneq_s16 (int32_t a, int16_t b, int16x8_t v, const int lane)  A64: SQDMLSL Sd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmlsls_lane_s32 (int64_t a, int32_t b, int32x2_t v, const int lane)  A64: SQDMLSL Dd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_doubling_widening_scalar_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqdmlsls_laneq_s32 (int64_t a, int32_t b, int32x4_t v, const int lane)  A64: SQDMLSL Dd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vmulx_f32 (float32x2_t a, float32x2_t b)  A64: FMULX Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def multiply_extended(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmulxq_f64 (float64x2_t a, float64x2_t b)  A64: FMULX Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def multiply_extended(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vmulxq_f32 (float32x4_t a, float32x4_t b)  A64: FMULX Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        def multiply_extended_by_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmulxq_lane_f64 (float64x2_t a, float64x1_t v, const int lane)  A64: FMULX Vd.2D, Vn.2D, Vm.D[0]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vmulx_lane_f32 (float32x2_t a, float32x2_t v, const int lane)  A64: FMULX Vd.2S, Vn.2S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vmulx_laneq_f32 (float32x2_t a, float32x4_t v, const int lane)  A64: FMULX Vd.2S, Vn.2S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vmulxq_laneq_f64 (float64x2_t a, float64x2_t v, const int lane)  A64: FMULX Vd.2D, Vn.2D, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vmulxq_lane_f32 (float32x4_t a, float32x2_t v, const int lane)  A64: FMULX Vd.4S, Vn.4S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vmulxq_laneq_f32 (float32x4_t a, float32x4_t v, const int lane)  A64: FMULX Vd.4S, Vn.4S, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vmulx_f64 (float64x1_t a, float64x1_t b)  A64: FMULX Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def multiply_extended_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vmulxs_f32 (float32_t a, float32_t b)  A64: FMULX Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_extended_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vmulxd_laneq_f64 (float64_t a, float64x2_t v, const int lane)  A64: FMULX Dd, Dn, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vmulxs_lane_f32 (float32_t a, float32x2_t v, const int lane)  A64: FMULX Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_extended_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vmulxs_laneq_f32 (float32_t a, float32x4_t v, const int lane)  A64: FMULX Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_saturate_high_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmulhh_s16 (int16_t a, int16_t b)  A64: SQRDMULH Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_saturate_high_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmulhs_s32 (int32_t a, int32_t b)  A64: SQRDMULH Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmulhh_lane_s16 (int16_t a, int16x4_t v, const int lane)  A64: SQRDMULH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmulhh_laneq_s16 (int16_t a, int16x8_t v, const int lane)  A64: SQRDMULH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmulhs_lane_s32 (int32_t a, int32x2_t v, const int lane)  A64: SQRDMULH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmulhs_laneq_s32 (int32_t a, int32x4_t v, const int lane)  A64: SQRDMULH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        def multiply_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vmuld_laneq_f64 (float64_t a, float64x2_t v, const int lane)  A64: FMUL Dd, Dn, Vm.D[lane]"""
            ...

        @staticmethod
        @overload
        def negate(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vnegq_f64 (float64x2_t a)  A64: FNEG Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def negate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vnegq_s64 (int64x2_t a)  A64: NEG Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def negate_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vqnegq_s64 (int64x2_t a)  A64: SQNEG Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def negate_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqnegh_s16 (int16_t a)  A64: SQNEG Hd, Hn"""
            ...

        @staticmethod
        @overload
        def negate_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqnegs_s32 (int32_t a)  A64: SQNEG Sd, Sn"""
            ...

        @staticmethod
        @overload
        def negate_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64_t vqnegd_s64 (int64_t a)  A64: SQNEG Dd, Dn"""
            ...

        @staticmethod
        @overload
        def negate_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqnegb_s8 (int8_t a)  A64: SQNEG Bd, Bn"""
            ...

        @staticmethod
        def negate_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int64x1_t vneg_s64 (int64x1_t a)  A64: NEG Dd, Dn"""
            ...

        @staticmethod
        def reciprocal_estimate(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrecpeq_f64 (float64x2_t a)  A64: FRECPE Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def reciprocal_estimate_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vrecpe_f64 (float64x1_t a)  A64: FRECPE Dd, Dn"""
            ...

        @staticmethod
        @overload
        def reciprocal_estimate_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vrecpes_f32 (float32_t a)  A64: FRECPE Sd, Sn"""
            ...

        @staticmethod
        @overload
        def reciprocal_exponent_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64_t vrecpxd_f64 (float64_t a)  A64: FRECPX Dd, Dn"""
            ...

        @staticmethod
        @overload
        def reciprocal_exponent_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vrecpxs_f32 (float32_t a)  A64: FRECPX Sd, Sn"""
            ...

        @staticmethod
        def reciprocal_square_root_estimate(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrsqrteq_f64 (float64x2_t a)  A64: FRSQRTE Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def reciprocal_square_root_estimate_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vrsqrte_f64 (float64x1_t a)  A64: FRSQRTE Dd, Dn"""
            ...

        @staticmethod
        @overload
        def reciprocal_square_root_estimate_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vrsqrtes_f32 (float32_t a)  A64: FRSQRTE Sd, Sn"""
            ...

        @staticmethod
        def reciprocal_square_root_step(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrsqrtsq_f64 (float64x2_t a, float64x2_t b)  A64: FRSQRTS Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def reciprocal_square_root_step_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vrsqrts_f64 (float64x1_t a, float64x1_t b)  A64: FRSQRTS Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def reciprocal_square_root_step_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vrsqrtss_f32 (float32_t a, float32_t b)  A64: FRSQRTS Sd, Sn, Sm"""
            ...

        @staticmethod
        def reciprocal_step(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrecpsq_f64 (float64x2_t a, float64x2_t b)  A64: FRECPS Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def reciprocal_step_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float64x1_t vrecps_f64 (float64x1_t a, float64x1_t b)  A64: FRECPS Dd, Dn, Dm"""
            ...

        @staticmethod
        @overload
        def reciprocal_step_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32_t vrecpss_f32 (float32_t a, float32_t b)  A64: FRECPS Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def reverse_element_bits(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vrbit_u8 (uint8x8_t a)  A64: RBIT Vd.8B, Vn.8B"""
            ...

        @staticmethod
        @overload
        def reverse_element_bits(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vrbit_s8 (int8x8_t a)  A64: RBIT Vd.8B, Vn.8B"""
            ...

        @staticmethod
        @overload
        def reverse_element_bits(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vrbitq_u8 (uint8x16_t a)  A64: RBIT Vd.16B, Vn.16B"""
            ...

        @staticmethod
        @overload
        def reverse_element_bits(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vrbitq_s8 (int8x16_t a)  A64: RBIT Vd.16B, Vn.16B"""
            ...

        @staticmethod
        def round_away_from_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrndaq_f64 (float64x2_t a)  A64: FRINTA Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def round_to_nearest(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrndnq_f64 (float64x2_t a)  A64: FRINTN Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrndmq_f64 (float64x2_t a)  A64: FRINTM Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrndpq_f64 (float64x2_t a)  A64: FRINTP Vd.2D, Vn.2D"""
            ...

        @staticmethod
        def round_to_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vrndq_f64 (float64x2_t a)  A64: FRINTZ Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def shift_arithmetic_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrshlh_s16 (int16_t a, int16_t b)  A64: SQRSHL Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def shift_arithmetic_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrshls_s32 (int32_t a, int32_t b)  A64: SQRSHL Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def shift_arithmetic_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqrshlb_s8 (int8_t a, int8_t b)  A64: SQRSHL Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def shift_arithmetic_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqshlh_s16 (int16_t a, int16_t b)  A64: SQSHL Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def shift_arithmetic_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqshls_s32 (int32_t a, int32_t b)  A64: SQSHL Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def shift_arithmetic_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqshlb_s8 (int8_t a, int8_t b)  A64: SQSHL Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqshlb_n_u8 (uint8_t a, const int n)  A64: UQSHL Bd, Bn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqshlh_n_s16 (int16_t a, const int n)  A64: SQSHL Hd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqshls_n_s32 (int32_t a, const int n)  A64: SQSHL Sd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqshlb_n_s8 (int8_t a, const int n)  A64: SQSHL Bd, Bn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqshlh_n_u16 (uint16_t a, const int n)  A64: UQSHL Hd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqshls_n_u32 (uint32_t a, const int n)  A64: UQSHL Sd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqshluh_n_s16 (int16_t a, const int n)  A64: SQSHLU Hd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqshlus_n_s32 (int32_t a, const int n)  A64: SQSHLU Sd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_left_logical_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqshlub_n_s8 (int8_t a, const int n)  A64: SQSHLU Bd, Bn, #n"""
            ...

        @staticmethod
        @overload
        def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqrshlb_u8 (uint8_t a, int8_t b)  A64: UQRSHL Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqrshlh_u16 (uint16_t a, int16_t b)  A64: UQRSHL Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqrshls_u32 (uint32_t a, int32_t b)  A64: UQRSHL Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqrshlb_u8 (uint8_t a, int8_t b)  A64: UQRSHL Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqrshlh_u16 (uint16_t a, int16_t b)  A64: UQRSHL Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqrshls_u32 (uint32_t a, int32_t b)  A64: UQRSHL Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqshlb_u8 (uint8_t a, int8_t b)  A64: UQSHL Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqshlh_u16 (uint16_t a, int16_t b)  A64: UQSHL Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqshls_u32 (uint32_t a, int32_t b)  A64: UQSHL Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqshlb_u8 (uint8_t a, int8_t b)  A64: UQSHL Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqshlh_u16 (uint16_t a, int16_t b)  A64: UQSHL Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqshls_u32 (uint32_t a, int32_t b)  A64: UQSHL Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqshrns_n_s32 (int32_t a, const int n)  A64: SQSHRN Hd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqshrnd_n_s64 (int64_t a, const int n)  A64: SQSHRN Sd, Dn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqshrnh_n_s16 (int16_t a, const int n)  A64: SQSHRN Bd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqshrunh_n_s16 (int16_t a, const int n)  A64: SQSHRUN Bd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqshruns_n_s32 (int32_t a, const int n)  A64: SQSHRUN Hd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqshrund_n_s64 (int64_t a, const int n)  A64: SQSHRUN Sd, Dn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrshrns_n_s32 (int32_t a, const int n)  A64: SQRSHRN Hd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrshrnd_n_s64 (int64_t a, const int n)  A64: SQRSHRN Sd, Dn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqrshrnh_n_s16 (int16_t a, const int n)  A64: SQRSHRN Bd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqrshrunh_n_s16 (int16_t a, const int n)  A64: SQRSHRUN Bd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqrshruns_n_s32 (int32_t a, const int n)  A64: SQRSHRUN Hd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqrshrund_n_s64 (int64_t a, const int n)  A64: SQRSHRUN Sd, Dn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqshrnh_n_u16 (uint16_t a, const int n)  A64: UQSHRN Bd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqshrns_n_u32 (uint32_t a, const int n)  A64: UQSHRN Hd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqshrnd_n_u64 (uint64_t a, const int n)  A64: UQSHRN Sd, Dn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqshrnh_n_u16 (uint16_t a, const int n)  A64: UQSHRN Bd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqshrns_n_u32 (uint32_t a, const int n)  A64: UQSHRN Hd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqshrnd_n_u64 (uint64_t a, const int n)  A64: UQSHRN Sd, Dn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqrshrnh_n_u16 (uint16_t a, const int n)  A64: UQRSHRN Bd, Hn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqrshrns_n_u32 (uint32_t a, const int n)  A64: UQRSHRN Hd, Sn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqrshrnd_n_u64 (uint64_t a, const int n)  A64: UQRSHRN Sd, Dn, #n"""
            ...

        @staticmethod
        @overload
        def shift_right_logical_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """
            uint16_t vqrshrns_n_u32 (uint32_t a, const int n)  A64: UQRSHRN Hd, Sn, #n
            
            Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
            """
            ...

        @staticmethod
        @overload
        def shift_right_logical_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """
            uint32_t vqrshrnd_n_u64 (uint64_t a, const int n)  A64: UQRSHRN Sd, Dn, #n
            
            Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
            """
            ...

        @staticmethod
        @overload
        def shift_right_logical_rounded_narrowing_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """
            uint8_t vqrshrnh_n_u16 (uint16_t a, const int n)  A64: UQRSHRN Bd, Hn, #n
            
            Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
            """
            ...

        @staticmethod
        @overload
        def sqrt(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vsqrt_f32 (float32x2_t a)  A64: FSQRT Vd.2S, Vn.2S"""
            ...

        @staticmethod
        @overload
        def sqrt(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vsqrtq_f64 (float64x2_t a)  A64: FSQRT Vd.2D, Vn.2D"""
            ...

        @staticmethod
        @overload
        def sqrt(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vsqrtq_f32 (float32x4_t a)  A64: FSQRT Vd.4S, Vn.4S"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST1 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST1 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[float], value_2: System.Runtime.Intrinsics.Vector64[float]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[float], value_2: System.Runtime.Intrinsics.Vector64[float]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[float], value_2: System.Runtime.Intrinsics.Vector128[float]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[float], value_2: System.Runtime.Intrinsics.Vector128[float]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[float], value_2: System.Runtime.Intrinsics.Vector64[float]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[float], value_2: System.Runtime.Intrinsics.Vector64[float]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP Dt1, Dt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[float], value_2: System.Runtime.Intrinsics.Vector128[float]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[float], value_2: System.Runtime.Intrinsics.Vector128[float]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector128[int], value_2: System.Runtime.Intrinsics.Vector128[int]) -> None:
            """A64: STNP Qt1, Qt2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_scalar(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_scalar(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[float], value_2: System.Runtime.Intrinsics.Vector64[float]) -> None:
            """A64: STP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_scalar(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_scalar_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_scalar_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[float], value_2: System.Runtime.Intrinsics.Vector64[float]) -> None:
            """A64: STNP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_pair_scalar_non_temporal(address: typing.Any, value_1: System.Runtime.Intrinsics.Vector64[int], value_2: System.Runtime.Intrinsics.Vector64[int]) -> None:
            """A64: STNP St1, St2, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst2_lane_s8 (int8_t * ptr, int8x16x2_t val, const int lane)  A64: ST2 { Vt.16B, Vt+1.16B }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst2_lane_s8 (int8_t * ptr, int8x16x2_t val, const int lane)  A64: ST2 { Vt.16B, Vt+1.16B }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst2_lane_s16 (int16_t * ptr, int16x8x2_t val, const int lane)  A64: ST2 { Vt.8H, Vt+1.8H }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst2_lane_s16 (int16_t * ptr, int16x8x2_t val, const int lane)  A64: ST2 { Vt.8H, Vt+1.8H }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst2_lane_s32 (int32_t * ptr, int32x4x2_t val, const int lane)  A64: ST2 { Vt.4S, Vt+1.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst2_lane_s32 (int32_t * ptr, int32x4x2_t val, const int lane)  A64: ST2 { Vt.4S, Vt+1.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """A64: ST2 { Vt.2D, Vt+1.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """A64: ST2 { Vt.2D, Vt+1.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int) -> None:
            """void vst2_lane_f32 (float32_t * ptr, float32x2x2_t val, const int lane)  A64: ST2 { Vt.4S, Vt+1.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int) -> None:
            """A64: ST2 { Vt.2D, Vt+1.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst3_lane_s8 (int8_t * ptr, int8x16x3_t val, const int lane)  A64: ST3 { Vt.16B, Vt+1.16B, Vt+2.16B }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst3_lane_s8 (int8_t * ptr, int8x16x3_t val, const int lane)  A64: ST3 { Vt.16B, Vt+1.16B, Vt+2.16B }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst3_lane_s16 (int16_t * ptr, int16x8x3_t val, const int lane)  A64: ST3 { Vt.8H, Vt+1.8H, Vt+2.8H }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst3_lane_s16 (int16_t * ptr, int16x8x3_t val, const int lane)  A64: ST3 { Vt.8H, Vt+1.8H, Vt+2.8H }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst3_lane_s32 (int32_t * ptr, int32x4x3_t val, const int lane)  A64: ST3 { Vt.4S, Vt+1.4S, Vt+2.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst3_lane_s32 (int32_t * ptr, int32x4x3_t val, const int lane)  A64: ST3 { Vt.4S, Vt+1.4S, Vt+2.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """A64: ST3 { Vt.2D, Vt+1.2D, Vt+2.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """A64: ST3 { Vt.2D, Vt+1.2D, Vt+2.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int) -> None:
            """void vst3_lane_f32 (float32_t * ptr, float32x2x3_t val, const int lane)  A64: ST3 { Vt.4S, Vt+1.4S, Vt+2.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int) -> None:
            """A64: ST3 { Vt.2D, Vt+1.2D, Vt+2.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst4_lane_s8 (int8_t * ptr, int8x16x4_t val, const int lane)  A64: ST4 { Vt.16B, Vt+1.16B, Vt+2.16B, Vt+3.16B }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst4_lane_s8 (int8_t * ptr, int8x16x4_t val, const int lane)  A64: ST4 { Vt.16B, Vt+1.16B, Vt+2.16B, Vt+3.16B }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst4_lane_s16 (int16_t * ptr, int16x8x4_t val, const int lane)  A64: ST4 { Vt.8H, Vt+1.8H, Vt+2.8H, Vt+3.8H }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst4_lane_s16 (int16_t * ptr, int16x8x4_t val, const int lane)  A64: ST4 { Vt.8H, Vt+1.8H, Vt+2.8H, Vt+3.8H }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst4_lane_s32 (int32_t * ptr, int32x4x4_t val, const int lane)  A64: ST4 { Vt.4S, Vt+1.4S, Vt+2.4S, Vt+3.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """void vst4_lane_s32 (int32_t * ptr, int32x4x4_t val, const int lane)  A64: ST4 { Vt.4S, Vt+1.4S, Vt+2.4S, Vt+3.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """A64: ST4 { Vt.2D, Vt+1.2D, Vt+2.2D, Vt+3.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], index: int) -> None:
            """A64: ST4 { Vt.2D, Vt+1.2D, Vt+2.2D, Vt+3.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int) -> None:
            """void vst4_lane_f32 (float32_t * ptr, float32x2x4_t val, const int lane)  A64: ST4 { Vt.4S, Vt+1.4S, Vt+2.4S, Vt+3.4S }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]], index: int) -> None:
            """A64: ST4 { Vt.2D, Vt+1.2D, Vt+2.2D, Vt+3.2D }[index], [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.16B, Vn+1.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.8H, Vn+1.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST2 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST2 { Vn.4S, Vn+1.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST2 { Vn.2D, Vn+1.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.16B, Vn+1.16B, Vn+2.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.8H, Vn+1.8H, Vn+2.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST3 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST3 { Vn.4S, Vn+1.4S, Vn+2.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST3 { Vn.2D, Vn+1.2D, Vn+2.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.8H, Vn+1.8H, Vn+2.8H, Vn+3.8H }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]]) -> None:
            """A64: ST4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.4S }, [Xn]"""
            ...

        @staticmethod
        @overload
        def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float], System.Runtime.Intrinsics.Vector128[float]]) -> None:
            """A64: ST4 { Vn.2D, Vn+1.2D, Vn+2.2D, Vn+3.2D }, [Xn]"""
            ...

        @staticmethod
        def subtract(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vsubq_f64 (float64x2_t a, float64x2_t b)  A64: FSUB Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8_t vqsubb_u8 (uint8_t a, uint8_t b)  A64: UQSUB Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqsubh_s16 (int16_t a, int16_t b)  A64: SQSUB Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqsubs_s32 (int32_t a, int32_t b)  A64: SQSUB Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8_t vqsubb_s8 (int8_t a, int8_t b)  A64: SQSUB Bd, Bn, Bm"""
            ...

        @staticmethod
        @overload
        def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16_t vqsubh_u16 (uint16_t a, uint16_t b)  A64: UQSUB Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32_t vqsubs_u32 (uint32_t a, uint32_t b)  A64: UQSUB Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vtrn1_u8(uint8x8_t a, uint8x8_t b)  A64: TRN1 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vtrn1_s16(int16x4_t a, int16x4_t b)  A64: TRN1 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vtrn1_s32(int32x2_t a, int32x2_t b)  A64: TRN1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vtrn1_s8(int8x8_t a, int8x8_t b)  A64: TRN1 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vtrn1_f32(float32x2_t a, float32x2_t b)  A64: TRN1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vtrn1_u16(uint16x4_t a, uint16x4_t b)  A64: TRN1 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vtrn1_u32(uint32x2_t a, uint32x2_t b)  A64: TRN1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vtrn1q_u8(uint8x16_t a, uint8x16_t b)  A64: TRN1 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vtrn1q_f64(float64x2_t a, float64x2_t b)  A64: TRN1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vtrn1q_s16(int16x8_t a, int16x8_t b)  A64: TRN1 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vtrn1q_s32(int32x4_t a, int32x4_t b)  A64: TRN1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vtrn1q_s64(int64x2_t a, int64x2_t b)  A64: TRN1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vtrn1q_u8(int8x16_t a, int8x16_t b)  A64: TRN1 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vtrn1q_f32(float32x4_t a, float32x4_t b)  A64: TRN1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vtrn1q_u16(uint16x8_t a, uint16x8_t b)  A64: TRN1 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vtrn1q_u32(uint32x4_t a, uint32x4_t b)  A64: TRN1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def transpose_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vtrn1q_u64(uint64x2_t a, uint64x2_t b)  A64: TRN1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vtrn2_u8(uint8x8_t a, uint8x8_t b)  A64: TRN2 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vtrn2_s16(int16x4_t a, int16x4_t b)  A64: TRN2 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vtrn2_s32(int32x2_t a, int32x2_t b)  A64: TRN2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vtrn2_s8(int8x8_t a, int8x8_t b)  A64: TRN2 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vtrn2_f32(float32x2_t a, float32x2_t b)  A64: TRN2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vtrn2_u16(uint16x4_t a, uint16x4_t b)  A64: TRN2 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vtrn2_u32(uint32x2_t a, uint32x2_t b)  A64: TRN2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vtrn2q_u8(uint8x16_t a, uint8x16_t b)  A64: TRN2 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vtrn2q_f64(float64x2_t a, float64x2_t b)  A64: TRN2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vtrn2q_s16(int16x8_t a, int16x8_t b)  A64: TRN2 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vtrn2q_s32(int32x4_t a, int32x4_t b)  A64: TRN2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vtrn2q_s64(int64x2_t a, int64x2_t b)  A64: TRN2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vtrn2q_u8(int8x16_t a, int8x16_t b)  A64: TRN2 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vtrn2q_f32(float32x4_t a, float32x4_t b)  A64: TRN2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vtrn2q_u16(uint16x8_t a, uint16x8_t b)  A64: TRN2 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vtrn2q_u32(uint32x4_t a, uint32x4_t b)  A64: TRN2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def transpose_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vtrn2q_u64(uint64x2_t a, uint64x2_t b)  A64: TRN2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vuzp1_u8(uint8x8_t a, uint8x8_t b)  A64: UZP1 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vuzp1_s16(int16x4_t a, int16x4_t b)  A64: UZP1 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vuzp1_s32(int32x2_t a, int32x2_t b)  A64: UZP1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vuzp1_s8(int8x8_t a, int8x8_t b)  A64: UZP1 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vuzp1_f32(float32x2_t a, float32x2_t b)  A64: UZP1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vuzp1_u16(uint16x4_t a, uint16x4_t b)  A64: UZP1 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vuzp1_u32(uint32x2_t a, uint32x2_t b)  A64: UZP1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vuzp1q_u8(uint8x16_t a, uint8x16_t b)  A64: UZP1 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vuzp1q_f64(float64x2_t a, float64x2_t b)  A64: UZP1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vuzp1q_s16(int16x8_t a, int16x8_t b)  A64: UZP1 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vuzp1q_s32(int32x4_t a, int32x4_t b)  A64: UZP1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vuzp1q_s64(int64x2_t a, int64x2_t b)  A64: UZP1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vuzp1q_u8(int8x16_t a, int8x16_t b)  A64: UZP1 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vuzp1q_f32(float32x4_t a, float32x4_t b)  A64: UZP1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vuzp1q_u16(uint16x8_t a, uint16x8_t b)  A64: UZP1 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vuzp1q_u32(uint32x4_t a, uint32x4_t b)  A64: UZP1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def unzip_even(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vuzp1q_u64(uint64x2_t a, uint64x2_t b)  A64: UZP1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vuzp2_u8(uint8x8_t a, uint8x8_t b)  A64: UZP2 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vuzp2_s16(int16x4_t a, int16x4_t b)  A64: UZP2 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vuzp2_s32(int32x2_t a, int32x2_t b)  A64: UZP2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vuzp2_s8(int8x8_t a, int8x8_t b)  A64: UZP2 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vuzp2_f32(float32x2_t a, float32x2_t b)  A64: UZP2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vuzp2_u16(uint16x4_t a, uint16x4_t b)  A64: UZP2 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vuzp2_u32(uint32x2_t a, uint32x2_t b)  A64: UZP2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vuzp2q_u8(uint8x16_t a, uint8x16_t b)  A64: UZP2 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vuzp2q_f64(float64x2_t a, float64x2_t b)  A64: UZP2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vuzp2q_s16(int16x8_t a, int16x8_t b)  A64: UZP2 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vuzp2q_s32(int32x4_t a, int32x4_t b)  A64: UZP2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vuzp2q_s64(int64x2_t a, int64x2_t b)  A64: UZP2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vuzp2q_u8(int8x16_t a, int8x16_t b)  A64: UZP2 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vuzp2q_f32(float32x4_t a, float32x4_t b)  A64: UZP2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vuzp2q_u16(uint16x8_t a, uint16x8_t b)  A64: UZP2 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vuzp2q_u32(uint32x4_t a, uint32x4_t b)  A64: UZP2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def unzip_odd(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vuzp2q_u64(uint64x2_t a, uint64x2_t b)  A64: UZP2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqvtbl1q_u8(uint8x16_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqvtbl1q_s8(int8x16_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqtbl2q_u8(uint8x16x2_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B, Vn+1.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqtbl2q_s8(int8x16x2_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B, Vn+1.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqtbl3q_u8(uint8x16x3_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqtbl3q_s8(int8x16x3_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqtbl4q_u8(uint8x16x4_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqtbl4q_s8(int8x16x4_t t, uint8x16_t idx)  A64: TBL Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqvtbx1q_u8(uint8x16_t r, int8x16_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqvtbx1q_s8(int8x16_t r, int8x16_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqtbx2q_u8(uint8x16x2_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B, Vn+1.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqtbx2q_s8(int8x16x2_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B, Vn+1.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqtbx3q_u8(uint8x16x3_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqtbx3q_s8(int8x16x3_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vqtbx4q_u8(uint8x16x4_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector128[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vqtbx4q_s8(int8x16x4_t t, uint8x16_t idx)  A64: TBX Vd.16B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.16B"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vzip2_u8(uint8x8_t a, uint8x8_t b)  A64: ZIP2 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vzip2_s16(int16x4_t a, int16x4_t b)  A64: ZIP2 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vzip2_s32(int32x2_t a, int32x2_t b)  A64: ZIP2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vzip2_s8(int8x8_t a, int8x8_t b)  A64: ZIP2 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vzip2_f32(float32x2_t a, float32x2_t b)  A64: ZIP2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vzip2_u16(uint16x4_t a, uint16x4_t b)  A64: ZIP2 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vzip2_u32(uint32x2_t a, uint32x2_t b)  A64: ZIP2 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vzip2q_u8(uint8x16_t a, uint8x16_t b)  A64: ZIP2 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vzip2q_f64(float64x2_t a, float64x2_t b)  A64: ZIP2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vzip2q_s16(int16x8_t a, int16x8_t b)  A64: ZIP2 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vzip2q_s32(int32x4_t a, int32x4_t b)  A64: ZIP2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vzip2q_s64(int64x2_t a, int64x2_t b)  A64: ZIP2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vzip2q_u8(int8x16_t a, int8x16_t b)  A64: ZIP2 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vzip2q_f32(float32x4_t a, float32x4_t b)  A64: ZIP2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vzip2q_u16(uint16x8_t a, uint16x8_t b)  A64: ZIP2 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vzip2q_u32(uint32x4_t a, uint32x4_t b)  A64: ZIP2 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def zip_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vzip2q_u64(uint64x2_t a, uint64x2_t b)  A64: ZIP2 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint8x8_t vzip1_u8(uint8x8_t a, uint8x8_t b)  A64: ZIP1 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16x4_t vzip1_s16(int16x4_t a, int16x4_t b)  A64: ZIP1 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32x2_t vzip1_s32(int32x2_t a, int32x2_t b)  A64: ZIP1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int8x8_t vzip1_s8(int8x8_t a, int8x8_t b)  A64: ZIP1 Vd.8B, Vn.8B, Vm.8B"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
            """float32x2_t vzip1_f32(float32x2_t a, float32x2_t b)  A64: ZIP1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint16x4_t vzip1_u16(uint16x4_t a, uint16x4_t b)  A64: ZIP1 Vd.4H, Vn.4H, Vm.4H"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """uint32x2_t vzip1_u32(uint32x2_t a, uint32x2_t b)  A64: ZIP1 Vd.2S, Vn.2S, Vm.2S"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint8x16_t vzip1q_u8(uint8x16_t a, uint8x16_t b)  A64: ZIP1 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float64x2_t vzip1q_f64(float64x2_t a, float64x2_t b)  A64: ZIP1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int16x8_t vzip1q_s16(int16x8_t a, int16x8_t b)  A64: ZIP1 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int32x4_t vzip1q_s32(int32x4_t a, int32x4_t b)  A64: ZIP1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int64x2_t vzip1q_s64(int64x2_t a, int64x2_t b)  A64: ZIP1 Vd.2D, Vn.2D, Vm.2D"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """int8x16_t vzip1q_u8(int8x16_t a, int8x16_t b)  A64: ZIP1 Vd.16B, Vn.16B, Vm.16B"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
            """float32x4_t vzip1q_f32(float32x4_t a, float32x4_t b)  A64: ZIP1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint16x8_t vzip1q_u16(uint16x8_t a, uint16x8_t b)  A64: ZIP1 Vd.8H, Vn.8H, Vm.8H"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint32x4_t vzip1q_u32(uint32x4_t a, uint32x4_t b)  A64: ZIP1 Vd.4S, Vn.4S, Vm.4S"""
            ...

        @staticmethod
        @overload
        def zip_low(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
            """uint64x2_t vzip1q_u64(uint64x2_t a, uint64x2_t b)  A64: ZIP1 Vd.2D, Vn.2D, Vm.2D"""
            ...

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vabs_s16 (int16x4_t a)  A32: VABS.S16 Dd, Dm  A64: ABS Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vabs_s32 (int32x2_t a)  A32: VABS.S32 Dd, Dm  A64: ABS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vabs_s8 (int8x8_t a)  A32: VABS.S8 Dd, Dm  A64: ABS Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vabs_f32 (float32x2_t a)  A32: VABS.F32 Dd, Dm  A64: FABS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vabsq_s16 (int16x8_t a)  A32: VABS.S16 Qd, Qm  A64: ABS Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vabsq_s32 (int32x4_t a)  A32: VABS.S32 Qd, Qm  A64: ABS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vabsq_s8 (int8x16_t a)  A32: VABS.S8 Qd, Qm  A64: ABS Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vabsq_f32 (float32x4_t a)  A32: VABS.F32 Qd, Qm  A64: FABS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vcagt_f32 (float32x2_t a, float32x2_t b)  A32: VACGT.F32 Dd, Dn, Dm  A64: FACGT Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcagtq_f32 (float32x4_t a, float32x4_t b)  A32: VACGT.F32 Qd, Qn, Qm  A64: FACGT Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vcage_f32 (float32x2_t a, float32x2_t b)  A32: VACGE.F32 Dd, Dn, Dm  A64: FACGE Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcageq_f32 (float32x4_t a, float32x4_t b)  A32: VACGE.F32 Qd, Qn, Qm  A64: FACGE Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vcalt_f32 (float32x2_t a, float32x2_t b)  A32: VACLT.F32 Dd, Dn, Dm  A64: FACGT Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcaltq_f32 (float32x4_t a, float32x4_t b)  A32: VACLT.F32 Qd, Qn, Qm  A64: FACGT Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vcale_f32 (float32x2_t a, float32x2_t b)  A32: VACLE.F32 Dd, Dn, Dm  A64: FACGE Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcaleq_f32 (float32x4_t a, float32x4_t b)  A32: VACLE.F32 Qd, Qn, Qm  A64: FACGE Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vabd_u8 (uint8x8_t a, uint8x8_t b)  A32: VABD.U8 Dd, Dn, Dm  A64: UABD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vabd_s16 (int16x4_t a, int16x4_t b)  A32: VABD.S16 Dd, Dn, Dm  A64: SABD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vabd_s32 (int32x2_t a, int32x2_t b)  A32: VABD.S32 Dd, Dn, Dm  A64: SABD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vabd_s8 (int8x8_t a, int8x8_t b)  A32: VABD.S8 Dd, Dn, Dm  A64: SABD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vabd_f32 (float32x2_t a, float32x2_t b)  A32: VABD.F32 Dd, Dn, Dm  A64: FABD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vabd_u16 (uint16x4_t a, uint16x4_t b)  A32: VABD.U16 Dd, Dn, Dm  A64: UABD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vabd_u32 (uint32x2_t a, uint32x2_t b)  A32: VABD.U32 Dd, Dn, Dm  A64: UABD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vabdq_u8 (uint8x16_t a, uint8x16_t b)  A32: VABD.U8 Qd, Qn, Qm  A64: UABD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vabdq_s16 (int16x8_t a, int16x8_t b)  A32: VABD.S16 Qd, Qn, Qm  A64: SABD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vabdq_s32 (int32x4_t a, int32x4_t b)  A32: VABD.S32 Qd, Qn, Qm  A64: SABD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vabdq_s8 (int8x16_t a, int8x16_t b)  A32: VABD.S8 Qd, Qn, Qm  A64: SABD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vabdq_f32 (float32x4_t a, float32x4_t b)  A32: VABD.F32 Qd, Qn, Qm  A64: FABD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vabdq_u16 (uint16x8_t a, uint16x8_t b)  A32: VABD.U16 Qd, Qn, Qm  A64: UABD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vabdq_u32 (uint32x4_t a, uint32x4_t b)  A32: VABD.U32 Qd, Qn, Qm  A64: UABD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vaba_u8 (uint8x8_t a, uint8x8_t b, uint8x8_t c)  A32: VABA.U8 Dd, Dn, Dm  A64: UABA Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vaba_s16 (int16x4_t a, int16x4_t b, int16x4_t c)  A32: VABA.S16 Dd, Dn, Dm  A64: SABA Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vaba_s32 (int32x2_t a, int32x2_t b, int32x2_t c)  A32: VABA.S32 Dd, Dn, Dm  A64: SABA Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vaba_s8 (int8x8_t a, int8x8_t b, int8x8_t c)  A32: VABA.S8 Dd, Dn, Dm  A64: SABA Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vaba_u16 (uint16x4_t a, uint16x4_t b, uint16x4_t c)  A32: VABA.U16 Dd, Dn, Dm  A64: UABA Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vaba_u32 (uint32x2_t a, uint32x2_t b, uint32x2_t c)  A32: VABA.U32 Dd, Dn, Dm  A64: UABA Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vabaq_u8 (uint8x16_t a, uint8x16_t b, uint8x16_t c)  A32: VABA.U8 Qd, Qn, Qm  A64: UABA Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vabaq_s16 (int16x8_t a, int16x8_t b, int16x8_t c)  A32: VABA.S16 Qd, Qn, Qm  A64: SABA Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vabaq_s32 (int32x4_t a, int32x4_t b, int32x4_t c)  A32: VABA.S32 Qd, Qn, Qm  A64: SABA Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vabaq_s8 (int8x16_t a, int8x16_t b, int8x16_t c)  A32: VABA.S8 Qd, Qn, Qm  A64: SABA Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vabaq_u16 (uint16x8_t a, uint16x8_t b, uint16x8_t c)  A32: VABA.U16 Qd, Qn, Qm  A64: UABA Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vabaq_u32 (uint32x4_t a, uint32x4_t b, uint32x4_t c)  A32: VABA.U32 Qd, Qn, Qm  A64: UABA Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vabdl_u8 (uint8x8_t a, uint8x8_t b)  A32: VABDL.U8 Qd, Dn, Dm  A64: UABDL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vabdl_s16 (int16x4_t a, int16x4_t b)  A32: VABDL.S16 Qd, Dn, Dm  A64: SABDL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vabdl_s32 (int32x2_t a, int32x2_t b)  A32: VABDL.S32 Qd, Dn, Dm  A64: SABDL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vabdl_s8 (int8x8_t a, int8x8_t b)  A32: VABDL.S8 Qd, Dn, Dm  A64: SABDL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vabdl_u16 (uint16x4_t a, uint16x4_t b)  A32: VABDL.U16 Qd, Dn, Dm  A64: UABDL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vabdl_u32 (uint32x2_t a, uint32x2_t b)  A32: VABDL.U32 Qd, Dn, Dm  A64: UABDL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vabal_u8 (uint16x8_t a, uint8x8_t b, uint8x8_t c)  A32: VABAL.U8 Qd, Dn, Dm  A64: UABAL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vabal_s16 (int32x4_t a, int16x4_t b, int16x4_t c)  A32: VABAL.S16 Qd, Dn, Dm  A64: SABAL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vabal_s32 (int64x2_t a, int32x2_t b, int32x2_t c)  A32: VABAL.S32 Qd, Dn, Dm  A64: SABAL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vabal_s8 (int16x8_t a, int8x8_t b, int8x8_t c)  A32: VABAL.S8 Qd, Dn, Dm  A64: SABAL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vabal_u16 (uint32x4_t a, uint16x4_t b, uint16x4_t c)  A32: VABAL.U16 Qd, Dn, Dm  A64: UABAL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vabal_u32 (uint64x2_t a, uint32x2_t b, uint32x2_t c)  A32: VABAL.U32 Qd, Dn, Dm  A64: UABAL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vabdl_high_u8 (uint8x16_t a, uint8x16_t b)  A32: VABDL.U8 Qd, Dn+1, Dm+1  A64: UABDL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vabdl_high_s16 (int16x8_t a, int16x8_t b)  A32: VABDL.S16 Qd, Dn+1, Dm+1  A64: SABDL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vabdl_high_s32 (int32x4_t a, int32x4_t b)  A32: VABDL.S32 Qd, Dn+1, Dm+1  A64: SABDL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vabdl_high_s8 (int8x16_t a, int8x16_t b)  A32: VABDL.S8 Qd, Dn+1, Dm+1  A64: SABDL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vabdl_high_u16 (uint16x8_t a, uint16x8_t b)  A32: VABDL.U16 Qd, Dn+1, Dm+1  A64: UABDL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vabdl_high_u32 (uint32x4_t a, uint32x4_t b)  A32: VABDL.U32 Qd, Dn+1, Dm+1  A64: UABDL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vabal_high_u8 (uint16x8_t a, uint8x16_t b, uint8x16_t c)  A32: VABAL.U8 Qd, Dn+1, Dm+1  A64: UABAL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vabal_high_s16 (int32x4_t a, int16x8_t b, int16x8_t c)  A32: VABAL.S16 Qd, Dn+1, Dm+1  A64: SABAL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vabal_high_s32 (int64x2_t a, int32x4_t b, int32x4_t c)  A32: VABAL.S32 Qd, Dn+1, Dm+1  A64: SABAL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vabal_high_s8 (int16x8_t a, int8x16_t b, int8x16_t c)  A32: VABAL.S8 Qd, Dn+1, Dm+1  A64: SABAL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vabal_high_u16 (uint32x4_t a, uint16x8_t b, uint16x8_t c)  A32: VABAL.U16 Qd, Dn+1, Dm+1  A64: UABAL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def absolute_difference_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vabal_high_u32 (uint64x2_t a, uint32x4_t b, uint32x4_t c)  A32: VABAL.U32 Qd, Dn+1, Dm+1  A64: UABAL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def abs_saturate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqabs_s16 (int16x4_t a)  A32: VQABS.S16 Dd, Dm  A64: SQABS Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def abs_saturate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqabs_s32 (int32x2_t a)  A32: VQABS.S32 Dd, Dm  A64: SQABS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def abs_saturate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqabs_s8 (int8x8_t a)  A32: VQABS.S8 Dd, Dm  A64: SQABS Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def abs_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqabsq_s16 (int16x8_t a)  A32: VQABS.S16 Qd, Qm  A64: SQABS Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def abs_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqabsq_s32 (int32x4_t a)  A32: VQABS.S32 Qd, Qm  A64: SQABS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def abs_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqabsq_s8 (int8x16_t a)  A32: VQABS.S8 Qd, Qm  A64: SQABS Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def abs_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vabs_f64 (float64x1_t a)  A32: VABS.F64 Dd, Dm  A64: FABS Dd, Dn"""
        ...

    @staticmethod
    @overload
    def abs_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vabss_f32 (float32_t a)  A32: VABS.F32 Sd, Sm  A64: FABS Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vadd_u8 (uint8x8_t a, uint8x8_t b)  A32: VADD.I8 Dd, Dn, Dm  A64: ADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vadd_s16 (int16x4_t a, int16x4_t b)  A32: VADD.I16 Dd, Dn, Dm  A64: ADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vadd_s32 (int32x2_t a, int32x2_t b)  A32: VADD.I32 Dd, Dn, Dm  A64: ADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vadd_s8 (int8x8_t a, int8x8_t b)  A32: VADD.I8 Dd, Dn, Dm  A64: ADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vadd_f32 (float32x2_t a, float32x2_t b)  A32: VADD.F32 Dd, Dn, Dm  A64: FADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vadd_u16 (uint16x4_t a, uint16x4_t b)  A32: VADD.I16 Dd, Dn, Dm  A64: ADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vadd_u32 (uint32x2_t a, uint32x2_t b)  A32: VADD.I32 Dd, Dn, Dm  A64: ADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaddq_u8 (uint8x16_t a, uint8x16_t b)  A32: VADD.I8 Qd, Qn, Qm  A64: ADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vaddq_s16 (int16x8_t a, int16x8_t b)  A32: VADD.I16 Qd, Qn, Qm  A64: ADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vaddq_s32 (int32x4_t a, int32x4_t b)  A32: VADD.I32 Qd, Qn, Qm  A64: ADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vaddq_s64 (int64x2_t a, int64x2_t b)  A32: VADD.I64 Qd, Qn, Qm  A64: ADD Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vaddq_s8 (int8x16_t a, int8x16_t b)  A32: VADD.I8 Qd, Qn, Qm  A64: ADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vaddq_f32 (float32x4_t a, float32x4_t b)  A32: VADD.F32 Qd, Qn, Qm  A64: FADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vaddq_u16 (uint16x8_t a, uint16x8_t b)  A32: VADD.I16 Qd, Qn, Qm  A64: ADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vaddq_u32 (uint32x4_t a, uint32x4_t b)  A32: VADD.I32 Qd, Qn, Qm  A64: ADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vaddq_u64 (uint64x2_t a, uint64x2_t b)  A32: VADD.I64 Qd, Qn, Qm  A64: ADD Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vaddhn_u16 (uint16x8_t a, uint16x8_t b)  A32: VADDHN.I16 Dd, Qn, Qm  A64: ADDHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vaddhn_s32 (int32x4_t a, int32x4_t b)  A32: VADDHN.I32 Dd, Qn, Qm  A64: ADDHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vaddhn_s64 (int64x2_t a, int64x2_t b)  A32: VADDHN.I64 Dd, Qn, Qm  A64: ADDHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vaddhn_s16 (int16x8_t a, int16x8_t b)  A32: VADDHN.I16 Dd, Qn, Qm  A64: ADDHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vaddhn_u32 (uint32x4_t a, uint32x4_t b)  A32: VADDHN.I32 Dd, Qn, Qm  A64: ADDHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vaddhn_u64 (uint64x2_t a, uint64x2_t b)  A32: VADDHN.I64 Dd, Qn, Qm  A64: ADDHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vaddhn_high_u16 (uint8x8_t r, uint16x8_t a, uint16x8_t b)  A32: VADDHN.I16 Dd+1, Qn, Qm  A64: ADDHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vaddhn_high_s32 (int16x4_t r, int32x4_t a, int32x4_t b)  A32: VADDHN.I32 Dd+1, Qn, Qm  A64: ADDHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vaddhn_high_s64 (int32x2_t r, int64x2_t a, int64x2_t b)  A32: VADDHN.I64 Dd+1, Qn, Qm  A64: ADDHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vaddhn_high_s16 (int8x8_t r, int16x8_t a, int16x8_t b)  A32: VADDHN.I16 Dd+1, Qn, Qm  A64: ADDHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vaddhn_high_u32 (uint16x4_t r, uint32x4_t a, uint32x4_t b)  A32: VADDHN.I32 Dd+1, Qn, Qm  A64: ADDHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vaddhn_high_u64 (uint32x2_t r, uint64x2_t a, uint64x2_t b)  A32: VADDHN.I64 Dd+1, Qn, Qm  A64: ADDHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vpadd_u8 (uint8x8_t a, uint8x8_t b)  A32: VPADD.I8 Dd, Dn, Dm  A64: ADDP Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vpadd_s16 (int16x4_t a, int16x4_t b)  A32: VPADD.I16 Dd, Dn, Dm  A64: ADDP Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vpadd_s32 (int32x2_t a, int32x2_t b)  A32: VPADD.I32 Dd, Dn, Dm  A64: ADDP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vpadd_s8 (int8x8_t a, int8x8_t b)  A32: VPADD.I8 Dd, Dn, Dm  A64: ADDP Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_pairwise(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vpadd_f32 (float32x2_t a, float32x2_t b)  A32: VPADD.F32 Dd, Dn, Dm  A64: FADDP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vpadd_u16 (uint16x4_t a, uint16x4_t b)  A32: VPADD.I16 Dd, Dn, Dm  A64: ADDP Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vpadd_u32 (uint32x2_t a, uint32x2_t b)  A32: VPADD.I32 Dd, Dn, Dm  A64: ADDP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vpaddl_u8 (uint8x8_t a)  A32: VPADDL.U8 Dd, Dm  A64: UADDLP Vd.4H, Vn.8B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vpaddl_s16 (int16x4_t a)  A32: VPADDL.S16 Dd, Dm  A64: SADDLP Vd.2S, Vn.4H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vpaddl_s8 (int8x8_t a)  A32: VPADDL.S8 Dd, Dm  A64: SADDLP Vd.4H, Vn.8B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vpaddl_u16 (uint16x4_t a)  A32: VPADDL.U16 Dd, Dm  A64: UADDLP Vd.2S, Vn.4H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vpaddlq_u8 (uint8x16_t a)  A32: VPADDL.U8 Qd, Qm  A64: UADDLP Vd.8H, Vn.16B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vpaddlq_s16 (int16x8_t a)  A32: VPADDL.S16 Qd, Qm  A64: SADDLP Vd.4S, Vn.8H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vpaddlq_s32 (int32x4_t a)  A32: VPADDL.S32 Qd, Qm  A64: SADDLP Vd.2D, Vn.4S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vpaddlq_s8 (int8x16_t a)  A32: VPADDL.S8 Qd, Qm  A64: SADDLP Vd.8H, Vn.16B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vpaddlq_u16 (uint16x8_t a)  A32: VPADDL.U16 Qd, Qm  A64: UADDLP Vd.4S, Vn.8H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vpaddlq_u32 (uint32x4_t a)  A32: VPADDL.U32 Qd, Qm  A64: UADDLP Vd.2D, Vn.4S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vpadal_u8 (uint16x4_t a, uint8x8_t b)  A32: VPADAL.U8 Dd, Dm  A64: UADALP Vd.4H, Vn.8B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vpadal_s16 (int32x2_t a, int16x4_t b)  A32: VPADAL.S16 Dd, Dm  A64: SADALP Vd.2S, Vn.4H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vpadal_s8 (int16x4_t a, int8x8_t b)  A32: VPADAL.S8 Dd, Dm  A64: SADALP Vd.4H, Vn.8B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vpadal_u16 (uint32x2_t a, uint16x4_t b)  A32: VPADAL.U16 Dd, Dm  A64: UADALP Vd.2S, Vn.4H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vpadalq_u8 (uint16x8_t a, uint8x16_t b)  A32: VPADAL.U8 Qd, Qm  A64: UADALP Vd.8H, Vn.16B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vpadalq_s16 (int32x4_t a, int16x8_t b)  A32: VPADAL.S16 Qd, Qm  A64: SADALP Vd.4S, Vn.8H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vpadalq_s32 (int64x2_t a, int32x4_t b)  A32: VPADAL.S32 Qd, Qm  A64: SADALP Vd.2D, Vn.4S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vpadalq_s8 (int16x8_t a, int8x16_t b)  A32: VPADAL.S8 Qd, Qm  A64: SADALP Vd.8H, Vn.16B"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vpadalq_u16 (uint32x4_t a, uint16x8_t b)  A32: VPADAL.U16 Qd, Qm  A64: UADALP Vd.4S, Vn.8H"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vpadalq_u32 (uint64x2_t a, uint32x4_t b)  A32: VPADAL.U32 Qd, Qm  A64: UADALP Vd.2D, Vn.4S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vpadal_s32 (int64x1_t a, int32x2_t b)  A32: VPADAL.S32 Dd, Dm  A64: SADALP Vd.1D, Vn.2S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_and_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vpadal_u32 (uint64x1_t a, uint32x2_t b)  A32: VPADAL.U32 Dd, Dm  A64: UADALP Vd.1D, Vn.2S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vpaddl_s32 (int32x2_t a)  A32: VPADDL.S32 Dd, Dm  A64: SADDLP Dd, Vn.2S"""
        ...

    @staticmethod
    @overload
    def add_pairwise_widening_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vpaddl_u32 (uint32x2_t a)  A32: VPADDL.U32 Dd, Dm  A64: UADDLP Dd, Vn.2S"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vraddhn_u16 (uint16x8_t a, uint16x8_t b)  A32: VRADDHN.I16 Dd, Qn, Qm  A64: RADDHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vraddhn_s32 (int32x4_t a, int32x4_t b)  A32: VRADDHN.I32 Dd, Qn, Qm  A64: RADDHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vraddhn_s64 (int64x2_t a, int64x2_t b)  A32: VRADDHN.I64 Dd, Qn, Qm  A64: RADDHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vraddhn_s16 (int16x8_t a, int16x8_t b)  A32: VRADDHN.I16 Dd, Qn, Qm  A64: RADDHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vraddhn_u32 (uint32x4_t a, uint32x4_t b)  A32: VRADDHN.I32 Dd, Qn, Qm  A64: RADDHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vraddhn_u64 (uint64x2_t a, uint64x2_t b)  A32: VRADDHN.I64 Dd, Qn, Qm  A64: RADDHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vraddhn_high_u16 (uint8x8_t r, uint16x8_t a, uint16x8_t b)  A32: VRADDHN.I16 Dd+1, Qn, Qm  A64: RADDHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vraddhn_high_s32 (int16x4_t r, int32x4_t a, int32x4_t b)  A32: VRADDHN.I32 Dd+1, Qn, Qm  A64: RADDHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vraddhn_high_s64 (int32x2_t r, int64x2_t a, int64x2_t b)  A32: VRADDHN.I64 Dd+1, Qn, Qm  A64: RADDHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vraddhn_high_s16 (int8x8_t r, int16x8_t a, int16x8_t b)  A32: VRADDHN.I16 Dd+1, Qn, Qm  A64: RADDHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vraddhn_high_u32 (uint16x4_t r, uint32x4_t a, uint32x4_t b)  A32: VRADDHN.I32 Dd+1, Qn, Qm  A64: RADDHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vraddhn_high_u64 (uint32x2_t r, uint64x2_t a, uint64x2_t b)  A32: VRADDHN.I64 Dd+1, Qn, Qm  A64: RADDHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqadd_u8 (uint8x8_t a, uint8x8_t b)  A32: VQADD.U8 Dd, Dn, Dm  A64: UQADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqadd_s16 (int16x4_t a, int16x4_t b)  A32: VQADD.S16 Dd, Dn, Dm  A64: SQADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqadd_s32 (int32x2_t a, int32x2_t b)  A32: VQADD.S32 Dd, Dn, Dm  A64: SQADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqadd_s8 (int8x8_t a, int8x8_t b)  A32: VQADD.S8 Dd, Dn, Dm  A64: SQADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqadd_u16 (uint16x4_t a, uint16x4_t b)  A32: VQADD.U16 Dd, Dn, Dm  A64: UQADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqadd_u32 (uint32x2_t a, uint32x2_t b)  A32: VQADD.U32 Dd, Dn, Dm  A64: UQADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqaddq_u8 (uint8x16_t a, uint8x16_t b)  A32: VQADD.U8 Qd, Qn, Qm  A64: UQADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqaddq_s16 (int16x8_t a, int16x8_t b)  A32: VQADD.S16 Qd, Qn, Qm  A64: SQADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqaddq_s32 (int32x4_t a, int32x4_t b)  A32: VQADD.S32 Qd, Qn, Qm  A64: SQADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqaddq_s64 (int64x2_t a, int64x2_t b)  A32: VQADD.S64 Qd, Qn, Qm  A64: SQADD Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqaddq_s8 (int8x16_t a, int8x16_t b)  A32: VQADD.S8 Qd, Qn, Qm  A64: SQADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqaddq_u16 (uint16x8_t a, uint16x8_t b)  A32: VQADD.U16 Qd, Qn, Qm  A64: UQADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqaddq_u32 (uint32x4_t a, uint32x4_t b)  A32: VQADD.U32 Qd, Qn, Qm  A64: UQADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqaddq_u64 (uint64x2_t a, uint64x2_t b)  A32: VQADD.U64 Qd, Qn, Qm  A64: UQADD Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vqadd_s64 (int64x1_t a, int64x1_t b)  A32: VQADD.S64 Dd, Dn, Dm  A64: SQADD Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def add_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqadd_u64 (uint64x1_t a, uint64x1_t b)  A32: VQADD.U64 Dd, Dn, Dm  A64: UQADD Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def add_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vadd_f64 (float64x1_t a, float64x1_t b)  A32: VADD.F64 Dd, Dn, Dm  A64: FADD Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def add_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vadd_s64 (int64x1_t a, int64x1_t b)  A32: VADD.I64 Dd, Dn, Dm  A64: ADD Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def add_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vadds_f32 (float32_t a, float32_t b)  A32: VADD.F32 Sd, Sn, Sm  A64: FADD Sd, Sn, SmThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def add_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vadd_u64 (uint64x1_t a, uint64x1_t b)  A32: VADD.I64 Dd, Dn, Dm  A64: ADD Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vaddl_u8 (uint8x8_t a, uint8x8_t b)  A32: VADDL.U8 Qd, Dn, Dm  A64: UADDL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vaddl_s16 (int16x4_t a, int16x4_t b)  A32: VADDL.S16 Qd, Dn, Dm  A64: SADDL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vaddl_s32 (int32x2_t a, int32x2_t b)  A32: VADDL.S32 Qd, Dn, Dm  A64: SADDL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vaddl_s8 (int8x8_t a, int8x8_t b)  A32: VADDL.S8 Qd, Dn, Dm  A64: SADDL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vaddl_u16 (uint16x4_t a, uint16x4_t b)  A32: VADDL.U16 Qd, Dn, Dm  A64: UADDL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vaddl_u32 (uint32x2_t a, uint32x2_t b)  A32: VADDL.U32 Qd, Dn, Dm  A64: UADDL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vaddw_s8 (int16x8_t a, int8x8_t b)  A32: VADDW.S8 Qd, Qn, Dm  A64: SADDW Vd.8H, Vn.8H, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vaddw_s16 (int32x4_t a, int16x4_t b)  A32: VADDW.S16 Qd, Qn, Dm  A64: SADDW Vd.4S, Vn.4S, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vaddw_s32 (int64x2_t a, int32x2_t b)  A32: VADDW.S32 Qd, Qn, Dm  A64: SADDW Vd.2D, Vn.2D, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vaddw_u8 (uint16x8_t a, uint8x8_t b)  A32: VADDW.U8 Qd, Qn, Dm  A64: UADDW Vd.8H, Vn.8H, Vm.8B"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vaddw_u16 (uint32x4_t a, uint16x4_t b)  A32: VADDW.U16 Qd, Qn, Dm  A64: UADDW Vd.4S, Vn.4S, Vm.4H"""
        ...

    @staticmethod
    @overload
    def add_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vaddw_u32 (uint64x2_t a, uint32x2_t b)  A32: VADDW.U32 Qd, Qn, Dm  A64: UADDW Vd.2D, Vn.2D, Vm.2S"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vaddl_high_u8 (uint8x16_t a, uint8x16_t b)  A32: VADDL.U8 Qd, Dn+1, Dm+1  A64: UADDL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vaddl_high_s16 (int16x8_t a, int16x8_t b)  A32: VADDL.S16 Qd, Dn+1, Dm+1  A64: SADDL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vaddw_high_s8 (int16x8_t a, int8x16_t b)  A32: VADDW.S8 Qd, Qn, Dm+1  A64: SADDW2 Vd.8H, Vn.8H, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vaddw_high_s16 (int32x4_t a, int16x8_t b)  A32: VADDW.S16 Qd, Qn, Dm+1  A64: SADDW2 Vd.4S, Vn.4S, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vaddl_high_s32 (int32x4_t a, int32x4_t b)  A32: VADDL.S32 Qd, Dn+1, Dm+1  A64: SADDL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vaddw_high_s32 (int64x2_t a, int32x4_t b)  A32: VADDW.S32 Qd, Qn, Dm+1  A64: SADDW2 Vd.2D, Vn.2D, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vaddl_high_s8 (int8x16_t a, int8x16_t b)  A32: VADDL.S8 Qd, Dn+1, Dm+1  A64: SADDL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vaddw_high_u8 (uint16x8_t a, uint8x16_t b)  A32: VADDW.U8 Qd, Qn, Dm+1  A64: UADDW2 Vd.8H, Vn.8H, Vm.16B"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vaddl_high_u16 (uint16x8_t a, uint16x8_t b)  A32: VADDL.U16 Qd, Dn+1, Dm+1  A64: UADDL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vaddw_high_u16 (uint32x4_t a, uint16x8_t b)  A32: VADDW.U16 Qd, Qn, Dm+1  A64: UADDW2 Vd.4S, Vn.4S, Vm.8H"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vaddl_high_u32 (uint32x4_t a, uint32x4_t b)  A32: VADDL.U32 Qd, Dn+1, Dm+1  A64: UADDL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def add_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vaddw_high_u32 (uint64x2_t a, uint32x4_t b)  A32: VADDW.U32 Qd, Qn, Dm+1  A64: UADDW2 Vd.2D, Vn.2D, Vm.4S"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vand_u8 (uint8x8_t a, uint8x8_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vand_f64 (float64x1_t a, float64x1_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vand_s16 (int16x4_t a, int16x4_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vand_s32 (int32x2_t a, int32x2_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vand_s64 (int64x1_t a, int64x1_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vand_s8 (int8x8_t a, int8x8_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vand_f32 (float32x2_t a, float32x2_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vand_u16 (uint16x4_t a, uint16x4_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vand_u32 (uint32x2_t a, uint32x2_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vand_u64 (uint64x1_t a, uint64x1_t b)  A32: VAND Dd, Dn, Dm  A64: AND Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vandq_u8 (uint8x16_t a, uint8x16_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vandq_f64 (float64x2_t a, float64x2_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vandq_s16 (int16x8_t a, int16x8_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vandq_s32 (int32x4_t a, int32x4_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vandq_s64 (int64x2_t a, int64x2_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vandq_s8 (int8x16_t a, int8x16_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vandq_f32 (float32x4_t a, float32x4_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vandq_u16 (uint16x8_t a, uint16x8_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vandq_u32 (uint32x4_t a, uint32x4_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def And(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vandq_u64 (uint64x2_t a, uint64x2_t b)  A32: VAND Qd, Qn, Qm  A64: AND Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vbic_u8 (uint8x8_t a, uint8x8_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[float], mask: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vbic_f64 (float64x1_t a, float64x1_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vbic_s16 (int16x4_t a, int16x4_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vbic_s32 (int32x2_t a, int32x2_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vbic_s64 (int64x1_t a, int64x1_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vbic_s8 (int8x8_t a, int8x8_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[float], mask: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vbic_f32 (float32x2_t a, float32x2_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vbic_u16 (uint16x4_t a, uint16x4_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vbic_u32 (uint32x2_t a, uint32x2_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector64[int], mask: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vbic_u64 (uint64x1_t a, uint64x1_t b)  A32: VBIC Dd, Dn, Dm  A64: BIC Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vbicq_u8 (uint8x16_t a, uint8x16_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[float], mask: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vbicq_f64 (float64x2_t a, float64x2_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vbicq_s16 (int16x8_t a, int16x8_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vbicq_s32 (int32x4_t a, int32x4_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vbicq_s64 (int64x2_t a, int64x2_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vbicq_s8 (int8x16_t a, int8x16_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[float], mask: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vbicq_f32 (float32x4_t a, float32x4_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vbicq_u16 (uint16x8_t a, uint16x8_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vbicq_u32 (uint32x4_t a, uint32x4_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(value: System.Runtime.Intrinsics.Vector128[int], mask: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vbicq_u64 (uint64x2_t a, uint64x2_t b)  A32: VBIC Qd, Qn, Qm  A64: BIC Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vbsl_u8 (uint8x8_t a, uint8x8_t b, uint8x8_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vbsl_f64 (uint64x1_t a, float64x1_t b, float64x1_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vbsl_s16 (uint16x4_t a, int16x4_t b, int16x4_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vbsl_s32 (uint32x2_t a, int32x2_t b, int32x2_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vbsl_s64 (uint64x1_t a, int64x1_t b, int64x1_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vbsl_s8 (uint8x8_t a, int8x8_t b, int8x8_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vbsl_f32 (uint32x2_t a, float32x2_t b, float32x2_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vbsl_u16 (uint16x4_t a, uint16x4_t b, uint16x4_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vbsl_u32 (uint32x2_t a, uint32x2_t b, uint32x2_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vbsl_u64 (uint64x1_t a, uint64x1_t b, uint64x1_t c)  A32: VBSL Dd, Dn, Dm  A64: BSL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vbslq_u8 (uint8x16_t a, uint8x16_t b, uint8x16_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vbslq_f64 (uint64x2_t a, float64x2_t b, float64x2_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vbslq_s16 (uint16x8_t a, int16x8_t b, int16x8_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vbslq_s32 (uint32x4_t a, int32x4_t b, int32x4_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vbslq_s64 (uint64x2_t a, int64x2_t b, int64x2_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vbslq_s8 (uint8x16_t a, int8x16_t b, int8x16_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vbslq_f32 (uint32x4_t a, float32x4_t b, float32x4_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vbslq_u16 (uint16x8_t a, uint16x8_t b, uint16x8_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vbslq_u32 (uint32x4_t a, uint32x4_t b, uint32x4_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def bitwise_select(select: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vbslq_u64 (uint64x2_t a, uint64x2_t b, uint64x2_t c)  A32: VBSL Qd, Qn, Qm  A64: BSL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def ceiling(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrndp_f32 (float32x2_t a)  A32: VRINTP.F32 Dd, Dm  A64: FRINTP Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def ceiling(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrndpq_f32 (float32x4_t a)  A32: VRINTP.F32 Qd, Qm  A64: FRINTP Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def ceiling_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vrndp_f64 (float64x1_t a)  A32: VRINTP.F64 Dd, Dm  A64: FRINTP Dd, Dn"""
        ...

    @staticmethod
    @overload
    def ceiling_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vrndps_f32 (float32_t a)  A32: VRINTP.F32 Sd, Sm  A64: FRINTP Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vceq_u8 (uint8x8_t a, uint8x8_t b)  A32: VCEQ.I8 Dd, Dn, Dm  A64: CMEQ Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vceq_s16 (int16x4_t a, int16x4_t b)  A32: VCEQ.I16 Dd, Dn, Dm  A64: CMEQ Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vceq_s32 (int32x2_t a, int32x2_t b)  A32: VCEQ.I32 Dd, Dn, Dm  A64: CMEQ Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vceq_s8 (int8x8_t a, int8x8_t b)  A32: VCEQ.I8 Dd, Dn, Dm  A64: CMEQ Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vceq_f32 (float32x2_t a, float32x2_t b)  A32: VCEQ.F32 Dd, Dn, Dm  A64: FCMEQ Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vceq_u16 (uint16x4_t a, uint16x4_t b)  A32: VCEQ.I16 Dd, Dn, Dm  A64: CMEQ Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vceq_u32 (uint32x2_t a, uint32x2_t b)  A32: VCEQ.I32 Dd, Dn, Dm  A64: CMEQ Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vceqq_u8 (uint8x16_t a, uint8x16_t b)  A32: VCEQ.I8 Qd, Qn, Qm  A64: CMEQ Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vceqq_s16 (int16x8_t a, int16x8_t b)  A32: VCEQ.I16 Qd, Qn, Qm  A64: CMEQ Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vceqq_s32 (int32x4_t a, int32x4_t b)  A32: VCEQ.I32 Qd, Qn, Qm  A64: CMEQ Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vceqq_s8 (int8x16_t a, int8x16_t b)  A32: VCEQ.I8 Qd, Qn, Qm  A64: CMEQ Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vceqq_f32 (float32x4_t a, float32x4_t b)  A32: VCEQ.F32 Qd, Qn, Qm  A64: FCMEQ Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vceqq_u16 (uint16x8_t a, uint16x8_t b)  A32: VCEQ.I16 Qd, Qn, Qm  A64: CMEQ Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vceqq_u32 (uint32x4_t a, uint32x4_t b)  A32: VCEQ.I32 Qd, Qn, Qm  A64: CMEQ Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vcgt_u8 (uint8x8_t a, uint8x8_t b)  A32: VCGT.U8 Dd, Dn, Dm  A64: CMHI Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vcgt_s16 (int16x4_t a, int16x4_t b)  A32: VCGT.S16 Dd, Dn, Dm  A64: CMGT Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcgt_s32 (int32x2_t a, int32x2_t b)  A32: VCGT.S32 Dd, Dn, Dm  A64: CMGT Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vcgt_s8 (int8x8_t a, int8x8_t b)  A32: VCGT.S8 Dd, Dn, Dm  A64: CMGT Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vcgt_f32 (float32x2_t a, float32x2_t b)  A32: VCGT.F32 Dd, Dn, Dm  A64: FCMGT Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vcgt_u16 (uint16x4_t a, uint16x4_t b)  A32: VCGT.U16 Dd, Dn, Dm  A64: CMHI Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcgt_u32 (uint32x2_t a, uint32x2_t b)  A32: VCGT.U32 Dd, Dn, Dm  A64: CMHI Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcgtq_u8 (uint8x16_t a, uint8x16_t b)  A32: VCGT.U8 Qd, Qn, Qm  A64: CMHI Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcgtq_s16 (int16x8_t a, int16x8_t b)  A32: VCGT.S16 Qd, Qn, Qm  A64: CMGT Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcgtq_s32 (int32x4_t a, int32x4_t b)  A32: VCGT.S32 Qd, Qn, Qm  A64: CMGT Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcgtq_s8 (int8x16_t a, int8x16_t b)  A32: VCGT.S8 Qd, Qn, Qm  A64: CMGT Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcgtq_f32 (float32x4_t a, float32x4_t b)  A32: VCGT.F32 Qd, Qn, Qm  A64: FCMGT Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcgtq_u16 (uint16x8_t a, uint16x8_t b)  A32: VCGT.U16 Qd, Qn, Qm  A64: CMHI Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcgtq_u32 (uint32x4_t a, uint32x4_t b)  A32: VCGT.U32 Qd, Qn, Qm  A64: CMHI Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vcge_u8 (uint8x8_t a, uint8x8_t b)  A32: VCGE.U8 Dd, Dn, Dm  A64: CMHS Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vcge_s16 (int16x4_t a, int16x4_t b)  A32: VCGE.S16 Dd, Dn, Dm  A64: CMGE Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcge_s32 (int32x2_t a, int32x2_t b)  A32: VCGE.S32 Dd, Dn, Dm  A64: CMGE Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vcge_s8 (int8x8_t a, int8x8_t b)  A32: VCGE.S8 Dd, Dn, Dm  A64: CMGE Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vcge_f32 (float32x2_t a, float32x2_t b)  A32: VCGE.F32 Dd, Dn, Dm  A64: FCMGE Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vcge_u16 (uint16x4_t a, uint16x4_t b)  A32: VCGE.U16 Dd, Dn, Dm  A64: CMHS Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcge_u32 (uint32x2_t a, uint32x2_t b)  A32: VCGE.U32 Dd, Dn, Dm  A64: CMHS Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcgeq_u8 (uint8x16_t a, uint8x16_t b)  A32: VCGE.U8 Qd, Qn, Qm  A64: CMHS Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcgeq_s16 (int16x8_t a, int16x8_t b)  A32: VCGE.S16 Qd, Qn, Qm  A64: CMGE Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcgeq_s32 (int32x4_t a, int32x4_t b)  A32: VCGE.S32 Qd, Qn, Qm  A64: CMGE Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcgeq_s8 (int8x16_t a, int8x16_t b)  A32: VCGE.S8 Qd, Qn, Qm  A64: CMGE Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcgeq_f32 (float32x4_t a, float32x4_t b)  A32: VCGE.F32 Qd, Qn, Qm  A64: FCMGE Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcgeq_u16 (uint16x8_t a, uint16x8_t b)  A32: VCGE.U16 Qd, Qn, Qm  A64: CMHS Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcgeq_u32 (uint32x4_t a, uint32x4_t b)  A32: VCGE.U32 Qd, Qn, Qm  A64: CMHS Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vclt_u8 (uint8x8_t a, uint8x8_t b)  A32: VCLT.U8 Dd, Dn, Dm  A64: CMHI Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vclt_s16 (int16x4_t a, int16x4_t b)  A32: VCLT.S16 Dd, Dn, Dm  A64: CMGT Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vclt_s32 (int32x2_t a, int32x2_t b)  A32: VCLT.S32 Dd, Dn, Dm  A64: CMGT Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vclt_s8 (int8x8_t a, int8x8_t b)  A32: VCLT.S8 Dd, Dn, Dm  A64: CMGT Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vclt_f32 (float32x2_t a, float32x2_t b)  A32: VCLT.F32 Dd, Dn, Dm  A64: FCMGT Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vclt_u16 (uint16x4_t a, uint16x4_t b)  A32: VCLT.U16 Dd, Dn, Dm  A64: CMHI Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vclt_u32 (uint32x2_t a, uint32x2_t b)  A32: VCLT.U32 Dd, Dn, Dm  A64: CMHI Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcltq_u8 (uint8x16_t a, uint8x16_t b)  A32: VCLT.U8 Qd, Qn, Qm  A64: CMHI Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcltq_s16 (int16x8_t a, int16x8_t b)  A32: VCLT.S16 Qd, Qn, Qm  A64: CMGT Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcltq_s32 (int32x4_t a, int32x4_t b)  A32: VCLT.S32 Qd, Qn, Qm  A64: CMGT Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcltq_s8 (int8x16_t a, int8x16_t b)  A32: VCLT.S8 Qd, Qn, Qm  A64: CMGT Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcltq_f32 (float32x4_t a, float32x4_t b)  A32: VCLT.F32 Qd, Qn, Qm  A64: FCMGT Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcltq_u16 (uint16x8_t a, uint16x8_t b)  A32: VCLT.U16 Qd, Qn, Qm  A64: CMHI Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcltq_u32 (uint32x4_t a, uint32x4_t b)  A32: VCLT.U32 Qd, Qn, Qm  A64: CMHI Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vcle_u8 (uint8x8_t a, uint8x8_t b)  A32: VCLE.U8 Dd, Dn, Dm  A64: CMHS Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vcle_s16 (int16x4_t a, int16x4_t b)  A32: VCLE.S16 Dd, Dn, Dm  A64: CMGE Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcle_s32 (int32x2_t a, int32x2_t b)  A32: VCLE.S32 Dd, Dn, Dm  A64: CMGE Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vcle_s8 (int8x8_t a, int8x8_t b)  A32: VCLE.S8 Dd, Dn, Dm  A64: CMGE Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vcle_f32 (float32x2_t a, float32x2_t b)  A32: VCLE.F32 Dd, Dn, Dm  A64: FCMGE Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vcle_u16 (uint16x4_t a, uint16x4_t b)  A32: VCLE.U16 Dd, Dn, Dm  A64: CMHS Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcle_u32 (uint32x2_t a, uint32x2_t b)  A32: VCLE.U32 Dd, Dn, Dm  A64: CMHS Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcleq_u8 (uint8x16_t a, uint8x16_t b)  A32: VCLE.U8 Qd, Qn, Qm  A64: CMHS Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcleq_s16 (int16x8_t a, int16x8_t b)  A32: VCLE.S16 Qd, Qn, Qm  A64: CMGE Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcleq_s32 (int32x4_t a, int32x4_t b)  A32: VCLE.S32 Qd, Qn, Qm  A64: CMGE Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcleq_s8 (int8x16_t a, int8x16_t b)  A32: VCLE.S8 Qd, Qn, Qm  A64: CMGE Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vcleq_f32 (float32x4_t a, float32x4_t b)  A32: VCLE.F32 Qd, Qn, Qm  A64: FCMGE Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vcleq_u16 (uint16x8_t a, uint16x8_t b)  A32: VCLE.U16 Qd, Qn, Qm  A64: CMHS Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcleq_u32 (uint32x4_t a, uint32x4_t b)  A32: VCLE.U32 Qd, Qn, Qm  A64: CMHS Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vtst_u8 (uint8x8_t a, uint8x8_t b)  A32: VTST.8 Dd, Dn, Dm  A64: CMTST Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vtst_s16 (int16x4_t a, int16x4_t b)  A32: VTST.16 Dd, Dn, Dm  A64: CMTST Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vtst_s32 (int32x2_t a, int32x2_t b)  A32: VTST.32 Dd, Dn, Dm  A64: CMTST Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vtst_s8 (int8x8_t a, int8x8_t b)  A32: VTST.8 Dd, Dn, Dm  A64: CMTST Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """uint32x2_t vtst_f32 (float32x2_t a, float32x2_t b)  A32: VTST.32 Dd, Dn, Dm  A64: CMTST Vd.2S, Vn.2S, Vm.2SThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vtst_u16 (uint16x4_t a, uint16x4_t b)  A32: VTST.16 Dd, Dn, Dm  A64: CMTST Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vtst_u32 (uint32x2_t a, uint32x2_t b)  A32: VTST.32 Dd, Dn, Dm  A64: CMTST Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vtstq_u8 (uint8x16_t a, uint8x16_t b)  A32: VTST.8 Qd, Qn, Qm  A64: CMTST Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vtstq_s16 (int16x8_t a, int16x8_t b)  A32: VTST.16 Qd, Qn, Qm  A64: CMTST Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vtstq_s32 (int32x4_t a, int32x4_t b)  A32: VTST.32 Qd, Qn, Qm  A64: CMTST Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vtstq_s8 (int8x16_t a, int8x16_t b)  A32: VTST.8 Qd, Qn, Qm  A64: CMTST Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """uint32x4_t vtstq_f32 (float32x4_t a, float32x4_t b)  A32: VTST.32 Qd, Qn, Qm  A64: CMTST Vd.4S, Vn.4S, Vm.4SThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vtstq_u16 (uint16x8_t a, uint16x8_t b)  A32: VTST.16 Qd, Qn, Qm  A64: CMTST Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def compare_test(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vtstq_u32 (uint32x4_t a, uint32x4_t b)  A32: VTST.32 Qd, Qn, Qm  A64: CMTST Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_away_from_zero(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vcvta_s32_f32 (float32x2_t a)  A32: VCVTA.S32.F32 Dd, Dm  A64: FCVTAS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_away_from_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vcvtaq_s32_f32 (float32x4_t a)  A32: VCVTA.S32.F32 Qd, Qm  A64: FCVTAS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_int_32_round_away_from_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32_t vcvtas_s32_f32 (float32_t a)  A32: VCVTA.S32.F32 Sd, Sm  A64: FCVTAS Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_even(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vcvtn_s32_f32 (float32x2_t a)  A32: VCVTN.S32.F32 Dd, Dm  A64: FCVTNS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_even(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vcvtnq_s32_f32 (float32x4_t a)  A32: VCVTN.S32.F32 Qd, Qm  A64: FCVTNS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_int_32_round_to_even_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32_t vcvtns_s32_f32 (float32_t a)  A32: VCVTN.S32.F32 Sd, Sm  A64: FCVTNS Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vcvtm_s32_f32 (float32x2_t a)  A32: VCVTM.S32.F32 Dd, Dm  A64: FCVTMS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vcvtmq_s32_f32 (float32x4_t a)  A32: VCVTM.S32.F32 Qd, Qm  A64: FCVTMS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_int_32_round_to_negative_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32_t vcvtms_s32_f32 (float32_t a)  A32: VCVTM.S32.F32 Sd, Sm  A64: FCVTMS Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vcvtp_s32_f32 (float32x2_t a)  A32: VCVTP.S32.F32 Dd, Dm  A64: FCVTPS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vcvtpq_s32_f32 (float32x4_t a)  A32: VCVTP.S32.F32 Qd, Qm  A64: FCVTPS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_int_32_round_to_positive_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32_t vcvtps_s32_f32 (float32_t a)  A32: VCVTP.S32.F32 Sd, Sm  A64: FCVTPS Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_zero(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vcvt_s32_f32 (float32x2_t a)  A32: VCVT.S32.F32 Dd, Dm  A64: FCVTZS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32_round_to_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vcvtq_s32_f32 (float32x4_t a)  A32: VCVT.S32.F32 Qd, Qm  A64: FCVTZS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_int_32_round_to_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32_t vcvts_s32_f32 (float32_t a)  A32: VCVT.S32.F32 Sd, Sm  A64: FCVTZS Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vcvt_f32_s32 (int32x2_t a)  A32: VCVT.F32.S32 Dd, Dm  A64: SCVTF Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vcvt_f32_u32 (uint32x2_t a)  A32: VCVT.F32.U32 Dd, Dm  A64: UCVTF Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vcvtq_f32_s32 (int32x4_t a)  A32: VCVT.F32.S32 Qd, Qm  A64: SCVTF Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vcvtq_f32_u32 (uint32x4_t a)  A32: VCVT.F32.U32 Qd, Qm  A64: UCVTF Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def convert_to_single_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vcvts_f32_s32 (int32_t a)  A32: VCVT.F32.S32 Sd, Sm  A64: SCVTF Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_single_scalar(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vcvts_f32_u32 (uint32_t a)  A32: VCVT.F32.U32 Sd, Sm  A64: UCVTF Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_away_from_zero(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcvta_u32_f32 (float32x2_t a)  A32: VCVTA.U32.F32 Dd, Dm  A64: FCVTAU Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_away_from_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcvtaq_u32_f32 (float32x4_t a)  A32: VCVTA.U32.F32 Qd, Qm  A64: FCVTAU Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_u_int_32_round_away_from_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32_t vcvtas_u32_f32 (float32_t a)  A32: VCVTA.U32.F32 Sd, Sm  A64: FCVTAU Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_even(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcvtn_u32_f32 (float32x2_t a)  A32: VCVTN.U32.F32 Dd, Dm  A64: FCVTNU Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_even(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcvtnq_u32_f32 (float32x4_t a)  A32: VCVTN.U32.F32 Qd, Qm  A64: FCVTNU Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_u_int_32_round_to_even_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32_t vcvtns_u32_f32 (float32_t a)  A32: VCVTN.U32.F32 Sd, Sm  A64: FCVTNU Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcvtm_u32_f32 (float32x2_t a)  A32: VCVTM.U32.F32 Dd, Dm  A64: FCVTMU Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcvtmq_u32_f32 (float32x4_t a)  A32: VCVTM.U32.F32 Qd, Qm  A64: FCVTMU Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_u_int_32_round_to_negative_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32_t vcvtms_u32_f32 (float32_t a)  A32: VCVTM.U32.F32 Sd, Sm  A64: FCVTMU Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcvtp_u32_f32 (float32x2_t a)  A32: VCVTP.U32.F32 Dd, Dm  A64: FCVTPU Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcvtpq_u32_f32 (float32x4_t a)  A32: VCVTP.U32.F32 Qd, Qm  A64: FCVTPU Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_u_int_32_round_to_positive_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32_t vcvtps_u32_f32 (float32_t a)  A32: VCVTP.U32.F32 Sd, Sm  A64: FCVTPU Sd, Sn"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_zero(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vcvt_u32_f32 (float32x2_t a)  A32: VCVT.U32.F32 Dd, Dm  A64: FCVTZU Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32_round_to_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vcvtq_u32_f32 (float32x4_t a)  A32: VCVT.U32.F32 Qd, Qm  A64: FCVTZU Vd.4S, Vn.4S"""
        ...

    @staticmethod
    def convert_to_u_int_32_round_to_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32_t vcvts_u32_f32 (float32_t a)  A32: VCVT.U32.F32 Sd, Sm  A64: FCVTZU Sd, Sn"""
        ...

    @staticmethod
    @overload
    def divide_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vdiv_f64 (float64x1_t a, float64x1_t b)  A32: VDIV.F64 Dd, Dn, Dm  A64: FDIV Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def divide_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vdivs_f32 (float32_t a, float32_t b)  A32: VDIV.F32 Sd, Sn, Sm  A64: FDIV Sd, Sn, SmThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vdupq_lane_u8 (uint8x8_t vec, const int lane)  A32: VDUP.8 Qd, Dm[index]  A64: DUP Vd.16B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vdupq_lane_s16 (int16x4_t vec, const int lane)  A32: VDUP.16 Qd, Dm[index]  A64: DUP Vd.8H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdupq_lane_s32 (int32x2_t vec, const int lane)  A32: VDUP.32 Qd, Dm[index]  A64: DUP Vd.4S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector64[float], index: int) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vdupq_lane_f32 (float32x2_t vec, const int lane)  A32: VDUP.32 Qd, Dm[index]  A64: DUP Vd.4S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vdupq_lane_s8 (int8x8_t vec, const int lane)  A32: VDUP.8 Qd, Dm[index]  A64: DUP Vd.16B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vdupq_lane_u16 (uint16x4_t vec, const int lane)  A32: VDUP.16 Qd, Dm[index]  A64: DUP Vd.8H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdupq_lane_u32 (uint32x2_t vec, const int lane)  A32: VDUP.32 Qd, Dm[index]  A64: DUP Vd.4S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vdupq_lane_u8 (uint8x16_t vec, const int lane)  A32: VDUP.8 Qd, Dm[index]  A64: DUP Vd.16B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vdupq_lane_s16 (int16x8_t vec, const int lane)  A32: VDUP.16 Qd, Dm[index]  A64: DUP Vd.8H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdupq_lane_s32 (int32x4_t vec, const int lane)  A32: VDUP.32 Qd, Dm[index]  A64: DUP Vd.4S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[float], index: int) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vdupq_lane_f32 (float32x4_t vec, const int lane)  A32: VDUP.32 Qd, Dm[index]  A64: DUP Vd.4S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vdupq_lane_s8 (int8x16_t vec, const int lane)  A32: VDUP.8 Qd, Dm[index]  A64: DUP Vd.16B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vdupq_lane_u16 (uint16x8_t vec, const int lane)  A32: VDUP.16 Qd, Dm[index]  A64: DUP Vd.8H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_128(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdupq_lane_u32 (uint32x4_t vec, const int lane)  A32: VDUP.32 Qd, Dm[index]  A64: DUP Vd.4S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vdup_lane_u8 (uint8x8_t vec, const int lane)  A32: VDUP.8 Dd, Dm[index]  A64: DUP Vd.8B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vdup_lane_s16 (int16x4_t vec, const int lane)  A32: VDUP.16 Dd, Dm[index]  A64: DUP Vd.4H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdup_lane_s32 (int32x2_t vec, const int lane)  A32: VDUP.32 Dd, Dm[index]  A64: DUP Vd.2S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector64[float], index: int) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vdup_lane_f32 (float32x2_t vec, const int lane)  A32: VDUP.32 Dd, Dm[index]  A64: DUP Vd.2S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vdup_lane_s8 (int8x8_t vec, const int lane)  A32: VDUP.8 Dd, Dm[index]  A64: DUP Vd.8B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vdup_lane_u16 (uint16x4_t vec, const int lane)  A32: VDUP.16 Dd, Dm[index]  A64: DUP Vd.4H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdup_lane_u32 (uint32x2_t vec, const int lane)  A32: VDUP.32 Dd, Dm[index]  A64: DUP Vd.2S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vdup_laneq_u8 (uint8x16_t vec, const int lane)  A32: VDUP.8 Dd, Dm[index]  A64: DUP Vd.8B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vdup_laneq_s16 (int16x8_t vec, const int lane)  A32: VDUP.16 Dd, Dm[index]  A64: DUP Vd.4H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdup_laneq_s32 (int32x4_t vec, const int lane)  A32: VDUP.32 Dd, Dm[index]  A64: DUP Vd.2S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector128[float], index: int) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vdup_laneq_f32 (float32x4_t vec, const int lane)  A32: VDUP.32 Dd, Dm[index]  A64: DUP Vd.2S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vdup_laneq_s8 (int8x16_t vec, const int lane)  A32: VDUP.8 Dd, Dm[index]  A64: DUP Vd.8B, Vn.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vdup_laneq_u16 (uint16x8_t vec, const int lane)  A32: VDUP.16 Dd, Dm[index]  A64: DUP Vd.4H, Vn.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector_64(value: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdup_laneq_u32 (uint32x4_t vec, const int lane)  A32: VDUP.32 Dd, Dm[index]  A64: DUP Vd.2S, Vn.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vdupq_n_u8 (uint8_t value)  A32: VDUP.8 Qd, Rt  A64: DUP Vd.16B, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vdupq_n_s16 (int16_t value)  A32: VDUP.16 Qd, Rt  A64: DUP Vd.8H, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdupq_n_s32 (int32_t value)  A32: VDUP.32 Qd, Rt  A64: DUP Vd.4S, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vdupq_n_s8 (int8_t value)  A32: VDUP.8 Qd, Rt  A64: DUP Vd.16B, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_128(value: float) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vdupq_n_f32 (float32_t value)  A32: VDUP Qd, Dm[0]  A64: DUP Vd.4S, Vn.S[0]"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vdupq_n_u16 (uint16_t value)  A32: VDUP.16 Qd, Rt  A64: DUP Vd.8H, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_128(value: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdupq_n_u32 (uint32_t value)  A32: VDUP.32 Qd, Rt  A64: DUP Vd.4S, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_64(value: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vdup_n_u8 (uint8_t value)  A32: VDUP.8 Dd, Rt  A64: DUP Vd.8B, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_64(value: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vdup_n_s16 (int16_t value)  A32: VDUP.16 Dd, Rt  A64: DUP Vd.4H, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_64(value: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdup_n_s32 (int32_t value)  A32: VDUP.32 Dd, Rt  A64: DUP Vd.2S, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_64(value: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vdup_n_s8 (int8_t value)  A32: VDUP.8 Dd, Rt  A64: DUP Vd.8B, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_64(value: float) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vdup_n_f32 (float32_t value)  A32: VDUP Dd, Dm[0]  A64: DUP Vd.2S, Vn.S[0]"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_64(value: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vdup_n_u16 (uint16_t value)  A32: VDUP.16 Dd, Rt  A64: DUP Vd.4H, Rn"""
        ...

    @staticmethod
    @overload
    def duplicate_to_vector_64(value: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdup_n_u32 (uint32_t value)  A32: VDUP.32 Dd, Rt  A64: DUP Vd.2S, Rn"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector64[int], index: int) -> int:
        """uint8_t vget_lane_u8 (uint8x8_t v, const int lane)  A32: VMOV.U8 Rt, Dn[lane]  A64: UMOV Wd, Vn.B[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector64[int], index: int) -> int:
        """int16_t vget_lane_s16 (int16x4_t v, const int lane)  A32: VMOV.S16 Rt, Dn[lane]  A64: SMOV Wd, Vn.H[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector64[int], index: int) -> int:
        """int32_t vget_lane_s32 (int32x2_t v, const int lane)  A32: VMOV.32 Rt, Dn[lane]  A64: SMOV Wd, Vn.S[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector64[int], index: int) -> int:
        """int8_t vget_lane_s8 (int8x8_t v, const int lane)  A32: VMOV.S8 Rt, Dn[lane]  A64: SMOV Wd, Vn.B[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector64[float], index: int) -> float:
        """float32_t vget_lane_f32 (float32x2_t v, const int lane)  A32: VMOV.F32 Sd, Sm  A64: DUP Sd, Vn.S[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector64[int], index: int) -> int:
        """uint16_t vget_lane_u16 (uint16x4_t v, const int lane)  A32: VMOV.U16 Rt, Dn[lane]  A64: UMOV Wd, Vn.H[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector64[int], index: int) -> int:
        """uint32_t vget_lane_u32 (uint32x2_t v, const int lane)  A32: VMOV.32 Rt, Dn[lane]  A64: UMOV Wd, Vn.S[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """uint8_t vgetq_lane_u8 (uint8x16_t v, const int lane)  A32: VMOV.U8 Rt, Dn[lane]  A64: UMOV Wd, Vn.B[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[float], index: int) -> float:
        """float64_t vgetq_lane_f64 (float64x2_t v, const int lane)  A32: VMOV.F64 Dd, Dm  A64: DUP Dd, Vn.D[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """int16_t vgetq_lane_s16 (int16x8_t v, const int lane)  A32: VMOV.S16 Rt, Dn[lane]  A64: SMOV Wd, Vn.H[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """int32_t vgetq_lane_s32 (int32x4_t v, const int lane)  A32: VMOV.32 Rt, Dn[lane]  A64: SMOV Wd, Vn.S[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """int64_t vgetq_lane_s64 (int64x2_t v, const int lane)  A32: VMOV Rt, Rt2, Dm  A64: UMOV Xd, Vn.D[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """int8_t vgetq_lane_s8 (int8x16_t v, const int lane)  A32: VMOV.S8 Rt, Dn[lane]  A64: SMOV Wd, Vn.B[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[float], index: int) -> float:
        """float32_t vgetq_lane_f32 (float32x4_t v, const int lane)  A32: VMOV.F32 Sd, Sm  A64: DUP Sd, Vn.S[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """uint16_t vgetq_lane_u16 (uint16x8_t v, const int lane)  A32: VMOV.U16 Rt, Dn[lane]  A64: UMOV Wd, Vn.H[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """uint32_t vgetq_lane_u32 (uint32x4_t v, const int lane)  A32: VMOV.32 Rt, Dn[lane]  A64: UMOV Wd, Vn.S[lane]"""
        ...

    @staticmethod
    @overload
    def extract(vector: System.Runtime.Intrinsics.Vector128[int], index: int) -> int:
        """uint64_t vgetq_lane_u64 (uint64x2_t v, const int lane)  A32: VMOV Rt, Rt2, Dm  A64: UMOV Xd, Vn.D[lane]"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vmovn_u16 (uint16x8_t a)  A32: VMOVN.I16 Dd, Qm  A64: XTN Vd.8B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmovn_s32 (int32x4_t a)  A32: VMOVN.I32 Dd, Qm  A64: XTN Vd.4H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmovn_s64 (int64x2_t a)  A32: VMOVN.I64 Dd, Qm  A64: XTN Vd.2S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vmovn_s16 (int16x8_t a)  A32: VMOVN.I16 Dd, Qm  A64: XTN Vd.8B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmovn_u32 (uint32x4_t a)  A32: VMOVN.I32 Dd, Qm  A64: XTN Vd.4H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmovn_u64 (uint64x2_t a)  A32: VMOVN.I64 Dd, Qm  A64: XTN Vd.2S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqmovn_u16 (uint16x8_t a)  A32: VQMOVN.U16 Dd, Qm  A64: UQXTN Vd.8B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqmovn_s32 (int32x4_t a)  A32: VQMOVN.S32 Dd, Qm  A64: SQXTN Vd.4H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqmovn_s64 (int64x2_t a)  A32: VQMOVN.S64 Dd, Qm  A64: SQXTN Vd.2S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqmovn_s16 (int16x8_t a)  A32: VQMOVN.S16 Dd, Qm  A64: SQXTN Vd.8B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqmovn_u32 (uint32x4_t a)  A32: VQMOVN.U32 Dd, Qm  A64: UQXTN Vd.4H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqmovn_u64 (uint64x2_t a)  A32: VQMOVN.U64 Dd, Qm  A64: UQXTN Vd.2S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqmovun_s16 (int16x8_t a)  A32: VQMOVUN.S16 Dd, Qm  A64: SQXTUN Vd.8B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqmovun_s32 (int32x4_t a)  A32: VQMOVUN.S32 Dd, Qm  A64: SQXTUN Vd.4H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqmovun_s64 (int64x2_t a)  A32: VQMOVUN.S64 Dd, Qm  A64: SQXTUN Vd.2S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqmovun_high_s16 (uint8x8_t r, int16x8_t a)  A32: VQMOVUN.S16 Dd+1, Qm  A64: SQXTUN2 Vd.16B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqmovun_high_s32 (uint16x4_t r, int32x4_t a)  A32: VQMOVUN.S32 Dd+1, Qm  A64: SQXTUN2 Vd.8H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqmovun_high_s64 (uint32x2_t r, int64x2_t a)  A32: VQMOVUN.S64 Dd+1, Qm  A64: SQXTUN2 Vd.4S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqmovn_high_u16 (uint8x8_t r, uint16x8_t a)  A32: VQMOVN.U16 Dd+1, Qm  A64: UQXTN2 Vd.16B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqmovn_high_s32 (int16x4_t r, int32x4_t a)  A32: VQMOVN.S32 Dd+1, Qm  A64: SQXTN2 Vd.8H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqmovn_high_s64 (int32x2_t r, int64x2_t a)  A32: VQMOVN.S64 Dd+1, Qm  A64: SQXTN2 Vd.4S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqmovn_high_s16 (int8x8_t r, int16x8_t a)  A32: VQMOVN.S16 Dd+1, Qm  A64: SQXTN2 Vd.16B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqmovn_high_u32 (uint16x4_t r, uint32x4_t a)  A32: VQMOVN.U32 Dd+1, Qm  A64: UQXTN2 Vd.8H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqmovn_high_u64 (uint32x2_t r, uint64x2_t a)  A32: VQMOVN.U64 Dd+1, Qm  A64: UQXTN2 Vd.4S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vmovn_high_u16 (uint8x8_t r, uint16x8_t a)  A32: VMOVN.I16 Dd+1, Qm  A64: XTN2 Vd.16B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmovn_high_s32 (int16x4_t r, int32x4_t a)  A32: VMOVN.I32 Dd+1, Qm  A64: XTN2 Vd.8H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmovn_high_s64 (int32x2_t r, int64x2_t a)  A32: VMOVN.I64 Dd+1, Qm  A64: XTN2 Vd.4S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vmovn_high_s16 (int8x8_t r, int16x8_t a)  A32: VMOVN.I16 Dd+1, Qm  A64: XTN2 Vd.16B, Vn.8H"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmovn_high_u32 (uint16x4_t r, uint32x4_t a)  A32: VMOVN.I32 Dd+1, Qm  A64: XTN2 Vd.8H, Vn.4S"""
        ...

    @staticmethod
    @overload
    def extract_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmovn_high_u64 (uint32x2_t r, uint64x2_t a)  A32: VMOVN.I64 Dd+1, Qm  A64: XTN2 Vd.4S, Vn.2D"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vextq_s8 (uint8x16_t a, uint8x16_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #n  A64: EXT Vd.16B, Vn.16B, Vm.16B, #n"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[float], lower: System.Runtime.Intrinsics.Vector128[float], index: int) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vextq_f64 (float64x2_t a, float64x2_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*8)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*8)"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vextq_s16 (int16x8_t a, int16x8_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*2)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*2)"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vextq_s32 (int32x4_t a, int32x4_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*4)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*4)"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vextq_s64 (int64x2_t a, int64x2_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*8)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*8)"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vextq_s8 (int8x16_t a, int8x16_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #n  A64: EXT Vd.16B, Vn.16B, Vm.16B, #n"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[float], lower: System.Runtime.Intrinsics.Vector128[float], index: int) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vextq_f32 (float32x4_t a, float32x4_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*4)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*4)"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vextq_s16 (uint16x8_t a, uint16x8_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*2)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*2)"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vextq_s32 (uint32x4_t a, uint32x4_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*4)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*4)"""
        ...

    @staticmethod
    @overload
    def extract_vector_128(upper: System.Runtime.Intrinsics.Vector128[int], lower: System.Runtime.Intrinsics.Vector128[int], index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vextq_s64 (uint64x2_t a, uint64x2_t b, const int n)  A32: VEXT.8 Qd, Qn, Qm, #(n*8)  A64: EXT Vd.16B, Vn.16B, Vm.16B, #(n*8)"""
        ...

    @staticmethod
    @overload
    def extract_vector_64(upper: System.Runtime.Intrinsics.Vector64[int], lower: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vext_s8 (uint8x8_t a, uint8x8_t b, const int n)  A32: VEXT.8 Dd, Dn, Dm, #n  A64: EXT Vd.8B, Vn.8B, Vm.8B, #n"""
        ...

    @staticmethod
    @overload
    def extract_vector_64(upper: System.Runtime.Intrinsics.Vector64[int], lower: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vext_s16 (int16x4_t a, int16x4_t b, const int n)  A32: VEXT.8 Dd, Dn, Dm, #(n*2)  A64: EXT Vd.8B, Vn.8B, Vm.8B, #(n*2)"""
        ...

    @staticmethod
    @overload
    def extract_vector_64(upper: System.Runtime.Intrinsics.Vector64[int], lower: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vext_s32 (int32x2_t a, int32x2_t b, const int n)  A32: VEXT.8 Dd, Dn, Dm, #(n*4)  A64: EXT Vd.8B, Vn.8B, Vm.8B, #(n*4)"""
        ...

    @staticmethod
    @overload
    def extract_vector_64(upper: System.Runtime.Intrinsics.Vector64[int], lower: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vext_s8 (int8x8_t a, int8x8_t b, const int n)  A32: VEXT.8 Dd, Dn, Dm, #n  A64: EXT Vd.8B, Vn.8B, Vm.8B, #n"""
        ...

    @staticmethod
    @overload
    def extract_vector_64(upper: System.Runtime.Intrinsics.Vector64[float], lower: System.Runtime.Intrinsics.Vector64[float], index: int) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vext_f32 (float32x2_t a, float32x2_t b, const int n)  A32: VEXT.8 Dd, Dn, Dm, #(n*4)  A64: EXT Vd.8B, Vn.8B, Vm.8B, #(n*4)"""
        ...

    @staticmethod
    @overload
    def extract_vector_64(upper: System.Runtime.Intrinsics.Vector64[int], lower: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vext_s16 (uint16x4_t a, uint16x4_t b, const int n)  A32: VEXT.8 Dd, Dn, Dm, #(n*2)  A64: EXT Vd.8B, Vn.8B, Vm.8B, #(n*2)"""
        ...

    @staticmethod
    @overload
    def extract_vector_64(upper: System.Runtime.Intrinsics.Vector64[int], lower: System.Runtime.Intrinsics.Vector64[int], index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vext_s32 (uint32x2_t a, uint32x2_t b, const int n)  A32: VEXT.8 Dd, Dn, Dm, #(n*4)  A64: EXT Vd.8B, Vn.8B, Vm.8B, #(n*4)"""
        ...

    @staticmethod
    @overload
    def floor(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrndm_f32 (float32x2_t a)  A32: VRINTM.F32 Dd, Dm  A64: FRINTM Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def floor(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrndmq_f32 (float32x4_t a)  A32: VRINTM.F32 Qd, Qm  A64: FRINTM Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def floor_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vrndm_f64 (float64x1_t a)  A32: VRINTM.F64 Dd, Dm  A64: FRINTM Dd, Dn"""
        ...

    @staticmethod
    @overload
    def floor_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vrndms_f32 (float32_t a)  A32: VRINTM.F32 Sd, Sm  A64: FRINTM Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vhadd_u8 (uint8x8_t a, uint8x8_t b)  A32: VHADD.U8 Dd, Dn, Dm  A64: UHADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vhadd_s16 (int16x4_t a, int16x4_t b)  A32: VHADD.S16 Dd, Dn, Dm  A64: SHADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vhadd_s32 (int32x2_t a, int32x2_t b)  A32: VHADD.S32 Dd, Dn, Dm  A64: SHADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vhadd_s8 (int8x8_t a, int8x8_t b)  A32: VHADD.S8 Dd, Dn, Dm  A64: SHADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vhadd_u16 (uint16x4_t a, uint16x4_t b)  A32: VHADD.U16 Dd, Dn, Dm  A64: UHADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vhadd_u32 (uint32x2_t a, uint32x2_t b)  A32: VHADD.U32 Dd, Dn, Dm  A64: UHADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vhaddq_u8 (uint8x16_t a, uint8x16_t b)  A32: VHADD.U8 Qd, Qn, Qm  A64: UHADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vhaddq_s16 (int16x8_t a, int16x8_t b)  A32: VHADD.S16 Qd, Qn, Qm  A64: SHADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vhaddq_s32 (int32x4_t a, int32x4_t b)  A32: VHADD.S32 Qd, Qn, Qm  A64: SHADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vhaddq_s8 (int8x16_t a, int8x16_t b)  A32: VHADD.S8 Qd, Qn, Qm  A64: SHADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vhaddq_u16 (uint16x8_t a, uint16x8_t b)  A32: VHADD.U16 Qd, Qn, Qm  A64: UHADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def fused_add_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vhaddq_u32 (uint32x4_t a, uint32x4_t b)  A32: VHADD.U32 Qd, Qn, Qm  A64: UHADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrhadd_u8 (uint8x8_t a, uint8x8_t b)  A32: VRHADD.U8 Dd, Dn, Dm  A64: URHADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrhadd_s16 (int16x4_t a, int16x4_t b)  A32: VRHADD.S16 Dd, Dn, Dm  A64: SRHADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vrhadd_s32 (int32x2_t a, int32x2_t b)  A32: VRHADD.S32 Dd, Dn, Dm  A64: SRHADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrhadd_s8 (int8x8_t a, int8x8_t b)  A32: VRHADD.S8 Dd, Dn, Dm  A64: SRHADD Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrhadd_u16 (uint16x4_t a, uint16x4_t b)  A32: VRHADD.U16 Dd, Dn, Dm  A64: URHADD Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrhadd_u32 (uint32x2_t a, uint32x2_t b)  A32: VRHADD.U32 Dd, Dn, Dm  A64: URHADD Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrhaddq_u8 (uint8x16_t a, uint8x16_t b)  A32: VRHADD.U8 Qd, Qn, Qm  A64: URHADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrhaddq_s16 (int16x8_t a, int16x8_t b)  A32: VRHADD.S16 Qd, Qn, Qm  A64: SRHADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vrhaddq_s32 (int32x4_t a, int32x4_t b)  A32: VRHADD.S32 Qd, Qn, Qm  A64: SRHADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrhaddq_s8 (int8x16_t a, int8x16_t b)  A32: VRHADD.S8 Qd, Qn, Qm  A64: SRHADD Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrhaddq_u16 (uint16x8_t a, uint16x8_t b)  A32: VRHADD.U16 Qd, Qn, Qm  A64: URHADD Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def fused_add_rounded_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrhaddq_u32 (uint32x4_t a, uint32x4_t b)  A32: VRHADD.U32 Qd, Qn, Qm  A64: URHADD Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vfma_f32 (float32x2_t a, float32x2_t b, float32x2_t c)  A32: VFMA.F32 Dd, Dn, Dm  A64: FMLA Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add(addend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vfmaq_f32 (float32x4_t a, float32x4_t b, float32x4_t c)  A32: VFMA.F32 Qd, Qn, Qm  A64: FMLA Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_negated_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vfnma_f64 (float64x1_t a, float64x1_t b, float64x1_t c)  A32: VFNMA.F64 Dd, Dn, Dm  A64: FNMADD Dd, Dn, Dm, DaThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_negated_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vfnmas_f32 (float32_t a, float32_t b, float32_t c)  A32: VFNMA.F32 Sd, Sn, Sm  A64: FNMADD Sd, Sn, Sm, SaThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vfma_f64 (float64x1_t a, float64x1_t b, float64x1_t c)  A32: VFMA.F64 Dd, Dn, Dm  A64: FMADD Dd, Dn, Dm, Da"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_scalar(addend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vfmas_f32 (float32_t a, float32_t b, float32_t c)  A32: VFMA.F32 Sd, Sn, Sm  A64: FMADD Sd, Sn, Sm, SaThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vfms_f32 (float32x2_t a, float32x2_t b, float32x2_t c)  A32: VFMS.F32 Dd, Dn, Dm  A64: FMLS Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[float], left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vfmsq_f32 (float32x4_t a, float32x4_t b, float32x4_t c)  A32: VFMS.F32 Qd, Qn, Qm  A64: FMLS Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_negated_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vfnms_f64 (float64x1_t a, float64x1_t b, float64x1_t c)  A32: VFNMS.F64 Dd, Dn, Dm  A64: FNMSUB Dd, Dn, Dm, DaThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_negated_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vfnmss_f32 (float32_t a, float32_t b, float32_t c)  A32: VFNMS.F32 Sd, Sn, Sm  A64: FNMSUB Sd, Sn, Sm, SaThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vfms_f64 (float64x1_t a, float64x1_t b, float64x1_t c)  A32: VFMS.F64 Dd, Dn, Dm  A64: FMSUB Dd, Dn, Dm, Da"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_scalar(minuend: System.Runtime.Intrinsics.Vector64[float], left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vfmss_f32 (float32_t a, float32_t b, float32_t c)  A32: VFMS.F32 Sd, Sn, Sm  A64: FMSUB Sd, Sn, Sm, SaThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vhsub_u8 (uint8x8_t a, uint8x8_t b)  A32: VHSUB.U8 Dd, Dn, Dm  A64: UHSUB Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vhsub_s16 (int16x4_t a, int16x4_t b)  A32: VHSUB.S16 Dd, Dn, Dm  A64: SHSUB Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vhsub_s32 (int32x2_t a, int32x2_t b)  A32: VHSUB.S32 Dd, Dn, Dm  A64: SHSUB Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vhsub_s8 (int8x8_t a, int8x8_t b)  A32: VHSUB.S8 Dd, Dn, Dm  A64: SHSUB Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vhsub_u16 (uint16x4_t a, uint16x4_t b)  A32: VHSUB.U16 Dd, Dn, Dm  A64: UHSUB Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vhsub_u32 (uint32x2_t a, uint32x2_t b)  A32: VHSUB.U32 Dd, Dn, Dm  A64: UHSUB Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vhsubq_u8 (uint8x16_t a, uint8x16_t b)  A32: VHSUB.U8 Qd, Qn, Qm  A64: UHSUB Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vhsubq_s16 (int16x8_t a, int16x8_t b)  A32: VHSUB.S16 Qd, Qn, Qm  A64: SHSUB Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vhsubq_s32 (int32x4_t a, int32x4_t b)  A32: VHSUB.S32 Qd, Qn, Qm  A64: SHSUB Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vhsubq_s8 (int8x16_t a, int8x16_t b)  A32: VHSUB.S8 Qd, Qn, Qm  A64: SHSUB Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vhsubq_u16 (uint16x8_t a, uint16x8_t b)  A32: VHSUB.U16 Qd, Qn, Qm  A64: UHSUB Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def fused_subtract_halving(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vhsubq_u32 (uint32x4_t a, uint32x4_t b)  A32: VHSUB.U32 Qd, Qn, Qm  A64: UHSUB Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector64[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vset_lane_u8 (uint8_t a, uint8x8_t v, const int lane)  A32: VMOV.8 Dd[lane], Rt  A64: INS Vd.B[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector64[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vset_lane_s16 (int16_t a, int16x4_t v, const int lane)  A32: VMOV.16 Dd[lane], Rt  A64: INS Vd.H[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector64[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vset_lane_s32 (int32_t a, int32x2_t v, const int lane)  A32: VMOV.32 Dd[lane], Rt  A64: INS Vd.S[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector64[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vset_lane_s8 (int8_t a, int8x8_t v, const int lane)  A32: VMOV.8 Dd[lane], Rt  A64: INS Vd.B[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector64[float], index: int, data: float) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vset_lane_f32 (float32_t a, float32x2_t v, const int lane)  A32: VMOV.F32 Sd, Sm  A64: INS Vd.S[lane], Vn.S[0]"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector64[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vset_lane_u16 (uint16_t a, uint16x4_t v, const int lane)  A32: VMOV.16 Dd[lane], Rt  A64: INS Vd.H[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector64[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vset_lane_u32 (uint32_t a, uint32x2_t v, const int lane)  A32: VMOV.32 Dd[lane], Rt  A64: INS Vd.S[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vsetq_lane_u8 (uint8_t a, uint8x16_t v, const int lane)  A32: VMOV.8 Dd[lane], Rt  A64: INS Vd.B[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[float], index: int, data: float) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vsetq_lane_f64 (float64_t a, float64x2_t v, const int lane)  A32: VMOV.F64 Dd, Dm  A64: INS Vd.D[lane], Vn.D[0]"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsetq_lane_s16 (int16_t a, int16x8_t v, const int lane)  A32: VMOV.16 Dd[lane], Rt  A64: INS Vd.H[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsetq_lane_s32 (int32_t a, int32x4_t v, const int lane)  A32: VMOV.32 Dd[lane], Rt  A64: INS Vd.S[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsetq_lane_s64 (int64_t a, int64x2_t v, const int lane)  A32: VMOV.64 Dd, Rt, Rt2  A64: INS Vd.D[lane], Xn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vsetq_lane_s8 (int8_t a, int8x16_t v, const int lane)  A32: VMOV.8 Dd[lane], Rt  A64: INS Vd.B[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[float], index: int, data: float) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vsetq_lane_f32 (float32_t a, float32x4_t v, const int lane)  A32: VMOV.F32 Sd, Sm  A64: INS Vd.S[lane], Vn.S[0]"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsetq_lane_u16 (uint16_t a, uint16x8_t v, const int lane)  A32: VMOV.16 Dd[lane], Rt  A64: INS Vd.H[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsetq_lane_u32 (uint32_t a, uint32x4_t v, const int lane)  A32: VMOV.32 Dd[lane], Rt  A64: INS Vd.S[lane], Wn"""
        ...

    @staticmethod
    @overload
    def insert(vector: System.Runtime.Intrinsics.Vector128[int], index: int, data: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsetq_lane_u64 (uint64_t a, uint64x2_t v, const int lane)  A32: VMOV.64 Dd, Rt, Rt2  A64: INS Vd.D[lane], Xn"""
        ...

    @staticmethod
    @overload
    def insert_scalar(result: System.Runtime.Intrinsics.Vector128[float], result_index: int, value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vcopyq_lane_f64 (float64x2_t a, const int lane1, float64x1_t b, const int lane2)  A32: VMOV.F64 Dd, Dm  A64: INS Vd.D[lane1], Vn.D[0]"""
        ...

    @staticmethod
    @overload
    def insert_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vcopyq_lane_s64 (int64x2_t a, const int lane1, int64x1_t b, const int lane2)  A32: VMOV Dd, Dm  A64: INS Vd.D[lane1], Vn.D[0]"""
        ...

    @staticmethod
    @overload
    def insert_scalar(result: System.Runtime.Intrinsics.Vector128[int], result_index: int, value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vcopyq_lane_u64 (uint64x2_t a, const int lane1, uint64x1_t b, const int lane2)  A32: VMOV Dd, Dm  A64: INS Vd.D[lane1], Vn.D[0]"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vcls_s16 (int16x4_t a)  A32: VCLS.S16 Dd, Dm  A64: CLS Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vcls_s32 (int32x2_t a)  A32: VCLS.S32 Dd, Dm  A64: CLS Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vcls_s8 (int8x8_t a)  A32: VCLS.S8 Dd, Dm  A64: CLS Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vclsq_s16 (int16x8_t a)  A32: VCLS.S16 Qd, Qm  A64: CLS Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vclsq_s32 (int32x4_t a)  A32: VCLS.S32 Qd, Qm  A64: CLS Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vclsq_s8 (int8x16_t a)  A32: VCLS.S8 Qd, Qm  A64: CLS Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vclz_u8 (uint8x8_t a)  A32: VCLZ.I8 Dd, Dm  A64: CLZ Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vclz_s16 (int16x4_t a)  A32: VCLZ.I16 Dd, Dm  A64: CLZ Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vclz_s32 (int32x2_t a)  A32: VCLZ.I32 Dd, Dm  A64: CLZ Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vclz_s8 (int8x8_t a)  A32: VCLZ.I8 Dd, Dm  A64: CLZ Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vclz_u16 (uint16x4_t a)  A32: VCLZ.I16 Dd, Dm  A64: CLZ Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vclz_u32 (uint32x2_t a)  A32: VCLZ.I32 Dd, Dm  A64: CLZ Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vclzq_u8 (uint8x16_t a)  A32: VCLZ.I8 Qd, Qm  A64: CLZ Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vclzq_s16 (int16x8_t a)  A32: VCLZ.I16 Qd, Qm  A64: CLZ Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vclzq_s32 (int32x4_t a)  A32: VCLZ.I32 Qd, Qm  A64: CLZ Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vclzq_s8 (int8x16_t a)  A32: VCLZ.I8 Qd, Qm  A64: CLZ Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vclzq_u16 (uint16x8_t a)  A32: VCLZ.I16 Qd, Qm  A64: CLZ Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vclzq_u32 (uint32x4_t a)  A32: VCLZ.I32 Qd, Qm  A64: CLZ Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD1 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD2 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD1 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD3 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD1 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD1 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_64_and_unzip(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD4 { Vn.4S, Vn+1.4S, Vn+2.4S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector64[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vld1_lane_u8 (uint8_t const * ptr, uint8x8_t src, const int lane)  A32: VLD1.8 { Dd[index] }, [Rn]  A64: LD1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector64[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vld1_lane_s16 (int16_t const * ptr, int16x4_t src, const int lane)  A32: VLD1.16 { Dd[index] }, [Rn]  A64: LD1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector64[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vld1_lane_s32 (int32_t const * ptr, int32x2_t src, const int lane)  A32: VLD1.32 { Dd[index] }, [Rn]  A64: LD1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector64[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vld1_lane_s8 (int8_t const * ptr, int8x8_t src, const int lane)  A32: VLD1.8 { Dd[index] }, [Rn]  A64: LD1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector64[float], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vld1_lane_f32 (float32_t const * ptr, float32x2_t src, const int lane)  A32: VLD1.32 { Dd[index] }, [Rn]  A64: LD1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector64[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vld1_lane_u16 (uint16_t const * ptr, uint16x4_t src, const int lane)  A32: VLD1.16 { Dd[index] }, [Rn]  A64: LD1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector64[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vld1_lane_u32 (uint32_t const * ptr, uint32x2_t src, const int lane)  A32: VLD1.32 { Dd[index] }, [Rn]  A64: LD1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vld1q_lane_u8 (uint8_t const * ptr, uint8x16_t src, const int lane)  A32: VLD1.8 { Dd[index] }, [Rn]  A64: LD1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[float], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vld1q_lane_f64 (float64_t const * ptr, float64x2_t src, const int lane)  A32: VLDR.64 Dd, [Rn]  A64: LD1 { Vt.D }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vld1q_lane_s16 (int16_t const * ptr, int16x8_t src, const int lane)  A32: VLD1.16 { Dd[index] }, [Rn]  A64: LD1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vld1q_lane_s32 (int32_t const * ptr, int32x4_t src, const int lane)  A32: VLD1.32 { Dd[index] }, [Rn]  A64: LD1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vld1q_lane_s64 (int64_t const * ptr, int64x2_t src, const int lane)  A32: VLDR.64 Dd, [Rn]  A64: LD1 { Vt.D }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vld1q_lane_s8 (int8_t const * ptr, int8x16_t src, const int lane)  A32: VLD1.8 { Dd[index] }, [Rn]  A64: LD1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[float], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vld1q_lane_f32 (float32_t const * ptr, float32x4_t src, const int lane)  A32: VLD1.32 { Dd[index] }, [Rn]  A64: LD1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vld1q_lane_u16 (uint16_t const * ptr, uint16x8_t src, const int lane)  A32: VLD1.16 { Dd[index] }, [Rn]  A64: LD1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vld1q_lane_u32 (uint32_t const * ptr, uint32x4_t src, const int lane)  A32: VLD1.32 { Dd[index] }, [Rn]  A64: LD1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(value: System.Runtime.Intrinsics.Vector128[int], index: int, address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vld1q_lane_u64 (uint64_t const * ptr, uint64x2_t src, const int lane)  A32: VLDR.64 Dd, [Rn]  A64: LD1 { Vt.D }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.8B, Vn+1.8B }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.8B, Vn+1.8B }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.4H, Vn+1.4H }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.4H, Vn+1.4H }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.2S, Vn+1.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2 { Vn.2S, Vn+1.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD2 { Vn.2S, Vn+1.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.8B, Vn+1.8B, Vn+2.8B }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.8B, Vn+1.8B, Vn+2.8B }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.4H, Vn+1.4H, Vn+2.4H }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.4H, Vn+1.4H, Vn+2.4H }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.2S, Vn+1.2S, Vn+2.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3 { Vn.2S, Vn+1.2S, Vn+2.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD3 { Vn.2S, Vn+1.2S, Vn+2.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_insert_scalar(values: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]], index: int, address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }[Vm], [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vld1q_dup_u8 (uint8_t const * ptr)  A32: VLD1.8 { Dd[], Dd+1[] }, [Rn]  A64: LD1R { Vt.16B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vld1q_dup_s16 (int16_t const * ptr)  A32: VLD1.16 { Dd[], Dd+1[] }, [Rn]  A64: LD1R { Vt.8H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vld1q_dup_s32 (int32_t const * ptr)  A32: VLD1.32 { Dd[], Dd+1[] }, [Rn]  A64: LD1R { Vt.4S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vld1q_dup_s8 (int8_t const * ptr)  A32: VLD1.8 { Dd[], Dd+1[] }, [Rn]  A64: LD1R { Vt.16B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vld1q_dup_f32 (float32_t const * ptr)  A32: VLD1.32 { Dd[], Dd+1[] }, [Rn]  A64: LD1R { Vt.4S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vld1q_dup_u16 (uint16_t const * ptr)  A32: VLD1.16 { Dd[], Dd+1[] }, [Rn]  A64: LD1R { Vt.8H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vld1q_dup_u32 (uint32_t const * ptr)  A32: VLD1.32 { Dd[], Dd+1[] }, [Rn]  A64: LD1R { Vt.4S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vld1_dup_u8 (uint8_t const * ptr)  A32: VLD1.8 { Dd[] }, [Rn]  A64: LD1R { Vt.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vld1_dup_s16 (int16_t const * ptr)  A32: VLD1.16 { Dd[] }, [Rn]  A64: LD1R { Vt.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vld1_dup_s32 (int32_t const * ptr)  A32: VLD1.32 { Dd[] }, [Rn]  A64: LD1R { Vt.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vld1_dup_s8 (int8_t const * ptr)  A32: VLD1.8 { Dd[] }, [Rn]  A64: LD1R { Vt.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vld1_dup_f32 (float32_t const * ptr)  A32: VLD1.32 { Dd[] }, [Rn]  A64: LD1R { Vt.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vld1_dup_u16 (uint16_t const * ptr)  A32: VLD1.16 { Dd[] }, [Rn]  A64: LD1R { Vt.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vld1_dup_u32 (uint32_t const * ptr)  A32: VLD1.32 { Dd[] }, [Rn]  A64: LD1R { Vt.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2R { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2R { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2R { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2R { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2R { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD2R { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_2(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD2R { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3R { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3R { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3R { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3R { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3R { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD3R { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_3(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD3R { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4R { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4R { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4R { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4R { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4R { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]:
        """A64: LD4R { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_and_replicate_to_vector_64x_4(address: typing.Any) -> System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]:
        """A64: LD4R { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vld1q_u8 (uint8_t const * ptr)  A32: VLD1.8 Dd, Dd+1, [Rn]  A64: LD1 Vt.16B, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vld1q_f64 (float64_t const * ptr)  A32: VLD1.64 Dd, Dd+1, [Rn]  A64: LD1 Vt.2D, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vld1q_s16 (int16_t const * ptr)  A32: VLD1.16 Dd, Dd+1, [Rn]  A64: LD1 Vt.8H, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vld1q_s32 (int32_t const * ptr)  A32: VLD1.32 Dd, Dd+1, [Rn]  A64: LD1 Vt.4S, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vld1q_s64 (int64_t const * ptr)  A32: VLD1.64 Dd, Dd+1, [Rn]  A64: LD1 Vt.2D, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vld1q_s8 (int8_t const * ptr)  A32: VLD1.8 Dd, Dd+1, [Rn]  A64: LD1 Vt.16B, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vld1q_f32 (float32_t const * ptr)  A32: VLD1.32 Dd, Dd+1, [Rn]  A64: LD1 Vt.4S, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vld1q_s16 (uint16_t const * ptr)  A32: VLD1.16 Dd, Dd+1, [Rn]  A64: LD1 Vt.8H, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vld1q_s32 (uint32_t const * ptr)  A32: VLD1.32 Dd, Dd+1, [Rn]  A64: LD1 Vt.4S, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_128(address: typing.Any) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vld1q_u64 (uint64_t const * ptr)  A32: VLD1.64 Dd, Dd+1, [Rn]  A64: LD1 Vt.2D, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vld1_u8 (uint8_t const * ptr)  A32: VLD1.8 Dd, [Rn]  A64: LD1 Vt.8B, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vld1_f64 (float64_t const * ptr)  A32: VLD1.64 Dd, [Rn]  A64: LD1 Vt.1D, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vld1_s16 (int16_t const * ptr)  A32: VLD1.16 Dd, [Rn]  A64: LD1 Vt.4H, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vld1_s32 (int32_t const * ptr)  A32: VLD1.32 Dd, [Rn]  A64: LD1 Vt.2S, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vld1_s64 (int64_t const * ptr)  A32: VLD1.64 Dd, [Rn]  A64: LD1 Vt.1D, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vld1_s8 (int8_t const * ptr)  A32: VLD1.8 Dd, [Rn]  A64: LD1 Vt.8B, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vld1_f32 (float32_t const * ptr)  A32: VLD1.32 Dd, [Rn]  A64: LD1 Vt.2S, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vld1_u16 (uint16_t const * ptr)  A32: VLD1.16 Dd, [Rn]  A64: LD1 Vt.4H, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vld1_u32 (uint32_t const * ptr)  A32: VLD1.32 Dd, [Rn]  A64: LD1 Vt.2S, [Xn]"""
        ...

    @staticmethod
    @overload
    def load_vector_64(address: typing.Any) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vld1_u64 (uint64_t const * ptr)  A32: VLD1.64 Dd, [Rn]  A64: LD1 Vt.1D, [Xn]"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vmax_u8 (uint8x8_t a, uint8x8_t b)  A32: VMAX.U8 Dd, Dn, Dm  A64: UMAX Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmax_s16 (int16x4_t a, int16x4_t b)  A32: VMAX.S16 Dd, Dn, Dm  A64: SMAX Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmax_s32 (int32x2_t a, int32x2_t b)  A32: VMAX.S32 Dd, Dn, Dm  A64: SMAX Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vmax_s8 (int8x8_t a, int8x8_t b)  A32: VMAX.S8 Dd, Dn, Dm  A64: SMAX Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmax_f32 (float32x2_t a, float32x2_t b)  A32: VMAX.F32 Dd, Dn, Dm  A64: FMAX Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmax_u16 (uint16x4_t a, uint16x4_t b)  A32: VMAX.U16 Dd, Dn, Dm  A64: UMAX Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmax_u32 (uint32x2_t a, uint32x2_t b)  A32: VMAX.U32 Dd, Dn, Dm  A64: UMAX Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vmaxq_u8 (uint8x16_t a, uint8x16_t b)  A32: VMAX.U8 Qd, Qn, Qm  A64: UMAX Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmaxq_s16 (int16x8_t a, int16x8_t b)  A32: VMAX.S16 Qd, Qn, Qm  A64: SMAX Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmaxq_s32 (int32x4_t a, int32x4_t b)  A32: VMAX.S32 Qd, Qn, Qm  A64: SMAX Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vmaxq_s8 (int8x16_t a, int8x16_t b)  A32: VMAX.S8 Qd, Qn, Qm  A64: SMAX Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vmaxq_f32 (float32x4_t a, float32x4_t b)  A32: VMAX.F32 Qd, Qn, Qm  A64: FMAX Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmaxq_u16 (uint16x8_t a, uint16x8_t b)  A32: VMAX.U16 Qd, Qn, Qm  A64: UMAX Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def max(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmaxq_u32 (uint32x4_t a, uint32x4_t b)  A32: VMAX.U32 Qd, Qn, Qm  A64: UMAX Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def max_number(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmaxnm_f32 (float32x2_t a, float32x2_t b)  A32: VMAXNM.F32 Dd, Dn, Dm  A64: FMAXNM Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def max_number(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vmaxnmq_f32 (float32x4_t a, float32x4_t b)  A32: VMAXNM.F32 Qd, Qn, Qm  A64: FMAXNM Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def max_number_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vmaxnm_f64 (float64x1_t a, float64x1_t b)  A32: VMAXNM.F64 Dd, Dn, Dm  A64: FMAXNM Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def max_number_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vmaxnms_f32 (float32_t a, float32_t b)  A32: VMAXNM.F32 Sd, Sn, Sm  A64: FMAXNM Sd, Sn, Sm"""
        ...

    @staticmethod
    @overload
    def max_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vpmax_u8 (uint8x8_t a, uint8x8_t b)  A32: VPMAX.U8 Dd, Dn, Dm  A64: UMAXP Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def max_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vpmax_s16 (int16x4_t a, int16x4_t b)  A32: VPMAX.S16 Dd, Dn, Dm  A64: SMAXP Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def max_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vpmax_s32 (int32x2_t a, int32x2_t b)  A32: VPMAX.S32 Dd, Dn, Dm  A64: SMAXP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def max_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vpmax_s8 (int8x8_t a, int8x8_t b)  A32: VPMAX.S8 Dd, Dn, Dm  A64: SMAXP Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def max_pairwise(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vpmax_f32 (float32x2_t a, float32x2_t b)  A32: VPMAX.F32 Dd, Dn, Dm  A64: FMAXP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def max_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vpmax_u16 (uint16x4_t a, uint16x4_t b)  A32: VPMAX.U16 Dd, Dn, Dm  A64: UMAXP Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def max_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vpmax_u32 (uint32x2_t a, uint32x2_t b)  A32: VPMAX.U32 Dd, Dn, Dm  A64: UMAXP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vmin_u8 (uint8x8_t a, uint8x8_t b)  A32: VMIN.U8 Dd, Dn, Dm  A64: UMIN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmin_s16 (int16x4_t a, int16x4_t b)  A32: VMIN.S16 Dd, Dn, Dm  A64: SMIN Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmin_s32 (int32x2_t a, int32x2_t b)  A32: VMIN.S32 Dd, Dn, Dm  A64: SMIN Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vmin_s8 (int8x8_t a, int8x8_t b)  A32: VMIN.S8 Dd, Dn, Dm  A64: SMIN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmin_f32 (float32x2_t a, float32x2_t b)  A32: VMIN.F32 Dd, Dn, Dm  A64: FMIN Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmin_u16 (uint16x4_t a, uint16x4_t b)  A32: VMIN.U16 Dd, Dn, Dm  A64: UMIN Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmin_u32 (uint32x2_t a, uint32x2_t b)  A32: VMIN.U32 Dd, Dn, Dm  A64: UMIN Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vminq_u8 (uint8x16_t a, uint8x16_t b)  A32: VMIN.U8 Qd, Qn, Qm  A64: UMIN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vminq_s16 (int16x8_t a, int16x8_t b)  A32: VMIN.S16 Qd, Qn, Qm  A64: SMIN Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vminq_s32 (int32x4_t a, int32x4_t b)  A32: VMIN.S32 Qd, Qn, Qm  A64: SMIN Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vminq_s8 (int8x16_t a, int8x16_t b)  A32: VMIN.S8 Qd, Qn, Qm  A64: SMIN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vminq_f32 (float32x4_t a, float32x4_t b)  A32: VMIN.F32 Qd, Qn, Qm  A64: FMIN Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vminq_u16 (uint16x8_t a, uint16x8_t b)  A32: VMIN.U16 Qd, Qn, Qm  A64: UMIN Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def min(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vminq_u32 (uint32x4_t a, uint32x4_t b)  A32: VMIN.U32 Qd, Qn, Qm  A64: UMIN Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def min_number(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vminnm_f32 (float32x2_t a, float32x2_t b)  A32: VMINNM.F32 Dd, Dn, Dm  A64: FMINNM Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def min_number(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vminnmq_f32 (float32x4_t a, float32x4_t b)  A32: VMINNM.F32 Qd, Qn, Qm  A64: FMINNM Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def min_number_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vminnm_f64 (float64x1_t a, float64x1_t b)  A32: VMINNM.F64 Dd, Dn, Dm  A64: FMINNM Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def min_number_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vminnms_f32 (float32_t a, float32_t b)  A32: VMINNM.F32 Sd, Sn, Sm  A64: FMINNM Sd, Sn, Sm"""
        ...

    @staticmethod
    @overload
    def min_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vpmin_u8 (uint8x8_t a, uint8x8_t b)  A32: VPMIN.U8 Dd, Dn, Dm  A64: UMINP Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def min_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vpmin_s16 (int16x4_t a, int16x4_t b)  A32: VPMIN.S16 Dd, Dn, Dm  A64: SMINP Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def min_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vpmin_s32 (int32x2_t a, int32x2_t b)  A32: VPMIN.S32 Dd, Dn, Dm  A64: SMINP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def min_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vpmin_s8 (int8x8_t a, int8x8_t b)  A32: VPMIN.S8 Dd, Dn, Dm  A64: SMINP Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def min_pairwise(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vpmin_f32 (float32x2_t a, float32x2_t b)  A32: VPMIN.F32 Dd, Dn, Dm  A64: FMINP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def min_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vpmin_u16 (uint16x4_t a, uint16x4_t b)  A32: VPMIN.U16 Dd, Dn, Dm  A64: UMINP Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def min_pairwise(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vpmin_u32 (uint32x2_t a, uint32x2_t b)  A32: VPMIN.U32 Dd, Dn, Dm  A64: UMINP Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vmul_u8 (uint8x8_t a, uint8x8_t b)  A32: VMUL.I8 Dd, Dn, Dm  A64: MUL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmul_s16 (int16x4_t a, int16x4_t b)  A32: VMUL.I16 Dd, Dn, Dm  A64: MUL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmul_s32 (int32x2_t a, int32x2_t b)  A32: VMUL.I32 Dd, Dn, Dm  A64: MUL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vmul_s8 (int8x8_t a, int8x8_t b)  A32: VMUL.I8 Dd, Dn, Dm  A64: MUL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmul_f32 (float32x2_t a, float32x2_t b)  A32: VMUL.F32 Dd, Dn, Dm  A64: FMUL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmul_u16 (uint16x4_t a, uint16x4_t b)  A32: VMUL.I16 Dd, Dn, Dm  A64: MUL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmul_u32 (uint32x2_t a, uint32x2_t b)  A32: VMUL.I32 Dd, Dn, Dm  A64: MUL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vmulq_u8 (uint8x16_t a, uint8x16_t b)  A32: VMUL.I8 Qd, Qn, Qm  A64: MUL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmulq_s16 (int16x8_t a, int16x8_t b)  A32: VMUL.I16 Qd, Qn, Qm  A64: MUL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmulq_s32 (int32x4_t a, int32x4_t b)  A32: VMUL.I32 Qd, Qn, Qm  A64: MUL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vmulq_s8 (int8x16_t a, int8x16_t b)  A32: VMUL.I8 Qd, Qn, Qm  A64: MUL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vmulq_f32 (float32x4_t a, float32x4_t b)  A32: VMUL.F32 Qd, Qn, Qm  A64: FMUL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmulq_u16 (uint16x8_t a, uint16x8_t b)  A32: VMUL.I16 Qd, Qn, Qm  A64: MUL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmulq_u32 (uint32x4_t a, uint32x4_t b)  A32: VMUL.I32 Qd, Qn, Qm  A64: MUL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vmla_u8 (uint8x8_t a, uint8x8_t b, uint8x8_t c)  A32: VMLA.I8 Dd, Dn, Dm  A64: MLA Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmla_s16 (int16x4_t a, int16x4_t b, int16x4_t c)  A32: VMLA.I16 Dd, Dn, Dm  A64: MLA Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmla_s32 (int32x2_t a, int32x2_t b, int32x2_t c)  A32: VMLA.I32 Dd, Dn, Dm  A64: MLA Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vmla_s8 (int8x8_t a, int8x8_t b, int8x8_t c)  A32: VMLA.I8 Dd, Dn, Dm  A64: MLA Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmla_u16 (uint16x4_t a, uint16x4_t b, uint16x4_t c)  A32: VMLA.I16 Dd, Dn, Dm  A64: MLA Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmla_u32 (uint32x2_t a, uint32x2_t b, uint32x2_t c)  A32: VMLA.I32 Dd, Dn, Dm  A64: MLA Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vmlaq_u8 (uint8x16_t a, uint8x16_t b, uint8x16_t c)  A32: VMLA.I8 Qd, Qn, Qm  A64: MLA Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlaq_s16 (int16x8_t a, int16x8_t b, int16x8_t c)  A32: VMLA.I16 Qd, Qn, Qm  A64: MLA Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlaq_s32 (int32x4_t a, int32x4_t b, int32x4_t c)  A32: VMLA.I32 Qd, Qn, Qm  A64: MLA Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vmlaq_s8 (int8x16_t a, int8x16_t b, int8x16_t c)  A32: VMLA.I8 Qd, Qn, Qm  A64: MLA Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlaq_u16 (uint16x8_t a, uint16x8_t b, uint16x8_t c)  A32: VMLA.I16 Qd, Qn, Qm  A64: MLA Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlaq_u32 (uint32x4_t a, uint32x4_t b, uint32x4_t c)  A32: VMLA.I32 Qd, Qn, Qm  A64: MLA Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmla_n_s16 (int16x4_t a, int16x4_t b, int16_t c)  A32: VMLA.I16 Dd, Dn, Dm[0]  A64: MLA Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmla_n_s32 (int32x2_t a, int32x2_t b, int32_t c)  A32: VMLA.I32 Dd, Dn, Dm[0]  A64: MLA Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmla_n_u16 (uint16x4_t a, uint16x4_t b, uint16_t c)  A32: VMLA.I16 Dd, Dn, Dm[0]  A64: MLA Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmla_n_u32 (uint32x2_t a, uint32x2_t b, uint32_t c)  A32: VMLA.I32 Dd, Dn, Dm[0]  A64: MLA Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlaq_n_s16 (int16x8_t a, int16x8_t b, int16_t c)  A32: VMLA.I16 Qd, Qn, Dm[0]  A64: MLA Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlaq_n_s32 (int32x4_t a, int32x4_t b, int32_t c)  A32: VMLA.I32 Qd, Qn, Dm[0]  A64: MLA Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlaq_n_u16 (uint16x8_t a, uint16x8_t b, uint16_t c)  A32: VMLA.I16 Qd, Qn, Dm[0]  A64: MLA Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlaq_n_u32 (uint32x4_t a, uint32x4_t b, uint32_t c)  A32: VMLA.I32 Qd, Qn, Dm[0]  A64: MLA Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmla_lane_s16 (int16x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VMLA.I16 Dd, Dn, Dm[lane]  A64: MLA Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmla_laneq_s16 (int16x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VMLA.I16 Dd, Dn, Dm[lane]  A64: MLA Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmla_lane_s32 (int32x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VMLA.I32 Dd, Dn, Dm[lane]  A64: MLA Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmla_laneq_s32 (int32x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VMLA.I32 Dd, Dn, Dm[lane]  A64: MLA Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmla_lane_u16 (uint16x4_t a, uint16x4_t b, uint16x4_t v, const int lane)  A32: VMLA.I16 Dd, Dn, Dm[lane]  A64: MLA Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmla_laneq_u16 (uint16x4_t a, uint16x4_t b, uint16x8_t v, const int lane)  A32: VMLA.I16 Dd, Dn, Dm[lane]  A64: MLA Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmla_lane_u32 (uint32x2_t a, uint32x2_t b, uint32x2_t v, const int lane)  A32: VMLA.I32 Dd, Dn, Dm[lane]  A64: MLA Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmla_laneq_u32 (uint32x2_t a, uint32x2_t b, uint32x4_t v, const int lane)  A32: VMLA.I32 Dd, Dn, Dm[lane]  A64: MLA Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlaq_lane_s16 (int16x8_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VMLA.I16 Qd, Qn, Dm[lane]  A64: MLA Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlaq_laneq_s16 (int16x8_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VMLA.I16 Qd, Qn, Dm[lane]  A64: MLA Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlaq_lane_s32 (int32x4_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VMLA.I32 Qd, Qn, Dm[lane]  A64: MLA Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlaq_laneq_s32 (int32x4_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VMLA.I32 Qd, Qn, Dm[lane]  A64: MLA Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlaq_lane_u16 (uint16x8_t a, uint16x8_t b, uint16x4_t v, const int lane)  A32: VMLA.I16 Qd, Qn, Dm[lane]  A64: MLA Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlaq_laneq_u16 (uint16x8_t a, uint16x8_t b, uint16x8_t v, const int lane)  A32: VMLA.I16 Qd, Qn, Dm[lane]  A64: MLA Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlaq_lane_u32 (uint32x4_t a, uint32x4_t b, uint32x2_t v, const int lane)  A32: VMLA.I32 Qd, Qn, Dm[lane]  A64: MLA Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_add_by_selected_scalar(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlaq_laneq_u32 (uint32x4_t a, uint32x4_t b, uint32x4_t v, const int lane)  A32: VMLA.I32 Qd, Qn, Dm[lane]  A64: MLA Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmul_n_s16 (int16x4_t a, int16_t b)  A32: VMUL.I16 Dd, Dn, Dm[0]  A64: MUL Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmul_n_s32 (int32x2_t a, int32_t b)  A32: VMUL.I32 Dd, Dn, Dm[0]  A64: MUL Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmul_n_f32 (float32x2_t a, float32_t b)  A32: VMUL.F32 Dd, Dn, Dm[0]  A64: FMUL Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmul_n_u16 (uint16x4_t a, uint16_t b)  A32: VMUL.I16 Dd, Dn, Dm[0]  A64: MUL Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmul_n_u32 (uint32x2_t a, uint32_t b)  A32: VMUL.I32 Dd, Dn, Dm[0]  A64: MUL Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmulq_n_s16 (int16x8_t a, int16_t b)  A32: VMUL.I16 Qd, Qn, Dm[0]  A64: MUL Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmulq_n_s32 (int32x4_t a, int32_t b)  A32: VMUL.I32 Qd, Qn, Dm[0]  A64: MUL Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vmulq_n_f32 (float32x4_t a, float32_t b)  A32: VMUL.F32 Qd, Qn, Dm[0]  A64: FMUL Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmulq_n_u16 (uint16x8_t a, uint16_t b)  A32: VMUL.I16 Qd, Qn, Dm[0]  A64: MUL Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmulq_n_u32 (uint32x4_t a, uint32_t b)  A32: VMUL.I32 Qd, Qn, Dm[0]  A64: MUL Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmul_lane_s16 (int16x4_t a, int16x4_t v, const int lane)  A32: VMUL.I16 Dd, Dn, Dm[lane]  A64: MUL Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmul_laneq_s16 (int16x4_t a, int16x8_t v, const int lane)  A32: VMUL.I16 Dd, Dn, Dm[lane]  A64: MUL Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmul_lane_s32 (int32x2_t a, int32x2_t v, const int lane)  A32: VMUL.I32 Dd, Dn, Dm[lane]  A64: MUL Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmul_laneq_s32 (int32x2_t a, int32x4_t v, const int lane)  A32: VMUL.I32 Dd, Dn, Dm[lane]  A64: MUL Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmul_lane_f32 (float32x2_t a, float32x2_t v, const int lane)  A32: VMUL.F32 Dd, Dn, Dm[lane]  A64: FMUL Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmul_laneq_f32 (float32x2_t a, float32x4_t v, const int lane)  A32: VMUL.F32 Dd, Dn, Dm[lane]  A64: FMUL Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmul_lane_u16 (uint16x4_t a, uint16x4_t v, const int lane)  A32: VMUL.I16 Dd, Dn, Dm[lane]  A64: MUL Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmul_laneq_u16 (uint16x4_t a, uint16x8_t v, const int lane)  A32: VMUL.I16 Dd, Dn, Dm[lane]  A64: MUL Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmul_lane_u32 (uint32x2_t a, uint32x2_t v, const int lane)  A32: VMUL.I32 Dd, Dn, Dm[lane]  A64: MUL Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmul_laneq_u32 (uint32x2_t a, uint32x4_t v, const int lane)  A32: VMUL.I32 Dd, Dn, Dm[lane]  A64: MUL Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmulq_lane_s16 (int16x8_t a, int16x4_t v, const int lane)  A32: VMUL.I16 Qd, Qn, Dm[lane]  A64: MUL Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmulq_laneq_s16 (int16x8_t a, int16x8_t v, const int lane)  A32: VMUL.I16 Qd, Qn, Dm[lane]  A64: MUL Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmulq_lane_s32 (int32x4_t a, int32x2_t v, const int lane)  A32: VMUL.I32 Qd, Qn, Dm[lane]  A64: MUL Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmulq_laneq_s32 (int32x4_t a, int32x4_t v, const int lane)  A32: VMUL.I32 Qd, Qn, Dm[lane]  A64: MUL Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vmulq_lane_f32 (float32x4_t a, float32x2_t v, const int lane)  A32: VMUL.F32 Qd, Qn, Dm[lane]  A64: FMUL Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vmulq_laneq_f32 (float32x4_t a, float32x4_t v, const int lane)  A32: VMUL.F32 Qd, Qn, Dm[lane]  A64: FMUL Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmulq_lane_u16 (uint16x8_t a, uint16x4_t v, const int lane)  A32: VMUL.I16 Qd, Qn, Dm[lane]  A64: MUL Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmulq_laneq_u16 (uint16x8_t a, uint16x8_t v, const int lane)  A32: VMUL.I16 Qd, Qn, Dm[lane]  A64: MUL Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmulq_lane_u32 (uint32x4_t a, uint32x2_t v, const int lane)  A32: VMUL.I32 Qd, Qn, Dm[lane]  A64: MUL Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmulq_laneq_u32 (uint32x4_t a, uint32x4_t v, const int lane)  A32: VMUL.I32 Qd, Qn, Dm[lane]  A64: MUL Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmull_lane_s16 (int16x4_t a, int16x4_t v, const int lane)  A32: VMULL.S16 Qd, Dn, Dm[lane]  A64: SMULL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmull_laneq_s16 (int16x4_t a, int16x8_t v, const int lane)  A32: VMULL.S16 Qd, Dn, Dm[lane]  A64: SMULL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmull_lane_s32 (int32x2_t a, int32x2_t v, const int lane)  A32: VMULL.S32 Qd, Dn, Dm[lane]  A64: SMULL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmull_laneq_s32 (int32x2_t a, int32x4_t v, const int lane)  A32: VMULL.S32 Qd, Dn, Dm[lane]  A64: SMULL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmull_lane_u16 (uint16x4_t a, uint16x4_t v, const int lane)  A32: VMULL.U16 Qd, Dn, Dm[lane]  A64: UMULL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmull_laneq_u16 (uint16x4_t a, uint16x8_t v, const int lane)  A32: VMULL.U16 Qd, Dn, Dm[lane]  A64: UMULL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmull_lane_u32 (uint32x2_t a, uint32x2_t v, const int lane)  A32: VMULL.U32 Qd, Dn, Dm[lane]  A64: UMULL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmull_laneq_u32 (uint32x2_t a, uint32x4_t v, const int lane)  A32: VMULL.U32 Qd, Dn, Dm[lane]  A64: UMULL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlal_lane_s16 (int32x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VMLAL.S16 Qd, Dn, Dm[lane]  A64: SMLAL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlal_laneq_s16 (int32x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VMLAL.S16 Qd, Dn, Dm[lane]  A64: SMLAL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlal_lane_s32 (int64x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VMLAL.S32 Qd, Dn, Dm[lane]  A64: SMLAL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlal_laneq_s32 (int64x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VMLAL.S32 Qd, Dn, Dm[lane]  A64: SMLAL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlal_lane_u16 (uint32x4_t a, uint16x4_t b, uint16x4_t v, const int lane)  A32: VMLAL.U16 Qd, Dn, Dm[lane]  A64: UMLAL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlal_laneq_u16 (uint32x4_t a, uint16x4_t b, uint16x8_t v, const int lane)  A32: VMLAL.U16 Qd, Dn, Dm[lane]  A64: UMLAL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlal_lane_u32 (uint64x2_t a, uint32x2_t b, uint32x2_t v, const int lane)  A32: VMLAL.U32 Qd, Dn, Dm[lane]  A64: UMLAL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlal_laneq_u32 (uint64x2_t a, uint32x2_t b, uint32x4_t v, const int lane)  A32: VMLAL.U32 Qd, Dn, Dm[lane]  A64: UMLAL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsl_lane_s16 (int32x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VMLSL.S16 Qd, Dn, Dm[lane]  A64: SMLSL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsl_laneq_s16 (int32x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VMLSL.S16 Qd, Dn, Dm[lane]  A64: SMLSL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlsl_lane_s32 (int64x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VMLSL.S32 Qd, Dn, Dm[lane]  A64: SMLSL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlsl_laneq_s32 (int64x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VMLSL.S32 Qd, Dn, Dm[lane]  A64: SMLSL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsl_lane_u16 (uint32x4_t a, uint16x4_t b, uint16x4_t v, const int lane)  A32: VMLSL.U16 Qd, Dn, Dm[lane]  A64: UMLSL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsl_laneq_u16 (uint32x4_t a, uint16x4_t b, uint16x8_t v, const int lane)  A32: VMLSL.U16 Qd, Dn, Dm[lane]  A64: UMLSL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlsl_lane_u32 (uint64x2_t a, uint32x2_t b, uint32x2_t v, const int lane)  A32: VMLSL.U32 Qd, Dn, Dm[lane]  A64: UMLSL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlsl_laneq_u32 (uint64x2_t a, uint32x2_t b, uint32x4_t v, const int lane)  A32: VMLSL.U32 Qd, Dn, Dm[lane]  A64: UMLSL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmull_high_lane_s16 (int16x8_t a, int16x4_t v, const int lane)  A32: VMULL.S16 Qd, Dn+1, Dm[lane]  A64: SMULL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmull_high_laneq_s16 (int16x8_t a, int16x8_t v, const int lane)  A32: VMULL.S16 Qd, Dn+1, Dm[lane]  A64: SMULL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmull_high_lane_s32 (int32x4_t a, int32x2_t v, const int lane)  A32: VMULL.S32 Qd, Dn+1, Dm[lane]  A64: SMULL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmull_high_laneq_s32 (int32x4_t a, int32x4_t v, const int lane)  A32: VMULL.S32 Qd, Dn+1, Dm[lane]  A64: SMULL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmull_high_lane_u16 (uint16x8_t a, uint16x4_t v, const int lane)  A32: VMULL.U16 Qd, Dn+1, Dm[lane]  A64: UMULL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmull_high_laneq_u16 (uint16x8_t a, uint16x8_t v, const int lane)  A32: VMULL.U16 Qd, Dn+1, Dm[lane]  A64: UMULL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmull_high_lane_u32 (uint32x4_t a, uint32x2_t v, const int lane)  A32: VMULL.U32 Qd, Dn+1, Dm[lane]  A64: UMULL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmull_high_laneq_u32 (uint32x4_t a, uint32x4_t v, const int lane)  A32: VMULL.U32 Qd, Dn+1, Dm[lane]  A64: UMULL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlal_high_lane_s16 (int32x4_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VMLAL.S16 Qd, Dn+1, Dm[lane]  A64: SMLAL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlal_high_laneq_s16 (int32x4_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VMLAL.S16 Qd, Dn+1, Dm[lane]  A64: SMLAL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlal_high_lane_s32 (int64x2_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VMLAL.S32 Qd, Dn+1, Dm[lane]  A64: SMLAL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlal_high_laneq_s32 (int64x2_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VMLAL.S32 Qd, Dn+1, Dm[lane]  A64: SMLAL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlal_high_lane_u16 (uint32x4_t a, uint16x8_t b, uint16x4_t v, const int lane)  A32: VMLAL.U16 Qd, Dn+1, Dm[lane]  A64: UMLAL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlal_high_laneq_u16 (uint32x4_t a, uint16x8_t b, uint16x8_t v, const int lane)  A32: VMLAL.U16 Qd, Dn+1, Dm[lane]  A64: UMLAL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlal_high_lane_u32 (uint64x2_t a, uint32x4_t b, uint32x2_t v, const int lane)  A32: VMLAL.U32 Qd, Dn+1, Dm[lane]  A64: UMLAL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlal_high_laneq_u32 (uint64x2_t a, uint32x4_t b, uint32x4_t v, const int lane)  A32: VMLAL.U32 Qd, Dn+1, Dm[lane]  A64: UMLAL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsl_high_lane_s16 (int32x4_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VMLSL.S16 Qd, Dn+1, Dm[lane]  A64: SMLSL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsl_high_laneq_s16 (int32x4_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VMLSL.S16 Qd, Dn+1, Dm[lane]  A64: SMLSL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlsl_high_lane_s32 (int64x2_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VMLSL.S32 Qd, Dn+1, Dm[lane]  A64: SMLSL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlsl_high_laneq_s32 (int64x2_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VMLSL.S32 Qd, Dn+1, Dm[lane]  A64: SMLSL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsl_high_lane_u16 (uint32x4_t a, uint16x8_t b, uint16x4_t v, const int lane)  A32: VMLSL.U16 Qd, Dn+1, Dm[lane]  A64: UMLSL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsl_high_laneq_u16 (uint32x4_t a, uint16x8_t b, uint16x8_t v, const int lane)  A32: VMLSL.U16 Qd, Dn+1, Dm[lane]  A64: UMLSL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlsl_high_lane_u32 (uint64x2_t a, uint32x4_t b, uint32x2_t v, const int lane)  A32: VMLSL.U32 Qd, Dn+1, Dm[lane]  A64: UMLSL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlsl_high_laneq_u32 (uint64x2_t a, uint32x4_t b, uint32x4_t v, const int lane)  A32: VMLSL.U32 Qd, Dn+1, Dm[lane]  A64: UMLSL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqdmulh_n_s16 (int16x4_t a, int16_t b)  A32: VQDMULH.S16 Dd, Dn, Dm[0]  A64: SQDMULH Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqdmulh_n_s32 (int32x2_t a, int32_t b)  A32: VQDMULH.S32 Dd, Dn, Dm[0]  A64: SQDMULH Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqdmulhq_n_s16 (int16x8_t a, int16_t b)  A32: VQDMULH.S16 Qd, Qn, Dm[0]  A64: SQDMULH Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmulhq_n_s32 (int32x4_t a, int32_t b)  A32: VQDMULH.S32 Qd, Qn, Dm[0]  A64: SQDMULH Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqdmulh_lane_s16 (int16x4_t a, int16x4_t v, const int lane)  A32: VQDMULH.S16 Dd, Dn, Dm[lane]  A64: SQDMULH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqdmulh_laneq_s16 (int16x4_t a, int16x8_t v, const int lane)  A32: VQDMULH.S16 Dd, Dn, Dm[lane]  A64: SQDMULH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqdmulh_lane_s32 (int32x2_t a, int32x2_t v, const int lane)  A32: VQDMULH.S32 Dd, Dn, Dm[lane]  A64: SQDMULH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqdmulh_laneq_s32 (int32x2_t a, int32x4_t v, const int lane)  A32: VQDMULH.S32 Dd, Dn, Dm[lane]  A64: SQDMULH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqdmulhq_lane_s16 (int16x8_t a, int16x4_t v, const int lane)  A32: VQDMULH.S16 Qd, Qn, Dm[lane]  A64: SQDMULH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqdmulhq_laneq_s16 (int16x8_t a, int16x8_t v, const int lane)  A32: VQDMULH.S16 Qd, Qn, Dm[lane]  A64: SQDMULH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmulhq_lane_s32 (int32x4_t a, int32x2_t v, const int lane)  A32: VQDMULH.S32 Qd, Qn, Dm[lane]  A64: SQDMULH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmulhq_laneq_s32 (int32x4_t a, int32x4_t v, const int lane)  A32: VQDMULH.S32 Qd, Qn, Dm[lane]  A64: SQDMULH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqdmulh_s16 (int16x4_t a, int16x4_t b)  A32: VQDMULH.S16 Dd, Dn, Dm  A64: SQDMULH Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqdmulh_s32 (int32x2_t a, int32x2_t b)  A32: VQDMULH.S32 Dd, Dn, Dm  A64: SQDMULH Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqdmulhq_s16 (int16x8_t a, int16x8_t b)  A32: VQDMULH.S16 Qd, Qn, Qm  A64: SQDMULH Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmulhq_s32 (int32x4_t a, int32x4_t b)  A32: VQDMULH.S32 Qd, Qn, Qm  A64: SQDMULH Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_s16 (int32x4_t a, int16x4_t b, int16x4_t c)  A32: VQDMLAL.S16 Qd, Dn, Dm  A64: SQDMLAL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_s32 (int64x2_t a, int32x2_t b, int32x2_t c)  A32: VQDMLAL.S32 Qd, Dn, Dm  A64: SQDMLAL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_s16 (int32x4_t a, int16x4_t b, int16x4_t c)  A32: VQDMLSL.S16 Qd, Dn, Dm  A64: SQDMLSL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_s32 (int64x2_t a, int32x2_t b, int32x2_t c)  A32: VQDMLSL.S32 Qd, Dn, Dm  A64: SQDMLSL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_n_s16 (int32x4_t a, int16x4_t b, int16_t c)  A32: VQDMLAL.S16 Qd, Dn, Dm[0]  A64: SQDMLAL Vd.4S, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_n_s32 (int64x2_t a, int32x2_t b, int32_t c)  A32: VQDMLAL.S32 Qd, Dn, Dm[0]  A64: SQDMLAL Vd.2D, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_n_s16 (int32x4_t a, int16x4_t b, int16_t c)  A32: VQDMLSL.S16 Qd, Dn, Dm[0]  A64: SQDMLSL Vd.4S, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_n_s32 (int64x2_t a, int32x2_t b, int32_t c)  A32: VQDMLSL.S32 Qd, Dn, Dm[0]  A64: SQDMLSL Vd.2D, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_lane_s16 (int32x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VQDMLAL.S16 Qd, Dn, Dm[lane]  A64: SQDMLAL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_laneq_s16 (int32x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VQDMLAL.S16 Qd, Dn, Dm[lane]  A64: SQDMLAL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_lane_s32 (int64x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VQDMLAL.S32 Qd, Dn, Dm[lane]  A64: SQDMLAL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_laneq_s32 (int64x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VQDMLAL.S32 Qd, Dn, Dm[lane]  A64: SQDMLAL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_lane_s16 (int32x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VQDMLSL.S16 Qd, Dn, Dm[lane]  A64: SQDMLSL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_laneq_s16 (int32x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VQDMLSL.S16 Qd, Dn, Dm[lane]  A64: SQDMLSL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_lane_s32 (int64x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VQDMLSL.S32 Qd, Dn, Dm[lane]  A64: SQDMLSL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_lower_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_laneq_s32 (int64x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VQDMLSL.S32 Qd, Dn, Dm[lane]  A64: SQDMLSL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_s16 (int16x4_t a, int16x4_t b)  A32: VQDMULL.S16 Qd, Dn, Dm  A64: SQDMULL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_s32 (int32x2_t a, int32x2_t b)  A32: VQDMULL.S32 Qd, Dn, Dm  A64: SQDMULL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower_by_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_n_s16 (int16x4_t a, int16_t b)  A32: VQDMULL.S16 Qd, Dn, Dm[0]  A64: SQDMULL Vd.4S, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower_by_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_n_s32 (int32x2_t a, int32_t b)  A32: VQDMULL.S32 Qd, Dn, Dm[0]  A64: SQDMULL Vd.2D, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_lane_s16 (int16x4_t a, int16x4_t v, const int lane)  A32: VQDMULL.S16 Qd, Dn, Dm[lane]  A64: SQDMULL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_laneq_s16 (int16x4_t a, int16x8_t v, const int lane)  A32: VQDMULL.S16 Qd, Dn, Dm[lane]  A64: SQDMULL Vd.4S, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_lane_s32 (int32x2_t a, int32x2_t v, const int lane)  A32: VQDMULL.S32 Qd, Dn, Dm[lane]  A64: SQDMULL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_lower_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_laneq_s32 (int32x2_t a, int32x4_t v, const int lane)  A32: VQDMULL.S32 Qd, Dn, Dm[lane]  A64: SQDMULL Vd.2D, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_high_s16 (int16x8_t a, int16x8_t b)  A32: VQDMULL.S16 Qd, Dn+1, Dm+1  A64: SQDMULL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_high_s32 (int32x4_t a, int32x4_t b)  A32: VQDMULL.S32 Qd, Dn+1, Dm+1  A64: SQDMULL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper_by_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_high_n_s16 (int16x8_t a, int16_t b)  A32: VQDMULL.S16 Qd, Dn+1, Dm[0]  A64: SQDMULL2 Vd.4S, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper_by_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_high_n_s32 (int32x4_t a, int32_t b)  A32: VQDMULL.S32 Qd, Dn+1, Dm[0]  A64: SQDMULL2 Vd.2D, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_high_lane_s16 (int16x8_t a, int16x4_t v, const int lane)  A32: VQDMULL.S16 Qd, Dn+1, Dm[lane]  A64: SQDMULL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmull_high_laneq_s16 (int16x8_t a, int16x8_t v, const int lane)  A32: VQDMULL.S16 Qd, Dn+1, Dm[lane]  A64: SQDMULL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_high_lane_s32 (int32x4_t a, int32x2_t v, const int lane)  A32: VQDMULL.S32 Qd, Dn+1, Dm[lane]  A64: SQDMULL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_saturate_upper_by_selected_scalar(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmull_high_laneq_s32 (int32x4_t a, int32x4_t v, const int lane)  A32: VQDMULL.S32 Qd, Dn+1, Dm[lane]  A64: SQDMULL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_high_s16 (int32x4_t a, int16x8_t b, int16x8_t c)  A32: VQDMLAL.S16 Qd, Dn+1, Dm+1  A64: SQDMLAL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_high_s32 (int64x2_t a, int32x4_t b, int32x4_t c)  A32: VQDMLAL.S32 Qd, Dn+1, Dm+1  A64: SQDMLAL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_high_s16 (int32x4_t a, int16x8_t b, int16x8_t c)  A32: VQDMLSL.S16 Qd, Dn+1, Dm+1  A64: SQDMLSL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_high_s32 (int64x2_t a, int32x4_t b, int32x4_t c)  A32: VQDMLSL.S32 Qd, Dn+1, Dm+1  A64: SQDMLSL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_high_n_s16 (int32x4_t a, int16x8_t b, int16_t c)  A32: VQDMLAL.S16 Qd, Dn+1, Dm[0]  A64: SQDMLAL2 Vd.4S, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_high_n_s32 (int64x2_t a, int32x4_t b, int32_t c)  A32: VQDMLAL.S32 Qd, Dn+1, Dm[0]  A64: SQDMLAL2 Vd.2D, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_high_n_s16 (int32x4_t a, int16x8_t b, int16_t c)  A32: VQDMLSL.S16 Qd, Dn+1, Dm[0]  A64: SQDMLSL2 Vd.4S, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_high_n_s32 (int64x2_t a, int32x4_t b, int32_t c)  A32: VQDMLSL.S32 Qd, Dn+1, Dm[0]  A64: SQDMLSL2 Vd.2D, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_high_lane_s16 (int32x4_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VQDMLAL.S16 Qd, Dn+1, Dm[lane]  A64: SQDMLAL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlal_high_laneq_s16 (int32x4_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VQDMLAL.S16 Qd, Dn+1, Dm[lane]  A64: SQDMLAL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_high_lane_s32 (int64x2_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VQDMLAL.S32 Qd, Dn+1, Dm[lane]  A64: SQDMLAL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_add_saturate(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlal_high_laneq_s32 (int64x2_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VQDMLAL.S32 Qd, Dn+1, Dm[lane]  A64: SQDMLAL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_high_lane_s16 (int32x4_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VQDMLSL.S16 Qd, Dn+1, Dm[lane]  A64: SQDMLSL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqdmlsl_high_laneq_s16 (int32x4_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VQDMLSL.S16 Qd, Dn+1, Dm[lane]  A64: SQDMLSL2 Vd.4S, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_high_lane_s32 (int64x2_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VQDMLSL.S32 Qd, Dn+1, Dm[lane]  A64: SQDMLSL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_doubling_widening_upper_by_selected_scalar_and_subtract_saturate(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqdmlsl_high_laneq_s32 (int64x2_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VQDMLSL.S32 Qd, Dn+1, Dm[lane]  A64: SQDMLSL2 Vd.2D, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmulh_n_s16 (int16x4_t a, int16_t b)  A32: VQRDMULH.S16 Dd, Dn, Dm[0]  A64: SQRDMULH Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmulh_n_s32 (int32x2_t a, int32_t b)  A32: VQRDMULH.S32 Dd, Dn, Dm[0]  A64: SQRDMULH Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmulhq_n_s16 (int16x8_t a, int16_t b)  A32: VQRDMULH.S16 Qd, Qn, Dm[0]  A64: SQRDMULH Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmulhq_n_s32 (int32x4_t a, int32_t b)  A32: VQRDMULH.S32 Qd, Qn, Dm[0]  A64: SQRDMULH Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmulh_lane_s16 (int16x4_t a, int16x4_t v, const int lane)  A32: VQRDMULH.S16 Dd, Dn, Dm[lane]  A64: SQRDMULH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmulh_laneq_s16 (int16x4_t a, int16x8_t v, const int lane)  A32: VQRDMULH.S16 Dd, Dn, Dm[lane]  A64: SQRDMULH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmulh_lane_s32 (int32x2_t a, int32x2_t v, const int lane)  A32: VQRDMULH.S32 Dd, Dn, Dm[lane]  A64: SQRDMULH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmulh_laneq_s32 (int32x2_t a, int32x4_t v, const int lane)  A32: VQRDMULH.S32 Dd, Dn, Dm[lane]  A64: SQRDMULH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmulhq_lane_s16 (int16x8_t a, int16x4_t v, const int lane)  A32: VQRDMULH.S16 Qd, Qn, Dm[lane]  A64: SQRDMULH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmulhq_laneq_s16 (int16x8_t a, int16x8_t v, const int lane)  A32: VQRDMULH.S16 Qd, Qn, Dm[lane]  A64: SQRDMULH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmulhq_lane_s32 (int32x4_t a, int32x2_t v, const int lane)  A32: VQRDMULH.S32 Qd, Qn, Dm[lane]  A64: SQRDMULH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmulhq_laneq_s32 (int32x4_t a, int32x4_t v, const int lane)  A32: VQRDMULH.S32 Qd, Qn, Dm[lane]  A64: SQRDMULH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmulh_s16 (int16x4_t a, int16x4_t b)  A32: VQRDMULH.S16 Dd, Dn, Dm  A64: SQRDMULH Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmulh_s32 (int32x2_t a, int32x2_t b)  A32: VQRDMULH.S32 Dd, Dn, Dm  A64: SQRDMULH Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmulhq_s16 (int16x8_t a, int16x8_t b)  A32: VQRDMULH.S16 Qd, Qn, Qm  A64: SQRDMULH Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_saturate_high(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmulhq_s32 (int32x4_t a, int32x4_t b)  A32: VQRDMULH.S32 Qd, Qn, Qm  A64: SQRDMULH Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vmul_f64 (float64x1_t a, float64x1_t b)  A32: VMUL.F64 Dd, Dn, Dm  A64: FMUL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def multiply_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vmuls_f32 (float32_t a, float32_t b)  A32: VMUL.F32 Sd, Sn, Sm  A64: FMUL Sd, Sn, SmThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def multiply_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vmuls_lane_f32 (float32_t a, float32x2_t v, const int lane)  A32: VMUL.F32 Sd, Sn, Dm[lane]  A64: FMUL Sd, Sn, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_scalar_by_selected_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector128[float], right_index: int) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vmuls_laneq_f32 (float32_t a, float32x4_t v, const int lane)  A32: VMUL.F32 Sd, Sn, Dm[lane]  A64: FMUL Sd, Sn, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vmls_u8 (uint8x8_t a, uint8x8_t b, uint8x8_t c)  A32: VMLS.I8 Dd, Dn, Dm  A64: MLS Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmls_s16 (int16x4_t a, int16x4_t b, int16x4_t c)  A32: VMLS.I16 Dd, Dn, Dm  A64: MLS Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmls_s32 (int32x2_t a, int32x2_t b, int32x2_t c)  A32: VMLS.I32 Dd, Dn, Dm  A64: MLS Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vmls_s8 (int8x8_t a, int8x8_t b, int8x8_t c)  A32: VMLS.I8 Dd, Dn, Dm  A64: MLS Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmls_u16 (uint16x4_t a, uint16x4_t b, uint16x4_t c)  A32: VMLS.I16 Dd, Dn, Dm  A64: MLS Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmls_u32 (uint32x2_t a, uint32x2_t b, uint32x2_t c)  A32: VMLS.I32 Dd, Dn, Dm  A64: MLS Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vmlsq_u8 (uint8x16_t a, uint8x16_t b, uint8x16_t c)  A32: VMLS.I8 Qd, Qn, Qm  A64: MLS Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlsq_s16 (int16x8_t a, int16x8_t b, int16x8_t c)  A32: VMLS.I16 Qd, Qn, Qm  A64: MLS Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsq_s32 (int32x4_t a, int32x4_t b, int32x4_t c)  A32: VMLS.I32 Qd, Qn, Qm  A64: MLS Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vmlsq_s8 (int8x16_t a, int8x16_t b, int8x16_t c)  A32: VMLS.I8 Qd, Qn, Qm  A64: MLS Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlsq_u16 (uint16x8_t a, uint16x8_t b, uint16x8_t c)  A32: VMLS.I16 Qd, Qn, Qm  A64: MLS Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsq_u32 (uint32x4_t a, uint32x4_t b, uint32x4_t c)  A32: VMLS.I32 Qd, Qn, Qm  A64: MLS Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmls_n_s16 (int16x4_t a, int16x4_t b, int16_t c)  A32: VMLS.I16 Dd, Dn, Dm[0]  A64: MLS Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmls_n_s32 (int32x2_t a, int32x2_t b, int32_t c)  A32: VMLS.I32 Dd, Dn, Dm[0]  A64: MLS Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmls_n_u16 (uint16x4_t a, uint16x4_t b, uint16_t c)  A32: VMLS.I16 Dd, Dn, Dm[0]  A64: MLS Vd.4H, Vn.4H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmls_n_u32 (uint32x2_t a, uint32x2_t b, uint32_t c)  A32: VMLS.I32 Dd, Dn, Dm[0]  A64: MLS Vd.2S, Vn.2S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlsq_n_s16 (int16x8_t a, int16x8_t b, int16_t c)  A32: VMLS.I16 Qd, Qn, Dm[0]  A64: MLS Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsq_n_s32 (int32x4_t a, int32x4_t b, int32_t c)  A32: VMLS.I32 Qd, Qn, Dm[0]  A64: MLS Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlsq_n_u16 (uint16x8_t a, uint16x8_t b, uint16_t c)  A32: VMLS.I16 Qd, Qn, Dm[0]  A64: MLS Vd.8H, Vn.8H, Vm.H[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsq_n_u32 (uint32x4_t a, uint32x4_t b, uint32_t c)  A32: VMLS.I32 Qd, Qn, Dm[0]  A64: MLS Vd.4S, Vn.4S, Vm.S[0]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmls_lane_s16 (int16x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VMLS.I16 Dd, Dn, Dm[lane]  A64: MLS Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmls_laneq_s16 (int16x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VMLS.I16 Dd, Dn, Dm[lane]  A64: MLS Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmls_lane_s32 (int32x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VMLS.I32 Dd, Dn, Dm[lane]  A64: MLS Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmls_laneq_s32 (int32x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VMLS.I32 Dd, Dn, Dm[lane]  A64: MLS Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmls_lane_u16 (uint16x4_t a, uint16x4_t b, uint16x4_t v, const int lane)  A32: VMLS.I16 Dd, Dn, Dm[lane]  A64: MLS Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmls_laneq_u16 (uint16x4_t a, uint16x4_t b, uint16x8_t v, const int lane)  A32: VMLS.I16 Dd, Dn, Dm[lane]  A64: MLS Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmls_lane_u32 (uint32x2_t a, uint32x2_t b, uint32x2_t v, const int lane)  A32: VMLS.I32 Dd, Dn, Dm[lane]  A64: MLS Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmls_laneq_u32 (uint32x2_t a, uint32x2_t b, uint32x4_t v, const int lane)  A32: VMLS.I32 Dd, Dn, Dm[lane]  A64: MLS Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlsq_lane_s16 (int16x8_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VMLS.I16 Qd, Qn, Dm[lane]  A64: MLS Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlsq_laneq_s16 (int16x8_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VMLS.I16 Qd, Qn, Dm[lane]  A64: MLS Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsq_lane_s32 (int32x4_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VMLS.I32 Qd, Qn, Dm[lane]  A64: MLS Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsq_laneq_s32 (int32x4_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VMLS.I32 Qd, Qn, Dm[lane]  A64: MLS Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlsq_lane_u16 (uint16x8_t a, uint16x8_t b, uint16x4_t v, const int lane)  A32: VMLS.I16 Qd, Qn, Dm[lane]  A64: MLS Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlsq_laneq_u16 (uint16x8_t a, uint16x8_t b, uint16x8_t v, const int lane)  A32: VMLS.I16 Qd, Qn, Dm[lane]  A64: MLS Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsq_lane_u32 (uint32x4_t a, uint32x4_t b, uint32x2_t v, const int lane)  A32: VMLS.I32 Qd, Qn, Dm[lane]  A64: MLS Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_subtract_by_selected_scalar(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsq_laneq_u32 (uint32x4_t a, uint32x4_t b, uint32x4_t v, const int lane)  A32: VMLS.I32 Qd, Qn, Dm[lane]  A64: MLS Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmull_u8 (uint8x8_t a, uint8x8_t b)  A32: VMULL.U8 Qd, Dn, Dm  A64: UMULL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmull_s16 (int16x4_t a, int16x4_t b)  A32: VMULL.S16 Qd, Dn, Dm  A64: SMULL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmull_s32 (int32x2_t a, int32x2_t b)  A32: VMULL.S32 Qd, Dn, Dm  A64: SMULL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmull_s8 (int8x8_t a, int8x8_t b)  A32: VMULL.S8 Qd, Dn, Dm  A64: SMULL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmull_u16 (uint16x4_t a, uint16x4_t b)  A32: VMULL.U16 Qd, Dn, Dm  A64: UMULL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmull_u32 (uint32x2_t a, uint32x2_t b)  A32: VMULL.U32 Qd, Dn, Dm  A64: UMULL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlal_u8 (uint16x8_t a, uint8x8_t b, uint8x8_t c)  A32: VMLAL.U8 Qd, Dn, Dm  A64: UMLAL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlal_s16 (int32x4_t a, int16x4_t b, int16x4_t c)  A32: VMLAL.S16 Qd, Dn, Dm  A64: SMLAL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlal_s32 (int64x2_t a, int32x2_t b, int32x2_t c)  A32: VMLAL.S32 Qd, Dn, Dm  A64: SMLAL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlal_s8 (int16x8_t a, int8x8_t b, int8x8_t c)  A32: VMLAL.S8 Qd, Dn, Dm  A64: SMLAL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlal_u16 (uint32x4_t a, uint16x4_t b, uint16x4_t c)  A32: VMLAL.U16 Qd, Dn, Dm  A64: UMLAL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlal_u32 (uint64x2_t a, uint32x2_t b, uint32x2_t c)  A32: VMLAL.U32 Qd, Dn, Dm  A64: UMLAL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlsl_u8 (uint16x8_t a, uint8x8_t b, uint8x8_t c)  A32: VMLSL.U8 Qd, Dn, Dm  A64: UMLSL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsl_s16 (int32x4_t a, int16x4_t b, int16x4_t c)  A32: VMLSL.S16 Qd, Dn, Dm  A64: SMLSL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlsl_s32 (int64x2_t a, int32x2_t b, int32x2_t c)  A32: VMLSL.S32 Qd, Dn, Dm  A64: SMLSL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlsl_s8 (int16x8_t a, int8x8_t b, int8x8_t c)  A32: VMLSL.S8 Qd, Dn, Dm  A64: SMLSL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsl_u16 (uint32x4_t a, uint16x4_t b, uint16x4_t c)  A32: VMLSL.U16 Qd, Dn, Dm  A64: UMLSL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_lower_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlsl_u32 (uint64x2_t a, uint32x2_t b, uint32x2_t c)  A32: VMLSL.U32 Qd, Dn, Dm  A64: UMLSL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmull_high_u8 (uint8x16_t a, uint8x16_t b)  A32: VMULL.U8 Qd, Dn+1, Dm+1  A64: UMULL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmull_high_s16 (int16x8_t a, int16x8_t b)  A32: VMULL.S16 Qd, Dn+1, Dm+1  A64: SMULL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmull_high_s32 (int32x4_t a, int32x4_t b)  A32: VMULL.S32 Qd, Dn+1, Dm+1  A64: SMULL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmull_high_s8 (int8x16_t a, int8x16_t b)  A32: VMULL.S8 Qd, Dn+1, Dm+1  A64: SMULL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmull_high_u16 (uint16x8_t a, uint16x8_t b)  A32: VMULL.U16 Qd, Dn+1, Dm+1  A64: UMULL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmull_high_u32 (uint32x4_t a, uint32x4_t b)  A32: VMULL.U32 Qd, Dn+1, Dm+1  A64: UMULL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlal_high_u8 (uint16x8_t a, uint8x16_t b, uint8x16_t c)  A32: VMLAL.U8 Qd, Dn+1, Dm+1  A64: UMLAL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlal_high_s16 (int32x4_t a, int16x8_t b, int16x8_t c)  A32: VMLAL.S16 Qd, Dn+1, Dm+1  A64: SMLAL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlal_high_s32 (int64x2_t a, int32x4_t b, int32x4_t c)  A32: VMLAL.S32 Qd, Dn+1, Dm+1  A64: SMLAL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlal_high_s8 (int16x8_t a, int8x16_t b, int8x16_t c)  A32: VMLAL.S8 Qd, Dn+1, Dm+1  A64: SMLAL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlal_high_u16 (uint32x4_t a, uint16x8_t b, uint16x8_t c)  A32: VMLAL.U16 Qd, Dn+1, Dm+1  A64: UMLAL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_add(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlal_high_u32 (uint64x2_t a, uint32x4_t b, uint32x4_t c)  A32: VMLAL.U32 Qd, Dn+1, Dm+1  A64: UMLAL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmlsl_high_u8 (uint16x8_t a, uint8x16_t b, uint8x16_t c)  A32: VMLSL.U8 Qd, Dn+1, Dm+1  A64: UMLSL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmlsl_high_s16 (int32x4_t a, int16x8_t b, int16x8_t c)  A32: VMLSL.S16 Qd, Dn+1, Dm+1  A64: SMLSL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmlsl_high_s32 (int64x2_t a, int32x4_t b, int32x4_t c)  A32: VMLSL.S32 Qd, Dn+1, Dm+1  A64: SMLSL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmlsl_high_s8 (int16x8_t a, int8x16_t b, int8x16_t c)  A32: VMLSL.S8 Qd, Dn+1, Dm+1  A64: SMLSL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmlsl_high_u16 (uint32x4_t a, uint16x8_t b, uint16x8_t c)  A32: VMLSL.U16 Qd, Dn+1, Dm+1  A64: UMLSL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_widening_upper_and_subtract(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmlsl_high_u32 (uint64x2_t a, uint32x4_t b, uint32x4_t c)  A32: VMLSL.U32 Qd, Dn+1, Dm+1  A64: UMLSL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vneg_s16 (int16x4_t a)  A32: VNEG.S16 Dd, Dm  A64: NEG Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vneg_s32 (int32x2_t a)  A32: VNEG.S32 Dd, Dm  A64: NEG Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vneg_s8 (int8x8_t a)  A32: VNEG.S8 Dd, Dm  A64: NEG Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vneg_f32 (float32x2_t a)  A32: VNEG.F32 Dd, Dm  A64: FNEG Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vnegq_s16 (int16x8_t a)  A32: VNEG.S16 Qd, Qm  A64: NEG Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vnegq_s32 (int32x4_t a)  A32: VNEG.S32 Qd, Qm  A64: NEG Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vnegq_s8 (int8x16_t a)  A32: VNEG.S8 Qd, Qm  A64: NEG Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vnegq_f32 (float32x4_t a)  A32: VNEG.F32 Qd, Qm  A64: FNEG Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def negate_saturate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqneg_s16 (int16x4_t a)  A32: VQNEG.S16 Dd, Dm  A64: SQNEG Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def negate_saturate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqneg_s32 (int32x2_t a)  A32: VQNEG.S32 Dd, Dm  A64: SQNEG Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def negate_saturate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqneg_s8 (int8x8_t a)  A32: VQNEG.S8 Dd, Dm  A64: SQNEG Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def negate_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqnegq_s16 (int16x8_t a)  A32: VQNEG.S16 Qd, Qm  A64: SQNEG Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def negate_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqnegq_s32 (int32x4_t a)  A32: VQNEG.S32 Qd, Qm  A64: SQNEG Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def negate_saturate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqnegq_s8 (int8x16_t a)  A32: VQNEG.S8 Qd, Qm  A64: SQNEG Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def negate_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vneg_f64 (float64x1_t a)  A32: VNEG.F64 Dd, Dm  A64: FNEG Dd, Dn"""
        ...

    @staticmethod
    @overload
    def negate_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vnegs_f32 (float32_t a)  A32: VNEG.F32 Sd, Sm  A64: FNEG Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vmvn_u8 (uint8x8_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vmvn_f64 (float64x1_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vmvn_s16 (int16x4_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vmvn_s32 (int32x2_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vmvn_s64 (int64x1_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vmvn_s8 (int8x8_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vmvn_f32 (float32x2_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vmvn_u16 (uint16x4_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vmvn_u32 (uint32x2_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vmvn_u64 (uint64x1_t a)  A32: VMVN Dd, Dm  A64: MVN Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vmvnq_u8 (uint8x16_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vmvnq_f64 (float64x2_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmvnq_s16 (int16x8_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmvnq_s32 (int32x4_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmvnq_s64 (int64x2_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vmvnq_s8 (int8x16_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vmvnq_f32 (float32x4_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmvnq_u16 (uint16x8_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmvnq_u32 (uint32x4_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmvnq_u64 (uint64x2_t a)  A32: VMVN Qd, Qm  A64: MVN Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vorr_u8 (uint8x8_t a, uint8x8_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vorr_f64 (float64x1_t a, float64x1_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vorr_s16 (int16x4_t a, int16x4_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vorr_s32 (int32x2_t a, int32x2_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vorr_s64 (int64x1_t a, int64x1_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vorr_s8 (int8x8_t a, int8x8_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vorr_f32 (float32x2_t a, float32x2_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vorr_u16 (uint16x4_t a, uint16x4_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vorr_u32 (uint32x2_t a, uint32x2_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vorr_u64 (uint64x1_t a, uint64x1_t b)  A32: VORR Dd, Dn, Dm  A64: ORR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vorrq_u8 (uint8x16_t a, uint8x16_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vorrq_f64 (float64x2_t a, float64x2_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vorrq_s16 (int16x8_t a, int16x8_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vorrq_s32 (int32x4_t a, int32x4_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vorrq_s64 (int64x2_t a, int64x2_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vorrq_s8 (int8x16_t a, int8x16_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vorrq_f32 (float32x4_t a, float32x4_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vorrq_u16 (uint16x8_t a, uint16x8_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vorrq_u32 (uint32x4_t a, uint32x4_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vorrq_u64 (uint64x2_t a, uint64x2_t b)  A32: VORR Qd, Qn, Qm  A64: ORR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vorn_u8 (uint8x8_t a, uint8x8_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vorn_f64 (float64x1_t a, float64x1_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vorn_s16 (int16x4_t a, int16x4_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vorn_s32 (int32x2_t a, int32x2_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vorn_s64 (int64x1_t a, int64x1_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vorn_s8 (int8x8_t a, int8x8_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vorn_f32 (float32x2_t a, float32x2_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vorn_u16 (uint16x4_t a, uint16x4_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vorn_u32 (uint32x2_t a, uint32x2_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vorn_u64 (uint64x1_t a, uint64x1_t b)  A32: VORN Dd, Dn, Dm  A64: ORN Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vornq_u8 (uint8x16_t a, uint8x16_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t vornq_f64 (float64x2_t a, float64x2_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vornq_s16 (int16x8_t a, int16x8_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vornq_s32 (int32x4_t a, int32x4_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vornq_s64 (int64x2_t a, int64x2_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vornq_s8 (int8x16_t a, int8x16_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vornq_f32 (float32x4_t a, float32x4_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vornq_u16 (uint16x8_t a, uint16x8_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vornq_u32 (uint32x4_t a, uint32x4_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def or_not(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vornq_u64 (uint64x2_t a, uint64x2_t b)  A32: VORN Qd, Qn, Qm  A64: ORN Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """poly8x8_t vmul_p8 (poly8x8_t a, poly8x8_t b)  A32: VMUL.P8 Dd, Dn, Dm  A64: PMUL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """poly8x8_t vmul_p8 (poly8x8_t a, poly8x8_t b)  A32: VMUL.P8 Dd, Dn, Dm  A64: PMUL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly8x16_t vmulq_p8 (poly8x16_t a, poly8x16_t b)  A32: VMUL.P8 Qd, Qn, Qm  A64: PMUL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly8x16_t vmulq_p8 (poly8x16_t a, poly8x16_t b)  A32: VMUL.P8 Qd, Qn, Qm  A64: PMUL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly16x8_t vmull_p8 (poly8x8_t a, poly8x8_t b)  A32: VMULL.P8 Qd, Dn, Dm  A64: PMULL Vd.16B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly16x8_t vmull_p8 (poly8x8_t a, poly8x8_t b)  A32: VMULL.P8 Qd, Dn, Dm  A64: PMULL Vd.16B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly16x8_t vmull_high_p8 (poly8x16_t a, poly8x16_t b)  A32: VMULL.P8 Qd, Dn+1, Dm+1  A64: PMULL2 Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def polynomial_multiply_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """poly16x8_t vmull_high_p8 (poly8x16_t a, poly8x16_t b)  A32: VMULL.P8 Qd, Dn+1, Dm+1  A64: PMULL2 Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vcnt_u8 (uint8x8_t a)  A32: VCNT.I8 Dd, Dm  A64: CNT Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vcnt_s8 (int8x8_t a)  A32: VCNT.I8 Dd, Dm  A64: CNT Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vcntq_u8 (uint8x16_t a)  A32: VCNT.I8 Qd, Qm  A64: CNT Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vcntq_s8 (int8x16_t a)  A32: VCNT.I8 Qd, Qm  A64: CNT Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def reciprocal_estimate(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrecpe_f32 (float32x2_t a)  A32: VRECPE.F32 Dd, Dm  A64: FRECPE Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def reciprocal_estimate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrecpe_u32 (uint32x2_t a)  A32: VRECPE.U32 Dd, Dm  A64: URECPE Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def reciprocal_estimate(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrecpeq_f32 (float32x4_t a)  A32: VRECPE.F32 Qd, Qm  A64: FRECPE Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def reciprocal_estimate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrecpeq_u32 (uint32x4_t a)  A32: VRECPE.U32 Qd, Qm  A64: URECPE Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def reciprocal_square_root_estimate(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrsqrte_f32 (float32x2_t a)  A32: VRSQRTE.F32 Dd, Dm  A64: FRSQRTE Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def reciprocal_square_root_estimate(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrsqrte_u32 (uint32x2_t a)  A32: VRSQRTE.U32 Dd, Dm  A64: URSQRTE Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def reciprocal_square_root_estimate(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrsqrteq_f32 (float32x4_t a)  A32: VRSQRTE.F32 Qd, Qm  A64: FRSQRTE Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def reciprocal_square_root_estimate(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrsqrteq_u32 (uint32x4_t a)  A32: VRSQRTE.U32 Qd, Qm  A64: URSQRTE Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def reciprocal_square_root_step(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrsqrts_f32 (float32x2_t a, float32x2_t b)  A32: VRSQRTS.F32 Dd, Dn, Dm  A64: FRSQRTS Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def reciprocal_square_root_step(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrsqrtsq_f32 (float32x4_t a, float32x4_t b)  A32: VRSQRTS.F32 Qd, Qn, Qm  A64: FRSQRTS Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def reciprocal_step(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrecps_f32 (float32x2_t a, float32x2_t b)  A32: VRECPS.F32 Dd, Dn, Dm  A64: FRECPS Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def reciprocal_step(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrecpsq_f32 (float32x4_t a, float32x4_t b)  A32: VRECPS.F32 Qd, Qn, Qm  A64: FRECPS Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrev32_s16 (int16x4_t vec)  A32: VREV32.16 Dd, Dm  A64: REV32 Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrev64_s16 (int16x4_t vec)  A32: VREV64.16 Dd, Dm  A64: REV64 Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrev32_u16 (uint16x4_t vec)  A32: VREV32.16 Dd, Dm  A64: REV32 Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrev64_u16 (uint16x4_t vec)  A32: VREV64.16 Dd, Dm  A64: REV64 Vd.4H, Vn.4H"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrev32q_s16 (int16x8_t vec)  A32: VREV32.16 Qd, Qm  A64: REV32 Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrev64q_s16 (int16x8_t vec)  A32: VREV64.16 Qd, Qm  A64: REV64 Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrev32q_u16 (uint16x8_t vec)  A32: VREV32.16 Qd, Qm  A64: REV32 Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrev64q_u16 (uint16x8_t vec)  A32: VREV64.16 Qd, Qm  A64: REV64 Vd.8H, Vn.8H"""
        ...

    @staticmethod
    @overload
    def reverse_element_32(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vrev64_s32 (int32x2_t vec)  A32: VREV64.32 Dd, Dm  A64: REV64 Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def reverse_element_32(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrev64_u32 (uint32x2_t vec)  A32: VREV64.32 Dd, Dm  A64: REV64 Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def reverse_element_32(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vrev64q_s32 (int32x4_t vec)  A32: VREV64.32 Qd, Qm  A64: REV64 Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def reverse_element_32(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrev64q_u32 (uint32x4_t vec)  A32: VREV64.32 Qd, Qm  A64: REV64 Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrev16_s8 (int8x8_t vec)  A32: VREV16.8 Dd, Dm  A64: REV16 Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrev32_s8 (int8x8_t vec)  A32: VREV32.8 Dd, Dm  A64: REV32 Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrev64_s8 (int8x8_t vec)  A32: VREV64.8 Dd, Dm  A64: REV64 Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrev16_u8 (uint8x8_t vec)  A32: VREV16.8 Dd, Dm  A64: REV16 Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrev32_u8 (uint8x8_t vec)  A32: VREV32.8 Dd, Dm  A64: REV32 Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrev64_u8 (uint8x8_t vec)  A32: VREV64.8 Dd, Dm  A64: REV64 Vd.8B, Vn.8B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrev16q_s8 (int8x16_t vec)  A32: VREV16.8 Qd, Qm  A64: REV16 Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrev32q_s8 (int8x16_t vec)  A32: VREV32.8 Qd, Qm  A64: REV32 Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrev64q_s8 (int8x16_t vec)  A32: VREV64.8 Qd, Qm  A64: REV64 Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrev16q_u8 (uint8x16_t vec)  A32: VREV16.8 Qd, Qm  A64: REV16 Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrev32q_u8 (uint8x16_t vec)  A32: VREV32.8 Qd, Qm  A64: REV32 Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrev64q_u8 (uint8x16_t vec)  A32: VREV64.8 Qd, Qm  A64: REV64 Vd.16B, Vn.16B"""
        ...

    @staticmethod
    @overload
    def round_away_from_zero(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrnda_f32 (float32x2_t a)  A32: VRINTA.F32 Dd, Dm  A64: FRINTA Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def round_away_from_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrndaq_f32 (float32x4_t a)  A32: VRINTA.F32 Qd, Qm  A64: FRINTA Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def round_away_from_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vrnda_f64 (float64x1_t a)  A32: VRINTA.F64 Dd, Dm  A64: FRINTA Dd, Dn"""
        ...

    @staticmethod
    @overload
    def round_away_from_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vrndas_f32 (float32_t a)  A32: VRINTA.F32 Sd, Sm  A64: FRINTA Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def round_to_nearest(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrndn_f32 (float32x2_t a)  A32: VRINTN.F32 Dd, Dm  A64: FRINTN Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def round_to_nearest(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrndnq_f32 (float32x4_t a)  A32: VRINTN.F32 Qd, Qm  A64: FRINTN Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def round_to_nearest_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vrndn_f64 (float64x1_t a)  A32: VRINTN.F64 Dd, Dm  A64: FRINTN Dd, Dn"""
        ...

    @staticmethod
    @overload
    def round_to_nearest_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vrndns_f32 (float32_t a)  A32: VRINTN.F32 Sd, Sm  A64: FRINTN Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrndm_f32 (float32x2_t a)  A32: VRINTM.F32 Dd, Dm  A64: FRINTM Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def round_to_negative_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrndmq_f32 (float32x4_t a)  A32: VRINTM.F32 Qd, Qm  A64: FRINTM Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def round_to_negative_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vrndm_f64 (float64x1_t a)  A32: VRINTM.F64 Dd, Dm  A64: FRINTM Dd, Dn"""
        ...

    @staticmethod
    @overload
    def round_to_negative_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vrndms_f32 (float32_t a)  A32: VRINTM.F32 Sd, Sm  A64: FRINTM Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrndp_f32 (float32x2_t a)  A32: VRINTP.F32 Dd, Dm  A64: FRINTP Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def round_to_positive_infinity(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrndpq_f32 (float32x4_t a)  A32: VRINTP.F32 Qd, Qm  A64: FRINTP Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def round_to_positive_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vrndp_f64 (float64x1_t a)  A32: VRINTP.F64 Dd, Dm  A64: FRINTP Dd, Dn"""
        ...

    @staticmethod
    @overload
    def round_to_positive_infinity_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vrndps_f32 (float32_t a)  A32: VRINTP.F32 Sd, Sm  A64: FRINTP Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def round_to_zero(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vrnd_f32 (float32x2_t a)  A32: VRINTZ.F32 Dd, Dm  A64: FRINTZ Vd.2S, Vn.2S"""
        ...

    @staticmethod
    @overload
    def round_to_zero(value: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vrndq_f32 (float32x4_t a)  A32: VRINTZ.F32 Qd, Qm  A64: FRINTZ Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def round_to_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vrnd_f64 (float64x1_t a)  A32: VRINTZ.F64 Dd, Dm  A64: FRINTZ Dd, Dn"""
        ...

    @staticmethod
    @overload
    def round_to_zero_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vrnds_f32 (float32_t a)  A32: VRINTZ.F32 Sd, Sm  A64: FRINTZ Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def shift_arithmetic(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vshl_s16 (int16x4_t a, int16x4_t b)  A32: VSHL.S16 Dd, Dn, Dm  A64: SSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vshl_s32 (int32x2_t a, int32x2_t b)  A32: VSHL.S32 Dd, Dn, Dm  A64: SSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vshl_s8 (int8x8_t a, int8x8_t b)  A32: VSHL.S8 Dd, Dn, Dm  A64: SSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vshlq_s16 (int16x8_t a, int16x8_t b)  A32: VSHL.S16 Qd, Qn, Qm  A64: SSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vshlq_s32 (int32x4_t a, int32x4_t b)  A32: VSHL.S32 Qd, Qn, Qm  A64: SSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vshlq_s64 (int64x2_t a, int64x2_t b)  A32: VSHL.S64 Qd, Qn, Qm  A64: SSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vshlq_s8 (int8x16_t a, int8x16_t b)  A32: VSHL.S8 Qd, Qn, Qm  A64: SSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrshl_s16 (int16x4_t a, int16x4_t b)  A32: VRSHL.S16 Dd, Dn, Dm  A64: SRSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vrshl_s32 (int32x2_t a, int32x2_t b)  A32: VRSHL.S32 Dd, Dn, Dm  A64: SRSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrshl_s8 (int8x8_t a, int8x8_t b)  A32: VRSHL.S8 Dd, Dn, Dm  A64: SRSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrshlq_s16 (int16x8_t a, int16x8_t b)  A32: VRSHL.S16 Qd, Qn, Qm  A64: SRSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vrshlq_s32 (int32x4_t a, int32x4_t b)  A32: VRSHL.S32 Qd, Qn, Qm  A64: SRSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vrshlq_s64 (int64x2_t a, int64x2_t b)  A32: VRSHL.S64 Qd, Qn, Qm  A64: SRSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrshlq_s8 (int8x16_t a, int8x16_t b)  A32: VRSHL.S8 Qd, Qn, Qm  A64: SRSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrshl_s16 (int16x4_t a, int16x4_t b)  A32: VQRSHL.S16 Dd, Dn, Dm  A64: SQRSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrshl_s32 (int32x2_t a, int32x2_t b)  A32: VQRSHL.S32 Dd, Dn, Dm  A64: SQRSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqrshl_s8 (int8x8_t a, int8x8_t b)  A32: VQRSHL.S8 Dd, Dn, Dm  A64: SQRSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrshlq_s16 (int16x8_t a, int16x8_t b)  A32: VQRSHL.S16 Qd, Qn, Qm  A64: SQRSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrshlq_s32 (int32x4_t a, int32x4_t b)  A32: VQRSHL.S32 Qd, Qn, Qm  A64: SQRSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqrshlq_s64 (int64x2_t a, int64x2_t b)  A32: VQRSHL.S64 Qd, Qn, Qm  A64: SQRSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqrshlq_s8 (int8x16_t a, int8x16_t b)  A32: VQRSHL.S8 Qd, Qn, Qm  A64: SQRSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    def shift_arithmetic_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vqrshl_s64 (int64x1_t a, int64x1_t b)  A32: VQRSHL.S64 Dd, Dn, Dm  A64: SQRSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    def shift_arithmetic_rounded_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vrshl_s64 (int64x1_t a, int64x1_t b)  A32: VRSHL.S64 Dd, Dn, Dm  A64: SRSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqshl_s16 (int16x4_t a, int16x4_t b)  A32: VQSHL.S16 Dd, Dn, Dm  A64: SQSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqshl_s32 (int32x2_t a, int32x2_t b)  A32: VQSHL.S32 Dd, Dn, Dm  A64: SQSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqshl_s8 (int8x8_t a, int8x8_t b)  A32: VQSHL.S8 Dd, Dn, Dm  A64: SQSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqshlq_s16 (int16x8_t a, int16x8_t b)  A32: VQSHL.S16 Qd, Qn, Qm  A64: SQSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqshlq_s32 (int32x4_t a, int32x4_t b)  A32: VQSHL.S32 Qd, Qn, Qm  A64: SQSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqshlq_s64 (int64x2_t a, int64x2_t b)  A32: VQSHL.S64 Qd, Qn, Qm  A64: SQSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_arithmetic_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqshlq_s8 (int8x16_t a, int8x16_t b)  A32: VQSHL.S8 Qd, Qn, Qm  A64: SQSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    def shift_arithmetic_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vqshl_s64 (int64x1_t a, int64x1_t b)  A32: VQSHL.S64 Dd, Dn, Dm  A64: SQSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    def shift_arithmetic_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vshl_s64 (int64x1_t a, int64x1_t b)  A32: VSHL.S64 Dd, Dn, Dm  A64: SSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vsli_n_u8(uint8x8_t a, uint8x8_t b, __builtin_constant_p(n))  A32: VSLI.8 Dd, Dm, #n  A64: SLI Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vsli_n_s16(int16x4_t a, int16x4_t b, __builtin_constant_p(n))  A32: VSLI.16 Dd, Dm, #n  A64: SLI Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vsli_n_s32(int32x2_t a, int32x2_t b, __builtin_constant_p(n))  A32: VSLI.32 Dd, Dm, #n  A64: SLI Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vsli_n_s8(int8x8_t a, int8x8_t b, __builtin_constant_p(n))  A32: VSLI.8 Dd, Dm, #n  A64: SLI Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vsli_n_u16(uint16x4_t a, uint16x4_t b, __builtin_constant_p(n))  A32: VSLI.16 Dd, Dm, #n  A64: SLI Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vsli_n_u32(uint32x2_t a, uint32x2_t b, __builtin_constant_p(n))  A32: VSLI.32 Dd, Dm, #n  A64: SLI Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vsliq_n_u8(uint8x16_t a, uint8x16_t b, __builtin_constant_p(n))  A32: VSLI.8 Qd, Qm, #n  A64: SLI Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsliq_n_s16(int16x8_t a, int16x8_t b, __builtin_constant_p(n))  A32: VSLI.16 Qd, Qm, #n  A64: SLI Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsliq_n_s32(int32x4_t a, int32x4_t b, __builtin_constant_p(n))  A32: VSLI.32 Qd, Qm, #n  A64: SLI Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsliq_n_s64(int64x2_t a, int64x2_t b, __builtin_constant_p(n))  A32: VSLI.64 Qd, Qm, #n  A64: SLI Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vsliq_n_s8(int8x16_t a, int8x16_t b, __builtin_constant_p(n))  A32: VSLI.8 Qd, Qm, #n  A64: SLI Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsliq_n_u16(uint16x8_t a, uint16x8_t b, __builtin_constant_p(n))  A32: VSLI.16 Qd, Qm, #n  A64: SLI Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsliq_n_u32(uint32x4_t a, uint32x4_t b, __builtin_constant_p(n))  A32: VSLI.32 Qd, Qm, #n  A64: SLI Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsliq_n_u64(uint64x2_t a, uint64x2_t b, __builtin_constant_p(n))  A32: VSLI.64 Qd, Qm, #n  A64: SLI Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64_t vslid_n_s64(int64_t a, int64_t b, __builtin_constant_p(n))  A32: VSLI.64 Dd, Dm, #n  A64: SLI Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_and_insert_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64_t vslid_n_u64(uint64_t a, uint64_t b, __builtin_constant_p(n))  A32: VSLI.64 Dd, Dm, #n  A64: SLI Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vshl_n_u8 (uint8x8_t a, const int n)  A32: VSHL.I8 Dd, Dm, #n  A64: SHL Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vshl_n_s16 (int16x4_t a, const int n)  A32: VSHL.I16 Dd, Dm, #n  A64: SHL Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vshl_n_s32 (int32x2_t a, const int n)  A32: VSHL.I32 Dd, Dm, #n  A64: SHL Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vshl_n_s8 (int8x8_t a, const int n)  A32: VSHL.I8 Dd, Dm, #n  A64: SHL Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vshl_n_u16 (uint16x4_t a, const int n)  A32: VSHL.I16 Dd, Dm, #n  A64: SHL Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vshl_n_u32 (uint32x2_t a, const int n)  A32: VSHL.I32 Dd, Dm, #n  A64: SHL Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vshlq_n_u8 (uint8x16_t a, const int n)  A32: VSHL.I8 Qd, Qm, #n  A64: SHL Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vshlq_n_s16 (int16x8_t a, const int n)  A32: VSHL.I16 Qd, Qm, #n  A64: SHL Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vshlq_n_s64 (int64x2_t a, const int n)  A32: VSHL.I64 Qd, Qm, #n  A64: SHL Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vshlq_n_s8 (int8x16_t a, const int n)  A32: VSHL.I8 Qd, Qm, #n  A64: SHL Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshlq_n_u16 (uint16x8_t a, const int n)  A32: VSHL.I16 Qd, Qm, #n  A64: SHL Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshlq_n_u32 (uint32x4_t a, const int n)  A32: VSHL.I32 Qd, Qm, #n  A64: SHL Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vshlq_n_u64 (uint64x2_t a, const int n)  A32: VSHL.I64 Qd, Qm, #n  A64: SHL Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqshl_n_u8 (uint8x8_t a, const int n)  A32: VQSHL.U8 Dd, Dm, #n  A64: UQSHL Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqshl_n_s16 (int16x4_t a, const int n)  A32: VQSHL.S16 Dd, Dm, #n  A64: SQSHL Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqshl_n_s32 (int32x2_t a, const int n)  A32: VQSHL.S32 Dd, Dm, #n  A64: SQSHL Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqshl_n_s8 (int8x8_t a, const int n)  A32: VQSHL.S8 Dd, Dm, #n  A64: SQSHL Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqshl_n_u16 (uint16x4_t a, const int n)  A32: VQSHL.U16 Dd, Dm, #n  A64: UQSHL Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqshl_n_u32 (uint32x2_t a, const int n)  A32: VQSHL.U32 Dd, Dm, #n  A64: UQSHL Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqshlq_n_u8 (uint8x16_t a, const int n)  A32: VQSHL.U8 Qd, Qm, #n  A64: UQSHL Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqshlq_n_s16 (int16x8_t a, const int n)  A32: VQSHL.S16 Qd, Qm, #n  A64: SQSHL Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqshlq_n_s32 (int32x4_t a, const int n)  A32: VQSHL.S32 Qd, Qm, #n  A64: SQSHL Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqshlq_n_s64 (int64x2_t a, const int n)  A32: VQSHL.S64 Qd, Qm, #n  A64: SQSHL Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqshlq_n_s8 (int8x16_t a, const int n)  A32: VQSHL.S8 Qd, Qm, #n  A64: SQSHL Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqshlq_n_u16 (uint16x8_t a, const int n)  A32: VQSHL.U16 Qd, Qm, #n  A64: UQSHL Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqshlq_n_u32 (uint32x4_t a, const int n)  A32: VQSHL.U32 Qd, Qm, #n  A64: UQSHL Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqshlq_n_u64 (uint64x2_t a, const int n)  A32: VQSHL.U64 Qd, Qm, #n  A64: UQSHL Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vqshl_n_s64 (int64x1_t a, const int n)  A32: VQSHL.S64 Dd, Dm, #n  A64: SQSHL Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqshl_n_u64 (uint64x1_t a, const int n)  A32: VQSHL.U64 Dd, Dm, #n  A64: UQSHL Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_unsigned(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqshlu_n_s16 (int16x4_t a, const int n)  A32: VQSHLU.S16 Dd, Dm, #n  A64: SQSHLU Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_unsigned(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqshlu_n_s32 (int32x2_t a, const int n)  A32: VQSHLU.S32 Dd, Dm, #n  A64: SQSHLU Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_unsigned(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqshlu_n_s8 (int8x8_t a, const int n)  A32: VQSHLU.S8 Dd, Dm, #n  A64: SQSHLU Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_unsigned(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqshluq_n_s16 (int16x8_t a, const int n)  A32: VQSHLU.S16 Qd, Qm, #n  A64: SQSHLU Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_unsigned(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqshluq_n_s32 (int32x4_t a, const int n)  A32: VQSHLU.S32 Qd, Qm, #n  A64: SQSHLU Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_unsigned(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqshluq_n_s64 (int64x2_t a, const int n)  A32: VQSHLU.S64 Qd, Qm, #n  A64: SQSHLU Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_saturate_unsigned(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqshluq_n_s8 (int8x16_t a, const int n)  A32: VQSHLU.S8 Qd, Qm, #n  A64: SQSHLU Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    def shift_left_logical_saturate_unsigned_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqshlu_n_s64 (int64x1_t a, const int n)  A32: VQSHLU.S64 Dd, Dm, #n  A64: SQSHLU Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vshl_n_s64 (int64x1_t a, const int n)  A32: VSHL.I64 Dd, Dm, #n  A64: SHL Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vshl_n_u64 (uint64x1_t a, const int n)  A32: VSHL.I64 Dd, Dm, #n  A64: SHL Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_lower(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshll_n_u8 (uint8x8_t a, const int n)  A32: VSHLL.U8 Qd, Dm, #n  A64: USHLL Vd.8H, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_lower(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vshll_n_s16 (int16x4_t a, const int n)  A32: VSHLL.S16 Qd, Dm, #n  A64: SSHLL Vd.4S, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_lower(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vshll_n_s32 (int32x2_t a, const int n)  A32: VSHLL.S32 Qd, Dm, #n  A64: SSHLL Vd.2D, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_lower(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vshll_n_s8 (int8x8_t a, const int n)  A32: VSHLL.S8 Qd, Dm, #n  A64: SSHLL Vd.8H, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_lower(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshll_n_u16 (uint16x4_t a, const int n)  A32: VSHLL.U16 Qd, Dm, #n  A64: USHLL Vd.4S, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_lower(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vshll_n_u32 (uint32x2_t a, const int n)  A32: VSHLL.U32 Qd, Dm, #n  A64: USHLL Vd.2D, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_upper(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshll_high_n_u8 (uint8x16_t a, const int n)  A32: VSHLL.U8 Qd, Dm+1, #n  A64: USHLL2 Vd.8H, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_upper(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vshll_high_n_s16 (int16x8_t a, const int n)  A32: VSHLL.S16 Qd, Dm+1, #n  A64: SSHLL2 Vd.4S, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_upper(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vshll_high_n_s32 (int32x4_t a, const int n)  A32: VSHLL.S32 Qd, Dm+1, #n  A64: SSHLL2 Vd.2D, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_upper(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vshll_high_n_s8 (int8x16_t a, const int n)  A32: VSHLL.S8 Qd, Dm+1, #n  A64: SSHLL2 Vd.8H, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_upper(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshll_high_n_u16 (uint16x8_t a, const int n)  A32: VSHLL.U16 Qd, Dm+1, #n  A64: USHLL2 Vd.4S, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_left_logical_widening_upper(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vshll_high_n_u32 (uint32x4_t a, const int n)  A32: VSHLL.U32 Qd, Dm+1, #n  A64: USHLL2 Vd.2D, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vshl_u8 (uint8x8_t a, int8x8_t b)  A32: VSHL.U8 Dd, Dn, Dm  A64: USHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vshl_u16 (uint16x4_t a, int16x4_t b)  A32: VSHL.U16 Dd, Dn, Dm  A64: USHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vshl_u32 (uint32x2_t a, int32x2_t b)  A32: VSHL.U32 Dd, Dn, Dm  A64: USHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vshl_u8 (uint8x8_t a, int8x8_t b)  A32: VSHL.U8 Dd, Dn, Dm  A64: USHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vshl_u16 (uint16x4_t a, int16x4_t b)  A32: VSHL.U16 Dd, Dn, Dm  A64: USHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vshl_u32 (uint32x2_t a, int32x2_t b)  A32: VSHL.U32 Dd, Dn, Dm  A64: USHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VSHL.U8 Qd, Qn, Qm  A64: USHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VSHL.U16 Qd, Qn, Qm  A64: USHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VSHL.U32 Qd, Qn, Qm  A64: USHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VSHL.U64 Qd, Qn, Qm  A64: USHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VSHL.U8 Qd, Qn, Qm  A64: USHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VSHL.U16 Qd, Qn, Qm  A64: USHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VSHL.U32 Qd, Qn, Qm  A64: USHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VSHL.U64 Qd, Qn, Qm  A64: USHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrshl_u8 (uint8x8_t a, int8x8_t b)  A32: VRSHL.U8 Dd, Dn, Dm  A64: URSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrshl_u16 (uint16x4_t a, int16x4_t b)  A32: VRSHL.U16 Dd, Dn, Dm  A64: URSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrshl_u32 (uint32x2_t a, int32x2_t b)  A32: VRSHL.U32 Dd, Dn, Dm  A64: URSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrshl_u8 (uint8x8_t a, int8x8_t b)  A32: VRSHL.U8 Dd, Dn, Dm  A64: URSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrshl_u16 (uint16x4_t a, int16x4_t b)  A32: VRSHL.U16 Dd, Dn, Dm  A64: URSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrshl_u32 (uint32x2_t a, int32x2_t b)  A32: VRSHL.U32 Dd, Dn, Dm  A64: URSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VRSHL.U8 Qd, Qn, Qm  A64: URSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VRSHL.U16 Qd, Qn, Qm  A64: URSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VRSHL.U32 Qd, Qn, Qm  A64: URSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vrshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VRSHL.U64 Qd, Qn, Qm  A64: URSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VRSHL.U8 Qd, Qn, Qm  A64: URSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VRSHL.U16 Qd, Qn, Qm  A64: URSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VRSHL.U32 Qd, Qn, Qm  A64: URSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vrshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VRSHL.U64 Qd, Qn, Qm  A64: URSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqrshl_u8 (uint8x8_t a, int8x8_t b)  A32: VQRSHL.U8 Dd, Dn, Dm  A64: UQRSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqrshl_u16 (uint16x4_t a, int16x4_t b)  A32: VQRSHL.U16 Dd, Dn, Dm  A64: UQRSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqrshl_u32 (uint32x2_t a, int32x2_t b)  A32: VQRSHL.U32 Dd, Dn, Dm  A64: UQRSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqrshl_u8 (uint8x8_t a, int8x8_t b)  A32: VQRSHL.U8 Dd, Dn, Dm  A64: UQRSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqrshl_u16 (uint16x4_t a, int16x4_t b)  A32: VQRSHL.U16 Dd, Dn, Dm  A64: UQRSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqrshl_u32 (uint32x2_t a, int32x2_t b)  A32: VQRSHL.U32 Dd, Dn, Dm  A64: UQRSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqrshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VQRSHL.U8 Qd, Qn, Qm  A64: UQRSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqrshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VQRSHL.U16 Qd, Qn, Qm  A64: UQRSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqrshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VQRSHL.U32 Qd, Qn, Qm  A64: UQRSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqrshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VQRSHL.U64 Qd, Qn, Qm  A64: UQRSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqrshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VQRSHL.U8 Qd, Qn, Qm  A64: UQRSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqrshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VQRSHL.U16 Qd, Qn, Qm  A64: UQRSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqrshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VQRSHL.U32 Qd, Qn, Qm  A64: UQRSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqrshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VQRSHL.U64 Qd, Qn, Qm  A64: UQRSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqrshl_u64 (uint64x1_t a, int64x1_t b)  A32: VQRSHL.U64 Dd, Dn, Dm  A64: UQRSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqrshl_u64 (uint64x1_t a, int64x1_t b)  A32: VQRSHL.U64 Dd, Dn, Dm  A64: UQRSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vrshl_u64 (uint64x1_t a, int64x1_t b)  A32: VRSHL.U64 Dd, Dn, Dm  A64: URSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_logical_rounded_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vrshl_u64 (uint64x1_t a, int64x1_t b)  A32: VRSHL.U64 Dd, Dn, Dm  A64: URSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqshl_u8 (uint8x8_t a, int8x8_t b)  A32: VQSHL.U8 Dd, Dn, Dm  A64: UQSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqshl_u16 (uint16x4_t a, int16x4_t b)  A32: VQSHL.U16 Dd, Dn, Dm  A64: UQSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqshl_u32 (uint32x2_t a, int32x2_t b)  A32: VQSHL.U32 Dd, Dn, Dm  A64: UQSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqshl_u8 (uint8x8_t a, int8x8_t b)  A32: VQSHL.U8 Dd, Dn, Dm  A64: UQSHL Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqshl_u16 (uint16x4_t a, int16x4_t b)  A32: VQSHL.U16 Dd, Dn, Dm  A64: UQSHL Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqshl_u32 (uint32x2_t a, int32x2_t b)  A32: VQSHL.U32 Dd, Dn, Dm  A64: UQSHL Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VQSHL.U8 Qd, Qn, Qm  A64: UQSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VQSHL.U16 Qd, Qn, Qm  A64: UQSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VQSHL.U32 Qd, Qn, Qm  A64: UQSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VQSHL.U64 Qd, Qn, Qm  A64: UQSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqshlq_u8 (uint8x16_t a, int8x16_t b)  A32: VQSHL.U8 Qd, Qn, Qm  A64: UQSHL Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqshlq_u16 (uint16x8_t a, int16x8_t b)  A32: VQSHL.U16 Qd, Qn, Qm  A64: UQSHL Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqshlq_u32 (uint32x4_t a, int32x4_t b)  A32: VQSHL.U32 Qd, Qn, Qm  A64: UQSHL Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate(value: System.Runtime.Intrinsics.Vector128[int], count: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqshlq_u64 (uint64x2_t a, int64x2_t b)  A32: VQSHL.U64 Qd, Qn, Qm  A64: UQSHL Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqshl_u64 (uint64x1_t a, int64x1_t b)  A32: VQSHL.U64 Dd, Dn, Dm  A64: UQSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_logical_saturate_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqshl_u64 (uint64x1_t a, int64x1_t b)  A32: VQSHL.U64 Dd, Dn, Dm  A64: UQSHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_logical_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vshl_u64 (uint64x1_t a, int64x1_t b)  A32: VSHL.U64 Dd, Dn, Dm  A64: USHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_logical_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vshl_u64 (uint64x1_t a, int64x1_t b)  A32: VSHL.U64 Dd, Dn, Dm  A64: USHL Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vsri_n_u8(uint8x8_t a, uint8x8_t b, __builtin_constant_p(n))  A32: VSRI.8 Dd, Dm, #n  A64: SRI Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vsri_n_s16(int16x4_t a, int16x4_t b, __builtin_constant_p(n))  A32: VSRI.16 Dd, Dm, #n  A64: SRI Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vsri_n_s32(int32x2_t a, int32x2_t b, __builtin_constant_p(n))  A32: VSRI.32 Dd, Dm, #n  A64: SRI Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vsri_n_s8(int8x8_t a, int8x8_t b, __builtin_constant_p(n))  A32: VSRI.8 Dd, Dm, #n  A64: SRI Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vsri_n_u16(uint16x4_t a, uint16x4_t b, __builtin_constant_p(n))  A32: VSRI.16 Dd, Dm, #n  A64: SRI Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vsri_n_u32(uint32x2_t a, uint32x2_t b, __builtin_constant_p(n))  A32: VSRI.32 Dd, Dm, #n  A64: SRI Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vsriq_n_u8(uint8x16_t a, uint8x16_t b, __builtin_constant_p(n))  A32: VSRI.8 Qd, Qm, #n  A64: SRI Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsriq_n_s16(int16x8_t a, int16x8_t b, __builtin_constant_p(n))  A32: VSRI.16 Qd, Qm, #n  A64: SRI Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsriq_n_s32(int32x4_t a, int32x4_t b, __builtin_constant_p(n))  A32: VSRI.32 Qd, Qm, #n  A64: SRI Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsriq_n_s64(int64x2_t a, int64x2_t b, __builtin_constant_p(n))  A32: VSRI.64 Qd, Qm, #n  A64: SRI Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vsriq_n_s8(int8x16_t a, int8x16_t b, __builtin_constant_p(n))  A32: VSRI.8 Qd, Qm, #n  A64: SRI Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsriq_n_u16(uint16x8_t a, uint16x8_t b, __builtin_constant_p(n))  A32: VSRI.16 Qd, Qm, #n  A64: SRI Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsriq_n_u32(uint32x4_t a, uint32x4_t b, __builtin_constant_p(n))  A32: VSRI.32 Qd, Qm, #n  A64: SRI Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], shift: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsriq_n_u64(uint64x2_t a, uint64x2_t b, __builtin_constant_p(n))  A32: VSRI.64 Qd, Qm, #n  A64: SRI Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64_t vsrid_n_s64(int64_t a, int64_t b, __builtin_constant_p(n))  A32: VSRI.64 Dd, Dm, #n  A64: SRI Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_and_insert_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], shift: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64_t vsrid_n_u64(uint64_t a, uint64_t b, __builtin_constant_p(n))  A32: VSRI.64 Dd, Dm, #n  A64: SRI Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vshr_n_s16 (int16x4_t a, const int n)  A32: VSHR.S16 Dd, Dm, #n  A64: SSHR Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vshr_n_s32 (int32x2_t a, const int n)  A32: VSHR.S32 Dd, Dm, #n  A64: SSHR Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vshr_n_s8 (int8x8_t a, const int n)  A32: VSHR.S8 Dd, Dm, #n  A64: SSHR Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vshrq_n_s16 (int16x8_t a, const int n)  A32: VSHR.S16 Qd, Qm, #n  A64: SSHR Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vshrq_n_s32 (int32x4_t a, const int n)  A32: VSHR.S32 Qd, Qm, #n  A64: SSHR Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vshrq_n_s64 (int64x2_t a, const int n)  A32: VSHR.S64 Qd, Qm, #n  A64: SSHR Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vshrq_n_s8 (int8x16_t a, const int n)  A32: VSHR.S8 Qd, Qm, #n  A64: SSHR Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vsra_n_s16 (int16x4_t a, int16x4_t b, const int n)  A32: VSRA.S16 Dd, Dm, #n  A64: SSRA Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vsra_n_s32 (int32x2_t a, int32x2_t b, const int n)  A32: VSRA.S32 Dd, Dm, #n  A64: SSRA Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vsra_n_s8 (int8x8_t a, int8x8_t b, const int n)  A32: VSRA.S8 Dd, Dm, #n  A64: SSRA Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsraq_n_s16 (int16x8_t a, int16x8_t b, const int n)  A32: VSRA.S16 Qd, Qm, #n  A64: SSRA Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsraq_n_s32 (int32x4_t a, int32x4_t b, const int n)  A32: VSRA.S32 Qd, Qm, #n  A64: SSRA Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsraq_n_s64 (int64x2_t a, int64x2_t b, const int n)  A32: VSRA.S64 Qd, Qm, #n  A64: SSRA Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vsraq_n_s8 (int8x16_t a, int8x16_t b, const int n)  A32: VSRA.S8 Qd, Qm, #n  A64: SSRA Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    def shift_right_arithmetic_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vsra_n_s64 (int64x1_t a, int64x1_t b, const int n)  A32: VSRA.S64 Dd, Dm, #n  A64: SSRA Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqshrn_n_s32 (int32x4_t a, const int n)  A32: VQSHRN.S32 Dd, Qm, #n  A64: SQSHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqshrn_n_s64 (int64x2_t a, const int n)  A32: VQSHRN.S64 Dd, Qm, #n  A64: SQSHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqshrn_n_s16 (int16x8_t a, const int n)  A32: VQSHRN.S16 Dd, Qm, #n  A64: SQSHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqshrun_n_s16 (int16x8_t a, const int n)  A32: VQSHRUN.S16 Dd, Qm, #n  A64: SQSHRUN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqshrun_n_s32 (int32x4_t a, const int n)  A32: VQSHRUN.S32 Dd, Qm, #n  A64: SQSHRUN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqshrun_n_s64 (int64x2_t a, const int n)  A32: VQSHRUN.S64 Dd, Qm, #n  A64: SQSHRUN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqshrun_high_n_s16 (uint8x8_t r, int16x8_t a, const int n)  A32: VQSHRUN.S16 Dd+1, Dn, #n  A64: SQSHRUN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqshrun_high_n_s32 (uint16x4_t r, int32x4_t a, const int n)  A32: VQSHRUN.S32 Dd+1, Dn, #n  A64: SQSHRUN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqshrun_high_n_s64 (uint32x2_t r, int64x2_t a, const int n)  A32: VQSHRUN.S64 Dd+1, Dn, #n  A64: SQSHRUN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqshrn_high_n_s32 (int16x4_t r, int32x4_t a, const int n)  A32: VQSHRN.S32 Dd+1, Qm, #n  A64: SQSHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqshrn_high_n_s64 (int32x2_t r, int64x2_t a, const int n)  A32: VQSHRN.S64 Dd+1, Qm, #n  A64: SQSHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqshrn_high_n_s16 (int8x8_t r, int16x8_t a, const int n)  A32: VQSHRN.S16 Dd+1, Qm, #n  A64: SQSHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrshr_n_s16 (int16x4_t a, const int n)  A32: VRSHR.S16 Dd, Dm, #n  A64: SRSHR Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vrshr_n_s32 (int32x2_t a, const int n)  A32: VRSHR.S32 Dd, Dm, #n  A64: SRSHR Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrshr_n_s8 (int8x8_t a, const int n)  A32: VRSHR.S8 Dd, Dm, #n  A64: SRSHR Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrshrq_n_s16 (int16x8_t a, const int n)  A32: VRSHR.S16 Qd, Qm, #n  A64: SRSHR Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vrshrq_n_s32 (int32x4_t a, const int n)  A32: VRSHR.S32 Qd, Qm, #n  A64: SRSHR Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vrshrq_n_s64 (int64x2_t a, const int n)  A32: VRSHR.S64 Qd, Qm, #n  A64: SRSHR Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrshrq_n_s8 (int8x16_t a, const int n)  A32: VRSHR.S8 Qd, Qm, #n  A64: SRSHR Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrsra_n_s16 (int16x4_t a, int16x4_t b, const int n)  A32: VRSRA.S16 Dd, Dm, #n  A64: SRSRA Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vrsra_n_s32 (int32x2_t a, int32x2_t b, const int n)  A32: VRSRA.S32 Dd, Dm, #n  A64: SRSRA Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrsra_n_s8 (int8x8_t a, int8x8_t b, const int n)  A32: VRSRA.S8 Dd, Dm, #n  A64: SRSRA Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrsraq_n_s16 (int16x8_t a, int16x8_t b, const int n)  A32: VRSRA.S16 Qd, Qm, #n  A64: SRSRA Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vrsraq_n_s32 (int32x4_t a, int32x4_t b, const int n)  A32: VRSRA.S32 Qd, Qm, #n  A64: SRSRA Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vrsraq_n_s64 (int64x2_t a, int64x2_t b, const int n)  A32: VRSRA.S64 Qd, Qm, #n  A64: SRSRA Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrsraq_n_s8 (int8x16_t a, int8x16_t b, const int n)  A32: VRSRA.S8 Qd, Qm, #n  A64: SRSRA Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    def shift_right_arithmetic_rounded_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vrsra_n_s64 (int64x1_t a, int64x1_t b, const int n)  A32: VRSRA.S64 Dd, Dm, #n  A64: SRSRA Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrshrn_n_s32 (int32x4_t a, const int n)  A32: VQRSHRN.S32 Dd, Qm, #n  A64: SQRSHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrshrn_n_s64 (int64x2_t a, const int n)  A32: VQRSHRN.S64 Dd, Qm, #n  A64: SQRSHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqrshrn_n_s16 (int16x8_t a, const int n)  A32: VQRSHRN.S16 Dd, Qm, #n  A64: SQRSHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqrshrun_n_s16 (int16x8_t a, const int n)  A32: VQRSHRUN.S16 Dd, Qm, #n  A64: SQRSHRUN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqrshrun_n_s32 (int32x4_t a, const int n)  A32: VQRSHRUN.S32 Dd, Qm, #n  A64: SQRSHRUN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqrshrun_n_s64 (int64x2_t a, const int n)  A32: VQRSHRUN.S64 Dd, Qm, #n  A64: SQRSHRUN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqrshrun_high_n_s16 (uint8x8_t r, int16x8_t a, const int n)  A32: VQRSHRUN.S16 Dd+1, Dn, #n  A64: SQRSHRUN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqrshrun_high_n_s32 (uint16x4_t r, int32x4_t a, const int n)  A32: VQRSHRUN.S32 Dd+1, Dn, #n  A64: SQRSHRUN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_unsigned_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqrshrun_high_n_s64 (uint32x2_t r, int64x2_t a, const int n)  A32: VQRSHRUN.S64 Dd+1, Dn, #n  A64: SQRSHRUN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrshrn_high_n_s32 (int16x4_t r, int32x4_t a, const int n)  A32: VQRSHRN.S32 Dd+1, Dn, #n  A64: SQRSHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrshrn_high_n_s64 (int32x2_t r, int64x2_t a, const int n)  A32: VQRSHRN.S64 Dd+1, Dn, #n  A64: SQRSHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqrshrn_high_n_s16 (int8x8_t r, int16x8_t a, const int n)  A32: VQRSHRN.S16 Dd+1, Dn, #n  A64: SQRSHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    def shift_right_arithmetic_rounded_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vrshr_n_s64 (int64x1_t a, const int n)  A32: VRSHR.S64 Dd, Dm, #n  A64: SRSHR Dd, Dn, #n"""
        ...

    @staticmethod
    def shift_right_arithmetic_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vshr_n_s64 (int64x1_t a, const int n)  A32: VSHR.S64 Dd, Dm, #n  A64: SSHR Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vshr_n_u8 (uint8x8_t a, const int n)  A32: VSHR.U8 Dd, Dm, #n  A64: USHR Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vshr_n_u16 (uint16x4_t a, const int n)  A32: VSHR.U16 Dd, Dm, #n  A64: USHR Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vshr_n_u32 (uint32x2_t a, const int n)  A32: VSHR.U32 Dd, Dm, #n  A64: USHR Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vshr_n_u8 (uint8x8_t a, const int n)  A32: VSHR.U8 Dd, Dm, #n  A64: USHR Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vshr_n_u16 (uint16x4_t a, const int n)  A32: VSHR.U16 Dd, Dm, #n  A64: USHR Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vshr_n_u32 (uint32x2_t a, const int n)  A32: VSHR.U32 Dd, Dm, #n  A64: USHR Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vshrq_n_u8 (uint8x16_t a, const int n)  A32: VSHR.U8 Qd, Qm, #n  A64: USHR Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshrq_n_u16 (uint16x8_t a, const int n)  A32: VSHR.U16 Qd, Qm, #n  A64: USHR Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshrq_n_u32 (uint32x4_t a, const int n)  A32: VSHR.U32 Qd, Qm, #n  A64: USHR Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vshrq_n_u64 (uint64x2_t a, const int n)  A32: VSHR.U64 Qd, Qm, #n  A64: USHR Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vshrq_n_u8 (uint8x16_t a, const int n)  A32: VSHR.U8 Qd, Qm, #n  A64: USHR Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshrq_n_u16 (uint16x8_t a, const int n)  A32: VSHR.U16 Qd, Qm, #n  A64: USHR Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshrq_n_u32 (uint32x4_t a, const int n)  A32: VSHR.U32 Qd, Qm, #n  A64: USHR Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vshrq_n_u64 (uint64x2_t a, const int n)  A32: VSHR.U64 Qd, Qm, #n  A64: USHR Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vsra_n_u8 (uint8x8_t a, uint8x8_t b, const int n)  A32: VSRA.U8 Dd, Dm, #n  A64: USRA Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vsra_n_u16 (uint16x4_t a, uint16x4_t b, const int n)  A32: VSRA.U16 Dd, Dm, #n  A64: USRA Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vsra_n_u32 (uint32x2_t a, uint32x2_t b, const int n)  A32: VSRA.U32 Dd, Dm, #n  A64: USRA Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vsra_n_u8 (uint8x8_t a, uint8x8_t b, const int n)  A32: VSRA.U8 Dd, Dm, #n  A64: USRA Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vsra_n_u16 (uint16x4_t a, uint16x4_t b, const int n)  A32: VSRA.U16 Dd, Dm, #n  A64: USRA Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vsra_n_u32 (uint32x2_t a, uint32x2_t b, const int n)  A32: VSRA.U32 Dd, Dm, #n  A64: USRA Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vsraq_n_u8 (uint8x16_t a, uint8x16_t b, const int n)  A32: VSRA.U8 Qd, Qm, #n  A64: USRA Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsraq_n_u16 (uint16x8_t a, uint16x8_t b, const int n)  A32: VSRA.U16 Qd, Qm, #n  A64: USRA Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsraq_n_u32 (uint32x4_t a, uint32x4_t b, const int n)  A32: VSRA.U32 Qd, Qm, #n  A64: USRA Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsraq_n_u64 (uint64x2_t a, uint64x2_t b, const int n)  A32: VSRA.U64 Qd, Qm, #n  A64: USRA Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vsraq_n_u8 (uint8x16_t a, uint8x16_t b, const int n)  A32: VSRA.U8 Qd, Qm, #n  A64: USRA Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsraq_n_u16 (uint16x8_t a, uint16x8_t b, const int n)  A32: VSRA.U16 Qd, Qm, #n  A64: USRA Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsraq_n_u32 (uint32x4_t a, uint32x4_t b, const int n)  A32: VSRA.U32 Qd, Qm, #n  A64: USRA Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsraq_n_u64 (uint64x2_t a, uint64x2_t b, const int n)  A32: VSRA.U64 Qd, Qm, #n  A64: USRA Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vsra_n_u64 (uint64x1_t a, uint64x1_t b, const int n)  A32: VSRA.U64 Dd, Dm, #n  A64: USRA Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vsra_n_u64 (uint64x1_t a, uint64x1_t b, const int n)  A32: VSRA.U64 Dd, Dm, #n  A64: USRA Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vshrn_n_u16 (uint16x8_t a, const int n)  A32: VSHRN.I16 Dd, Qm, #n  A64: SHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vshrn_n_s32 (int32x4_t a, const int n)  A32: VSHRN.I32 Dd, Qm, #n  A64: SHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vshrn_n_s64 (int64x2_t a, const int n)  A32: VSHRN.I64 Dd, Qm, #n  A64: SHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vshrn_n_s16 (int16x8_t a, const int n)  A32: VSHRN.I16 Dd, Qm, #n  A64: SHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vshrn_n_u32 (uint32x4_t a, const int n)  A32: VSHRN.I32 Dd, Qm, #n  A64: SHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vshrn_n_u64 (uint64x2_t a, const int n)  A32: VSHRN.I64 Dd, Qm, #n  A64: SHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqshrn_n_u16 (uint16x8_t a, const int n)  A32: VQSHRN.U16 Dd, Qm, #n  A64: UQSHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqshrn_n_u32 (uint32x4_t a, const int n)  A32: VQSHRN.U32 Dd, Qm, #n  A64: UQSHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqshrn_n_u64 (uint64x2_t a, const int n)  A32: VQSHRN.U64 Dd, Qm, #n  A64: UQSHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqshrn_n_u16 (uint16x8_t a, const int n)  A32: VQSHRN.U16 Dd, Qm, #n  A64: UQSHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqshrn_n_u32 (uint32x4_t a, const int n)  A32: VQSHRN.U32 Dd, Qm, #n  A64: UQSHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqshrn_n_u64 (uint64x2_t a, const int n)  A32: VQSHRN.U64 Dd, Qm, #n  A64: UQSHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqshrn_high_n_u16 (uint8x8_t r, uint16x8_t a, const int n)  A32: VQSHRN.U16 Dd+1, Qm, #n  A64: UQSHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqshrn_high_n_u32 (uint16x4_t r, uint32x4_t a, const int n)  A32: VQSHRN.U32 Dd+1, Qm, #n  A64: UQSHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqshrn_high_n_u64 (uint32x2_t r, uint64x2_t a, const int n)  A32: VQSHRN.U64 Dd+1, Qm, #n  A64: UQSHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqshrn_high_n_u16 (uint8x8_t r, uint16x8_t a, const int n)  A32: VQSHRN.U16 Dd+1, Qm, #n  A64: UQSHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqshrn_high_n_u32 (uint16x4_t r, uint32x4_t a, const int n)  A32: VQSHRN.U32 Dd+1, Qm, #n  A64: UQSHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqshrn_high_n_u64 (uint32x2_t r, uint64x2_t a, const int n)  A32: VQSHRN.U64 Dd+1, Qm, #n  A64: UQSHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vshrn_high_n_u16 (uint8x8_t r, uint16x8_t a, const int n)  A32: VSHRN.I16 Dd+1, Qm, #n  A64: SHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vshrn_high_n_s32 (int16x4_t r, int32x4_t a, const int n)  A32: VSHRN.I32 Dd+1, Qm, #n  A64: SHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vshrn_high_n_s64 (int32x2_t r, int64x2_t a, const int n)  A32: VSHRN.I64 Dd+1, Qm, #n  A64: SHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vshrn_high_n_s16 (int8x8_t r, int16x8_t a, const int n)  A32: VSHRN.I16 Dd+1, Qm, #n  A64: SHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vshrn_high_n_u32 (uint16x4_t r, uint32x4_t a, const int n)  A32: VSHRN.I32 Dd+1, Qm, #n  A64: SHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vshrn_high_n_u64 (uint32x2_t r, uint64x2_t a, const int n)  A32: VSHRN.I64 Dd+1, Qm, #n  A64: SHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrshr_n_u8 (uint8x8_t a, const int n)  A32: VRSHR.U8 Dd, Dm, #n  A64: URSHR Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrshr_n_u16 (uint16x4_t a, const int n)  A32: VRSHR.U16 Dd, Dm, #n  A64: URSHR Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrshr_n_u32 (uint32x2_t a, const int n)  A32: VRSHR.U32 Dd, Dm, #n  A64: URSHR Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrshr_n_u8 (uint8x8_t a, const int n)  A32: VRSHR.U8 Dd, Dm, #n  A64: URSHR Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrshr_n_u16 (uint16x4_t a, const int n)  A32: VRSHR.U16 Dd, Dm, #n  A64: URSHR Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrshr_n_u32 (uint32x2_t a, const int n)  A32: VRSHR.U32 Dd, Dm, #n  A64: URSHR Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrshrq_n_u8 (uint8x16_t a, const int n)  A32: VRSHR.U8 Qd, Qm, #n  A64: URSHR Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrshrq_n_u16 (uint16x8_t a, const int n)  A32: VRSHR.U16 Qd, Qm, #n  A64: URSHR Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrshrq_n_u32 (uint32x4_t a, const int n)  A32: VRSHR.U32 Qd, Qm, #n  A64: URSHR Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vrshrq_n_u64 (uint64x2_t a, const int n)  A32: VRSHR.U64 Qd, Qm, #n  A64: URSHR Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrshrq_n_u8 (uint8x16_t a, const int n)  A32: VRSHR.U8 Qd, Qm, #n  A64: URSHR Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrshrq_n_u16 (uint16x8_t a, const int n)  A32: VRSHR.U16 Qd, Qm, #n  A64: URSHR Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrshrq_n_u32 (uint32x4_t a, const int n)  A32: VRSHR.U32 Qd, Qm, #n  A64: URSHR Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vrshrq_n_u64 (uint64x2_t a, const int n)  A32: VRSHR.U64 Qd, Qm, #n  A64: URSHR Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrsra_n_u8 (uint8x8_t a, uint8x8_t b, const int n)  A32: VRSRA.U8 Dd, Dm, #n  A64: URSRA Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrsra_n_u16 (uint16x4_t a, uint16x4_t b, const int n)  A32: VRSRA.U16 Dd, Dm, #n  A64: URSRA Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrsra_n_u32 (uint32x2_t a, uint32x2_t b, const int n)  A32: VRSRA.U32 Dd, Dm, #n  A64: URSRA Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrsra_n_u8 (uint8x8_t a, uint8x8_t b, const int n)  A32: VRSRA.U8 Dd, Dm, #n  A64: URSRA Vd.8B, Vn.8B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrsra_n_u16 (uint16x4_t a, uint16x4_t b, const int n)  A32: VRSRA.U16 Dd, Dm, #n  A64: URSRA Vd.4H, Vn.4H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrsra_n_u32 (uint32x2_t a, uint32x2_t b, const int n)  A32: VRSRA.U32 Dd, Dm, #n  A64: URSRA Vd.2S, Vn.2S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrsraq_n_u8 (uint8x16_t a, uint8x16_t b, const int n)  A32: VRSRA.U8 Qd, Qm, #n  A64: URSRA Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrsraq_n_u16 (uint16x8_t a, uint16x8_t b, const int n)  A32: VRSRA.U16 Qd, Qm, #n  A64: URSRA Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrsraq_n_u32 (uint32x4_t a, uint32x4_t b, const int n)  A32: VRSRA.U32 Qd, Qm, #n  A64: URSRA Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vrsraq_n_u64 (uint64x2_t a, uint64x2_t b, const int n)  A32: VRSRA.U64 Qd, Qm, #n  A64: URSRA Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrsraq_n_u8 (uint8x16_t a, uint8x16_t b, const int n)  A32: VRSRA.U8 Qd, Qm, #n  A64: URSRA Vd.16B, Vn.16B, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrsraq_n_u16 (uint16x8_t a, uint16x8_t b, const int n)  A32: VRSRA.U16 Qd, Qm, #n  A64: URSRA Vd.8H, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrsraq_n_u32 (uint32x4_t a, uint32x4_t b, const int n)  A32: VRSRA.U32 Qd, Qm, #n  A64: URSRA Vd.4S, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add(addend: System.Runtime.Intrinsics.Vector128[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vrsraq_n_u64 (uint64x2_t a, uint64x2_t b, const int n)  A32: VRSRA.U64 Qd, Qm, #n  A64: URSRA Vd.2D, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vrsra_n_u64 (uint64x1_t a, uint64x1_t b, const int n)  A32: VRSRA.U64 Dd, Dm, #n  A64: URSRA Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_add_scalar(addend: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vrsra_n_u64 (uint64x1_t a, uint64x1_t b, const int n)  A32: VRSRA.U64 Dd, Dm, #n  A64: URSRA Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrshrn_n_u16 (uint16x8_t a, const int n)  A32: VRSHRN.I16 Dd, Qm, #n  A64: RSHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrshrn_n_s32 (int32x4_t a, const int n)  A32: VRSHRN.I32 Dd, Qm, #n  A64: RSHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vrshrn_n_s64 (int64x2_t a, const int n)  A32: VRSHRN.I64 Dd, Qm, #n  A64: RSHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrshrn_n_s16 (int16x8_t a, const int n)  A32: VRSHRN.I16 Dd, Qm, #n  A64: RSHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrshrn_n_u32 (uint32x4_t a, const int n)  A32: VRSHRN.I32 Dd, Qm, #n  A64: RSHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrshrn_n_u64 (uint64x2_t a, const int n)  A32: VRSHRN.I64 Dd, Qm, #n  A64: RSHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqrshrn_n_u16 (uint16x8_t a, const int n)  A32: VQRSHRN.U16 Dd, Qm, #n  A64: UQRSHRN Vd.8B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqrshrn_n_u32 (uint32x4_t a, const int n)  A32: VQRSHRN.U32 Dd, Qm, #n  A64: UQRSHRN Vd.4H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqrshrn_n_u64 (uint64x2_t a, const int n)  A32: VQRSHRN.U64 Dd, Qm, #n  A64: UQRSHRN Vd.2S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """
        uint16x4_t vqrshrn_n_u32 (uint32x4_t a, const int n)  A32: VQRSHRN.U32 Dd, Qm, #n  A64: UQRSHRN Vd.4H, Vn.4S, #n
        
        Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
        """
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """
        uint32x2_t vqrshrn_n_u64 (uint64x2_t a, const int n)  A32: VQRSHRN.U64 Dd, Qm, #n  A64: UQRSHRN Vd.2S, Vn.2D, #n
        
        Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
        """
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_lower(value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """
        uint8x8_t vqrshrn_n_u16 (uint16x8_t a, const int n)  A32: VQRSHRN.U16 Dd, Qm, #n  A64: UQRSHRN Vd.8B, Vn.8H, #n
        
        Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
        """
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqrshrn_high_n_u16 (uint8x8_t r, uint16x8_t a, const int n)  A32: VQRSHRN.U16 Dd+1, Dn, #n  A64: UQRSHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqrshrn_high_n_u32 (uint16x4_t r, uint32x4_t a, const int n)  A32: VQRSHRN.U32 Dd+1, Dn, #n  A64: UQRSHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqrshrn_high_n_u64 (uint32x2_t r, uint64x2_t a, const int n)  A32: VQRSHRN.U64 Dd+1, Dn, #n  A64: UQRSHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """
        uint16x8_t vqrshrn_high_n_u32 (uint16x4_t r, uint32x4_t a, const int n)  A32: VQRSHRN.U32 Dd+1, Dn, #n  A64: UQRSHRN2 Vd.8H, Vn.4S, #n
        
        Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
        """
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """
        uint32x4_t vqrshrn_high_n_u64 (uint32x2_t r, uint64x2_t a, const int n)  A32: VQRSHRN.U64 Dd+1, Dn, #n  A64: UQRSHRN2 Vd.4S, Vn.2D, #n
        
        Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
        """
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_saturate_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """
        uint8x16_t vqrshrn_high_n_u16 (uint8x8_t r, uint16x8_t a, const int n)  A32: VQRSHRN.U16 Dd+1, Dn, #n  A64: UQRSHRN2 Vd.16B, Vn.8H, #n
        
        Obsoletions.ArmIntrinsicPerformsUnsignedOperationMessage
        """
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrshrn_high_n_u16 (uint8x8_t r, uint16x8_t a, const int n)  A32: VRSHRN.I16 Dd+1, Qm, #n  A64: RSHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrshrn_high_n_s32 (int16x4_t r, int32x4_t a, const int n)  A32: VRSHRN.I32 Dd+1, Qm, #n  A64: RSHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vrshrn_high_n_s64 (int32x2_t r, int64x2_t a, const int n)  A32: VRSHRN.I64 Dd+1, Qm, #n  A64: RSHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrshrn_high_n_s16 (int8x8_t r, int16x8_t a, const int n)  A32: VRSHRN.I16 Dd+1, Qm, #n  A64: RSHRN2 Vd.16B, Vn.8H, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrshrn_high_n_u32 (uint16x4_t r, uint32x4_t a, const int n)  A32: VRSHRN.I32 Dd+1, Qm, #n  A64: RSHRN2 Vd.8H, Vn.4S, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], value: System.Runtime.Intrinsics.Vector128[int], count: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrshrn_high_n_u64 (uint32x2_t r, uint64x2_t a, const int n)  A32: VRSHRN.I64 Dd+1, Qm, #n  A64: RSHRN2 Vd.4S, Vn.2D, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vrshr_n_u64 (uint64x1_t a, const int n)  A32: VRSHR.U64 Dd, Dm, #n  A64: URSHR Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_rounded_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vrshr_n_u64 (uint64x1_t a, const int n)  A32: VRSHR.U64 Dd, Dm, #n  A64: URSHR Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vshr_n_u64 (uint64x1_t a, const int n)  A32: VSHR.U64 Dd, Dm, #n  A64: USHR Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def shift_right_logical_scalar(value: System.Runtime.Intrinsics.Vector64[int], count: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vshr_n_u64 (uint64x1_t a, const int n)  A32: VSHR.U64 Dd, Dm, #n  A64: USHR Dd, Dn, #n"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmovl_s16 (int16x4_t a)  A32: VMOVL.S16 Qd, Dm  A64: SXTL Vd.4S, Vn.4H"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmovl_s32 (int32x2_t a)  A32: VMOVL.S32 Qd, Dm  A64: SXTL Vd.2D, Vn.2S"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmovl_s8 (int8x8_t a)  A32: VMOVL.S8 Qd, Dm  A64: SXTL Vd.8H, Vn.8B"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vmovl_high_s16 (int16x8_t a)  A32: VMOVL.S16 Qd, Dm+1  A64: SXTL2 Vd.4S, Vn.8H"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vmovl_high_s32 (int32x4_t a)  A32: VMOVL.S32 Qd, Dm+1  A64: SXTL2 Vd.2D, Vn.4S"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vmovl_high_s8 (int8x16_t a)  A32: VMOVL.S8 Qd, Dm+1  A64: SXTL2 Vd.8H, Vn.16B"""
        ...

    @staticmethod
    @overload
    def sqrt_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vsqrt_f64 (float64x1_t a)  A32: VSQRT.F64 Dd, Dm  A64: FSQRT Dd, Dn"""
        ...

    @staticmethod
    @overload
    def sqrt_scalar(value: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vsqrts_f32 (float32_t a)  A32: VSQRT.F32 Sd, Sm  A64: FSQRT Sd, SnThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_u8 (uint8_t * ptr, uint8x8_t val)  A32: VST1.8 { Dd }, [Rn]  A64: ST1 { Vt.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[float]) -> None:
        """void vst1_f64 (float64_t * ptr, float64x1_t val)  A32: VST1.64 { Dd }, [Rn]  A64: ST1 { Vt.1D }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_s16 (int16_t * ptr, int16x4_t val)  A32: VST1.16 { Dd }, [Rn]  A64: ST1 {Vt.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_s32 (int32_t * ptr, int32x2_t val)  A32: VST1.32 { Dd }, [Rn]  A64: ST1 { Vt.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_s64 (int64_t * ptr, int64x1_t val)  A32: VST1.64 { Dd }, [Rn]  A64: ST1 { Vt.1D }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_s8 (int8_t * ptr, int8x8_t val)  A32: VST1.8 { Dd }, [Rn]  A64: ST1 { Vt.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[float]) -> None:
        """void vst1_f32 (float32_t * ptr, float32x2_t val)  A32: VST1.32 { Dd }, [Rn]  A64: ST1 { Vt.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_u16 (uint16_t * ptr, uint16x4_t val)  A32: VST1.16 { Dd }, [Rn]  A64: ST1 { Vt.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_u32 (uint32_t * ptr, uint32x2_t val)  A32: VST1.32 { Dd }, [Rn]  A64: ST1 { Vt.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector64[int]) -> None:
        """void vst1_u64 (uint64_t * ptr, uint64x1_t val)  A32: VST1.64 { Dd }, [Rn]  A64: ST1 { Vt.1D }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_u8 (uint8_t * ptr, uint8x16_t val)  A32: VST1.8 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.16B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[float]) -> None:
        """void vst1q_f64 (float64_t * ptr, float64x2_t val)  A32: VST1.64 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.2D }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_s16 (int16_t * ptr, int16x8_t val)  A32: VST1.16 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.8H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_s32 (int32_t * ptr, int32x4_t val)  A32: VST1.32 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.4S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_s64 (int64_t * ptr, int64x2_t val)  A32: VST1.64 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.2D }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_s8 (int8_t * ptr, int8x16_t val)  A32: VST1.8 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.16B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[float]) -> None:
        """void vst1q_f32 (float32_t * ptr, float32x4_t val)  A32: VST1.32 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.4S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_u16 (uint16_t * ptr, uint16x8_t val)  A32: VST1.16 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.8H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_u32 (uint32_t * ptr, uint32x4_t val)  A32: VST1.32 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.4S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, source: System.Runtime.Intrinsics.Vector128[int]) -> None:
        """void vst1q_u64 (uint64_t * ptr, uint64x2_t val)  A32: VST1.64 { Dd, Dd+1 }, [Rn]  A64: ST1 { Vt.2D }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]) -> None:
        """A64: ST1 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector64[int], index: int) -> None:
        """void vst1_lane_u8 (uint8_t * ptr, uint8x8_t val, const int lane)  A32: VST1.8 { Dd[index] }, [Rn]  A64: ST1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector64[int], index: int) -> None:
        """void vst1_lane_s16 (int16_t * ptr, int16x4_t val, const int lane)  A32: VST1.16 { Dd[index] }, [Rn]  A64: ST1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector64[int], index: int) -> None:
        """void vst1_lane_s32 (int32_t * ptr, int32x2_t val, const int lane)  A32: VST1.32 { Dd[index] }, [Rn]  A64: ST1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector64[int], index: int) -> None:
        """void vst1_lane_s8 (int8_t * ptr, int8x8_t val, const int lane)  A32: VST1.8 { Dd[index] }, [Rn]  A64: ST1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector64[float], index: int) -> None:
        """void vst1_lane_f32 (float32_t * ptr, float32x2_t val, const int lane)  A32: VST1.32 { Dd[index] }, [Rn]  A64: ST1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector64[int], index: int) -> None:
        """void vst1_lane_u16 (uint16_t * ptr, uint16x4_t val, const int lane)  A32: VST1.16 { Dd[index] }, [Rn]  A64: ST1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector64[int], index: int) -> None:
        """void vst1_lane_u32 (uint32_t * ptr, uint32x2_t val, const int lane)  A32: VST1.32 { Dd[index] }, [Rn]  A64: ST1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_u8 (uint8_t * ptr, uint8x16_t val, const int lane)  A32: VST1.8 { Dd[index] }, [Rn]  A64: ST1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[float], index: int) -> None:
        """void vst1q_lane_f64 (float64_t * ptr, float64x2_t val, const int lane)  A32: VSTR.64 Dd, [Rn]  A64: ST1 { Vt.D }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_s16 (int16_t * ptr, int16x8_t val, const int lane)  A32: VST1.16 { Dd[index] }, [Rn]  A64: ST1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_s32 (int32_t * ptr, int32x4_t val, const int lane)  A32: VST1.32 { Dd[index] }, [Rn]  A64: ST1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_s64 (int64_t * ptr, int64x2_t val, const int lane)  A32: VSTR.64 Dd, [Rn]  A64: ST1 { Vt.D }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_s8 (int8_t * ptr, int8x16_t val, const int lane)  A32: VST1.8 { Dd[index] }, [Rn]  A64: ST1 { Vt.B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[float], index: int) -> None:
        """void vst1q_lane_f32 (float32_t * ptr, float32x4_t val, const int lane)  A32: VST1.32 { Dd[index] }, [Rn]  A64: ST1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_u16 (uint16_t * ptr, uint16x8_t val, const int lane)  A32: VST1.16 { Dd[index] }, [Rn]  A64: ST1 { Vt.H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_u32 (uint32_t * ptr, uint32x4_t val, const int lane)  A32: VST1.32 { Dd[index] }, [Rn]  A64: ST1 { Vt.S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.Runtime.Intrinsics.Vector128[int], index: int) -> None:
        """void vst1q_lane_u64 (uint64_t * ptr, uint64x2_t val, const int lane)  A32: VSTR.64 Dd, [Rn]  A64: ST1 { Vt.D }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST2 { Vt.8B, Vt+1.8B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST2 { Vt.8B, Vt+1.8B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST2 { Vt.4H, Vt+1.4H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST2 { Vt.4H, Vt+1.4H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST2 { Vt.2S, Vt+1.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST2 { Vt.2S, Vt+1.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]], index: int) -> None:
        """A64: ST2 { Vt.2S, Vt+1.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST3 { Vt.8B, Vt+1.8B, Vt+2.8B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST3 { Vt.8B, Vt+1.8B, Vt+2.8B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST3 { Vt.4H, Vt+1.4H, Vt+2.4H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST3 { Vt.4H, Vt+1.4H, Vt+2.4H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST3 { Vt.2S, Vt+1.2S, Vt+2.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST3 { Vt.2S, Vt+1.2S, Vt+2.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]], index: int) -> None:
        """A64: ST2 { Vt.2S, Vt+1.2S, Vt+2.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST4 { Vt.8B, Vt+1.8B, Vt+2.8B, Vt+3.8B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST4 { Vt.8B, Vt+1.8B, Vt+2.8B, Vt+3.8B }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST4 { Vt.4H, Vt+1.4H, Vt+2.4H, Vt+3.4H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST4 { Vt.4H, Vt+1.4H, Vt+2.4H, Vt+3.4H }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST4 { Vt.2S, Vt+1.2S, Vt+2.2S, Vt+3.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]], index: int) -> None:
        """A64: ST4 { Vt.2S, Vt+1.2S, Vt+2.2S, Vt+3.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_selected_scalar(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]], index: int) -> None:
        """A64: ST4 { Vt.2S, Vt+1.2S, Vt+2.2S, Vt+3.2S }[index], [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST2 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST2 { Vn.8B, Vn+1.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST2 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST2 { Vn.4H, Vn+1.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST2 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST2 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]) -> None:
        """A64: ST2 { Vn.2S, Vn+1.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST3 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST3 { Vn.8B, Vn+1.8B, Vn+2.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST3 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST3 { Vn.4H, Vn+1.4H, Vn+2.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST3 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST3 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]) -> None:
        """A64: ST3 { Vn.2S, Vn+1.2S, Vn+2.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST4 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST4 { Vn.8B, Vn+1.8B, Vn+2.8B, Vn+3.8B }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST4 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST4 { Vn.4H, Vn+1.4H, Vn+2.4H, Vn+3.4H }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int], System.Runtime.Intrinsics.Vector64[int]]) -> None:
        """A64: ST4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def store_vector_and_zip(address: typing.Any, value: System.ValueTuple[System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float], System.Runtime.Intrinsics.Vector64[float]]) -> None:
        """A64: ST4 { Vn.2S, Vn+1.2S, Vn+2.2S, Vn+3.2S }, [Xn]"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vsub_u8 (uint8x8_t a, uint8x8_t b)  A32: VSUB.I8 Dd, Dn, Dm  A64: SUB Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vsub_s16 (int16x4_t a, int16x4_t b)  A32: VSUB.I16 Dd, Dn, Dm  A64: SUB Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vsub_s32 (int32x2_t a, int32x2_t b)  A32: VSUB.I32 Dd, Dn, Dm  A64: SUB Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vsub_s8 (int8x8_t a, int8x8_t b)  A32: VSUB.I8 Dd, Dn, Dm  A64: SUB Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t vsub_f32 (float32x2_t a, float32x2_t b)  A32: VSUB.F32 Dd, Dn, Dm  A64: FSUB Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vsub_u16 (uint16x4_t a, uint16x4_t b)  A32: VSUB.I16 Dd, Dn, Dm  A64: SUB Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vsub_u32 (uint32x2_t a, uint32x2_t b)  A32: VSUB.I32 Dd, Dn, Dm  A64: SUB Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vsubq_u8 (uint8x16_t a, uint8x16_t b)  A32: VSUB.I8 Qd, Qn, Qm  A64: SUB Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsubq_s16 (int16x8_t a, int16x8_t b)  A32: VSUB.I16 Qd, Qn, Qm  A64: SUB Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsubq_s32 (int32x4_t a, int32x4_t b)  A32: VSUB.I32 Qd, Qn, Qm  A64: SUB Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsubq_s64 (int64x2_t a, int64x2_t b)  A32: VSUB.I64 Qd, Qn, Qm  A64: SUB Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vsubq_s8 (int8x16_t a, int8x16_t b)  A32: VSUB.I8 Qd, Qn, Qm  A64: SUB Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t vsubq_f32 (float32x4_t a, float32x4_t b)  A32: VSUB.F32 Qd, Qn, Qm  A64: FSUB Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsubq_u16 (uint16x8_t a, uint16x8_t b)  A32: VSUB.I16 Qd, Qn, Qm  A64: SUB Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsubq_u32 (uint32x4_t a, uint32x4_t b)  A32: VSUB.I32 Qd, Qn, Qm  A64: SUB Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsubq_u64 (uint64x2_t a, uint64x2_t b)  A32: VSUB.I64 Qd, Qn, Qm  A64: SUB Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vsubhn_u16 (uint16x8_t a, uint16x8_t b)  A32: VSUBHN.I16 Dd, Qn, Qm  A64: SUBHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vsubhn_s32 (int32x4_t a, int32x4_t b)  A32: VSUBHN.I32 Dd, Qn, Qm  A64: SUBHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vsubhn_s64 (int64x2_t a, int64x2_t b)  A32: VSUBHN.I64 Dd, Qn, Qm  A64: SUBHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vsubhn_s16 (int16x8_t a, int16x8_t b)  A32: VSUBHN.I16 Dd, Qn, Qm  A64: SUBHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vsubhn_u32 (uint32x4_t a, uint32x4_t b)  A32: VSUBHN.I32 Dd, Qn, Qm  A64: SUBHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vsubhn_u64 (uint64x2_t a, uint64x2_t b)  A32: VSUBHN.I64 Dd, Qn, Qm  A64: SUBHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vsubhn_high_u16 (uint8x8_t r, uint16x8_t a, uint16x8_t b)  A32: VSUBHN.I16 Dd+1, Qn, Qm  A64: SUBHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsubhn_high_s32 (int16x4_t r, int32x4_t a, int32x4_t b)  A32: VSUBHN.I32 Dd+1, Qn, Qm  A64: SUBHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsubhn_high_s64 (int32x2_t r, int64x2_t a, int64x2_t b)  A32: VSUBHN.I64 Dd+1, Qn, Qm  A64: SUBHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vsubhn_high_s16 (int8x8_t r, int16x8_t a, int16x8_t b)  A32: VSUBHN.I16 Dd+1, Qn, Qm  A64: SUBHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsubhn_high_u32 (uint16x4_t r, uint32x4_t a, uint32x4_t b)  A32: VSUBHN.I32 Dd+1, Qn, Qm  A64: SUBHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsubhn_high_u64 (uint32x2_t r, uint64x2_t a, uint64x2_t b)  A32: VSUBHN.I64 Dd+1, Qn, Qm  A64: SUBHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vrsubhn_u16 (uint16x8_t a, uint16x8_t b)  A32: VRSUBHN.I16 Dd, Qn, Qm  A64: RSUBHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vrsubhn_s32 (int32x4_t a, int32x4_t b)  A32: VRSUBHN.I32 Dd, Qn, Qm  A64: RSUBHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vrsubhn_s64 (int64x2_t a, int64x2_t b)  A32: VRSUBHN.I64 Dd, Qn, Qm  A64: RSUBHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vrsubhn_s16 (int16x8_t a, int16x8_t b)  A32: VRSUBHN.I16 Dd, Qn, Qm  A64: RSUBHN Vd.8B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vrsubhn_u32 (uint32x4_t a, uint32x4_t b)  A32: VRSUBHN.I32 Dd, Qn, Qm  A64: RSUBHN Vd.4H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vrsubhn_u64 (uint64x2_t a, uint64x2_t b)  A32: VRSUBHN.I64 Dd, Qn, Qm  A64: RSUBHN Vd.2S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vrsubhn_high_u16 (uint8x8_t r, uint16x8_t a, uint16x8_t b)  A32: VRSUBHN.I16 Dd+1, Qn, Qm  A64: RSUBHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vrsubhn_high_s32 (int16x4_t r, int32x4_t a, int32x4_t b)  A32: VRSUBHN.I32 Dd+1, Qn, Qm  A64: RSUBHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vrsubhn_high_s64 (int32x2_t r, int64x2_t a, int64x2_t b)  A32: VRSUBHN.I64 Dd+1, Qn, Qm  A64: RSUBHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vrsubhn_high_s16 (int8x8_t r, int16x8_t a, int16x8_t b)  A32: VRSUBHN.I16 Dd+1, Qn, Qm  A64: RSUBHN2 Vd.16B, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vrsubhn_high_u32 (uint16x4_t r, uint32x4_t a, uint32x4_t b)  A32: VRSUBHN.I32 Dd+1, Qn, Qm  A64: RSUBHN2 Vd.8H, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_rounded_high_narrowing_upper(lower: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vrsubhn_high_u64 (uint32x2_t r, uint64x2_t a, uint64x2_t b)  A32: VRSUBHN.I64 Dd+1, Qn, Qm  A64: RSUBHN2 Vd.4S, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqsub_u8 (uint8x8_t a, uint8x8_t b)  A32: VQSUB.U8 Dd, Dn, Dm  A64: UQSUB Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqsub_s16 (int16x4_t a, int16x4_t b)  A32: VQSUB.S16 Dd, Dn, Dm  A64: SQSUB Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqsub_s32 (int32x2_t a, int32x2_t b)  A32: VQSUB.S32 Dd, Dn, Dm  A64: SQSUB Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqsub_s8 (int8x8_t a, int8x8_t b)  A32: VQSUB.S8 Dd, Dn, Dm  A64: SQSUB Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t vqsub_u16 (uint16x4_t a, uint16x4_t b)  A32: VQSUB.U16 Dd, Dn, Dm  A64: UQSUB Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vqsub_u32 (uint32x2_t a, uint32x2_t b)  A32: VQSUB.U32 Dd, Dn, Dm  A64: UQSUB Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t vqsubq_u8 (uint8x16_t a, uint8x16_t b)  A32: VQSUB.U8 Qd, Qn, Qm  A64: UQSUB Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqsubq_s16 (int16x8_t a, int16x8_t b)  A32: VQSUB.S16 Qd, Qn, Qm  A64: SQSUB Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqsubq_s32 (int32x4_t a, int32x4_t b)  A32: VQSUB.S32 Qd, Qn, Qm  A64: SQSUB Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vqsubq_s64 (int64x2_t a, int64x2_t b)  A32: VQSUB.S64 Qd, Qn, Qm  A64: SQSUB Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t vqsubq_s8 (int8x16_t a, int8x16_t b)  A32: VQSUB.S8 Qd, Qn, Qm  A64: SQSUB Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vqsubq_u16 (uint16x8_t a, uint16x8_t b)  A32: VQSUB.U16 Qd, Qn, Qm  A64: UQSUB Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vqsubq_u32 (uint32x4_t a, uint32x4_t b)  A32: VQSUB.U32 Qd, Qn, Qm  A64: UQSUB Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vqsubq_u64 (uint64x2_t a, uint64x2_t b)  A32: VQSUB.U64 Qd, Qn, Qm  A64: UQSUB Vd.2D, Vn.2D, Vm.2D"""
        ...

    @staticmethod
    @overload
    def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vqsub_s64 (int64x1_t a, int64x1_t b)  A32: VQSUB.S64 Dd, Dn, Dm  A64: SQSUB Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def subtract_saturate_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vqsub_u64 (uint64x1_t a, uint64x1_t b)  A32: VQSUB.U64 Dd, Dn, Dm  A64: UQSUB Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def subtract_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t vsub_f64 (float64x1_t a, float64x1_t b)  A32: VSUB.F64 Dd, Dn, Dm  A64: FSUB Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def subtract_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t vsub_s64 (int64x1_t a, int64x1_t b)  A32: VSUB.I64 Dd, Dn, Dm  A64: SUB Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def subtract_scalar(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32_t vsubs_f32 (float32_t a, float32_t b)  A32: VSUB.F32 Sd, Sn, Sm  A64: FSUB Sd, Sn, SmThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def subtract_scalar(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t vsub_u64 (uint64x1_t a, uint64x1_t b)  A32: VSUB.I64 Dd, Dn, Dm  A64: SUB Dd, Dn, Dm"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsubl_u8 (uint8x8_t a, uint8x8_t b)  A32: VSUBL.U8 Qd, Dn, Dm  A64: USUBL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsubl_s16 (int16x4_t a, int16x4_t b)  A32: VSUBL.S16 Qd, Dn, Dm  A64: SSUBL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsubl_s32 (int32x2_t a, int32x2_t b)  A32: VSUBL.S32 Qd, Dn, Dm  A64: SSUBL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsubl_s8 (int8x8_t a, int8x8_t b)  A32: VSUBL.S8 Qd, Dn, Dm  A64: SSUBL Vd.8H, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsubl_u16 (uint16x4_t a, uint16x4_t b)  A32: VSUBL.U16 Qd, Dn, Dm  A64: USUBL Vd.4S, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsubl_u32 (uint32x2_t a, uint32x2_t b)  A32: VSUBL.U32 Qd, Dn, Dm  A64: USUBL Vd.2D, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsubw_s8 (int16x8_t a, int8x8_t b)  A32: VSUBW.S8 Qd, Qn, Dm  A64: SSUBW Vd.8H, Vn.8H, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsubw_s16 (int32x4_t a, int16x4_t b)  A32: VSUBW.S16 Qd, Qn, Dm  A64: SSUBW Vd.4S, Vn.4S, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsubw_s32 (int64x2_t a, int32x2_t b)  A32: VSUBW.S32 Qd, Qn, Dm  A64: SSUBW Vd.2D, Vn.2D, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsubw_u8 (uint16x8_t a, uint8x8_t b)  A32: VSUBW.U8 Qd, Qn, Dm  A64: USUBW Vd.8H, Vn.8H, Vm.8B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsubw_u16 (uint32x4_t a, uint16x4_t b)  A32: VSUBW.U16 Qd, Qn, Dm  A64: USUBW Vd.4S, Vn.4S, Vm.4H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_lower(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsubw_u32 (uint64x2_t a, uint32x2_t b)  A32: VSUBW.U32 Qd, Qn, Dm  A64: USUBW Vd.2D, Vn.2D, Vm.2S"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsubl_high_u8 (uint8x16_t a, uint8x16_t b)  A32: VSUBL.U8 Qd, Dn+1, Dm+1  A64: USUBL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsubl_high_s16 (int16x8_t a, int16x8_t b)  A32: VSUBL.S16 Qd, Dn+1, Dm+1  A64: SSUBL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsubw_high_s8 (int16x8_t a, int8x16_t b)  A32: VSUBW.S8 Qd, Qn, Dm+1  A64: SSUBW2 Vd.8H, Vn.8H, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vsubw_high_s16 (int32x4_t a, int16x8_t b)  A32: VSUBW.S16 Qd, Qn, Dm+1  A64: SSUBW2 Vd.4S, Vn.4S, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsubl_high_s32 (int32x4_t a, int32x4_t b)  A32: VSUBL.S32 Qd, Dn+1, Dm+1  A64: SSUBL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t vsubw_high_s32 (int64x2_t a, int32x4_t b)  A32: VSUBW.S32 Qd, Qn, Dm+1  A64: SSUBW2 Vd.2D, Vn.2D, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vsubl_high_s8 (int8x16_t a, int8x16_t b)  A32: VSUBL.S8 Qd, Dn+1, Dm+1  A64: SSUBL2 Vd.8H, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vsubw_high_u8 (uint16x8_t a, uint8x16_t b)  A32: VSUBW.U8 Qd, Qn, Dm+1  A64: USUBW2 Vd.8H, Vn.8H, Vm.16B"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsubl_high_u16 (uint16x8_t a, uint16x8_t b)  A32: VSUBL.U16 Qd, Dn+1, Dm+1  A64: USUBL2 Vd.4S, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsubw_high_u16 (uint32x4_t a, uint16x8_t b)  A32: VSUBW.U16 Qd, Qn, Dm+1  A64: USUBW2 Vd.4S, Vn.4S, Vm.8H"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsubl_high_u32 (uint32x4_t a, uint32x4_t b)  A32: VSUBL.U32 Qd, Dn+1, Dm+1  A64: USUBL2 Vd.2D, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def subtract_widening_upper(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vsubw_high_u32 (uint64x2_t a, uint32x4_t b)  A32: VSUBW.U32 Qd, Qn, Dm+1  A64: USUBW2 Vd.2D, Vn.2D, Vm.4S"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqvtbl1_u8(uint8x16_t t, uint8x8_t idx)  A32: VTBL Dd, {Dn, Dn+1}, Dm  A64: TBL Vd.8B, {Vn.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqvtbl1_s8(int8x16_t t, uint8x8_t idx)  A32: VTBL Dd, {Dn, Dn+1}, Dm  A64: TBL Vd.8B, {Vn.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqtbl2q_u8(uint8x16x2_t t, uint8x8_t idx)  A64: TBL Vd.8B, {Vn.16B, Vn+1.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqtbl2q_u8(int8x16x2_t t, uint8x8_t idx)  A64: TBL Vd.8B, {Vn.16B, Vn+1.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqtbl3q_u8(uint8x16x3_t t, uint8x8_t idx)  A64: TBL Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqtbl3q_u8(int8x16x3_t t, uint8x8_t idx)  A64: TBL Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqtbl4q_u8(uint8x16x4_t t, uint8x8_t idx)  A64: TBL Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqtbl4q_u8(int8x16x4_t t, uint8x8_t idx)  A64: TBL Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqvtbx1_u8(uint8x8_t r, uint8x16_t t, uint8x8_t idx)  A32: VTBX Dd, {Dn, Dn+1}, Dm  A64: TBX Vd.8B, {Vn.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.Runtime.Intrinsics.Vector128[int], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqvtbx1_s8(int8x8_t r, int8x16_t t, uint8x8_t idx)  A32: VTBX Dd, {Dn, Dn+1}, Dm  A64: TBX Vd.8B, {Vn.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqtbx2q_u8(uint8x16x2_t t, uint8x8_t idx)  A64: TBX Vd.8B, {Vn.16B, Vn+1.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqtbx2q_u8(int8x16x2_t t, uint8x8_t idx)  A64: TBX Vd.8B, {Vn.16B, Vn+1.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqtbx3q_u8(uint8x16x3_t t, uint8x8_t idx)  A64: TBX Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqtbx3q_u8(int8x16x3_t t, uint8x8_t idx)  A64: TBX Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t vqtbx4q_u8(uint8x16x4_t t, uint8x8_t idx)  A64: TBX Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup_extension(default_values: System.Runtime.Intrinsics.Vector64[int], table: System.ValueTuple[System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int], System.Runtime.Intrinsics.Vector128[int]], byte_indexes: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t vqtbx4q_u8(int8x16x4_t t, uint8x8_t idx)  A64: TBX Vd.8B, {Vn.16B, Vn+1.16B, Vn+2.16B, Vn+3.16B}, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint8x8_t veor_u8 (uint8x8_t a, uint8x8_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float64x1_t veor_f64 (float64x1_t a, float64x1_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t veor_s16 (int16x4_t a, int16x4_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t veor_s32 (int32x2_t a, int32x2_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int64x1_t veor_s64 (int64x1_t a, int64x1_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int8x8_t veor_s8 (int8x8_t a, int8x8_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[float], right: System.Runtime.Intrinsics.Vector64[float]) -> System.Runtime.Intrinsics.Vector64[float]:
        """float32x2_t veor_f32 (float32x2_t a, float32x2_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint16x4_t veor_u16 (uint16x4_t a, uint16x4_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t veor_u32 (uint32x2_t a, uint32x2_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint64x1_t veor_u64 (uint64x1_t a, uint64x1_t b)  A32: VEOR Dd, Dn, Dm  A64: EOR Vd.8B, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint8x16_t veorq_u8 (uint8x16_t a, uint8x16_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float64x2_t veorq_f64 (float64x2_t a, float64x2_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t veorq_s16 (int16x8_t a, int16x8_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t veorq_s32 (int32x4_t a, int32x4_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int64x2_t veorq_s64 (int64x2_t a, int64x2_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int8x16_t veorq_s8 (int8x16_t a, int8x16_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[float], right: System.Runtime.Intrinsics.Vector128[float]) -> System.Runtime.Intrinsics.Vector128[float]:
        """float32x4_t veorq_f32 (float32x4_t a, float32x4_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16BThe above native signature does not exist. We provide this additional overload for consistency with the other scalar APIs."""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t veorq_u16 (uint16x8_t a, uint16x8_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t veorq_u32 (uint32x4_t a, uint32x4_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t veorq_u64 (uint64x2_t a, uint64x2_t b)  A32: VEOR Qd, Qn, Qm  A64: EOR Vd.16B, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmovl_u8 (uint8x8_t a)  A32: VMOVL.U8 Qd, Dm  A64: UXTL Vd.8H, Vn.8B"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmovl_u16 (uint16x4_t a)  A32: VMOVL.U16 Qd, Dm  A64: UXTL Vd.4S, Vn.4H"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmovl_u32 (uint32x2_t a)  A32: VMOVL.U32 Qd, Dm  A64: UXTL Vd.2D, Vn.2S"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmovl_u8 (uint8x8_t a)  A32: VMOVL.U8 Qd, Dm  A64: UXTL Vd.8H, Vn.8B"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmovl_u16 (uint16x4_t a)  A32: VMOVL.U16 Qd, Dm  A64: UXTL Vd.4S, Vn.4H"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmovl_u32 (uint32x2_t a)  A32: VMOVL.U32 Qd, Dm  A64: UXTL Vd.2D, Vn.2S"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmovl_high_u8 (uint8x16_t a)  A32: VMOVL.U8 Qd, Dm+1  A64: UXTL2 Vd.8H, Vn.16B"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmovl_high_u16 (uint16x8_t a)  A32: VMOVL.U16 Qd, Dm+1  A64: UXTL2 Vd.4S, Vn.8H"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmovl_high_u32 (uint32x4_t a)  A32: VMOVL.U32 Qd, Dm+1  A64: UXTL2 Vd.2D, Vn.4S"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint16x8_t vmovl_high_u8 (uint8x16_t a)  A32: VMOVL.U8 Qd, Dm+1  A64: UXTL2 Vd.8H, Vn.16B"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vmovl_high_u16 (uint16x8_t a)  A32: VMOVL.U16 Qd, Dm+1  A64: UXTL2 Vd.4S, Vn.8H"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint64x2_t vmovl_high_u32 (uint32x4_t a)  A32: VMOVL.U32 Qd, Dm+1  A64: UXTL2 Vd.2D, Vn.4S"""
        ...


class Rdm(System.Runtime.Intrinsics.Arm.AdvSimd, metaclass=abc.ABCMeta):
    """Provides access to the ARMv8.1-RDMA hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.AdvSimd.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARMv8.1-RDMA hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_add_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlahh_s16 (int16_t a, int16_t b, int16_t c)  A64: SQRDMLAH Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_add_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlahs_s32 (int32_t a, int32_t b, int32_t c)  A64: SQRDMLAH Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_add_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlahh_s16 (int16_t a, int16_t b, int16_t c)  A64: SQRDMLAH Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_add_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlahs_s32 (int32_t a, int32_t b, int32_t c)  A64: SQRDMLAH Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_subtract_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlshh_s16 (int16_t a, int16_t b, int16_t c)  A64: SQRDMLSH Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_subtract_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlshs_s32 (int32_t a, int32_t b, int32_t c)  A64: SQRDMLSH Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_subtract_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlshh_s16 (int16_t a, int16_t b, int16_t c)  A64: SQRDMLSH Hd, Hn, Hm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_and_subtract_saturate_high_scalar(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlshs_s32 (int32_t a, int32_t b, int32_t c)  A64: SQRDMLSH Sd, Sn, Sm"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlahh_lane_s16 (int16_t a, int16_t b, int16x4_t v, const int lane)  A64: SQRDMLAH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlahh_laneq_s16 (int16_t a, int16_t b, int16x8_t v, const int lane)  A64: SQRDMLAH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlahs_lane_s32 (int32_t a, int32_t b, int32x2_t v, const int lane)  A64: SQRDMLAH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlahs_laneq_s32 (int32_t a, int32_t b, int32x4_t v, const int lane)  A64: SQRDMLAH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlahh_lane_s16 (int16_t a, int16_t b, int16x4_t v, const int lane)  A64: SQRDMLAH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlahh_laneq_s16 (int16_t a, int16_t b, int16x8_t v, const int lane)  A64: SQRDMLAH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlahs_lane_s32 (int32_t a, int32_t b, int32x2_t v, const int lane)  A64: SQRDMLAH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlahs_laneq_s32 (int32_t a, int32_t b, int32x4_t v, const int lane)  A64: SQRDMLAH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlshh_lane_s16 (int16_t a, int16_t b, int16x4_t v, const int lane)  A64: SQRDMLSH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlshh_laneq_s16 (int16_t a, int16_t b, int16x8_t v, const int lane)  A64: SQRDMLSH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlshs_lane_s32 (int32_t a, int32_t b, int32x2_t v, const int lane)  A64: SQRDMLSH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlshs_laneq_s32 (int32_t a, int32_t b, int32x4_t v, const int lane)  A64: SQRDMLSH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlshh_lane_s16 (int16_t a, int16_t b, int16x4_t v, const int lane)  A64: SQRDMLSH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int16_t vqrdmlshh_laneq_s16 (int16_t a, int16_t b, int16x8_t v, const int lane)  A64: SQRDMLSH Hd, Hn, Vm.H[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlshs_lane_s32 (int32_t a, int32_t b, int32x2_t v, const int lane)  A64: SQRDMLSH Sd, Sn, Vm.S[lane]"""
            ...

        @staticmethod
        @overload
        def multiply_rounded_doubling_scalar_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
            """int32_t vqrdmlshs_laneq_s32 (int32_t a, int32_t b, int32x4_t v, const int lane)  A64: SQRDMLSH Sd, Sn, Vm.S[lane]"""
            ...

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlah_s16 (int16x4_t a, int16x4_t b, int16x4_t c)  A32: VQRDMLAH.S16 Dd, Dn, Dm  A64: SQRDMLAH Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlah_s32 (int32x2_t a, int32x2_t b, int32x2_t c)  A32: VQRDMLAH.S32 Dd, Dn, Dm  A64: SQRDMLAH Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlahq_s16 (int16x8_t a, int16x8_t b, int16x8_t c)  A32: VQRDMLAH.S16 Qd, Qn, Qm  A64: SQRDMLAH Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlahq_s32 (int32x4_t a, int32x4_t b, int32x4_t c)  A32: VQRDMLAH.S32 Qd, Qn, Qm  A64: SQRDMLAH Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlah_s16 (int16x4_t a, int16x4_t b, int16x4_t c)  A32: VQRDMLAH.S16 Dd, Dn, Dm  A64: SQRDMLAH Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlah_s32 (int32x2_t a, int32x2_t b, int32x2_t c)  A32: VQRDMLAH.S32 Dd, Dn, Dm  A64: SQRDMLAH Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlahq_s16 (int16x8_t a, int16x8_t b, int16x8_t c)  A32: VQRDMLAH.S16 Qd, Qn, Qm  A64: SQRDMLAH Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlahq_s32 (int32x4_t a, int32x4_t b, int32x4_t c)  A32: VQRDMLAH.S32 Qd, Qn, Qm  A64: SQRDMLAH Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlsh_s16 (int16x4_t a, int16x4_t b, int16x4_t c)  A32: VQRDMLSH.S16 Dd, Dn, Dm  A64: SQRDMLSH Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlsh_s32 (int32x2_t a, int32x2_t b, int32x2_t c)  A32: VQRDMLSH.S32 Dd, Dn, Dm  A64: SQRDMLSH Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlshq_s16 (int16x8_t a, int16x8_t b, int16x8_t c)  A32: VQRDMLSH.S16 Qd, Qn, Qm  A64: SQRDMLSH Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlshq_s32 (int32x4_t a, int32x4_t b, int32x4_t c)  A32: VQRDMLSH.S32 Qd, Qn, Qm  A64: SQRDMLSH Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlsh_s16 (int16x4_t a, int16x4_t b, int16x4_t c)  A32: VQRDMLSH.S16 Dd, Dn, Dm  A64: SQRDMLSH Vd.4H, Vn.4H, Vm.4H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlsh_s32 (int32x2_t a, int32x2_t b, int32x2_t c)  A32: VQRDMLSH.S32 Dd, Dn, Dm  A64: SQRDMLSH Vd.2S, Vn.2S, Vm.2S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlshq_s16 (int16x8_t a, int16x8_t b, int16x8_t c)  A32: VQRDMLSH.S16 Qd, Qn, Qm  A64: SQRDMLSH Vd.8H, Vn.8H, Vm.8H"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlshq_s32 (int32x4_t a, int32x4_t b, int32x4_t c)  A32: VQRDMLSH.S32 Qd, Qn, Qm  A64: SQRDMLSH Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlah_lane_s16 (int16x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VQRDMLAH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlah_laneq_s16 (int16x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VQRDMLAH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlah_lane_s32 (int32x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VQRDMLAH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlah_laneq_s32 (int32x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VQRDMLAH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlahq_lane_s16 (int16x8_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VQRDMLAH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlahq_laneq_s16 (int16x8_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VQRDMLAH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlahq_lane_s32 (int32x4_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VQRDMLAH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlahq_laneq_s32 (int32x4_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VQRDMLAH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlah_lane_s16 (int16x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VQRDMLAH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlah_laneq_s16 (int16x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VQRDMLAH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlah_lane_s32 (int32x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VQRDMLAH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlah_laneq_s32 (int32x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VQRDMLAH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLAH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlahq_lane_s16 (int16x8_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VQRDMLAH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlahq_laneq_s16 (int16x8_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VQRDMLAH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlahq_lane_s32 (int32x4_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VQRDMLAH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_add_saturate_high(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlahq_laneq_s32 (int32x4_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VQRDMLAH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLAH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlsh_lane_s16 (int16x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VQRDMLSH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlsh_laneq_s16 (int16x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VQRDMLSH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlsh_lane_s32 (int32x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VQRDMLSH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlsh_laneq_s32 (int32x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VQRDMLSH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlshq_lane_s16 (int16x8_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VQRDMLSH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlshq_laneq_s16 (int16x8_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VQRDMLSH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlshq_lane_s32 (int32x4_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VQRDMLSH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlshq_laneq_s32 (int32x4_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VQRDMLSH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlsh_lane_s16 (int16x4_t a, int16x4_t b, int16x4_t v, const int lane)  A32: VQRDMLSH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int16x4_t vqrdmlsh_laneq_s16 (int16x4_t a, int16x4_t b, int16x8_t v, const int lane)  A32: VQRDMLSH.S16 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.4H, Vn.4H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlsh_lane_s32 (int32x2_t a, int32x2_t b, int32x2_t v, const int lane)  A32: VQRDMLSH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vqrdmlsh_laneq_s32 (int32x2_t a, int32x2_t b, int32x4_t v, const int lane)  A32: VQRDMLSH.S32 Dd, Dn, Dm[lane]  A64: SQRDMLSH Vd.2S, Vn.2S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlshq_lane_s16 (int16x8_t a, int16x8_t b, int16x4_t v, const int lane)  A32: VQRDMLSH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int16x8_t vqrdmlshq_laneq_s16 (int16x8_t a, int16x8_t b, int16x8_t v, const int lane)  A32: VQRDMLSH.S16 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.8H, Vn.8H, Vm.H[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlshq_lane_s32 (int32x4_t a, int32x4_t b, int32x2_t v, const int lane)  A32: VQRDMLSH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...

    @staticmethod
    @overload
    def multiply_rounded_doubling_by_selected_scalar_and_subtract_saturate_high(minuend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vqrdmlshq_laneq_s32 (int32x4_t a, int32x4_t b, int32x4_t v, const int lane)  A32: VQRDMLSH.S32 Qd, Qn, Dm[lane]  A64: SQRDMLSH Vd.4S, Vn.4S, Vm.S[lane]"""
        ...


class Dp(System.Runtime.Intrinsics.Arm.AdvSimd, metaclass=abc.ABCMeta):
    """Provides access to the ARMv8.2-DotProd hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.AdvSimd.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARMv8.2-DotProd hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdot_s32 (int32x2_t r, int8x8_t a, int8x8_t b)  A32: VSDOT.S8 Dd, Dn, Dm  A64: SDOT Vd.2S, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdot_u32 (uint32x2_t r, uint8x8_t a, uint8x8_t b)  A32: VUDOT.U8 Dd, Dn, Dm  A64: UDOT Vd.2S, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdotq_s32 (int32x4_t r, int8x16_t a, int8x16_t b)  A32: VSDOT.S8 Qd, Qn, Qm  A64: SDOT Vd.4S, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdotq_u32 (uint32x4_t r, uint8x16_t a, uint8x16_t b)  A32: VUDOT.U8 Qd, Qn, Qm  A64: UDOT Vd.4S, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdot_s32 (int32x2_t r, int8x8_t a, int8x8_t b)  A32: VSDOT.S8 Dd, Dn, Dm  A64: SDOT Vd.2S, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdot_u32 (uint32x2_t r, uint8x8_t a, uint8x8_t b)  A32: VUDOT.U8 Dd, Dn, Dm  A64: UDOT Vd.2S, Vn.8B, Vm.8B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdotq_s32 (int32x4_t r, int8x16_t a, int8x16_t b)  A32: VSDOT.S8 Qd, Qn, Qm  A64: SDOT Vd.4S, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdotq_u32 (uint32x4_t r, uint8x16_t a, uint8x16_t b)  A32: VUDOT.U8 Qd, Qn, Qm  A64: UDOT Vd.4S, Vn.16B, Vm.16B"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdot_lane_s32 (int32x2_t r, int8x8_t a, int8x8_t b, const int lane)  A32: VSDOT.S8 Dd, Dn, Dm[lane]  A64: SDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdot_laneq_s32 (int32x2_t r, int8x8_t a, int8x16_t b, const int lane)  A32: VSDOT.S8 Dd, Dn, Dm[lane]  A64: SDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdot_lane_u32 (uint32x2_t r, uint8x8_t a, uint8x8_t b, const int lane)  A32: VUDOT.U8 Dd, Dn, Dm[lane]  A64: UDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdot_laneq_u32 (uint32x2_t r, uint8x8_t a, uint8x16_t b, const int lane)  A32: VUDOT.U8 Dd, Dn, Dm[lane]  A64: UDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdotq_laneq_s32 (int32x4_t r, int8x16_t a, int8x16_t b, const int lane)  A32: VSDOT.S8 Qd, Qn, Dm[lane]  A64: SDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdotq_lane_s32 (int32x4_t r, int8x16_t a, int8x8_t b, const int lane)  A32: VSDOT.S8 Qd, Qn, Dm[lane]  A64: SDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdotq_laneq_u32 (uint32x4_t r, uint8x16_t a, uint8x16_t b, const int lane)  A32: VUDOT.U8 Qd, Qn, Dm[lane]  A64: UDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdotq_lane_u32 (uint32x4_t r, uint8x16_t a, uint8x8_t b, const int lane)  A32: VUDOT.U8 Qd, Qn, Dm[lane]  A64: UDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdot_lane_s32 (int32x2_t r, int8x8_t a, int8x8_t b, const int lane)  A32: VSDOT.S8 Dd, Dn, Dm[lane]  A64: SDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """int32x2_t vdot_laneq_s32 (int32x2_t r, int8x8_t a, int8x16_t b, const int lane)  A32: VSDOT.S8 Dd, Dn, Dm[lane]  A64: SDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdot_lane_u32 (uint32x2_t r, uint8x8_t a, uint8x8_t b, const int lane)  A32: VUDOT.U8 Dd, Dn, Dm[lane]  A64: UDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector64[int], left: System.Runtime.Intrinsics.Vector64[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32x2_t vdot_laneq_u32 (uint32x2_t r, uint8x8_t a, uint8x16_t b, const int lane)  A32: VUDOT.U8 Dd, Dn, Dm[lane]  A64: UDOT Vd.2S, Vn.8B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdotq_laneq_s32 (int32x4_t r, int8x16_t a, int8x16_t b, const int lane)  A32: VSDOT.S8 Qd, Qn, Dm[lane]  A64: SDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """int32x4_t vdotq_lane_s32 (int32x4_t r, int8x16_t a, int8x8_t b, const int lane)  A32: VSDOT.S8 Qd, Qn, Dm[lane]  A64: SDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector128[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdotq_laneq_u32 (uint32x4_t r, uint8x16_t a, uint8x16_t b, const int lane)  A32: VUDOT.U8 Qd, Qn, Dm[lane]  A64: UDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_quadruplet(addend: System.Runtime.Intrinsics.Vector128[int], left: System.Runtime.Intrinsics.Vector128[int], right: System.Runtime.Intrinsics.Vector64[int], right_scaled_index: int) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vdotq_lane_u32 (uint32x4_t r, uint8x16_t a, uint8x8_t b, const int lane)  A32: VUDOT.U8 Qd, Qn, Dm[lane]  A64: UDOT Vd.4S, Vn.16B, Vm.4B[lane]"""
        ...


class Sha256(System.Runtime.Intrinsics.Arm.ArmBase, metaclass=abc.ABCMeta):
    """Provides access to the ARM SHA256 hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.ArmBase.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARM SHA256 hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def hash_update_1(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_efgh: System.Runtime.Intrinsics.Vector128[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256hq_u32 (uint32x4_t hash_abcd, uint32x4_t hash_efgh, uint32x4_t wk)  A32: SHA256H.32 Qd, Qn, Qm  A64: SHA256H Qd, Qn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_1(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_efgh: System.Runtime.Intrinsics.Vector128[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256hq_u32 (uint32x4_t hash_abcd, uint32x4_t hash_efgh, uint32x4_t wk)  A32: SHA256H.32 Qd, Qn, Qm  A64: SHA256H Qd, Qn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_2(hash_efgh: System.Runtime.Intrinsics.Vector128[int], hash_abcd: System.Runtime.Intrinsics.Vector128[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256h2q_u32 (uint32x4_t hash_efgh, uint32x4_t hash_abcd, uint32x4_t wk)  A32: SHA256H2.32 Qd, Qn, Qm  A64: SHA256H2 Qd, Qn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_2(hash_efgh: System.Runtime.Intrinsics.Vector128[int], hash_abcd: System.Runtime.Intrinsics.Vector128[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256h2q_u32 (uint32x4_t hash_efgh, uint32x4_t hash_abcd, uint32x4_t wk)  A32: SHA256H2.32 Qd, Qn, Qm  A64: SHA256H2 Qd, Qn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_0(w_0_3: System.Runtime.Intrinsics.Vector128[int], w_4_7: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256su0q_u32 (uint32x4_t w_0_3, uint32x4_t w_4_7)  A32: SHA256SU0.32 Qd, Qm  A64: SHA256SU0 Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_0(w_0_3: System.Runtime.Intrinsics.Vector128[int], w_4_7: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256su0q_u32 (uint32x4_t w_0_3, uint32x4_t w_4_7)  A32: SHA256SU0.32 Qd, Qm  A64: SHA256SU0 Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_1(w_0_3: System.Runtime.Intrinsics.Vector128[int], w_8_11: System.Runtime.Intrinsics.Vector128[int], w_12_15: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256su1q_u32 (uint32x4_t w_0_3, uint32x4_t w_8_11, uint32x4_t w_12_15)  A32: SHA256SU1.32 Qd, Qn, Qm  A64: SHA256SU1 Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_1(w_0_3: System.Runtime.Intrinsics.Vector128[int], w_8_11: System.Runtime.Intrinsics.Vector128[int], w_12_15: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha256su1q_u32 (uint32x4_t w_0_3, uint32x4_t w_8_11, uint32x4_t w_12_15)  A32: SHA256SU1.32 Qd, Qn, Qm  A64: SHA256SU1 Vd.4S, Vn.4S, Vm.4S"""
        ...


class Sha1(System.Runtime.Intrinsics.Arm.ArmBase, metaclass=abc.ABCMeta):
    """Provides access to the ARM SHA1 hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.ArmBase.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARM SHA1 hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def fixed_rotate(hash_e: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32_t vsha1h_u32 (uint32_t hash_e)  A32: SHA1H.32 Qd, Qm  A64: SHA1H Sd, Sn"""
        ...

    @staticmethod
    @overload
    def fixed_rotate(hash_e: System.Runtime.Intrinsics.Vector64[int]) -> System.Runtime.Intrinsics.Vector64[int]:
        """uint32_t vsha1h_u32 (uint32_t hash_e)  A32: SHA1H.32 Qd, Qm  A64: SHA1H Sd, Sn"""
        ...

    @staticmethod
    @overload
    def hash_update_choose(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_e: System.Runtime.Intrinsics.Vector64[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1cq_u32 (uint32x4_t hash_abcd, uint32_t hash_e, uint32x4_t wk)  A32: SHA1C.32 Qd, Qn, Qm  A64: SHA1C Qd, Sn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_choose(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_e: System.Runtime.Intrinsics.Vector64[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1cq_u32 (uint32x4_t hash_abcd, uint32_t hash_e, uint32x4_t wk)  A32: SHA1C.32 Qd, Qn, Qm  A64: SHA1C Qd, Sn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_majority(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_e: System.Runtime.Intrinsics.Vector64[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1mq_u32 (uint32x4_t hash_abcd, uint32_t hash_e, uint32x4_t wk)  A32: SHA1M.32 Qd, Qn, Qm  A64: SHA1M Qd, Sn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_majority(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_e: System.Runtime.Intrinsics.Vector64[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1mq_u32 (uint32x4_t hash_abcd, uint32_t hash_e, uint32x4_t wk)  A32: SHA1M.32 Qd, Qn, Qm  A64: SHA1M Qd, Sn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_parity(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_e: System.Runtime.Intrinsics.Vector64[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1pq_u32 (uint32x4_t hash_abcd, uint32_t hash_e, uint32x4_t wk)  A32: SHA1P.32 Qd, Qn, Qm  A64: SHA1P Qd, Sn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def hash_update_parity(hash_abcd: System.Runtime.Intrinsics.Vector128[int], hash_e: System.Runtime.Intrinsics.Vector64[int], wk: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1pq_u32 (uint32x4_t hash_abcd, uint32_t hash_e, uint32x4_t wk)  A32: SHA1P.32 Qd, Qn, Qm  A64: SHA1P Qd, Sn, Vm.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_0(w_0_3: System.Runtime.Intrinsics.Vector128[int], w_4_7: System.Runtime.Intrinsics.Vector128[int], w_8_11: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1su0q_u32 (uint32x4_t w_0_3, uint32x4_t w_4_7, uint32x4_t w_8_11)  A32: SHA1SU0.32 Qd, Qn, Qm  A64: SHA1SU0 Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_0(w_0_3: System.Runtime.Intrinsics.Vector128[int], w_4_7: System.Runtime.Intrinsics.Vector128[int], w_8_11: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1su0q_u32 (uint32x4_t w_0_3, uint32x4_t w_4_7, uint32x4_t w_8_11)  A32: SHA1SU0.32 Qd, Qn, Qm  A64: SHA1SU0 Vd.4S, Vn.4S, Vm.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_1(tw_0_3: System.Runtime.Intrinsics.Vector128[int], w_12_15: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1su1q_u32 (uint32x4_t tw_0_3, uint32x4_t w_12_15)  A32: SHA1SU1.32 Qd, Qm  A64: SHA1SU1 Vd.4S, Vn.4S"""
        ...

    @staticmethod
    @overload
    def schedule_update_1(tw_0_3: System.Runtime.Intrinsics.Vector128[int], w_12_15: System.Runtime.Intrinsics.Vector128[int]) -> System.Runtime.Intrinsics.Vector128[int]:
        """uint32x4_t vsha1su1q_u32 (uint32x4_t tw_0_3, uint32x4_t w_12_15)  A32: SHA1SU1.32 Qd, Qm  A64: SHA1SU1 Vd.4S, Vn.4S"""
        ...


class Sve(System.Runtime.Intrinsics.Arm.AdvSimd, metaclass=abc.ABCMeta):
    """Provides access to the ARM SVE hardware instructions via intrinsics."""

    class Arm64(System.Runtime.Intrinsics.Arm.AdvSimd.Arm64, metaclass=abc.ABCMeta):
        """Provides access to the ARM SVE hardware instructions, that are only available to 64-bit processes, via intrinsics."""

        IS_SUPPORTED: bool
        """Gets a value that indicates whether the APIs in this class are supported."""

    IS_SUPPORTED: bool
    """Gets a value that indicates whether the APIs in this class are supported."""

    @staticmethod
    @overload
    def abs(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svabs[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svabs[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svabs[_f64]_z(svbool_t pg, svfloat64_t op)  FABS Ztied.D, Pg/M, Zop.D  FABS Ztied.D, Pg/M, Ztied.D"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svabs[_s16]_m(svint16_t inactive, svbool_t pg, svint16_t op)svint16_t svabs[_s16]_x(svbool_t pg, svint16_t op)svint16_t svabs[_s16]_z(svbool_t pg, svint16_t op)  ABS Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svabs[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svabs[_s32]_x(svbool_t pg, svint32_t op)svint32_t svabs[_s32]_z(svbool_t pg, svint32_t op)  ABS Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svabs[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svabs[_s64]_x(svbool_t pg, svint64_t op)svint64_t svabs[_s64]_z(svbool_t pg, svint64_t op)  ABS Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svabs[_s8]_m(svint8_t inactive, svbool_t pg, svint8_t op)svint8_t svabs[_s8]_x(svbool_t pg, svint8_t op)svint8_t svabs[_s8]_z(svbool_t pg, svint8_t op)  ABS Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def abs(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svabs[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svabs[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svabs[_f32]_z(svbool_t pg, svfloat32_t op)  FABS Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svacgt[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FACGT Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svacgt[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FACGT Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svacge[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FACGE Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def absolute_compare_greater_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svacge[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FACGE Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svaclt[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FACLT Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svaclt[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FACLT Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svacle[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FACLE Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def absolute_compare_less_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svacle[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FACLE Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svabd[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svabd[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)  UABD Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svabd[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svabd[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svabd[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FABD Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svabd[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svabd[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svabd[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  SABD Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svabd[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svabd[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svabd[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  SABD Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svabd[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svabd[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svabd[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  SABD Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svabd[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svabd[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svabd[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  SABD Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svabd[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svabd[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svabd[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FABD Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svabd[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svabd[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svabd[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  UABD Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svabd[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svabd[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svabd[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  UABD Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def absolute_difference(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svabd[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svabd[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svabd[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  UABD Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svadd[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svadd[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svadd[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  ADD Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svadd[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svadd[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svadd[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FADD Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svadd[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svadd[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svadd[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  ADD Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svadd[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svadd[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svadd[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  ADD Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svadd[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svadd[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svadd[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  ADD Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svadd[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svadd[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svadd[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  ADD Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svadd[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svadd[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svadd[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FADD Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svadd[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svadd[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svadd[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  ADD Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadd[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svadd[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svadd[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  ADD Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def add(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadd[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svadd[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svadd[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  ADD Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float64_t svaddv[_f64](svbool_t pg, svfloat64_t op)  FADDV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svaddv[_s16](svbool_t pg, svint16_t op)  SADDV Dresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svaddv[_s32](svbool_t pg, svint32_t op)  SADDV Dresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svaddv[_s8](svbool_t pg, svint8_t op)  SADDV Dresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svaddv[_s64](svbool_t pg, svint64_t op)  UADDV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float32_t svaddv[_f32](svbool_t pg, svfloat32_t op)  FADDV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svaddv[_u8](svbool_t pg, svuint8_t op)  UADDV Dresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svaddv[_u16](svbool_t pg, svuint16_t op)  UADDV Dresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svaddv[_u32](svbool_t pg, svuint32_t op)  UADDV Dresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def add_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svaddv[_u64](svbool_t pg, svuint64_t op)  UADDV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def add_rotate_complex(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], rotation: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svcadd[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2, uint64_t imm_rotation)svfloat64_t svcadd[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2, uint64_t imm_rotation)svfloat64_t svcadd[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2, uint64_t imm_rotation)  FCADD Ztied1.D, Pg/M, Ztied1.D, Zop2.D, #imm_rotation"""
        ...

    @staticmethod
    @overload
    def add_rotate_complex(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], rotation: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svcadd[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2, uint64_t imm_rotation)svfloat32_t svcadd[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2, uint64_t imm_rotation)svfloat32_t svcadd[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2, uint64_t imm_rotation)  FCADD Ztied1.S, Pg/M, Ztied1.S, Zop2.S, #imm_rotation  FCADD Ztied1.S, Pg/M, Ztied1.S, Zop2.S, #imm_rotation"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svqadd[_u8](svuint8_t op1, svuint8_t op2)  UQADD Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svqadd[_s16](svint16_t op1, svint16_t op2)  SQADD Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svqadd[_s32](svint32_t op1, svint32_t op2)  SQADD Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svqadd[_s64](svint64_t op1, svint64_t op2)  SQADD Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svqadd[_s8](svint8_t op1, svint8_t op2)  SQADD Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svqadd[_u16](svuint16_t op1, svuint16_t op2)  UQADD Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svqadd[_u32](svuint32_t op1, svuint32_t op2)  UQADD Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def add_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svqadd[_u64](svuint64_t op1, svuint64_t op2)  UQADD Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def add_sequential_across(initial: System.Numerics.Vector[float], value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float64_t svadda[_f64](svbool_t pg, float64_t initial, svfloat64_t op)  FADDA Dtied, Pg, Dtied, Zop.D"""
        ...

    @staticmethod
    @overload
    def add_sequential_across(initial: System.Numerics.Vector[float], value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float32_t svadda[_f32](svbool_t pg, float32_t initial, svfloat32_t op)  FADDA Stied, Pg, Stied, Zop.S"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svand[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svand[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svand[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svand[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svand[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svand[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svand[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svand[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svand[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svand[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svand[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svand[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svand[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svand[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svand[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svand[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svand[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svand[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svand[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svand[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svand[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def And(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svand[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svand[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svand[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  AND Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint8_t svandv[_u8](svbool_t pg, svuint8_t op)  ANDV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int16_t svandv[_s16](svbool_t pg, svint16_t op)  ANDV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int32_t svandv[_s32](svbool_t pg, svint32_t op)  ANDV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svandv[_s64](svbool_t pg, svint64_t op)  ANDV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int8_t svandv[_s8](svbool_t pg, svint8_t op)  ANDV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint16_t svandv[_u16](svbool_t pg, svuint16_t op)  ANDV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint32_t svandv[_u32](svbool_t pg, svuint32_t op)  ANDV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def and_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svandv[_u64](svbool_t pg, svuint64_t op)  ANDV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svbic[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svbic[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svbic[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  BIC Ztied1.B, Pg/M, Ztied1.B, Zop2.B  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svbic[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svbic[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svbic[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  BIC Ztied1.H, Pg/M, Ztied1.H, Zop2.H  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svbic[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svbic[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svbic[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  BIC Ztied1.S, Pg/M, Ztied1.S, Zop2.S  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svbic[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svbic[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svbic[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  BIC Ztied1.D, Pg/M, Ztied1.D, Zop2.D  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svbic[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svbic[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svbic[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  BIC Ztied1.B, Pg/M, Ztied1.B, Zop2.B  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svbic[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svbic[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svbic[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  BIC Ztied1.H, Pg/M, Ztied1.H, Zop2.H  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svbic[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svbic[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svbic[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  BIC Ztied1.S, Pg/M, Ztied1.S, Zop2.S  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def bitwise_clear(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svbic[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svbic[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svbic[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  BIC Ztied1.D, Pg/M, Ztied1.D, Zop2.D  BIC Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svcnot[_u8]_m(svuint8_t inactive, svbool_t pg, svuint8_t op)svuint8_t svcnot[_u8]_x(svbool_t pg, svuint8_t op)svuint8_t svcnot[_u8]_z(svbool_t pg, svuint8_t op)  CNOT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svcnot[_s16]_m(svint16_t inactive, svbool_t pg, svint16_t op)svint16_t svcnot[_s16]_x(svbool_t pg, svint16_t op)svint16_t svcnot[_s16]_z(svbool_t pg, svint16_t op)  CNOT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svcnot[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svcnot[_s32]_x(svbool_t pg, svint32_t op)svint32_t svcnot[_s32]_z(svbool_t pg, svint32_t op)  CNOT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svcnot[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svcnot[_s64]_x(svbool_t pg, svint64_t op)svint64_t svcnot[_s64]_z(svbool_t pg, svint64_t op)  CNOT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svcnot[_s8]_m(svint8_t inactive, svbool_t pg, svint8_t op)svint8_t svcnot[_s8]_x(svbool_t pg, svint8_t op)svint8_t svcnot[_s8]_z(svbool_t pg, svint8_t op)  CNOT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svcnot[_u16]_m(svuint16_t inactive, svbool_t pg, svuint16_t op)svuint16_t svcnot[_u16]_x(svbool_t pg, svuint16_t op)svuint16_t svcnot[_u16]_z(svbool_t pg, svuint16_t op)  CNOT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svcnot[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svcnot[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svcnot[_u32]_z(svbool_t pg, svuint32_t op)  CNOT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def boolean_not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svcnot[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svcnot[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svcnot[_u64]_z(svbool_t pg, svuint64_t op)  CNOT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def compact(mask: System.Numerics.Vector[float], value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svcompact[_f64](svbool_t pg, svfloat64_t op)  COMPACT Zresult.D, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def compact(mask: System.Numerics.Vector[int], value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svcompact[_s32](svbool_t pg, svint32_t op)  COMPACT Zresult.S, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def compact(mask: System.Numerics.Vector[int], value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svcompact[_s64](svbool_t pg, svint64_t op)  COMPACT Zresult.D, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def compact(mask: System.Numerics.Vector[float], value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svcompact[_f32](svbool_t pg, svfloat32_t op)  COMPACT Zresult.S, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def compact(mask: System.Numerics.Vector[int], value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svcompact[_u32](svbool_t pg, svuint32_t op)  COMPACT Zresult.S, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def compact(mask: System.Numerics.Vector[int], value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svcompact[_u64](svbool_t pg, svuint64_t op)  COMPACT Zresult.D, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)  CMPEQ Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpeq[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FCMEQ Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_s16](svbool_t pg, svint16_t op1, svint16_t op2)  CMPEQ Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq_wide[_s16](svbool_t pg, svint16_t op1, svint64_t op2)  CMPEQ Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_s32](svbool_t pg, svint32_t op1, svint32_t op2)  CMPEQ Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq_wide[_s32](svbool_t pg, svint32_t op1, svint64_t op2)  CMPEQ Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_s64](svbool_t pg, svint64_t op1, svint64_t op2)  CMPEQ Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_s8](svbool_t pg, svint8_t op1, svint8_t op2)  CMPEQ Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq_wide[_s8](svbool_t pg, svint8_t op1, svint64_t op2)  CMPEQ Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpeq[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FCMEQ Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)  CMPEQ Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)  CMPEQ Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpeq[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)  CMPEQ Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)  CMPHI Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt_wide[_u8](svbool_t pg, svuint8_t op1, svuint64_t op2)  CMPHI Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpgt[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FCMGT Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_s16](svbool_t pg, svint16_t op1, svint16_t op2)  CMPGT Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt_wide[_s16](svbool_t pg, svint16_t op1, svint64_t op2)  CMPGT Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_s32](svbool_t pg, svint32_t op1, svint32_t op2)  CMPGT Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt_wide[_s32](svbool_t pg, svint32_t op1, svint64_t op2)  CMPGT Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_s64](svbool_t pg, svint64_t op1, svint64_t op2)  CMPGT Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_s8](svbool_t pg, svint8_t op1, svint8_t op2)  CMPGT Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt_wide[_s8](svbool_t pg, svint8_t op1, svint64_t op2)  CMPGT Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpgt[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FCMGT Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)  CMPHI Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt_wide[_u16](svbool_t pg, svuint16_t op1, svuint64_t op2)  CMPHI Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)  CMPHI Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt_wide[_u32](svbool_t pg, svuint32_t op1, svuint64_t op2)  CMPHI Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpgt[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)  CMPHI Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)  CMPHS Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge_wide[_u8](svbool_t pg, svuint8_t op1, svuint64_t op2)  CMPHS Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpge[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FCMGE Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_s16](svbool_t pg, svint16_t op1, svint16_t op2)  CMPGE Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge_wide[_s16](svbool_t pg, svint16_t op1, svint64_t op2)  CMPGE Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_s32](svbool_t pg, svint32_t op1, svint32_t op2)  CMPGE Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge_wide[_s32](svbool_t pg, svint32_t op1, svint64_t op2)  CMPGE Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_s64](svbool_t pg, svint64_t op1, svint64_t op2)  CMPGE Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_s8](svbool_t pg, svint8_t op1, svint8_t op2)  CMPGE Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge_wide[_s8](svbool_t pg, svint8_t op1, svint64_t op2)  CMPGE Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpge[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FCMGE Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)  CMPHS Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge_wide[_u16](svbool_t pg, svuint16_t op1, svuint64_t op2)  CMPHS Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)  CMPHS Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge_wide[_u32](svbool_t pg, svuint32_t op1, svuint64_t op2)  CMPHS Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_greater_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpge[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)  CMPHS Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)  CMPHI Presult.B, Pg/Z, Zop2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt_wide[_u8](svbool_t pg, svuint8_t op1, svuint64_t op2)  CMPLO Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmplt[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FCMGT Presult.D, Pg/Z, Zop2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_s16](svbool_t pg, svint16_t op1, svint16_t op2)  CMPGT Presult.H, Pg/Z, Zop2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt_wide[_s16](svbool_t pg, svint16_t op1, svint64_t op2)  CMPLT Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_s32](svbool_t pg, svint32_t op1, svint32_t op2)  CMPGT Presult.S, Pg/Z, Zop2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt_wide[_s32](svbool_t pg, svint32_t op1, svint64_t op2)  CMPLT Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_s64](svbool_t pg, svint64_t op1, svint64_t op2)  CMPGT Presult.D, Pg/Z, Zop2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_s8](svbool_t pg, svint8_t op1, svint8_t op2)  CMPGT Presult.B, Pg/Z, Zop2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt_wide[_s8](svbool_t pg, svint8_t op1, svint64_t op2)  CMPLT Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmplt[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FCMGT Presult.S, Pg/Z, Zop2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)  CMPHI Presult.H, Pg/Z, Zop2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt_wide[_u16](svbool_t pg, svuint16_t op1, svuint64_t op2)  CMPLO Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)  CMPHI Presult.S, Pg/Z, Zop2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt_wide[_u32](svbool_t pg, svuint32_t op1, svuint64_t op2)  CMPLO Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmplt[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)  CMPHI Presult.D, Pg/Z, Zop2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)  CMPHS Presult.B, Pg/Z, Zop2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple_wide[_u8](svbool_t pg, svuint8_t op1, svuint64_t op2)  CMPLS Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmple[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FCMGE Presult.D, Pg/Z, Zop2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_s16](svbool_t pg, svint16_t op1, svint16_t op2)  CMPGE Presult.H, Pg/Z, Zop2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple_wide[_s16](svbool_t pg, svint16_t op1, svint64_t op2)  CMPLE Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_s32](svbool_t pg, svint32_t op1, svint32_t op2)  CMPGE Presult.S, Pg/Z, Zop2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple_wide[_s32](svbool_t pg, svint32_t op1, svint64_t op2)  CMPLE Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_s64](svbool_t pg, svint64_t op1, svint64_t op2)  CMPGE Presult.D, Pg/Z, Zop2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_s8](svbool_t pg, svint8_t op1, svint8_t op2)  CMPGE Presult.B, Pg/Z, Zop2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple_wide[_s8](svbool_t pg, svint8_t op1, svint64_t op2)  CMPLE Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmple[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FCMGE Presult.S, Pg/Z, Zop2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)  CMPHS Presult.H, Pg/Z, Zop2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple_wide[_u16](svbool_t pg, svuint16_t op1, svuint64_t op2)  CMPLS Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)  CMPHS Presult.S, Pg/Z, Zop2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple_wide[_u32](svbool_t pg, svuint32_t op1, svuint64_t op2)  CMPLS Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_less_than_or_equal(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmple[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)  CMPHS Presult.D, Pg/Z, Zop2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)  CMPNE Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpne[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FCMNE Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_s16](svbool_t pg, svint16_t op1, svint16_t op2)  CMPNE Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne_wide[_s16](svbool_t pg, svint16_t op1, svint64_t op2)  CMPNE Presult.H, Pg/Z, Zop1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_s32](svbool_t pg, svint32_t op1, svint32_t op2)  CMPNE Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne_wide[_s32](svbool_t pg, svint32_t op1, svint64_t op2)  CMPNE Presult.S, Pg/Z, Zop1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_s64](svbool_t pg, svint64_t op1, svint64_t op2)  CMPNE Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_s8](svbool_t pg, svint8_t op1, svint8_t op2)  CMPNE Presult.B, Pg/Z, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne_wide[_s8](svbool_t pg, svint8_t op1, svint64_t op2)  CMPNE Presult.B, Pg/Z, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpne[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FCMNE Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)  CMPNE Presult.H, Pg/Z, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)  CMPNE Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compare_not_equal_to(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svcmpne[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)  CMPNE Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_unordered(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpuo[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FCMUO Presult.D, Pg/Z, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def compare_unordered(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svbool_t svcmpuo[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FCMUO Presult.S, Pg/Z, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def compute_16_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrh[_u32base]_[s32]index(svuint32_t bases, svint32_t indices)  ADR Zresult.S, [Zbases.S, Zindices.S, LSL #1]"""
        ...

    @staticmethod
    @overload
    def compute_16_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrh[_u32base]_[u32]index(svuint32_t bases, svuint32_t indices)  ADR Zresult.S, [Zbases.S, Zindices.S, LSL #1]"""
        ...

    @staticmethod
    @overload
    def compute_16_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrh[_u64base]_[s64]index(svuint64_t bases, svint64_t indices)  ADR Zresult.D, [Zbases.D, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def compute_16_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrh[_u64base]_[u64]index(svuint64_t bases, svuint64_t indices)  ADR Zresult.D, [Zbases.D, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def compute_32_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrw[_u32base]_[s32]index(svuint32_t bases, svint32_t indices)  ADR Zresult.S, [Zbases.S, Zindices.S, LSL #2]"""
        ...

    @staticmethod
    @overload
    def compute_32_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrw[_u32base]_[u32]index(svuint32_t bases, svuint32_t indices)  ADR Zresult.S, [Zbases.S, Zindices.S, LSL #2]"""
        ...

    @staticmethod
    @overload
    def compute_32_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrw[_u64base]_[s64]index(svuint64_t bases, svint64_t indices)  ADR Zresult.D, [Zbases.D, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def compute_32_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrw[_u64base]_[u64]index(svuint64_t bases, svuint64_t indices)  ADR Zresult.D, [Zbases.D, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def compute_64_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrd[_u32base]_[s32]index(svuint32_t bases, svint32_t indices)  ADR Zresult.S, [Zbases.S, Zindices.S, LSL #3]"""
        ...

    @staticmethod
    @overload
    def compute_64_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrd[_u32base]_[u32]index(svuint32_t bases, svuint32_t indices)  ADR Zresult.S, [Zbases.S, Zindices.S, LSL #3]"""
        ...

    @staticmethod
    @overload
    def compute_64_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrd[_u64base]_[s64]index(svuint64_t bases, svint64_t indices)  ADR Zresult.D, [Zbases.D, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def compute_64_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrd[_u64base]_[u64]index(svuint64_t bases, svuint64_t indices)  ADR Zresult.D, [Zbases.D, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def compute_8_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrb[_u32base]_[s32]offset(svuint32_t bases, svint32_t offsets)  ADR Zresult.S, [Zbases.S, Zoffsets.S]"""
        ...

    @staticmethod
    @overload
    def compute_8_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svadrb[_u32base]_[u32]offset(svuint32_t bases, svuint32_t offsets)  ADR Zresult.S, [Zbases.S, Zoffsets.S]"""
        ...

    @staticmethod
    @overload
    def compute_8_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrb[_u64base]_[s64]offset(svuint64_t bases, svint64_t offsets)  ADR Zresult.D, [Zbases.D, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def compute_8_bit_addresses(bases: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svadrb[_u64base]_[u64]offset(svuint64_t bases, svuint64_t offsets)  ADR Zresult.D, [Zbases.D, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svclasta[_u8](svbool_t pg, svuint8_t fallback, svuint8_t data)  CLASTA Btied, Pg, Btied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint8_t svclasta[_n_u8](svbool_t pg, uint8_t fallback, svuint8_t data)  CLASTA Wtied, Pg, Wtied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[float], default_value: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svclasta[_f64](svbool_t pg, svfloat64_t fallback, svfloat64_t data)  CLASTA Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[float], default_values: float, data: System.Numerics.Vector[float]) -> float:
        """float64_t svclasta[_n_f64](svbool_t pg, float64_t fallback, svfloat64_t data)  CLASTA Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svclasta[_s16](svbool_t pg, svint16_t fallback, svint16_t data)  CLASTA Htied, Pg, Htied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int16_t svclasta[_n_s16](svbool_t pg, int16_t fallback, svint16_t data)  CLASTA Wtied, Pg, Wtied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svclasta[_s32](svbool_t pg, svint32_t fallback, svint32_t data)  CLASTA Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int32_t svclasta[_n_s32](svbool_t pg, int32_t fallback, svint32_t data)  CLASTA Wtied, Pg, Wtied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svclasta[_s64](svbool_t pg, svint64_t fallback, svint64_t data)  CLASTA Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int64_t svclasta[_n_s64](svbool_t pg, int64_t fallback, svint64_t data)  CLASTA Xtied, Pg, Xtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svclasta[_s8](svbool_t pg, svint8_t fallback, svint8_t data)  CLASTA Btied, Pg, Btied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int8_t svclasta[_n_s8](svbool_t pg, int8_t fallback, svint8_t data)  CLASTA Wtied, Pg, Wtied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[float], default_value: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svclasta[_f32](svbool_t pg, svfloat32_t fallback, svfloat32_t data)  CLASTA Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[float], default_values: float, data: System.Numerics.Vector[float]) -> float:
        """float32_t svclasta[_n_f32](svbool_t pg, float32_t fallback, svfloat32_t data)  CLASTA Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svclasta[_u16](svbool_t pg, svuint16_t fallback, svuint16_t data)  CLASTA Htied, Pg, Htied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint16_t svclasta[_n_u16](svbool_t pg, uint16_t fallback, svuint16_t data)  CLASTA Wtied, Pg, Wtied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svclasta[_u32](svbool_t pg, svuint32_t fallback, svuint32_t data)  CLASTA Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint32_t svclasta[_n_u32](svbool_t pg, uint32_t fallback, svuint32_t data)  CLASTA Wtied, Pg, Wtied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svclasta[_u64](svbool_t pg, svuint64_t fallback, svuint64_t data)  CLASTA Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint64_t svclasta[_n_u64](svbool_t pg, uint64_t fallback, svuint64_t data)  CLASTA Xtied, Pg, Xtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svclasta[_u8](svbool_t pg, svuint8_t fallback, svuint8_t data)  CLASTA Ztied.B, Pg, Ztied.B, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[float], default_scalar: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svclasta[_f64](svbool_t pg, svfloat64_t fallback, svfloat64_t data)  CLASTA Ztied.D, Pg, Ztied.D, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svclasta[_s16](svbool_t pg, svint16_t fallback, svint16_t data)  CLASTA Ztied.H, Pg, Ztied.H, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svclasta[_s32](svbool_t pg, svint32_t fallback, svint32_t data)  CLASTA Ztied.S, Pg, Ztied.S, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svclasta[_s64](svbool_t pg, svint64_t fallback, svint64_t data)  CLASTA Ztied.D, Pg, Ztied.D, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svclasta[_s8](svbool_t pg, svint8_t fallback, svint8_t data)  CLASTA Ztied.B, Pg, Ztied.B, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[float], default_scalar: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svclasta[_f32](svbool_t pg, svfloat32_t fallback, svfloat32_t data)  CLASTA Ztied.S, Pg, Ztied.S, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svclasta[_u16](svbool_t pg, svuint16_t fallback, svuint16_t data)  CLASTA Ztied.H, Pg, Ztied.H, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svclasta[_u32](svbool_t pg, svuint32_t fallback, svuint32_t data)  CLASTA Ztied.S, Pg, Ztied.S, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_after_last_active_element_and_replicate(mask: System.Numerics.Vector[int], default_scalar: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svclasta[_u64](svbool_t pg, svuint64_t fallback, svuint64_t data)  CLASTA Ztied.D, Pg, Ztied.D, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svclastb[_u8](svbool_t pg, svuint8_t fallback, svuint8_t data)  CLASTB Btied, Pg, Btied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint8_t svclastb[_n_u8](svbool_t pg, uint8_t fallback, svuint8_t data)  CLASTB Wtied, Pg, Wtied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[float], default_value: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svclastb[_f64](svbool_t pg, svfloat64_t fallback, svfloat64_t data)  CLASTB Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[float], default_values: float, data: System.Numerics.Vector[float]) -> float:
        """float64_t svclastb[_n_f64](svbool_t pg, float64_t fallback, svfloat64_t data)  CLASTB Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svclastb[_s16](svbool_t pg, svint16_t fallback, svint16_t data)  CLASTB Htied, Pg, Htied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int16_t svclastb[_n_s16](svbool_t pg, int16_t fallback, svint16_t data)  CLASTB Wtied, Pg, Wtied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svclastb[_s32](svbool_t pg, svint32_t fallback, svint32_t data)  CLASTB Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int32_t svclastb[_n_s32](svbool_t pg, int32_t fallback, svint32_t data)  CLASTB Wtied, Pg, Wtied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svclastb[_s64](svbool_t pg, svint64_t fallback, svint64_t data)  CLASTB Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int64_t svclastb[_n_s64](svbool_t pg, int64_t fallback, svint64_t data)  CLASTB Xtied, Pg, Xtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svclastb[_s8](svbool_t pg, svint8_t fallback, svint8_t data)  CLASTB Btied, Pg, Btied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """int8_t svclastb[_n_s8](svbool_t pg, int8_t fallback, svint8_t data)  CLASTB Wtied, Pg, Wtied, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[float], default_value: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svclastb[_f32](svbool_t pg, svfloat32_t fallback, svfloat32_t data)  CLASTB Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[float], default_values: float, data: System.Numerics.Vector[float]) -> float:
        """float32_t svclastb[_n_f32](svbool_t pg, float32_t fallback, svfloat32_t data)  CLASTB Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svclastb[_u16](svbool_t pg, svuint16_t fallback, svuint16_t data)  CLASTB Htied, Pg, Htied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint16_t svclastb[_n_u16](svbool_t pg, uint16_t fallback, svuint16_t data)  CLASTB Wtied, Pg, Wtied, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svclastb[_u32](svbool_t pg, svuint32_t fallback, svuint32_t data)  CLASTB Stied, Pg, Stied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint32_t svclastb[_n_u32](svbool_t pg, uint32_t fallback, svuint32_t data)  CLASTB Wtied, Pg, Wtied, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_value: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svclastb[_u64](svbool_t pg, svuint64_t fallback, svuint64_t data)  CLASTB Dtied, Pg, Dtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element(mask: System.Numerics.Vector[int], default_values: int, data: System.Numerics.Vector[int]) -> int:
        """uint64_t svclastb[_n_u64](svbool_t pg, uint64_t fallback, svuint64_t data)  CLASTB Xtied, Pg, Xtied, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svclastb[_u8](svbool_t pg, svuint8_t fallback, svuint8_t data)  CLASTB Ztied.B, Pg, Ztied.B, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[float], fallback: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svclastb[_f64](svbool_t pg, svfloat64_t fallback, svfloat64_t data)  CLASTB Ztied.D, Pg, Ztied.D, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svclastb[_s16](svbool_t pg, svint16_t fallback, svint16_t data)  CLASTB Ztied.H, Pg, Ztied.H, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svclastb[_s32](svbool_t pg, svint32_t fallback, svint32_t data)  CLASTB Ztied.S, Pg, Ztied.S, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svclastb[_s64](svbool_t pg, svint64_t fallback, svint64_t data)  CLASTB Ztied.D, Pg, Ztied.D, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svclastb[_s8](svbool_t pg, svint8_t fallback, svint8_t data)  CLASTB Ztied.B, Pg, Ztied.B, Zdata.B"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[float], fallback: System.Numerics.Vector[float], data: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svclastb[_f32](svbool_t pg, svfloat32_t fallback, svfloat32_t data)  CLASTB Ztied.S, Pg, Ztied.S, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svclastb[_u16](svbool_t pg, svuint16_t fallback, svuint16_t data)  CLASTB Ztied.H, Pg, Ztied.H, Zdata.H"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svclastb[_u32](svbool_t pg, svuint32_t fallback, svuint32_t data)  CLASTB Ztied.S, Pg, Ztied.S, Zdata.S"""
        ...

    @staticmethod
    @overload
    def conditional_extract_last_active_element_and_replicate(mask: System.Numerics.Vector[int], fallback: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svclastb[_u64](svbool_t pg, svuint64_t fallback, svuint64_t data)  CLASTB Ztied.D, Pg, Ztied.D, Zdata.D"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svsel[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.B, Pg, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svsel[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  SEL Zresult.D, Pg, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svsel[_s16](svbool_t pg, svint16_t op1, svint16_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.H, Pg, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svsel[_s32](svbool_t pg, svint32_t op1, svint32_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.S, Pg, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svsel[_s64](svbool_t pg, svint64_t op1, svint64_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.D, Pg, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svsel[_s8](svbool_t pg, svint8_t op1, svint8_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.B, Pg, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svsel[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  SEL Zresult.S, Pg, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svsel[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.H, Pg, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svsel[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.S, Pg, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def conditional_select(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svsel[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)svbool_t svsel[_b](svbool_t pg, svbool_t op1, svbool_t op2)  SEL Zresult.D, Pg, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def convert_to_double(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svcvt_f64[_s32]_m(svfloat64_t inactive, svbool_t pg, svint32_t op)svfloat64_t svcvt_f64[_s32]_x(svbool_t pg, svint32_t op)svfloat64_t svcvt_f64[_s32]_z(svbool_t pg, svint32_t op)  SCVTF Zresult.D, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_double(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svcvt_f64[_s64]_m(svfloat64_t inactive, svbool_t pg, svint64_t op)svfloat64_t svcvt_f64[_s64]_x(svbool_t pg, svint64_t op)svfloat64_t svcvt_f64[_s64]_z(svbool_t pg, svint64_t op)  SCVTF Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_double(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svcvt_f64[_f32]_m(svfloat64_t inactive, svbool_t pg, svfloat32_t op)svfloat64_t svcvt_f64[_f32]_x(svbool_t pg, svfloat32_t op)svfloat64_t svcvt_f64[_f32]_z(svbool_t pg, svfloat32_t op)  FCVT Zresult.D, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_double(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svcvt_f64[_u32]_m(svfloat64_t inactive, svbool_t pg, svuint32_t op)svfloat64_t svcvt_f64[_u32]_x(svbool_t pg, svuint32_t op)svfloat64_t svcvt_f64[_u32]_z(svbool_t pg, svuint32_t op)  UCVTF Zresult.D, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_double(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svcvt_f64[_u64]_m(svfloat64_t inactive, svbool_t pg, svuint64_t op)svfloat64_t svcvt_f64[_u64]_x(svbool_t pg, svuint64_t op)svfloat64_t svcvt_f64[_u64]_z(svbool_t pg, svuint64_t op)  UCVTF Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svint32_t svcvt_s32[_f64]_m(svint32_t inactive, svbool_t pg, svfloat64_t op)svint32_t svcvt_s32[_f64]_x(svbool_t pg, svfloat64_t op)svint32_t svcvt_s32[_f64]_z(svbool_t pg, svfloat64_t op)  FCVTZS Zresult.S, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_int_32(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svint32_t svcvt_s32[_f32]_m(svint32_t inactive, svbool_t pg, svfloat32_t op)svint32_t svcvt_s32[_f32]_x(svbool_t pg, svfloat32_t op)svint32_t svcvt_s32[_f32]_z(svbool_t pg, svfloat32_t op)  FCVTZS Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_int_64(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svint64_t svcvt_s64[_f64]_m(svint64_t inactive, svbool_t pg, svfloat64_t op)svint64_t svcvt_s64[_f64]_x(svbool_t pg, svfloat64_t op)svint64_t svcvt_s64[_f64]_z(svbool_t pg, svfloat64_t op)  FCVTZS Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_int_64(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svint64_t svcvt_s64[_f32]_m(svint64_t inactive, svbool_t pg, svfloat32_t op)svint64_t svcvt_s64[_f32]_x(svbool_t pg, svfloat32_t op)svint64_t svcvt_s64[_f32]_z(svbool_t pg, svfloat32_t op)  FCVTZS Zresult.D, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svcvt_f32[_f64]_m(svfloat32_t inactive, svbool_t pg, svfloat64_t op)svfloat32_t svcvt_f32[_f64]_x(svbool_t pg, svfloat64_t op)svfloat32_t svcvt_f32[_f64]_z(svbool_t pg, svfloat64_t op)  FCVT Zresult.S, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svcvt_f32[_s32]_m(svfloat32_t inactive, svbool_t pg, svint32_t op)svfloat32_t svcvt_f32[_s32]_x(svbool_t pg, svint32_t op)svfloat32_t svcvt_f32[_s32]_z(svbool_t pg, svint32_t op)  SCVTF Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svcvt_f32[_s64]_m(svfloat32_t inactive, svbool_t pg, svint64_t op)svfloat32_t svcvt_f32[_s64]_x(svbool_t pg, svint64_t op)svfloat32_t svcvt_f32[_s64]_z(svbool_t pg, svint64_t op)  SCVTF Zresult.S, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svcvt_f32[_u32]_m(svfloat32_t inactive, svbool_t pg, svuint32_t op)svfloat32_t svcvt_f32[_u32]_x(svbool_t pg, svuint32_t op)svfloat32_t svcvt_f32[_u32]_z(svbool_t pg, svuint32_t op)  UCVTF Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_single(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svcvt_f32[_u64]_m(svfloat32_t inactive, svbool_t pg, svuint64_t op)svfloat32_t svcvt_f32[_u64]_x(svbool_t pg, svuint64_t op)svfloat32_t svcvt_f32[_u64]_z(svbool_t pg, svuint64_t op)  UCVTF Zresult.S, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svuint32_t svcvt_u32[_f64]_m(svuint32_t inactive, svbool_t pg, svfloat64_t op)svuint32_t svcvt_u32[_f64]_x(svbool_t pg, svfloat64_t op)svuint32_t svcvt_u32[_f64]_z(svbool_t pg, svfloat64_t op)  FCVTZU Zresult.S, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_32(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svuint32_t svcvt_u32[_f32]_m(svuint32_t inactive, svbool_t pg, svfloat32_t op)svuint32_t svcvt_u32[_f32]_x(svbool_t pg, svfloat32_t op)svuint32_t svcvt_u32[_f32]_z(svbool_t pg, svfloat32_t op)  FCVTZU Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_64(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svuint64_t svcvt_u64[_f64]_m(svuint64_t inactive, svbool_t pg, svfloat64_t op)svuint64_t svcvt_u64[_f64]_x(svbool_t pg, svfloat64_t op)svuint64_t svcvt_u64[_f64]_z(svbool_t pg, svfloat64_t op)  FCVTZU Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def convert_to_u_int_64(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svuint64_t svcvt_u64[_f32]_m(svuint64_t inactive, svbool_t pg, svfloat32_t op)svuint64_t svcvt_u64[_f32]_x(svbool_t pg, svfloat32_t op)svuint64_t svcvt_u64[_f32]_z(svbool_t pg, svfloat32_t op)  FCVTZU Zresult.D, Pg/M, Zop.S"""
        ...

    @staticmethod
    def count_16_bit_elements(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svcnth_pat(enum svpattern pattern)  CNTH Xresult, pattern"""
        ...

    @staticmethod
    def count_32_bit_elements(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svcntw_pat(enum svpattern pattern)  CNTW Xresult, pattern"""
        ...

    @staticmethod
    def count_64_bit_elements(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svcntd_pat(enum svpattern pattern)  CNTD Xresult, pattern"""
        ...

    @staticmethod
    def count_8_bit_elements(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svcntb_pat(enum svpattern pattern)  CNTB Xresult, pattern"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrka[_b]_z(svbool_t pg, svbool_t op)  BRKA Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_after_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpa[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPA Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_mask(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkb[_b]_z(svbool_t pg, svbool_t op)  BRKB Presult.B, Pg/Z, Pop.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_before_propagate_mask(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkpb[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKPB Presult.B, Pg/Z, Pop1.B, Pop2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    @overload
    def create_break_propagate_mask(total_mask: System.Numerics.Vector[int], from_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svbrkn[_b]_z(svbool_t pg, svbool_t op1, svbool_t op2)  BRKN Ptied2.B, Pg/Z, Pop1.B, Ptied2.B"""
        ...

    @staticmethod
    def create_false_mask_byte() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_double() -> System.Numerics.Vector[float]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_int_16() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_int_32() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_int_64() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_s_byte() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_single() -> System.Numerics.Vector[float]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_u_int_16() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_u_int_32() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    def create_false_mask_u_int_64() -> System.Numerics.Vector[int]:
        """svbool_t svpfalse[_b]()  PFALSE Presult.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_first_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpfirst[_b](svbool_t pg, svbool_t op)  PFIRST Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_next_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpnext_b8(svbool_t pg, svbool_t op)  PNEXT Ptied.B, Pg, Ptied.B"""
        ...

    @staticmethod
    @overload
    def create_mask_for_next_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpnext_b16(svbool_t pg, svbool_t op)  PNEXT Ptied.H, Pg, Ptied.H"""
        ...

    @staticmethod
    @overload
    def create_mask_for_next_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpnext_b32(svbool_t pg, svbool_t op)  PNEXT Ptied.S, Pg, Ptied.S"""
        ...

    @staticmethod
    @overload
    def create_mask_for_next_active_element(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svbool_t svpnext_b64(svbool_t pg, svbool_t op)  PNEXT Ptied.D, Pg, Ptied.D"""
        ...

    @staticmethod
    def create_true_mask_byte(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b8(enum svpattern pattern)  PTRUE Presult.B, pattern"""
        ...

    @staticmethod
    def create_true_mask_double(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[float]:
        """svbool_t svptrue_pat_b8(enum svpattern pattern)  PTRUE Presult.B, pattern"""
        ...

    @staticmethod
    def create_true_mask_int_16(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b8(enum svpattern pattern)  PTRUE Presult.B, pattern"""
        ...

    @staticmethod
    def create_true_mask_int_32(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b8(enum svpattern pattern)  PTRUE Presult.B, pattern"""
        ...

    @staticmethod
    def create_true_mask_int_64(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b8(enum svpattern pattern)  PTRUE Presult.B, pattern"""
        ...

    @staticmethod
    def create_true_mask_s_byte(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b8(enum svpattern pattern)  PTRUE Presult.B, pattern"""
        ...

    @staticmethod
    def create_true_mask_single(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[float]:
        """svbool_t svptrue_pat_b8(enum svpattern pattern)  PTRUE Presult.B, pattern"""
        ...

    @staticmethod
    def create_true_mask_u_int_16(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b16(enum svpattern pattern)  PTRUE Presult.H, pattern"""
        ...

    @staticmethod
    def create_true_mask_u_int_32(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b32(enum svpattern pattern)  PTRUE Presult.S, pattern"""
        ...

    @staticmethod
    def create_true_mask_u_int_64(pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svbool_t svptrue_pat_b64(enum svpattern pattern)  PTRUE Presult.D, pattern"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b16[_s32](int32_t op1, int32_t op2)  WHILELT Presult.H, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b16[_s64](int64_t op1, int64_t op2)  WHILELT Presult.H, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b16[_u32](uint32_t op1, uint32_t op2)  WHILELO Presult.H, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b16[_u64](uint64_t op1, uint64_t op2)  WHILELO Presult.H, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b32[_s32](int32_t op1, int32_t op2)  WHILELT Presult.S, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b32[_s64](int64_t op1, int64_t op2)  WHILELT Presult.S, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b32[_u32](uint32_t op1, uint32_t op2)  WHILELO Presult.S, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b32[_u64](uint64_t op1, uint64_t op2)  WHILELO Presult.S, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b64[_s32](int32_t op1, int32_t op2)  WHILELT Presult.D, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b64[_s64](int64_t op1, int64_t op2)  WHILELT Presult.D, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b64[_u32](uint32_t op1, uint32_t op2)  WHILELO Presult.D, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b64[_u64](uint64_t op1, uint64_t op2)  WHILELO Presult.D, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b8[_s32](int32_t op1, int32_t op2)  WHILELT Presult.B, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b8[_s64](int64_t op1, int64_t op2)  WHILELT Presult.B, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b8[_u32](uint32_t op1, uint32_t op2)  WHILELO Presult.B, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilelt_b8[_u64](uint64_t op1, uint64_t op2)  WHILELO Presult.B, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b16[_s32](int32_t op1, int32_t op2)  WHILELE Presult.H, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b16[_s64](int64_t op1, int64_t op2)  WHILELE Presult.H, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b16[_u32](uint32_t op1, uint32_t op2)  WHILELS Presult.H, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_16_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b16[_u64](uint64_t op1, uint64_t op2)  WHILELS Presult.H, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b32[_s32](int32_t op1, int32_t op2)  WHILELE Presult.S, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b32[_s64](int64_t op1, int64_t op2)  WHILELE Presult.S, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b32[_u32](uint32_t op1, uint32_t op2)  WHILELS Presult.S, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_32_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b32[_u64](uint64_t op1, uint64_t op2)  WHILELS Presult.S, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b64[_s32](int32_t op1, int32_t op2)  WHILELE Presult.D, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b64[_s64](int64_t op1, int64_t op2)  WHILELE Presult.D, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b64[_u32](uint32_t op1, uint32_t op2)  WHILELS Presult.D, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_64_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b64[_u64](uint64_t op1, uint64_t op2)  WHILELS Presult.D, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b8[_s32](int32_t op1, int32_t op2)  WHILELE Presult.B, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b8[_s64](int64_t op1, int64_t op2)  WHILELE Presult.B, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b8[_u32](uint32_t op1, uint32_t op2)  WHILELS Presult.B, Wop1, Wop2"""
        ...

    @staticmethod
    @overload
    def create_while_less_than_or_equal_mask_8_bit(left: int, right: int) -> System.Numerics.Vector[int]:
        """svbool_t svwhilele_b8[_u64](uint64_t op1, uint64_t op2)  WHILELS Presult.B, Xop1, Xop2"""
        ...

    @staticmethod
    @overload
    def divide(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svdiv[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svdiv[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svdiv[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FDIV Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def divide(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svdiv[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svdiv[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svdiv[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FDIV Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svdot[_s32](svint32_t op1, svint8_t op2, svint8_t op3)  SDOT Ztied1.S, Zop2.B, Zop3.B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svdot[_s64](svint64_t op1, svint16_t op2, svint16_t op3)  SDOT Ztied1.D, Zop2.H, Zop3.H"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svdot[_u32](svuint32_t op1, svuint8_t op2, svuint8_t op3)  UDOT Ztied1.S, Zop2.B, Zop3.B"""
        ...

    @staticmethod
    @overload
    def dot_product(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svdot[_u64](svuint64_t op1, svuint16_t op2, svuint16_t op3)  UDOT Ztied1.D, Zop2.H, Zop3.H"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_scalar(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int], right_index: int) -> System.Numerics.Vector[int]:
        """svint32_t svdot_lane[_s32](svint32_t op1, svint8_t op2, svint8_t op3, uint64_t imm_index)  SDOT Ztied1.S, Zop2.B, Zop3.B[imm_index]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_scalar(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int], right_index: int) -> System.Numerics.Vector[int]:
        """svint64_t svdot_lane[_s64](svint64_t op1, svint16_t op2, svint16_t op3, uint64_t imm_index)  SDOT Ztied1.D, Zop2.H, Zop3.H[imm_index]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_scalar(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int], right_index: int) -> System.Numerics.Vector[int]:
        """svuint32_t svdot_lane[_u32](svuint32_t op1, svuint8_t op2, svuint8_t op3, uint64_t imm_index)  UDOT Ztied1.S, Zop2.B, Zop3.B[imm_index]"""
        ...

    @staticmethod
    @overload
    def dot_product_by_selected_scalar(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int], right_index: int) -> System.Numerics.Vector[int]:
        """svuint64_t svdot_lane[_u64](svuint64_t op1, svuint16_t op2, svuint16_t op3, uint64_t imm_index)  UDOT Ztied1.D, Zop2.H, Zop3.H[imm_index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint8_t svdup_lane[_u8](svuint8_t data, uint8_t index)  DUP Zresult.B, Zdata.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[float], index: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svdup_lane[_f64](svfloat64_t data, uint64_t index)  DUP Zresult.D, Zdata.D[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint16_t svdup_lane[_s16](svint16_t data, uint16_t index)  DUP Zresult.H, Zdata.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint32_t svdup_lane[_s32](svint32_t data, uint32_t index)  DUP Zresult.S, Zdata.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint64_t svdup_lane[_s64](svint64_t data, uint64_t index)  DUP Zresult.D, Zdata.D[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint8_t svdup_lane[_s8](svint8_t data, uint8_t index)  DUP Zresult.B, Zdata.B[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[float], index: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svdup_lane[_f32](svfloat32_t data, uint32_t index)  DUP Zresult.S, Zdata.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint16_t svdup_lane[_u16](svuint16_t data, uint16_t index)  DUP Zresult.H, Zdata.H[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint32_t svdup_lane[_u32](svuint32_t data, uint32_t index)  DUP Zresult.S, Zdata.S[index]"""
        ...

    @staticmethod
    @overload
    def duplicate_selected_scalar_to_vector(data: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint64_t svdup_lane[_u64](svuint64_t data, uint64_t index)  DUP Zresult.D, Zdata.D[index]"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint8_t svext[_u8](svuint8_t op1, svuint8_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[float], lower: System.Numerics.Vector[float], index: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svext[_f64](svfloat64_t op1, svfloat64_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 8"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint16_t svext[_s16](svint16_t op1, svint16_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 2"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint32_t svext[_s32](svint32_t op1, svint32_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 4"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint64_t svext[_s64](svint64_t op1, svint64_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 8"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svint8_t svext[_s8](svint8_t op1, svint8_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[float], lower: System.Numerics.Vector[float], index: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svext[_f32](svfloat32_t op1, svfloat32_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 4"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint16_t svext[_u16](svuint16_t op1, svuint16_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 2"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint32_t svext[_u32](svuint32_t op1, svuint32_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 4"""
        ...

    @staticmethod
    @overload
    def extract_vector(upper: System.Numerics.Vector[int], lower: System.Numerics.Vector[int], index: int) -> System.Numerics.Vector[int]:
        """svuint64_t svext[_u64](svuint64_t op1, svuint64_t op2, uint64_t imm3)  EXT Ztied1.B, Ztied1.B, Zop2.B, #imm3 * 8"""
        ...

    @staticmethod
    @overload
    def floating_point_exponential_accelerator(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svexpa[_f64](svuint64_t op)  FEXPA Zresult.D, Zop.D"""
        ...

    @staticmethod
    @overload
    def floating_point_exponential_accelerator(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svexpa[_f32](svuint32_t op)  FEXPA Zresult.S, Zop.S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svmla[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svmla[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svmla[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)  FMLA Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svmla[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svmla[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svmla[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)  FMLA Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_by_selected_scalar(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], right_index: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svmla_lane[_f64](svfloat64_t op1, svfloat64_t op2, svfloat64_t op3, uint64_t imm_index)  FMLA Ztied1.D, Zop2.D, Zop3.D[imm_index]"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_by_selected_scalar(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], right_index: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svmla_lane[_f32](svfloat32_t op1, svfloat32_t op2, svfloat32_t op3, uint64_t imm_index)  FMLA Ztied1.S, Zop2.S, Zop3.S[imm_index]"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_negated(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svnmla[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svnmla[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svnmla[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)  FNMLA Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def fused_multiply_add_negated(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svnmla[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svnmla[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svnmla[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)  FNMLA Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract(minuend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svmls[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svmls[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svmls[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)  FMLS Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract(minuend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svmls[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svmls[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svmls[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)  FMLS Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_by_selected_scalar(minuend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], right_index: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svmls_lane[_f64](svfloat64_t op1, svfloat64_t op2, svfloat64_t op3, uint64_t imm_index)  FMLS Ztied1.D, Zop2.D, Zop3.D[imm_index]"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_by_selected_scalar(minuend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], right_index: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svmls_lane[_f32](svfloat32_t op1, svfloat32_t op2, svfloat32_t op3, uint64_t imm_index)  FMLS Ztied1.S, Zop2.S, Zop3.S[imm_index]"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_negated(minuend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svnmls[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svnmls[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)svfloat64_t svnmls[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3)  FNMLS Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def fused_multiply_subtract_negated(minuend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svnmls[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svnmls[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)svfloat32_t svnmls[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3)  FNMLS Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[s32]index(svbool_t pg, const void *base, svint32_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[s64]index(svbool_t pg, const void *base, svint64_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[u32]index(svbool_t pg, const void *base, svuint32_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFH op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[u64]index(svbool_t pg, const void *base, svuint64_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[s32]index(svbool_t pg, const void *base, svint32_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[s64]index(svbool_t pg, const void *base, svint64_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[u32]index(svbool_t pg, const void *base, svuint32_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFH op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_16_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh_gather_[u64]index(svbool_t pg, const void *base, svuint64_t indices, enum svprfop op)  PRFH op, Pg, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[s32]index(svbool_t pg, const void *base, svint32_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[s64]index(svbool_t pg, const void *base, svint64_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[u32]index(svbool_t pg, const void *base, svuint32_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFW op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[u64]index(svbool_t pg, const void *base, svuint64_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[s32]index(svbool_t pg, const void *base, svint32_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[s64]index(svbool_t pg, const void *base, svint64_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[u32]index(svbool_t pg, const void *base, svuint32_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFW op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_32_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw_gather_[u64]index(svbool_t pg, const void *base, svuint64_t indices, enum svprfop op)  PRFW op, Pg, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[s32]index(svbool_t pg, const void *base, svint32_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.S, SXTW #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[s64]index(svbool_t pg, const void *base, svint64_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[u32]index(svbool_t pg, const void *base, svuint32_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.S, UXTW #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFD op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[u64]index(svbool_t pg, const void *base, svuint64_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[s32]index(svbool_t pg, const void *base, svint32_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.S, SXTW #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[s64]index(svbool_t pg, const void *base, svint64_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[u32]index(svbool_t pg, const void *base, svuint32_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.S, UXTW #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFD op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_64_bit(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd_gather_[u64]index(svbool_t pg, const void *base, svuint64_t indices, enum svprfop op)  PRFD op, Pg, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[s32]offset(svbool_t pg, const void *base, svint32_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[s64]offset(svbool_t pg, const void *base, svint64_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[u32]offset(svbool_t pg, const void *base, svuint32_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFB op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[u64]offset(svbool_t pg, const void *base, svuint64_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[s32]offset(svbool_t pg, const void *base, svint32_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[s64]offset(svbool_t pg, const void *base, svint64_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[u32]offset(svbool_t pg, const void *base, svuint32_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather[_u64base](svbool_t pg, svuint64_t bases, enum svprfop op)  PRFB op, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_prefetch_8_bit(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb_gather_[u64]offset(svbool_t pg, const void *base, svuint64_t offsets, enum svprfop op)  PRFB op, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svld1_gather_[s64]index[_f64](svbool_t pg, const float64_t *base, svint64_t indices)  LD1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[float], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svld1_gather[_u64base]_f64(svbool_t pg, svuint64_t bases)  LD1D Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svld1_gather_[u64]index[_f64](svbool_t pg, const float64_t *base, svuint64_t indices)  LD1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1_gather_[s32]index[_s32](svbool_t pg, const int32_t *base, svint32_t indices)  LD1W Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1_gather_[u32]index[_s32](svbool_t pg, const int32_t *base, svuint32_t indices)  LD1W Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1_gather_[s64]index[_s64](svbool_t pg, const int64_t *base, svint64_t indices)  LD1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LD1D Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1_gather_[u64]index[_s64](svbool_t pg, const int64_t *base, svuint64_t indices)  LD1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svld1_gather_[s32]index[_f32](svbool_t pg, const float32_t *base, svint32_t indices)  LD1W Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svld1_gather_[u32]index[_f32](svbool_t pg, const float32_t *base, svuint32_t indices)  LD1W Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1_gather_[s32]index[_u32](svbool_t pg, const uint32_t *base, svint32_t indices)  LD1W Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1_gather_[u32]index[_u32](svbool_t pg, const uint32_t *base, svuint32_t indices)  LD1W Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1_gather_[s64]index[_u64](svbool_t pg, const uint64_t *base, svint64_t indices)  LD1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LD1D Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1_gather_[u64]index[_u64](svbool_t pg, const uint64_t *base, svuint64_t indices)  LD1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1ub_gather_[s32]offset_s32(svbool_t pg, const uint8_t *base, svint32_t offsets)  LD1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1ub_gather_[u32]offset_s32(svbool_t pg, const uint8_t *base, svuint32_t offsets)  LD1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1ub_gather_[s64]offset_s64(svbool_t pg, const uint8_t *base, svint64_t offsets)  LD1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1ub_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LD1B Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1ub_gather_[u64]offset_s64(svbool_t pg, const uint8_t *base, svuint64_t offsets)  LD1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1ub_gather_[s32]offset_u32(svbool_t pg, const uint8_t *base, svint32_t offsets)  LD1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1ub_gather_[u32]offset_u32(svbool_t pg, const uint8_t *base, svuint32_t offsets)  LD1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1ub_gather_[s64]offset_u64(svbool_t pg, const uint8_t *base, svint64_t offsets)  LD1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1ub_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LD1B Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1ub_gather_[u64]offset_u64(svbool_t pg, const uint8_t *base, svuint64_t offsets)  LD1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1ub_gather_[s32]offset_s32(svbool_t pg, const uint8_t *base, svint32_t offsets)  LDFF1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1ub_gather_[u32]offset_s32(svbool_t pg, const uint8_t *base, svuint32_t offsets)  LDFF1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1ub_gather_[s64]offset_s64(svbool_t pg, const uint8_t *base, svint64_t offsets)  LDFF1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1ub_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LDFF1B Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1ub_gather_[u64]offset_s64(svbool_t pg, const uint8_t *base, svuint64_t offsets)  LDFF1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1ub_gather_[s32]offset_u32(svbool_t pg, const uint8_t *base, svint32_t offsets)  LDFF1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1ub_gather_[u32]offset_u32(svbool_t pg, const uint8_t *base, svuint32_t offsets)  LDFF1B Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1ub_gather_[s64]offset_u64(svbool_t pg, const uint8_t *base, svint64_t offsets)  LDFF1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1ub_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LDFF1B Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1ub_gather_[u64]offset_u64(svbool_t pg, const uint8_t *base, svuint64_t offsets)  LDFF1B Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svldff1_gather_[s64]index[_f64](svbool_t pg, const float64_t *base, svint64_t indices)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[float], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svldff1_gather[_u64base]_f64(svbool_t pg, svuint64_t bases)  LDFF1D Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svldff1_gather_[u64]index[_f64](svbool_t pg, const float64_t *base, svuint64_t indices)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1_gather_[s32]index[_s32](svbool_t pg, const int32_t *base, svint32_t indices)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1_gather_[u32]index[_s32](svbool_t pg, const int32_t *base, svuint32_t indices)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LDFF1D Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1_gather_[s64]index[_s64](svbool_t pg, const int64_t *base, svint64_t indices)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1_gather_[u64]index[_s64](svbool_t pg, const int64_t *base, svuint64_t indices)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svldff1_gather_[s32]index[_f32](svbool_t pg, const float32_t *base, svint32_t indices)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svldff1_gather_[u32]index[_f32](svbool_t pg, const float32_t *base, svuint32_t indices)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1_gather_[s32]index[_u32](svbool_t pg, const uint32_t *base, svint32_t indices)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1_gather_[u32]index[_u32](svbool_t pg, const uint32_t *base, svuint32_t indices)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LDFF1D Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1_gather_[s64]index[_u64](svbool_t pg, const uint64_t *base, svint64_t indices)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1_gather_[u64]index[_u64](svbool_t pg, const uint64_t *base, svuint64_t indices)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #3]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1sh_gather_[s32]index_s32(svbool_t pg, const int16_t *base, svint32_t indices)  LD1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1sh_gather_[u32]index_s32(svbool_t pg, const int16_t *base, svuint32_t indices)  LD1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sh_gather_[s64]index_s64(svbool_t pg, const int16_t *base, svint64_t indices)  LD1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sh_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LD1SH Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sh_gather_[u64]index_s64(svbool_t pg, const int16_t *base, svuint64_t indices)  LD1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sh_gather_[s32]index_u32(svbool_t pg, const int16_t *base, svint32_t indices)  LD1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sh_gather_[u32]index_u32(svbool_t pg, const int16_t *base, svuint32_t indices)  LD1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sh_gather_[s64]index_u64(svbool_t pg, const int16_t *base, svint64_t indices)  LD1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sh_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LD1SH Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sh_gather_[u64]index_u64(svbool_t pg, const int16_t *base, svuint64_t indices)  LD1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sh_gather_[s32]index_s32(svbool_t pg, const int16_t *base, svint32_t indices)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sh_gather_[u32]index_s32(svbool_t pg, const int16_t *base, svuint32_t indices)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sh_gather_[s64]index_s64(svbool_t pg, const int16_t *base, svint64_t indices)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sh_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LDFF1SH Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sh_gather_[u64]index_s64(svbool_t pg, const int16_t *base, svuint64_t indices)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sh_gather_[s32]index_u32(svbool_t pg, const int16_t *base, svint32_t indices)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sh_gather_[u32]index_u32(svbool_t pg, const int16_t *base, svuint32_t indices)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sh_gather_[s64]index_u64(svbool_t pg, const int16_t *base, svint64_t indices)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sh_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LDFF1SH Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sh_gather_[u64]index_u64(svbool_t pg, const int16_t *base, svuint64_t indices)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1sh_gather_[s32]offset_s32(svbool_t pg, const int16_t *base, svint32_t offsets)  LD1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1sh_gather_[u32]offset_s32(svbool_t pg, const int16_t *base, svuint32_t offsets)  LD1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sh_gather_[s64]offset_s64(svbool_t pg, const int16_t *base, svint64_t offsets)  LD1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sh_gather_[u64]offset_s64(svbool_t pg, const int16_t *base, svuint64_t offsets)  LD1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sh_gather_[s32]offset_u32(svbool_t pg, const int16_t *base, svint32_t offsets)  LD1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sh_gather_[u32]offset_u32(svbool_t pg, const int16_t *base, svuint32_t offsets)  LD1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sh_gather_[s64]offset_u64(svbool_t pg, const int16_t *base, svint64_t offsets)  LD1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sh_gather_[u64]offset_u64(svbool_t pg, const int16_t *base, svuint64_t offsets)  LD1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sh_gather_[s32]offset_s32(svbool_t pg, const int16_t *base, svint32_t offsets)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sh_gather_[u32]offset_s32(svbool_t pg, const int16_t *base, svuint32_t offsets)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sh_gather_[s64]offset_s64(svbool_t pg, const int16_t *base, svint64_t offsets)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sh_gather_[u64]offset_s64(svbool_t pg, const int16_t *base, svuint64_t offsets)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sh_gather_[s32]offset_u32(svbool_t pg, const int16_t *base, svint32_t offsets)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sh_gather_[u32]offset_u32(svbool_t pg, const int16_t *base, svuint32_t offsets)  LDFF1SH Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sh_gather_[s64]offset_u64(svbool_t pg, const int16_t *base, svint64_t offsets)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_16_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sh_gather_[u64]offset_u64(svbool_t pg, const int16_t *base, svuint64_t offsets)  LDFF1SH Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sw_gather_[s64]index_s64(svbool_t pg, const int32_t *base, svint64_t indices)  LD1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sw_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LD1SW Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sw_gather_[u64]index_s64(svbool_t pg, const int32_t *base, svuint64_t indices)  LD1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sw_gather_[s64]index_u64(svbool_t pg, const int32_t *base, svint64_t indices)  LD1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sw_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LD1SW Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sw_gather_[u64]index_u64(svbool_t pg, const int32_t *base, svuint64_t indices)  LD1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sw_gather_[s64]index_s64(svbool_t pg, const int32_t *base, svint64_t indices)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sw_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LDFF1SW Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sw_gather_[u64]index_s64(svbool_t pg, const int32_t *base, svuint64_t indices)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sw_gather_[s64]index_u64(svbool_t pg, const int32_t *base, svint64_t indices)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sw_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LDFF1SW Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sw_gather_[u64]index_u64(svbool_t pg, const int32_t *base, svuint64_t indices)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sw_gather_[s64]offset_s64(svbool_t pg, const int32_t *base, svint64_t offsets)  LD1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sw_gather_[u64]offset_s64(svbool_t pg, const int32_t *base, svuint64_t offsets)  LD1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sw_gather_[s64]offset_u64(svbool_t pg, const int32_t *base, svint64_t offsets)  LD1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sw_gather_[u64]offset_u64(svbool_t pg, const int32_t *base, svuint64_t offsets)  LD1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sw_gather_[s64]offset_s64(svbool_t pg, const int32_t *base, svint64_t offsets)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sw_gather_[u64]offset_s64(svbool_t pg, const int32_t *base, svuint64_t offsets)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sw_gather_[s64]offset_u64(svbool_t pg, const int32_t *base, svint64_t offsets)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_int_32_with_byte_offsets_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sw_gather_[u64]offset_u64(svbool_t pg, const int32_t *base, svuint64_t offsets)  LDFF1SW Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1sb_gather_[s32]offset_s32(svbool_t pg, const int8_t *base, svint32_t offsets)  LD1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1sb_gather_[u32]offset_s32(svbool_t pg, const int8_t *base, svuint32_t offsets)  LD1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sb_gather_[s64]offset_s64(svbool_t pg, const int8_t *base, svint64_t offsets)  LD1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sb_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LD1SB Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1sb_gather_[u64]offset_s64(svbool_t pg, const int8_t *base, svuint64_t offsets)  LD1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sb_gather_[s32]offset_u32(svbool_t pg, const int8_t *base, svint32_t offsets)  LD1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sb_gather_[u32]offset_u32(svbool_t pg, const int8_t *base, svuint32_t offsets)  LD1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sb_gather_[s64]offset_u64(svbool_t pg, const int8_t *base, svint64_t offsets)  LD1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sb_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LD1SB Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sb_gather_[u64]offset_u64(svbool_t pg, const int8_t *base, svuint64_t offsets)  LD1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sb_gather_[s32]offset_s32(svbool_t pg, const int8_t *base, svint32_t offsets)  LDFF1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sb_gather_[u32]offset_s32(svbool_t pg, const int8_t *base, svuint32_t offsets)  LDFF1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sb_gather_[s64]offset_s64(svbool_t pg, const int8_t *base, svint64_t offsets)  LDFF1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sb_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LDFF1SB Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sb_gather_[u64]offset_s64(svbool_t pg, const int8_t *base, svuint64_t offsets)  LDFF1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sb_gather_[s32]offset_u32(svbool_t pg, const int8_t *base, svint32_t offsets)  LDFF1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sb_gather_[u32]offset_u32(svbool_t pg, const int8_t *base, svuint32_t offsets)  LDFF1SB Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sb_gather_[s64]offset_u64(svbool_t pg, const int8_t *base, svint64_t offsets)  LDFF1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sb_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LDFF1SB Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sb_gather_[u64]offset_u64(svbool_t pg, const int8_t *base, svuint64_t offsets)  LDFF1SB Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1uh_gather_[s32]offset_s32(svbool_t pg, const uint16_t *base, svint32_t offsets)  LD1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1uh_gather_[u32]offset_s32(svbool_t pg, const uint16_t *base, svuint32_t offsets)  LD1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uh_gather_[s64]offset_s64(svbool_t pg, const uint16_t *base, svint64_t offsets)  LD1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uh_gather_[u64]offset_s64(svbool_t pg, const uint16_t *base, svuint64_t offsets)  LD1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1uh_gather_[s32]offset_u32(svbool_t pg, const uint16_t *base, svint32_t offsets)  LD1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1uh_gather_[u32]offset_u32(svbool_t pg, const uint16_t *base, svuint32_t offsets)  LD1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uh_gather_[s64]offset_u64(svbool_t pg, const uint16_t *base, svint64_t offsets)  LD1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uh_gather_[u64]offset_u64(svbool_t pg, const uint16_t *base, svuint64_t offsets)  LD1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1uh_gather_[s32]offset_s32(svbool_t pg, const uint16_t *base, svint32_t offsets)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1uh_gather_[u32]offset_s32(svbool_t pg, const uint16_t *base, svuint32_t offsets)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uh_gather_[s64]offset_s64(svbool_t pg, const uint16_t *base, svint64_t offsets)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uh_gather_[u64]offset_s64(svbool_t pg, const uint16_t *base, svuint64_t offsets)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1uh_gather_[s32]offset_u32(svbool_t pg, const uint16_t *base, svint32_t offsets)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1uh_gather_[u32]offset_u32(svbool_t pg, const uint16_t *base, svuint32_t offsets)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uh_gather_[s64]offset_u64(svbool_t pg, const uint16_t *base, svint64_t offsets)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uh_gather_[u64]offset_u64(svbool_t pg, const uint16_t *base, svuint64_t offsets)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1uh_gather_[s32]index_s32(svbool_t pg, const uint16_t *base, svint32_t indices)  LD1H Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1uh_gather_[u32]index_s32(svbool_t pg, const uint16_t *base, svuint32_t indices)  LD1H Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uh_gather_[s64]index_s64(svbool_t pg, const uint16_t *base, svint64_t indices)  LD1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uh_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LD1H Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uh_gather_[u64]index_s64(svbool_t pg, const uint16_t *base, svuint64_t indices)  LD1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1uh_gather_[s32]index_u32(svbool_t pg, const uint16_t *base, svint32_t indices)  LD1H Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1uh_gather_[u32]index_u32(svbool_t pg, const uint16_t *base, svuint32_t indices)  LD1H Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uh_gather_[s64]index_u64(svbool_t pg, const uint16_t *base, svint64_t indices)  LD1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uh_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LD1H Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uh_gather_[u64]index_u64(svbool_t pg, const uint16_t *base, svuint64_t indices)  LD1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1uh_gather_[s32]index_s32(svbool_t pg, const uint16_t *base, svint32_t indices)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1uh_gather_[u32]index_s32(svbool_t pg, const uint16_t *base, svuint32_t indices)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uh_gather_[s64]index_s64(svbool_t pg, const uint16_t *base, svint64_t indices)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uh_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LDFF1H Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uh_gather_[u64]index_s64(svbool_t pg, const uint16_t *base, svuint64_t indices)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1uh_gather_[s32]index_u32(svbool_t pg, const uint16_t *base, svint32_t indices)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zindices.S, SXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1uh_gather_[u32]index_u32(svbool_t pg, const uint16_t *base, svuint32_t indices)  LDFF1H Zresult.S, Pg/Z, [Xbase, Zindices.S, UXTW #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uh_gather_[s64]index_u64(svbool_t pg, const uint16_t *base, svint64_t indices)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uh_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LDFF1H Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uh_gather_[u64]index_u64(svbool_t pg, const uint16_t *base, svuint64_t indices)  LDFF1H Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #1]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[s64]offset_s64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[u64]offset_s64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[s64]offset_s64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[u64]offset_s64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[s64]offset_u64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[u64]offset_u64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[s64]offset_u64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[u64]offset_u64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LD1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[s64]offset_s64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[u64]offset_s64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[s64]offset_s64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[u64]offset_s64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[s64]offset_u64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[u64]offset_u64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[s64]offset_u64(svbool_t pg, const uint32_t *base, svint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_with_byte_offsets_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[u64]offset_u64(svbool_t pg, const uint32_t *base, svuint64_t offsets)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[s64]index_s64(svbool_t pg, const uint32_t *base, svint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[u64]index_s64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[s64]index_s64(svbool_t pg, const uint32_t *base, svint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LD1W Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_gather_[u64]index_s64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[s64]index_u64(svbool_t pg, const uint32_t *base, svint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[u64]index_u64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[s64]index_u64(svbool_t pg, const uint32_t *base, svint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LD1W Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_gather_[u64]index_u64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LD1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[s64]index_s64(svbool_t pg, const uint32_t *base, svint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[u64]index_s64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[s64]index_s64(svbool_t pg, const uint32_t *base, svint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather[_u64base]_s64(svbool_t pg, svuint64_t bases)  LDFF1W Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_gather_[u64]index_s64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[s64]index_u64(svbool_t pg, const uint32_t *base, svint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[u64]index_u64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[s64]index_u64(svbool_t pg, const uint32_t *base, svint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather[_u64base]_u64(svbool_t pg, svuint64_t bases)  LDFF1W Zresult.D, Pg/Z, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def gather_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_gather_[u64]index_u64(svbool_t pg, const uint32_t *base, svuint64_t indices)  LDFF1W Zresult.D, Pg/Z, [Xbase, Zindices.D, LSL #2]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svldff1_gather_[s64]offset[_f64](svbool_t pg, const float64_t *base, svint64_t offsets)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svldff1_gather_[u64]offset[_f64](svbool_t pg, const float64_t *base, svuint64_t offsets)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1_gather_[s32]offset[_s32](svbool_t pg, const int32_t *base, svint32_t offsets)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svldff1_gather_[u32]offset[_s32](svbool_t pg, const int32_t *base, svuint32_t offsets)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1_gather_[s64]offset[_s64](svbool_t pg, const int64_t *base, svint64_t offsets)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svldff1_gather_[u64]offset[_s64](svbool_t pg, const int64_t *base, svuint64_t offsets)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svldff1_gather_[s32]offset[_f32](svbool_t pg, const float32_t *base, svint32_t offsets)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svldff1_gather_[u32]offset[_f32](svbool_t pg, const float32_t *base, svuint32_t offsets)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1_gather_[s32]offset[_u32](svbool_t pg, const uint32_t *base, svint32_t offsets)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1_gather_[u32]offset[_u32](svbool_t pg, const uint32_t *base, svuint32_t offsets)  LDFF1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1_gather_[s64]offset[_u64](svbool_t pg, const uint64_t *base, svint64_t offsets)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offset_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1_gather_[u64]offset[_u64](svbool_t pg, const uint64_t *base, svuint64_t offsets)  LDFF1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svld1_gather_[s64]offset[_f64](svbool_t pg, const float64_t *base, svint64_t offsets)  LD1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svld1_gather_[u64]offset[_f64](svbool_t pg, const float64_t *base, svuint64_t offsets)  LD1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1_gather_[s32]offset[_s32](svbool_t pg, const int32_t *base, svint32_t offsets)  LD1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svld1_gather_[u32]offset[_s32](svbool_t pg, const int32_t *base, svuint32_t offsets)  LD1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1_gather_[s64]offset[_s64](svbool_t pg, const int64_t *base, svint64_t offsets)  LD1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svld1_gather_[u64]offset[_s64](svbool_t pg, const int64_t *base, svuint64_t offsets)  LD1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svld1_gather_[s32]offset[_f32](svbool_t pg, const float32_t *base, svint32_t offsets)  LD1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[float], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svld1_gather_[u32]offset[_f32](svbool_t pg, const float32_t *base, svuint32_t offsets)  LD1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1_gather_[s32]offset[_u32](svbool_t pg, const uint32_t *base, svint32_t offsets)  LD1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svld1_gather_[u32]offset[_u32](svbool_t pg, const uint32_t *base, svuint32_t offsets)  LD1W Zresult.S, Pg/Z, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1_gather_[s64]offset[_u64](svbool_t pg, const uint64_t *base, svint64_t offsets)  LD1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def gather_vector_with_byte_offsets(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svld1_gather_[u64]offset[_u64](svbool_t pg, const uint64_t *base, svuint64_t offsets)  LD1D Zresult.D, Pg/Z, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b8(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.B"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[float], _from: System.Numerics.Vector[float]) -> int:
        """uint64_t svcntp_b8(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.B"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b8(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.B"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b8(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.B"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b8(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.B"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b8(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.B"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[float], _from: System.Numerics.Vector[float]) -> int:
        """uint64_t svcntp_b8(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.B"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b16(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.H"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b32(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.S"""
        ...

    @staticmethod
    @overload
    def get_active_element_count(mask: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svcntp_b64(svbool_t pg, svbool_t op)  CNTP Xresult, Pg, Pop.D"""
        ...

    @staticmethod
    def get_ffr_byte() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    def get_ffr_int_16() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    def get_ffr_int_32() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    def get_ffr_int_64() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    def get_ffr_s_byte() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    def get_ffr_u_int_16() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    def get_ffr_u_int_32() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    def get_ffr_u_int_64() -> System.Numerics.Vector[int]:
        """svbool_t svrdffr()  RDFFR Presult.B"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svuint8_t svinsr[_n_u8](svuint8_t op1, uint8_t op2)  INSR Ztied1.B, Wop2  INSR Ztied1.B, Bop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[float], right: float) -> System.Numerics.Vector[float]:
        """svfloat64_t svinsr[_n_f64](svfloat64_t op1, float64_t op2)  INSR Ztied1.D, Xop2  INSR Ztied1.D, Dop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svint16_t svinsr[_n_s16](svint16_t op1, int16_t op2)  INSR Ztied1.H, Wop2  INSR Ztied1.H, Hop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svint32_t svinsr[_n_s32](svint32_t op1, int32_t op2)  INSR Ztied1.S, Wop2  INSR Ztied1.S, Sop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svint64_t svinsr[_n_s64](svint64_t op1, int64_t op2)  INSR Ztied1.D, Xop2  INSR Ztied1.D, Dop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svint8_t svinsr[_n_s8](svint8_t op1, int8_t op2)  INSR Ztied1.B, Wop2  INSR Ztied1.B, Bop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[float], right: float) -> System.Numerics.Vector[float]:
        """svfloat32_t svinsr[_n_f32](svfloat32_t op1, float32_t op2)  INSR Ztied1.S, Wop2  INSR Ztied1.S, Sop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svuint16_t svinsr[_n_u16](svuint16_t op1, uint16_t op2)  INSR Ztied1.H, Wop2  INSR Ztied1.H, Hop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svuint32_t svinsr[_n_u32](svuint32_t op1, uint32_t op2)  INSR Ztied1.S, Wop2  INSR Ztied1.S, Sop2"""
        ...

    @staticmethod
    @overload
    def insert_into_shifted_vector(left: System.Numerics.Vector[int], right: int) -> System.Numerics.Vector[int]:
        """svuint64_t svinsr[_n_u64](svuint64_t op1, uint64_t op2)  INSR Ztied1.D, Xop2  INSR Ztied1.D, Dop2"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svcls[_s8]_m(svuint8_t inactive, svbool_t pg, svint8_t op)svuint8_t svcls[_s8]_x(svbool_t pg, svint8_t op)svuint8_t svcls[_s8]_z(svbool_t pg, svint8_t op)  CLS Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svcls[_s16]_m(svuint16_t inactive, svbool_t pg, svint16_t op)svuint16_t svcls[_s16]_x(svbool_t pg, svint16_t op)svuint16_t svcls[_s16]_z(svbool_t pg, svint16_t op)  CLS Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svcls[_s32]_m(svuint32_t inactive, svbool_t pg, svint32_t op)svuint32_t svcls[_s32]_x(svbool_t pg, svint32_t op)svuint32_t svcls[_s32]_z(svbool_t pg, svint32_t op)  CLS Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def leading_sign_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svcls[_s64]_m(svuint64_t inactive, svbool_t pg, svint64_t op)svuint64_t svcls[_s64]_x(svbool_t pg, svint64_t op)svuint64_t svcls[_s64]_z(svbool_t pg, svint64_t op)  CLS Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svclz[_s8]_m(svuint8_t inactive, svbool_t pg, svint8_t op)svuint8_t svclz[_s8]_x(svbool_t pg, svint8_t op)svuint8_t svclz[_s8]_z(svbool_t pg, svint8_t op)  CLZ Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svclz[_u8]_m(svuint8_t inactive, svbool_t pg, svuint8_t op)svuint8_t svclz[_u8]_x(svbool_t pg, svuint8_t op)svuint8_t svclz[_u8]_z(svbool_t pg, svuint8_t op)  CLZ Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svclz[_s16]_m(svuint16_t inactive, svbool_t pg, svint16_t op)svuint16_t svclz[_s16]_x(svbool_t pg, svint16_t op)svuint16_t svclz[_s16]_z(svbool_t pg, svint16_t op)  CLZ Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svclz[_u16]_m(svuint16_t inactive, svbool_t pg, svuint16_t op)svuint16_t svclz[_u16]_x(svbool_t pg, svuint16_t op)svuint16_t svclz[_u16]_z(svbool_t pg, svuint16_t op)  CLZ Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svclz[_s32]_m(svuint32_t inactive, svbool_t pg, svint32_t op)svuint32_t svclz[_s32]_x(svbool_t pg, svint32_t op)svuint32_t svclz[_s32]_z(svbool_t pg, svint32_t op)  CLZ Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svclz[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svclz[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svclz[_u32]_z(svbool_t pg, svuint32_t op)  CLZ Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svclz[_s64]_m(svuint64_t inactive, svbool_t pg, svint64_t op)svuint64_t svclz[_s64]_x(svbool_t pg, svint64_t op)svuint64_t svclz[_s64]_z(svbool_t pg, svint64_t op)  CLZ Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def leading_zero_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svclz[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svclz[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svclz[_u64]_z(svbool_t pg, svuint64_t op)  CLZ Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint8x2_t svld2[_u8](svbool_t pg, const uint8_t *base)  LD2B {Zresult0.B, Zresult1.B}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[float], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float]]:
        """svfloat64x2_t svld2[_f64](svbool_t pg, const float64_t *base)  LD2D {Zresult0.D, Zresult1.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint16x2_t svld2[_s16](svbool_t pg, const int16_t *base)  LD2H {Zresult0.H, Zresult1.H}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint32x2_t svld2[_s32](svbool_t pg, const int32_t *base)  LD2W {Zresult0.S, Zresult1.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint64x2_t svld2[_s64](svbool_t pg, const int64_t *base)  LD2D {Zresult0.D, Zresult1.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint8x2_t svld2[_s8](svbool_t pg, const int8_t *base)  LD2B {Zresult0.B, Zresult1.B}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[float], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float]]:
        """svfloat32x2_t svld2[_f32](svbool_t pg, const float32_t *base)  LD2W {Zresult0.S, Zresult1.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint16x2_t svld2[_u16](svbool_t pg, const uint16_t *base)  LD2H {Zresult0.H, Zresult1.H}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint32x2_t svld2[_u32](svbool_t pg, const uint32_t *base)  LD2W {Zresult0.S, Zresult1.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_2_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint64x2_t svld2[_u64](svbool_t pg, const uint64_t *base)  LD2D {Zresult0.D, Zresult1.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint8x3_t svld3[_u8](svbool_t pg, const uint8_t *base)  LD3B {Zresult0.B - Zresult2.B}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[float], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]:
        """svfloat64x3_t svld3[_f64](svbool_t pg, const float64_t *base)  LD3D {Zresult0.D - Zresult2.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint16x3_t svld3[_s16](svbool_t pg, const int16_t *base)  LD3H {Zresult0.H - Zresult2.H}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint32x3_t svld3[_s32](svbool_t pg, const int32_t *base)  LD3W {Zresult0.S - Zresult2.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint64x3_t svld3[_s64](svbool_t pg, const int64_t *base)  LD3D {Zresult0.D - Zresult2.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint8x3_t svld3[_s8](svbool_t pg, const int8_t *base)  LD3B {Zresult0.B - Zresult2.B}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[float], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]:
        """svfloat32x3_t svld3[_f32](svbool_t pg, const float32_t *base)  LD3W {Zresult0.S - Zresult2.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint16x3_t svld3[_u16](svbool_t pg, const uint16_t *base)  LD3H {Zresult0.H - Zresult2.H}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint32x3_t svld3[_u32](svbool_t pg, const uint32_t *base)  LD3W {Zresult0.S - Zresult2.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_3_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint64x3_t svld3[_u64](svbool_t pg, const uint64_t *base)  LD3D {Zresult0.D - Zresult2.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint8x4_t svld4[_u8](svbool_t pg, const uint8_t *base)  LD4B {Zresult0.B - Zresult3.B}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[float], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]:
        """svfloat64x4_t svld4[_f64](svbool_t pg, const float64_t *base)  LD4D {Zresult0.D - Zresult3.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint16x4_t svld4[_s16](svbool_t pg, const int16_t *base)  LD4H {Zresult0.H - Zresult3.H}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint32x4_t svld4[_s32](svbool_t pg, const int32_t *base)  LD4W {Zresult0.S - Zresult3.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint64x4_t svld4[_s64](svbool_t pg, const int64_t *base)  LD4D {Zresult0.D - Zresult3.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svint8x4_t svld4[_s8](svbool_t pg, const int8_t *base)  LD4B {Zresult0.B - Zresult3.B}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[float], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]:
        """svfloat32x4_t svld4[_f32](svbool_t pg, const float32_t *base)  LD4W {Zresult0.S - Zresult3.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint16x4_t svld4[_u16](svbool_t pg, const uint16_t *base)  LD4H {Zresult0.H - Zresult3.H}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint32x4_t svld4[_u32](svbool_t pg, const uint32_t *base)  LD4W {Zresult0.S - Zresult3.S}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_4_x_vector_and_unzip(mask: System.Numerics.Vector[int], address: typing.Any) -> System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]:
        """svuint64x4_t svld4[_u64](svbool_t pg, const uint64_t *base)  LD4D {Zresult0.D - Zresult3.D}, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint8_t svld1[_u8](svbool_t pg, const uint8_t *base)  LD1B Zresult.B, Pg/Z, [Xarray, Xindex]  LD1B Zresult.B, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat64_t svld1[_f64](svbool_t pg, const float64_t *base)  LD1D Zresult.D, Pg/Z, [Xarray, Xindex, LSL #3]  LD1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svld1[_s16](svbool_t pg, const int16_t *base)  LD1H Zresult.H, Pg/Z, [Xarray, Xindex, LSL #1]  LD1H Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svld1[_s32](svbool_t pg, const int32_t *base)  LD1W Zresult.S, Pg/Z, [Xarray, Xindex, LSL #2]  LD1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1[_s64](svbool_t pg, const int64_t *base)  LD1D Zresult.D, Pg/Z, [Xarray, Xindex, LSL #3]  LD1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint8_t svld1[_s8](svbool_t pg, const int8_t *base)  LD1B Zresult.B, Pg/Z, [Xarray, Xindex]  LD1B Zresult.B, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat32_t svld1[_f32](svbool_t pg, const float32_t *base)  LD1W Zresult.S, Pg/Z, [Xarray, Xindex, LSL #2]  LD1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svld1[_u16](svbool_t pg, const uint16_t *base)  LD1H Zresult.H, Pg/Z, [Xarray, Xindex, LSL #1]  LD1H Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svld1[_u32](svbool_t pg, const uint32_t *base)  LD1W Zresult.S, Pg/Z, [Xarray, Xindex, LSL #2]  LD1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1[_u64](svbool_t pg, const uint64_t *base)  LD1D Zresult.D, Pg/Z, [Xarray, Xindex, LSL #3]  LD1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint8_t svld1rq[_u8](svbool_t pg, const uint8_t *base)  LD1RQB Zresult.B, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat64_t svld1rq[_f64](svbool_t pg, const float64_t *base)  LD1RQD Zresult.D, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svld1rq[_s16](svbool_t pg, const int16_t *base)  LD1RQH Zresult.H, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svld1rq[_s32](svbool_t pg, const int32_t *base)  LD1RQW Zresult.S, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1rq[_s64](svbool_t pg, const int64_t *base)  LD1RQD Zresult.D, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint8_t svld1rq[_s8](svbool_t pg, const int8_t *base)  LD1RQB Zresult.B, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat32_t svld1rq[_f32](svbool_t pg, const float32_t *base)  LD1RQW Zresult.S, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svld1rq[_u16](svbool_t pg, const uint16_t *base)  LD1RQH Zresult.H, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svld1rq[_u32](svbool_t pg, const uint32_t *base)  LD1RQW Zresult.S, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    @overload
    def load_vector_128_and_replicate_to_vector(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1rq[_u64](svbool_t pg, const uint64_t *base)  LD1RQD Zresult.D, Pg/Z, [Xbase, #0]"""
        ...

    @staticmethod
    def load_vector_byte_non_faulting_zero_extend_to_int_16(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svldnf1ub_s16(svbool_t pg, const uint8_t *base)  LDNF1B Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_non_faulting_zero_extend_to_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldnf1ub_s32(svbool_t pg, const uint8_t *base)  LDNF1B Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_non_faulting_zero_extend_to_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnf1ub_s64(svbool_t pg, const uint8_t *base)  LDNF1B Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_non_faulting_zero_extend_to_u_int_16(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svldnf1ub_u16(svbool_t pg, const uint8_t *base)  LDNF1B Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_non_faulting_zero_extend_to_u_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldnf1ub_u32(svbool_t pg, const uint8_t *base)  LDNF1B Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_non_faulting_zero_extend_to_u_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnf1ub_u64(svbool_t pg, const uint8_t *base)  LDNF1B Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svldff1ub_s16(svbool_t pg, const uint8_t *base)  LDFF1B Zresult.H, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldff1ub_s32(svbool_t pg, const uint8_t *base)  LDFF1B Zresult.S, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldff1ub_s64(svbool_t pg, const uint8_t *base)  LDFF1B Zresult.D, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svldff1ub_u16(svbool_t pg, const uint8_t *base)  LDFF1B Zresult.H, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1ub_u32(svbool_t pg, const uint8_t *base)  LDFF1B Zresult.S, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_byte_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1ub_u64(svbool_t pg, const uint8_t *base)  LDFF1B Zresult.D, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    def load_vector_byte_zero_extend_to_int_16(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svld1ub_s16(svbool_t pg, const uint8_t *base)  LD1B Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_zero_extend_to_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svld1ub_s32(svbool_t pg, const uint8_t *base)  LD1B Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_zero_extend_to_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1ub_s64(svbool_t pg, const uint8_t *base)  LD1B Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_zero_extend_to_u_int_16(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svld1ub_u16(svbool_t pg, const uint8_t *base)  LD1B Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_zero_extend_to_u_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svld1ub_u32(svbool_t pg, const uint8_t *base)  LD1B Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_byte_zero_extend_to_u_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1ub_u64(svbool_t pg, const uint8_t *base)  LD1B Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint8_t svldff1[_u8](svbool_t pg, const uint8_t *base)  LDFF1B Zresult.B, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat64_t svldff1[_f64](svbool_t pg, const float64_t *base)  LDFF1D Zresult.D, Pg/Z, [Xbase, XZR, LSL #3]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svldff1[_s16](svbool_t pg, const int16_t *base)  LDFF1H Zresult.H, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldff1[_s32](svbool_t pg, const int32_t *base)  LDFF1W Zresult.S, Pg/Z, [Xbase, XZR, LSL #2]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldff1[_s64](svbool_t pg, const int64_t *base)  LDFF1D Zresult.D, Pg/Z, [Xbase, XZR, LSL #3]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint8_t svldff1[_s8](svbool_t pg, const int8_t *base)  LDFF1B Zresult.B, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat32_t svldff1[_f32](svbool_t pg, const float32_t *base)  LDFF1W Zresult.S, Pg/Z, [Xbase, XZR, LSL #2]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svldff1[_u16](svbool_t pg, const uint16_t *base)  LDFF1H Zresult.H, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1[_u32](svbool_t pg, const uint32_t *base)  LDFF1W Zresult.S, Pg/Z, [Xbase, XZR, LSL #2]"""
        ...

    @staticmethod
    @overload
    def load_vector_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1[_u64](svbool_t pg, const uint64_t *base)  LDFF1D Zresult.D, Pg/Z, [Xbase, XZR, LSL #3]"""
        ...

    @staticmethod
    def load_vector_int_16_non_faulting_sign_extend_to_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldnf1sh_s32(svbool_t pg, const int16_t *base)  LDNF1SH Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_16_non_faulting_sign_extend_to_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnf1sh_s64(svbool_t pg, const int16_t *base)  LDNF1SH Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_16_non_faulting_sign_extend_to_u_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldnf1sh_u32(svbool_t pg, const int16_t *base)  LDNF1SH Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_16_non_faulting_sign_extend_to_u_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnf1sh_u64(svbool_t pg, const int16_t *base)  LDNF1SH Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sh_s32(svbool_t pg, const int16_t *base)  LDFF1SH Zresult.S, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sh_s64(svbool_t pg, const int16_t *base)  LDFF1SH Zresult.D, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sh_u32(svbool_t pg, const int16_t *base)  LDFF1SH Zresult.S, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_int_16_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sh_u64(svbool_t pg, const int16_t *base)  LDFF1SH Zresult.D, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    def load_vector_int_16_sign_extend_to_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svld1sh_s32(svbool_t pg, const int16_t *base)  LD1SH Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_16_sign_extend_to_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1sh_s64(svbool_t pg, const int16_t *base)  LD1SH Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_16_sign_extend_to_u_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sh_u32(svbool_t pg, const int16_t *base)  LD1SH Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_16_sign_extend_to_u_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sh_u64(svbool_t pg, const int16_t *base)  LD1SH Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_32_non_faulting_sign_extend_to_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnf1sw_s64(svbool_t pg, const int32_t *base)  LDNF1SW Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_32_non_faulting_sign_extend_to_u_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnf1sw_u64(svbool_t pg, const int32_t *base)  LDNF1SW Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sw_s64(svbool_t pg, const int32_t *base)  LDFF1SW Zresult.D, Pg/Z, [Xbase, XZR, LSL #2]"""
        ...

    @staticmethod
    @overload
    def load_vector_int_32_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sw_u64(svbool_t pg, const int32_t *base)  LDFF1SW Zresult.D, Pg/Z, [Xbase, XZR, LSL #2]"""
        ...

    @staticmethod
    def load_vector_int_32_sign_extend_to_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1sw_s64(svbool_t pg, const int32_t *base)  LD1SW Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_int_32_sign_extend_to_u_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sw_u64(svbool_t pg, const int32_t *base)  LD1SW Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint8_t svldnf1[_u8](svbool_t pg, const uint8_t *base)  LDNF1B Zresult.B, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat64_t svldnf1[_f64](svbool_t pg, const float64_t *base)  LDNF1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svldnf1[_s16](svbool_t pg, const int16_t *base)  LDNF1H Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldnf1[_s32](svbool_t pg, const int32_t *base)  LDNF1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnf1[_s64](svbool_t pg, const int64_t *base)  LDNF1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint8_t svldnf1[_s8](svbool_t pg, const int8_t *base)  LDNF1B Zresult.B, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat32_t svldnf1[_f32](svbool_t pg, const float32_t *base)  LDNF1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svldnf1[_u16](svbool_t pg, const uint16_t *base)  LDNF1H Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldnf1[_u32](svbool_t pg, const uint32_t *base)  LDNF1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_faulting(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnf1[_u64](svbool_t pg, const uint64_t *base)  LDNF1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint8_t svldnt1[_u8](svbool_t pg, const uint8_t *base)  LDNT1B Zresult.B, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat64_t svldnt1[_f64](svbool_t pg, const float64_t *base)  LDNT1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svldnt1[_s16](svbool_t pg, const int16_t *base)  LDNT1H Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldnt1[_s32](svbool_t pg, const int32_t *base)  LDNT1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnt1[_s64](svbool_t pg, const int64_t *base)  LDNT1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint8_t svldnt1[_s8](svbool_t pg, const int8_t *base)  LDNT1B Zresult.B, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[float], address: typing.Any) -> System.Numerics.Vector[float]:
        """svfloat32_t svldnt1[_f32](svbool_t pg, const float32_t *base)  LDNT1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svldnt1[_u16](svbool_t pg, const uint16_t *base)  LDNT1H Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldnt1[_u32](svbool_t pg, const uint32_t *base)  LDNT1W Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnt1[_u64](svbool_t pg, const uint64_t *base)  LDNT1D Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_non_faulting_sign_extend_to_int_16(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svldnf1sb_s16(svbool_t pg, const int8_t *base)  LDNF1SB Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_non_faulting_sign_extend_to_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldnf1sb_s32(svbool_t pg, const int8_t *base)  LDNF1SB Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_non_faulting_sign_extend_to_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnf1sb_s64(svbool_t pg, const int8_t *base)  LDNF1SB Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_non_faulting_sign_extend_to_u_int_16(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svldnf1sb_u16(svbool_t pg, const int8_t *base)  LDNF1SB Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_non_faulting_sign_extend_to_u_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldnf1sb_u32(svbool_t pg, const int8_t *base)  LDNF1SB Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_non_faulting_sign_extend_to_u_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnf1sb_u64(svbool_t pg, const int8_t *base)  LDNF1SB Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svldff1sb_s16(svbool_t pg, const int8_t *base)  LDFF1SB Zresult.H, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldff1sb_s32(svbool_t pg, const int8_t *base)  LDFF1SB Zresult.S, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldff1sb_s64(svbool_t pg, const int8_t *base)  LDFF1SB Zresult.D, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svldff1sb_u16(svbool_t pg, const int8_t *base)  LDFF1SB Zresult.H, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1sb_u32(svbool_t pg, const int8_t *base)  LDFF1SB Zresult.S, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    @overload
    def load_vector_s_byte_sign_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1sb_u64(svbool_t pg, const int8_t *base)  LDFF1SB Zresult.D, Pg/Z, [Xbase, XZR]"""
        ...

    @staticmethod
    def load_vector_s_byte_sign_extend_to_int_16(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint16_t svld1sb_s16(svbool_t pg, const int8_t *base)  LD1SB Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_sign_extend_to_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svld1sb_s32(svbool_t pg, const int8_t *base)  LD1SB Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_sign_extend_to_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1sb_s64(svbool_t pg, const int8_t *base)  LD1SB Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_sign_extend_to_u_int_16(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint16_t svld1sb_u16(svbool_t pg, const int8_t *base)  LD1SB Zresult.H, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_sign_extend_to_u_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svld1sb_u32(svbool_t pg, const int8_t *base)  LD1SB Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_s_byte_sign_extend_to_u_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1sb_u64(svbool_t pg, const int8_t *base)  LD1SB Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_16_non_faulting_zero_extend_to_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldnf1uh_s32(svbool_t pg, const uint16_t *base)  LDNF1H Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_16_non_faulting_zero_extend_to_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnf1uh_s64(svbool_t pg, const uint16_t *base)  LDNF1H Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_16_non_faulting_zero_extend_to_u_int_32(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldnf1uh_u32(svbool_t pg, const uint16_t *base)  LDNF1H Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_16_non_faulting_zero_extend_to_u_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnf1uh_u64(svbool_t pg, const uint16_t *base)  LDNF1H Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svldff1uh_s32(svbool_t pg, const uint16_t *base)  LDFF1H Zresult.S, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uh_s64(svbool_t pg, const uint16_t *base)  LDFF1H Zresult.D, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svldff1uh_u32(svbool_t pg, const uint16_t *base)  LDFF1H Zresult.S, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    @overload
    def load_vector_u_int_16_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uh_u64(svbool_t pg, const uint16_t *base)  LDFF1H Zresult.D, Pg/Z, [Xbase, XZR, LSL #1]"""
        ...

    @staticmethod
    def load_vector_u_int_16_zero_extend_to_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint32_t svld1uh_s32(svbool_t pg, const uint16_t *base)  LD1H Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_16_zero_extend_to_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1uh_s64(svbool_t pg, const uint16_t *base)  LD1H Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_16_zero_extend_to_u_int_32(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint32_t svld1uh_u32(svbool_t pg, const uint16_t *base)  LD1H Zresult.S, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_16_zero_extend_to_u_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uh_u64(svbool_t pg, const uint16_t *base)  LD1H Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_32_non_faulting_zero_extend_to_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldnf1uw_s64(svbool_t pg, const uint32_t *base)  LDNF1W Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_32_non_faulting_zero_extend_to_u_int_64(address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldnf1uw_u64(svbool_t pg, const uint32_t *base)  LDNF1W Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def load_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svldff1uw_s64(svbool_t pg, const uint32_t *base)  LDFF1W Zresult.D, Pg/Z, [Xbase, XZR, LSL #2]"""
        ...

    @staticmethod
    @overload
    def load_vector_u_int_32_zero_extend_first_faulting(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svldff1uw_u64(svbool_t pg, const uint32_t *base)  LDFF1W Zresult.D, Pg/Z, [Xbase, XZR, LSL #2]"""
        ...

    @staticmethod
    def load_vector_u_int_32_zero_extend_to_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svint64_t svld1uw_s64(svbool_t pg, const uint32_t *base)  LD1W Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def load_vector_u_int_32_zero_extend_to_u_int_64(mask: System.Numerics.Vector[int], address: typing.Any) -> System.Numerics.Vector[int]:
        """svuint64_t svld1uw_u64(svbool_t pg, const uint32_t *base)  LD1W Zresult.D, Pg/Z, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svmax[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svmax[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svmax[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  UMAX Ztied1.B, Pg/M, Ztied1.B, Zop2.B  UMAX Ztied2.B, Pg/M, Ztied2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svmax[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmax[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmax[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FMAX Ztied1.D, Pg/M, Ztied1.D, Zop2.D  FMAX Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svmax[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svmax[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svmax[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  SMAX Ztied1.H, Pg/M, Ztied1.H, Zop2.H  SMAX Ztied2.H, Pg/M, Ztied2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svmax[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svmax[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svmax[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  SMAX Ztied1.S, Pg/M, Ztied1.S, Zop2.S  SMAX Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svmax[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svmax[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svmax[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  SMAX Ztied1.D, Pg/M, Ztied1.D, Zop2.D  SMAX Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svmax[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svmax[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svmax[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  SMAX Ztied1.B, Pg/M, Ztied1.B, Zop2.B  SMAX Ztied2.B, Pg/M, Ztied2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svmax[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmax[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmax[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FMAX Ztied1.S, Pg/M, Ztied1.S, Zop2.S  FMAX Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svmax[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svmax[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svmax[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  UMAX Ztied1.H, Pg/M, Ztied1.H, Zop2.H  UMAX Ztied2.H, Pg/M, Ztied2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svmax[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svmax[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svmax[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  UMAX Ztied1.S, Pg/M, Ztied1.S, Zop2.S  UMAX Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def max(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svmax[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svmax[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svmax[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  UMAX Ztied1.D, Pg/M, Ztied1.D, Zop2.D  UMAX Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint8_t svmaxv[_u8](svbool_t pg, svuint8_t op)  UMAXV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float64_t svmaxv[_f64](svbool_t pg, svfloat64_t op)  FMAXV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int16_t svmaxv[_s16](svbool_t pg, svint16_t op)  SMAXV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int32_t svmaxv[_s32](svbool_t pg, svint32_t op)  SMAXV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svmaxv[_s64](svbool_t pg, svint64_t op)  SMAXV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int8_t svmaxv[_s8](svbool_t pg, svint8_t op)  SMAXV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float32_t svmaxv[_f32](svbool_t pg, svfloat32_t op)  FMAXV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint16_t svmaxv[_u16](svbool_t pg, svuint16_t op)  UMAXV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint32_t svmaxv[_u32](svbool_t pg, svuint32_t op)  UMAXV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def max_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svmaxv[_u64](svbool_t pg, svuint64_t op)  UMAXV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def max_number(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svmaxnm[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmaxnm[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmaxnm[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FMAXNM Ztied1.D, Pg/M, Ztied1.D, Zop2.D  FMAXNM Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def max_number(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svmaxnm[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmaxnm[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmaxnm[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FMAXNM Ztied1.S, Pg/M, Ztied1.S, Zop2.S  FMAXNM Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def max_number_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float64_t svmaxnmv[_f64](svbool_t pg, svfloat64_t op)  FMAXNMV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def max_number_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float32_t svmaxnmv[_f32](svbool_t pg, svfloat32_t op)  FMAXNMV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svmin[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svmin[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svmin[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  UMIN Ztied1.B, Pg/M, Ztied1.B, Zop2.B  UMIN Ztied2.B, Pg/M, Ztied2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svmin[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmin[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmin[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FMIN Ztied1.D, Pg/M, Ztied1.D, Zop2.D  FMIN Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svmin[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svmin[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svmin[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  SMIN Ztied1.H, Pg/M, Ztied1.H, Zop2.H  SMIN Ztied2.H, Pg/M, Ztied2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svmin[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svmin[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svmin[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  SMIN Ztied1.S, Pg/M, Ztied1.S, Zop2.S  SMIN Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svmin[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svmin[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svmin[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  SMIN Ztied1.D, Pg/M, Ztied1.D, Zop2.D  SMIN Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svmin[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svmin[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svmin[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  SMIN Ztied1.B, Pg/M, Ztied1.B, Zop2.B  SMIN Ztied2.B, Pg/M, Ztied2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svmin[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmin[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmin[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FMIN Ztied1.S, Pg/M, Ztied1.S, Zop2.S  FMIN Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svmin[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svmin[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svmin[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  UMIN Ztied1.H, Pg/M, Ztied1.H, Zop2.H  UMIN Ztied2.H, Pg/M, Ztied2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svmin[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svmin[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svmin[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  UMIN Ztied1.S, Pg/M, Ztied1.S, Zop2.S  UMIN Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def min(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svmin[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svmin[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svmin[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  UMIN Ztied1.D, Pg/M, Ztied1.D, Zop2.D  UMIN Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint8_t svminv[_u8](svbool_t pg, svuint8_t op)  UMINV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float64_t svminv[_f64](svbool_t pg, svfloat64_t op)  FMINV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int16_t svminv[_s16](svbool_t pg, svint16_t op)  SMINV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int32_t svminv[_s32](svbool_t pg, svint32_t op)  SMINV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svminv[_s64](svbool_t pg, svint64_t op)  SMINV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int8_t svminv[_s8](svbool_t pg, svint8_t op)  SMINV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float32_t svminv[_f32](svbool_t pg, svfloat32_t op)  FMINV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint16_t svminv[_u16](svbool_t pg, svuint16_t op)  UMINV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint32_t svminv[_u32](svbool_t pg, svuint32_t op)  UMINV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def min_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svminv[_u64](svbool_t pg, svuint64_t op)  UMINV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def min_number(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svminnm[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svminnm[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svminnm[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FMINNM Ztied1.D, Pg/M, Ztied1.D, Zop2.D  FMINNM Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def min_number(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svminnm[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svminnm[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svminnm[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FMINNM Ztied1.S, Pg/M, Ztied1.S, Zop2.S  FMINNM Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def min_number_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float64_t svminnmv[_f64](svbool_t pg, svfloat64_t op)  FMINNMV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def min_number_across(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """float32_t svminnmv[_f32](svbool_t pg, svfloat32_t op)  FMINNMV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svmul[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svmul[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svmul[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  MUL Ztied1.B, Pg/M, Ztied1.B, Zop2.B  MUL Ztied2.B, Pg/M, Ztied2.B, Zop1.Bsvuint8_t svmul[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svmul[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmul[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmul[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FMUL Ztied1.D, Pg/M, Ztied1.D, Zop2.D  FMUL Ztied2.D, Pg/M, Ztied2.D, Zop1.Dsvfloat64_t svmul[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svmul[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svmul[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svmul[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  MUL Ztied1.H, Pg/M, Ztied1.H, Zop2.H  MUL Ztied2.H, Pg/M, Ztied2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svmul[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svmul[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svmul[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  MUL Ztied1.S, Pg/M, Ztied1.S, Zop2.S  MUL Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svmul[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svmul[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svmul[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  MUL Ztied1.D, Pg/M, Ztied1.D, Zop2.D  MUL Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svmul[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svmul[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svmul[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  MUL Ztied1.B, Pg/M, Ztied1.B, Zop2.B  MUL Ztied2.B, Pg/M, Ztied2.B, Zop1.B"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svmul[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmul[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmul[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FMUL Ztied1.S, Pg/M, Ztied1.S, Zop2.S  FMUL Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svmul[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svmul[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svmul[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  MUL Ztied1.H, Pg/M, Ztied1.H, Zop2.H  MUL Ztied2.H, Pg/M, Ztied2.H, Zop1.H"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svmul[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svmul[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svmul[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  MUL Ztied1.S, Pg/M, Ztied1.S, Zop2.S  MUL Ztied2.S, Pg/M, Ztied2.S, Zop1.S"""
        ...

    @staticmethod
    @overload
    def multiply(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svmul[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svmul[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svmul[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  MUL Ztied1.D, Pg/M, Ztied1.D, Zop2.D  MUL Ztied2.D, Pg/M, Ztied2.D, Zop1.D"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svmla[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2, svuint8_t op3)svuint8_t svmla[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2, svuint8_t op3)svuint8_t svmla[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2, svuint8_t op3)  MLA Ztied1.B, Pg/M, Zop2.B, Zop3.B"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svmla[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2, svint16_t op3)svint16_t svmla[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2, svint16_t op3)svint16_t svmla[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2, svint16_t op3)  MLA Ztied1.H, Pg/M, Zop2.H, Zop3.H"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svmla[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2, svint32_t op3)svint32_t svmla[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2, svint32_t op3)svint32_t svmla[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2, svint32_t op3)  MLA Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svmla[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2, svint64_t op3)svint64_t svmla[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2, svint64_t op3)svint64_t svmla[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2, svint64_t op3)  MLA Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svmla[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2, svint8_t op3)svint8_t svmla[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2, svint8_t op3)svint8_t svmla[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2, svint8_t op3)  MLA Ztied1.B, Pg/M, Zop2.B, Zop3.B"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svmla[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2, svuint16_t op3)svuint16_t svmla[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2, svuint16_t op3)svuint16_t svmla[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2, svuint16_t op3)  MLA Ztied1.H, Pg/M, Zop2.H, Zop3.H"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svmla[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2, svuint32_t op3)svuint32_t svmla[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2, svuint32_t op3)svuint32_t svmla[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2, svuint32_t op3)  MLA Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def multiply_add(addend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svmla[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2, svuint64_t op3)svuint64_t svmla[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2, svuint64_t op3)svuint64_t svmla[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2, svuint64_t op3)  MLA Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def multiply_add_rotate_complex(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], rotation: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svcmla[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3, uint64_t imm_rotation)svfloat64_t svcmla[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3, uint64_t imm_rotation)svfloat64_t svcmla[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2, svfloat64_t op3, uint64_t imm_rotation)  FCMLA Ztied1.D, Pg/M, Zop2.D, Zop3.D, #imm_rotation"""
        ...

    @staticmethod
    @overload
    def multiply_add_rotate_complex(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], rotation: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svcmla[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3, uint64_t imm_rotation)svfloat32_t svcmla[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3, uint64_t imm_rotation)svfloat32_t svcmla[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2, svfloat32_t op3, uint64_t imm_rotation)  FCMLA Ztied1.S, Pg/M, Zop2.S, Zop3.S, #imm_rotation"""
        ...

    @staticmethod
    def multiply_add_rotate_complex_by_selected_scalar(addend: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], right_index: int, rotation: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svcmla_lane[_f32](svfloat32_t op1, svfloat32_t op2, svfloat32_t op3, uint64_t imm_index, uint64_t imm_rotation)  FCMLA Ztied1.S, Zop2.S, Zop3.S[imm_index], #imm_rotation"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], right_index: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svmul_lane[_f64](svfloat64_t op1, svfloat64_t op2, uint64_t imm_index)  FMUL Zresult.D, Zop1.D, Zop2.D[imm_index]"""
        ...

    @staticmethod
    @overload
    def multiply_by_selected_scalar(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], right_index: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svmul_lane[_f32](svfloat32_t op1, svfloat32_t op2, uint64_t imm_index)  FMUL Zresult.S, Zop1.S, Zop2.S[imm_index]"""
        ...

    @staticmethod
    @overload
    def multiply_extended(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svmulx[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmulx[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svmulx[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FMULX Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def multiply_extended(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svmulx[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmulx[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svmulx[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FMULX Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svmls[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2, svuint8_t op3)svuint8_t svmls[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2, svuint8_t op3)svuint8_t svmls[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2, svuint8_t op3)  MLS Ztied1.B, Pg/M, Zop2.B, Zop3.B"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svmls[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2, svint16_t op3)svint16_t svmls[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2, svint16_t op3)svint16_t svmls[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2, svint16_t op3)  MLS Ztied1.H, Pg/M, Zop2.H, Zop3.H"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svmls[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2, svint32_t op3)svint32_t svmls[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2, svint32_t op3)svint32_t svmls[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2, svint32_t op3)  MLS Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svmls[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2, svint64_t op3)svint64_t svmls[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2, svint64_t op3)svint64_t svmls[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2, svint64_t op3)  MLS Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svmls[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2, svint8_t op3)svint8_t svmls[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2, svint8_t op3)svint8_t svmls[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2, svint8_t op3)  MLS Ztied1.B, Pg/M, Zop2.B, Zop3.B"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svmls[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2, svuint16_t op3)svuint16_t svmls[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2, svuint16_t op3)svuint16_t svmls[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2, svuint16_t op3)  MLS Ztied1.H, Pg/M, Zop2.H, Zop3.H"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svmls[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2, svuint32_t op3)svuint32_t svmls[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2, svuint32_t op3)svuint32_t svmls[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2, svuint32_t op3)  MLS Ztied1.S, Pg/M, Zop2.S, Zop3.S"""
        ...

    @staticmethod
    @overload
    def multiply_subtract(minuend: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svmls[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2, svuint64_t op3)svuint64_t svmls[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2, svuint64_t op3)svuint64_t svmls[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2, svuint64_t op3)  MLS Ztied1.D, Pg/M, Zop2.D, Zop3.D"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svneg[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svneg[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svneg[_f64]_z(svbool_t pg, svfloat64_t op)  FNEG Ztied.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svneg[_s16]_m(svint16_t inactive, svbool_t pg, svint16_t op)svint16_t svneg[_s16]_x(svbool_t pg, svint16_t op)svint16_t svneg[_s16]_z(svbool_t pg, svint16_t op)  NEG Ztied.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svneg[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svneg[_s32]_x(svbool_t pg, svint32_t op)svint32_t svneg[_s32]_z(svbool_t pg, svint32_t op)  NEG Ztied.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svneg[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svneg[_s64]_x(svbool_t pg, svint64_t op)svint64_t svneg[_s64]_z(svbool_t pg, svint64_t op)  NEG Ztied.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svneg[_s8]_m(svint8_t inactive, svbool_t pg, svint8_t op)svint8_t svneg[_s8]_x(svbool_t pg, svint8_t op)svint8_t svneg[_s8]_z(svbool_t pg, svint8_t op)  NEG Ztied.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def negate(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svneg[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svneg[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svneg[_f32]_z(svbool_t pg, svfloat32_t op)  FNEG Ztied.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svnot[_u8]_m(svuint8_t inactive, svbool_t pg, svuint8_t op)svuint8_t svnot[_u8]_x(svbool_t pg, svuint8_t op)svuint8_t svnot[_u8]_z(svbool_t pg, svuint8_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svnot[_s16]_m(svint16_t inactive, svbool_t pg, svint16_t op)svint16_t svnot[_s16]_x(svbool_t pg, svint16_t op)svint16_t svnot[_s16]_z(svbool_t pg, svint16_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svnot[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svnot[_s32]_x(svbool_t pg, svint32_t op)svint32_t svnot[_s32]_z(svbool_t pg, svint32_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svnot[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svnot[_s64]_x(svbool_t pg, svint64_t op)svint64_t svnot[_s64]_z(svbool_t pg, svint64_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svnot[_s8]_m(svint8_t inactive, svbool_t pg, svint8_t op)svint8_t svnot[_s8]_x(svbool_t pg, svint8_t op)svint8_t svnot[_s8]_z(svbool_t pg, svint8_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svnot[_u16]_m(svuint16_t inactive, svbool_t pg, svuint16_t op)svuint16_t svnot[_u16]_x(svbool_t pg, svuint16_t op)svuint16_t svnot[_u16]_z(svbool_t pg, svuint16_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svnot[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svnot[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svnot[_u32]_z(svbool_t pg, svuint32_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def Not(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svnot[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svnot[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svnot[_u64]_z(svbool_t pg, svuint64_t op)svbool_t svnot[_b]_z(svbool_t pg, svbool_t op)  NOT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svorr[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svorr[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svorr[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svorr[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svorr[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svorr[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svorr[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svorr[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svorr[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svorr[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svorr[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svorr[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svorr[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svorr[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svorr[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svorr[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svorr[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svorr[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svorr[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svorr[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svorr[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def Or(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svorr[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svorr[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svorr[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  ORR Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint8_t svorv[_u8](svbool_t pg, svuint8_t op)  ORV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int16_t svorv[_s16](svbool_t pg, svint16_t op)  ORV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int32_t svorv[_s32](svbool_t pg, svint32_t op)  ORV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t svorv[_s64](svbool_t pg, svint64_t op)  ORV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int8_t svorv[_s8](svbool_t pg, svint8_t op)  ORV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint16_t svorv[_u16](svbool_t pg, svuint16_t op)  ORV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint32_t svorv[_u32](svbool_t pg, svuint32_t op)  ORV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def or_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t svorv[_u64](svbool_t pg, svuint64_t op)  ORV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svcnt[_s8]_m(svuint8_t inactive, svbool_t pg, svint8_t op)svuint8_t svcnt[_s8]_x(svbool_t pg, svint8_t op)svuint8_t svcnt[_s8]_z(svbool_t pg, svint8_t op)  CNT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svcnt[_u8]_m(svuint8_t inactive, svbool_t pg, svuint8_t op)svuint8_t svcnt[_u8]_x(svbool_t pg, svuint8_t op)svuint8_t svcnt[_u8]_z(svbool_t pg, svuint8_t op)  CNT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svcnt[_s16]_m(svuint16_t inactive, svbool_t pg, svint16_t op)svuint16_t svcnt[_s16]_x(svbool_t pg, svint16_t op)svuint16_t svcnt[_s16]_z(svbool_t pg, svint16_t op)  CNT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svcnt[_u16]_m(svuint16_t inactive, svbool_t pg, svuint16_t op)svuint16_t svcnt[_u16]_x(svbool_t pg, svuint16_t op)svuint16_t svcnt[_u16]_z(svbool_t pg, svuint16_t op)  CNT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svcnt[_s32]_m(svuint32_t inactive, svbool_t pg, svint32_t op)svuint32_t svcnt[_s32]_x(svbool_t pg, svint32_t op)svuint32_t svcnt[_s32]_z(svbool_t pg, svint32_t op)  CNT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svuint32_t svcnt[_f32]_m(svuint32_t inactive, svbool_t pg, svfloat32_t op)svuint32_t svcnt[_f32]_x(svbool_t pg, svfloat32_t op)svuint32_t svcnt[_f32]_z(svbool_t pg, svfloat32_t op)  CNT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svcnt[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svcnt[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svcnt[_u32]_z(svbool_t pg, svuint32_t op)  CNT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[int]:
        """svuint64_t svcnt[_f64]_m(svuint64_t inactive, svbool_t pg, svfloat64_t op)svuint64_t svcnt[_f64]_x(svbool_t pg, svfloat64_t op)svuint64_t svcnt[_f64]_z(svbool_t pg, svfloat64_t op)  CNT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svcnt[_s64]_m(svuint64_t inactive, svbool_t pg, svint64_t op)svuint64_t svcnt[_s64]_x(svbool_t pg, svint64_t op)svuint64_t svcnt[_s64]_z(svbool_t pg, svint64_t op)  CNT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def pop_count(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svcnt[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svcnt[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svcnt[_u64]_z(svbool_t pg, svuint64_t op)  CNT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    def prefetch_bytes(mask: System.Numerics.Vector[int], address: typing.Any, prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfb(svbool_t pg, const void *base, enum svprfop op)  PRFB op, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def prefetch_int_16(mask: System.Numerics.Vector[int], address: typing.Any, prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfh(svbool_t pg, const void *base, enum svprfop op)  PRFH op, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def prefetch_int_32(mask: System.Numerics.Vector[int], address: typing.Any, prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfw(svbool_t pg, const void *base, enum svprfop op)  PRFW op, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    def prefetch_int_64(mask: System.Numerics.Vector[int], address: typing.Any, prefetch_type: System.Runtime.Intrinsics.Arm.SvePrefetchType) -> None:
        """void svprfd(svbool_t pg, const void *base, enum svprfop op)  PRFD op, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def reciprocal_estimate(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrecpe[_f64](svfloat64_t op)  FRECPE Zresult.D, Zop.D"""
        ...

    @staticmethod
    @overload
    def reciprocal_estimate(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrecpe[_f32](svfloat32_t op)  FRECPE Zresult.S, Zop.S"""
        ...

    @staticmethod
    @overload
    def reciprocal_exponent(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrecpx[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svrecpx[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svrecpx[_f64]_z(svbool_t pg, svfloat64_t op)  FRECPX Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reciprocal_exponent(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrecpx[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svrecpx[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svrecpx[_f32]_z(svbool_t pg, svfloat32_t op)  FRECPX Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def reciprocal_sqrt_estimate(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrsqrte[_f64](svfloat64_t op)  FRSQRTE Zresult.D, Zop.D"""
        ...

    @staticmethod
    @overload
    def reciprocal_sqrt_estimate(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrsqrte[_f32](svfloat32_t op)  FRSQRTE Zresult.S, Zop.S"""
        ...

    @staticmethod
    @overload
    def reciprocal_sqrt_step(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrsqrts[_f64](svfloat64_t op1, svfloat64_t op2)  FRSQRTS Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def reciprocal_sqrt_step(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrsqrts[_f32](svfloat32_t op1, svfloat32_t op2)  FRSQRTS Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def reciprocal_step(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrecps[_f64](svfloat64_t op1, svfloat64_t op2)  FRECPS Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def reciprocal_step(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrecps[_f32](svfloat32_t op1, svfloat32_t op2)  FRECPS Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svrbit[_u8]_m(svuint8_t inactive, svbool_t pg, svuint8_t op)svuint8_t svrbit[_u8]_x(svbool_t pg, svuint8_t op)svuint8_t svrbit[_u8]_z(svbool_t pg, svuint8_t op)  RBIT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svrbit[_s16]_m(svint16_t inactive, svbool_t pg, svint16_t op)svint16_t svrbit[_s16]_x(svbool_t pg, svint16_t op)svint16_t svrbit[_s16]_z(svbool_t pg, svint16_t op)  RBIT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svrbit[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svrbit[_s32]_x(svbool_t pg, svint32_t op)svint32_t svrbit[_s32]_z(svbool_t pg, svint32_t op)  RBIT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svrbit[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svrbit[_s64]_x(svbool_t pg, svint64_t op)svint64_t svrbit[_s64]_z(svbool_t pg, svint64_t op)  RBIT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svrbit[_s8]_m(svint8_t inactive, svbool_t pg, svint8_t op)svint8_t svrbit[_s8]_x(svbool_t pg, svint8_t op)svint8_t svrbit[_s8]_z(svbool_t pg, svint8_t op)  RBIT Zresult.B, Pg/M, Zop.B"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svrbit[_u16]_m(svuint16_t inactive, svbool_t pg, svuint16_t op)svuint16_t svrbit[_u16]_x(svbool_t pg, svuint16_t op)svuint16_t svrbit[_u16]_z(svbool_t pg, svuint16_t op)  RBIT Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svrbit[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svrbit[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svrbit[_u32]_z(svbool_t pg, svuint32_t op)  RBIT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_bits(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svrbit[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svrbit[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svrbit[_u64]_z(svbool_t pg, svuint64_t op)  RBIT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svrev[_u8](svuint8_t op)  REV Zresult.B, Zop.B"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrev[_f64](svfloat64_t op)  REV Zresult.D, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svrev[_s16](svint16_t op)  REV Zresult.H, Zop.H"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svrev[_s32](svint32_t op)  REV Zresult.S, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svrev[_s64](svint64_t op)  REV Zresult.D, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svrev[_s8](svint8_t op)  REV Zresult.B, Zop.B"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrev[_f32](svfloat32_t op)  REV Zresult.S, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svrev[_u16](svuint16_t op)  REV Zresult.H, Zop.H"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svrev[_u32](svuint32_t op)  REV Zresult.S, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_element(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svrev[_u64](svuint64_t op)  REV Zresult.D, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svrevh[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svrevh[_s32]_x(svbool_t pg, svint32_t op)svint32_t svrevh[_s32]_z(svbool_t pg, svint32_t op)  REVH Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svrevh[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svrevh[_s64]_x(svbool_t pg, svint64_t op)svint64_t svrevh[_s64]_z(svbool_t pg, svint64_t op)  REVH Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svrevh[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svrevh[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svrevh[_u32]_z(svbool_t pg, svuint32_t op)  REVH Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_element_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svrevh[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svrevh[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svrevh[_u64]_z(svbool_t pg, svuint64_t op)  REVH Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element_32(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svrevw[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svrevw[_s64]_x(svbool_t pg, svint64_t op)svint64_t svrevw[_s64]_z(svbool_t pg, svint64_t op)  REVW Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element_32(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svrevw[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svrevw[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svrevw[_u64]_z(svbool_t pg, svuint64_t op)  REVW Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svrevb[_s16]_m(svint16_t inactive, svbool_t pg, svint16_t op)svint16_t svrevb[_s16]_x(svbool_t pg, svint16_t op)svint16_t svrevb[_s16]_z(svbool_t pg, svint16_t op)  REVB Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svrevb[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svrevb[_s32]_x(svbool_t pg, svint32_t op)svint32_t svrevb[_s32]_z(svbool_t pg, svint32_t op)  REVB Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svrevb[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svrevb[_s64]_x(svbool_t pg, svint64_t op)svint64_t svrevb[_s64]_z(svbool_t pg, svint64_t op)  REVB Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svrevb[_u16]_m(svuint16_t inactive, svbool_t pg, svuint16_t op)svuint16_t svrevb[_u16]_x(svbool_t pg, svuint16_t op)svuint16_t svrevb[_u16]_z(svbool_t pg, svuint16_t op)  REVB Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svrevb[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svrevb[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svrevb[_u32]_z(svbool_t pg, svuint32_t op)  REVB Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def reverse_element_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svrevb[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svrevb[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svrevb[_u64]_z(svbool_t pg, svuint64_t op)  REVB Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def round_away_from_zero(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrinta[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svrinta[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svrinta[_f64]_z(svbool_t pg, svfloat64_t op)  FRINTA Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def round_away_from_zero(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrinta[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svrinta[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svrinta[_f32]_z(svbool_t pg, svfloat32_t op)  FRINTA Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def round_to_nearest(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrintn[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svrintn[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svrintn[_f64]_z(svbool_t pg, svfloat64_t op)  FRINTN Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def round_to_nearest(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrintn[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svrintn[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svrintn[_f32]_z(svbool_t pg, svfloat32_t op)  FRINTN Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def round_to_negative_infinity(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrintm[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svrintm[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svrintm[_f64]_z(svbool_t pg, svfloat64_t op)  FRINTM Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def round_to_negative_infinity(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrintm[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svrintm[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svrintm[_f32]_z(svbool_t pg, svfloat32_t op)  FRINTM Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def round_to_positive_infinity(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrintp[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svrintp[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svrintp[_f64]_z(svbool_t pg, svfloat64_t op)  FRINTP Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def round_to_positive_infinity(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrintp[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svrintp[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svrintp[_f32]_z(svbool_t pg, svfloat32_t op)  FRINTP Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def round_to_zero(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svrintz[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svrintz[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svrintz[_f64]_z(svbool_t pg, svfloat64_t op)  FRINTZ Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def round_to_zero(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svrintz[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svrintz[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svrintz[_f32]_z(svbool_t pg, svfloat32_t op)  FRINTZ Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqdech_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECH Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqdech_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECH Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqdech_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECH Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqdech_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECH Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_16_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svint16_t svqdech_pat[_s16](svint16_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECH Ztied.H, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_16_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svuint16_t svqdech_pat[_u16](svuint16_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECH Ztied.H, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqdecw_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECW Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqdecw_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECW Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqdecw_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECW Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqdecw_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECW Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_32_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svint32_t svqdecw_pat[_s32](svint32_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECW Ztied.S, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_32_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svuint32_t svqdecw_pat[_u32](svuint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECW Ztied.S, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqdecd_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECD Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqdecd_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECD Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqdecd_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECD Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqdecd_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECD Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_64_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svint64_t svqdecd_pat[_s64](svint64_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECD Ztied.D, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_64_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svuint64_t svqdecd_pat[_u64](svuint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECD Ztied.D, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqdecb_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECB Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqdecb_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQDECB Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqdecb_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECB Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqdecb_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQDECB Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqdecp[_n_s32]_b8(int32_t op, svbool_t pg)  SQDECP Xtied, Pg.B, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqdecp[_n_s64]_b8(int64_t op, svbool_t pg)  SQDECP Xtied, Pg.B"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqdecp[_n_u32]_b8(uint32_t op, svbool_t pg)  UQDECP Wtied, Pg.B"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqdecp[_n_u64]_b8(uint64_t op, svbool_t pg)  UQDECP Xtied, Pg.B"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svqdecp[_s16](svint16_t op, svbool_t pg)  SQDECP Ztied.H, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svqdecp[_s32](svint32_t op, svbool_t pg)  SQDECP Ztied.S, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svqdecp[_s64](svint64_t op, svbool_t pg)  SQDECP Ztied.D, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqdecp[_n_s32]_b16(int32_t op, svbool_t pg)  SQDECP Xtied, Pg.H, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqdecp[_n_s64]_b16(int64_t op, svbool_t pg)  SQDECP Xtied, Pg.H"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqdecp[_n_u32]_b16(uint32_t op, svbool_t pg)  UQDECP Wtied, Pg.H"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqdecp[_n_u64]_b16(uint64_t op, svbool_t pg)  UQDECP Xtied, Pg.H"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svqdecp[_u16](svuint16_t op, svbool_t pg)  UQDECP Ztied.H, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqdecp[_n_s32]_b32(int32_t op, svbool_t pg)  SQDECP Xtied, Pg.S, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqdecp[_n_s64]_b32(int64_t op, svbool_t pg)  SQDECP Xtied, Pg.S"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqdecp[_n_u32]_b32(uint32_t op, svbool_t pg)  UQDECP Wtied, Pg.S"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqdecp[_n_u64]_b32(uint64_t op, svbool_t pg)  UQDECP Xtied, Pg.S"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svqdecp[_u32](svuint32_t op, svbool_t pg)  UQDECP Ztied.S, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqdecp[_n_s32]_b64(int32_t op, svbool_t pg)  SQDECP Xtied, Pg.D, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqdecp[_n_s64]_b64(int64_t op, svbool_t pg)  SQDECP Xtied, Pg.D"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqdecp[_n_u32]_b64(uint32_t op, svbool_t pg)  UQDECP Wtied, Pg.D"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqdecp[_n_u64]_b64(uint64_t op, svbool_t pg)  UQDECP Xtied, Pg.D"""
        ...

    @staticmethod
    @overload
    def saturating_decrement_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svqdecp[_u64](svuint64_t op, svbool_t pg)  UQDECP Ztied.D, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqinch_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCH Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqinch_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCH Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqinch_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCH Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_16_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqinch_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCH Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_16_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svint16_t svqinch_pat[_s16](svint16_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCH Ztied.H, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_16_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svuint16_t svqinch_pat[_u16](svuint16_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCH Ztied.H, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqincw_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCW Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqincw_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCW Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqincw_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCW Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_32_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqincw_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCW Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_32_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svint32_t svqincw_pat[_s32](svint32_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCW Ztied.S, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_32_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svuint32_t svqincw_pat[_u32](svuint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCW Ztied.S, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqincd_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCD Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqincd_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCD Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqincd_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCD Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_64_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqincd_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCD Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_64_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svint64_t svqincd_pat[_s64](svint64_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCD Ztied.D, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_64_bit_element_count(value: System.Numerics.Vector[int], scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> System.Numerics.Vector[int]:
        """svuint64_t svqincd_pat[_u64](svuint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCD Ztied.D, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int32_t svqincb_pat[_n_s32](int32_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCB Xtied, Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """int64_t svqincb_pat[_n_s64](int64_t op, enum svpattern pattern, uint64_t imm_factor)  SQINCB Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint32_t svqincb_pat[_n_u32](uint32_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCB Wtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_8_bit_element_count(value: int, scale: int, pattern: System.Runtime.Intrinsics.Arm.SveMaskPattern = ...) -> int:
        """uint64_t svqincb_pat[_n_u64](uint64_t op, enum svpattern pattern, uint64_t imm_factor)  UQINCB Xtied, pattern, MUL #imm_factor"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqincp[_n_s32]_b8(int32_t op, svbool_t pg)  SQINCP Xtied, Pg.B, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqincp[_n_s64]_b8(int64_t op, svbool_t pg)  SQINCP Xtied, Pg.B"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqincp[_n_u32]_b8(uint32_t op, svbool_t pg)  UQINCP Wtied, Pg.B"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqincp[_n_u64]_b8(uint64_t op, svbool_t pg)  UQINCP Xtied, Pg.B"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svqincp[_s16](svint16_t op, svbool_t pg)  SQINCP Ztied.H, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svqincp[_s32](svint32_t op, svbool_t pg)  SQINCP Ztied.S, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svqincp[_s64](svint64_t op, svbool_t pg)  SQINCP Ztied.D, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqincp[_n_s32]_b16(int32_t op, svbool_t pg)  SQINCP Xtied, Pg.H, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqincp[_n_s64]_b16(int64_t op, svbool_t pg)  SQINCP Xtied, Pg.H"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqincp[_n_u32]_b16(uint32_t op, svbool_t pg)  UQINCP Wtied, Pg.H"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqincp[_n_u64]_b16(uint64_t op, svbool_t pg)  UQINCP Xtied, Pg.H"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svqincp[_u16](svuint16_t op, svbool_t pg)  UQINCP Ztied.H, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqincp[_n_s32]_b32(int32_t op, svbool_t pg)  SQINCP Xtied, Pg.S, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqincp[_n_s64]_b32(int64_t op, svbool_t pg)  SQINCP Xtied, Pg.S"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqincp[_n_u32]_b32(uint32_t op, svbool_t pg)  UQINCP Wtied, Pg.S"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqincp[_n_u64]_b32(uint64_t op, svbool_t pg)  UQINCP Xtied, Pg.S"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svqincp[_u32](svuint32_t op, svbool_t pg)  UQINCP Ztied.S, Pg"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int32_t svqincp[_n_s32]_b64(int32_t op, svbool_t pg)  SQINCP Xtied, Pg.D, Wtied"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """int64_t svqincp[_n_s64]_b64(int64_t op, svbool_t pg)  SQINCP Xtied, Pg.D"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint32_t svqincp[_n_u32]_b64(uint32_t op, svbool_t pg)  UQINCP Wtied, Pg.D"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: int, _from: System.Numerics.Vector[int]) -> int:
        """uint64_t svqincp[_n_u64]_b64(uint64_t op, svbool_t pg)  UQINCP Xtied, Pg.D"""
        ...

    @staticmethod
    @overload
    def saturating_increment_by_active_element_count(value: System.Numerics.Vector[int], _from: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svqincp[_u64](svuint64_t op, svbool_t pg)  UQINCP Ztied.D, Pg"""
        ...

    @staticmethod
    @overload
    def scale(left: System.Numerics.Vector[float], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svscale[_f64]_m(svbool_t pg, svfloat64_t op1, svint64_t op2)svfloat64_t svscale[_f64]_x(svbool_t pg, svfloat64_t op1, svint64_t op2)svfloat64_t svscale[_f64]_z(svbool_t pg, svfloat64_t op1, svint64_t op2)  FSCALE Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def scale(left: System.Numerics.Vector[float], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svscale[_f32]_m(svbool_t pg, svfloat32_t op1, svint32_t op2)svfloat32_t svscale[_f32]_x(svbool_t pg, svfloat32_t op1, svint32_t op2)svfloat32_t svscale[_f32]_z(svbool_t pg, svfloat32_t op1, svint32_t op2)  FSCALE Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[float], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[float]) -> None:
        """void svst1_scatter_[s64]offset[_f64](svbool_t pg, float64_t *base, svint64_t offsets, svfloat64_t data)  ST1D Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[float], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[float]) -> None:
        """void svst1_scatter[_u64base_f64](svbool_t pg, svuint64_t bases, svfloat64_t data)  ST1D Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[float], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[float]) -> None:
        """void svst1_scatter_[u64]offset[_f64](svbool_t pg, float64_t *base, svuint64_t offsets, svfloat64_t data)  ST1D Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[s32]offset[_s32](svbool_t pg, int32_t *base, svint32_t offsets, svint32_t data)  ST1W Zdata.S, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[u32]offset[_s32](svbool_t pg, int32_t *base, svuint32_t offsets, svint32_t data)  ST1W Zdata.S, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[s64]offset[_s64](svbool_t pg, int64_t *base, svint64_t offsets, svint64_t data)  ST1D Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter[_u64base_s64](svbool_t pg, svuint64_t bases, svint64_t data)  ST1D Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[u64]offset[_s64](svbool_t pg, int64_t *base, svuint64_t offsets, svint64_t data)  ST1D Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[float], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[float]) -> None:
        """void svst1_scatter_[s32]offset[_f32](svbool_t pg, float32_t *base, svint32_t offsets, svfloat32_t data)  ST1W Zdata.S, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[float], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[float]) -> None:
        """void svst1_scatter_[u32]offset[_f32](svbool_t pg, float32_t *base, svuint32_t offsets, svfloat32_t data)  ST1W Zdata.S, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[s32]offset[_u32](svbool_t pg, uint32_t *base, svint32_t offsets, svuint32_t data)  ST1W Zdata.S, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[u32]offset[_u32](svbool_t pg, uint32_t *base, svuint32_t offsets, svuint32_t data)  ST1W Zdata.S, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[s64]offset[_u64](svbool_t pg, uint64_t *base, svint64_t offsets, svuint64_t data)  ST1D Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter[_u64base_u64](svbool_t pg, svuint64_t bases, svuint64_t data)  ST1D Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter(mask: System.Numerics.Vector[int], address: typing.Any, indicies: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1_scatter_[u64]offset[_u64](svbool_t pg, uint64_t *base, svuint64_t offsets, svuint64_t data)  ST1D Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_narrowing(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter[_u64base_s64](svbool_t pg, svuint64_t bases, svint64_t data)  ST1H Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_narrowing(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter[_u64base_u64](svbool_t pg, svuint64_t bases, svuint64_t data)  ST1H Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[s32]offset[_s32](svbool_t pg, int16_t *base, svint32_t offsets, svint32_t data)  ST1H Zdata.S, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[u32]offset[_s32](svbool_t pg, int16_t *base, svuint32_t offsets, svint32_t data)  ST1H Zdata.S, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[s64]offset[_s64](svbool_t pg, int16_t *base, svint64_t offsets, svint64_t data)  ST1H Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[u64]offset[_s64](svbool_t pg, int16_t *base, svuint64_t offsets, svint64_t data)  ST1H Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[s32]offset[_u32](svbool_t pg, uint16_t *base, svint32_t offsets, svuint32_t data)  ST1H Zdata.S, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[u32]offset[_u32](svbool_t pg, uint16_t *base, svuint32_t offsets, svuint32_t data)  ST1H Zdata.S, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[s64]offset[_u64](svbool_t pg, uint16_t *base, svint64_t offsets, svuint64_t data)  ST1H Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_16_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1h_scatter_[u64]offset[_u64](svbool_t pg, uint16_t *base, svuint64_t offsets, svuint64_t data)  ST1H Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_32_bit_narrowing(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1w_scatter[_u64base_s64](svbool_t pg, svuint64_t bases, svint64_t data)  ST1W Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter_32_bit_narrowing(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1w_scatter[_u64base_u64](svbool_t pg, svuint64_t bases, svuint64_t data)  ST1W Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter_32_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1w_scatter_[s64]offset[_s64](svbool_t pg, int32_t *base, svint64_t offsets, svint64_t data)  ST1W Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_32_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1w_scatter_[u64]offset[_s64](svbool_t pg, int32_t *base, svuint64_t offsets, svint64_t data)  ST1W Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_32_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1w_scatter_[s64]offset[_u64](svbool_t pg, uint32_t *base, svint64_t offsets, svuint64_t data)  ST1W Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_32_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1w_scatter_[u64]offset[_u64](svbool_t pg, uint32_t *base, svuint64_t offsets, svuint64_t data)  ST1W Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_narrowing(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter[_u64base_s64](svbool_t pg, svuint64_t bases, svint64_t data)  ST1B Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_narrowing(mask: System.Numerics.Vector[int], addresses: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter[_u64base_u64](svbool_t pg, svuint64_t bases, svuint64_t data)  ST1B Zdata.D, Pg, [Zbases.D, #0]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[s32]offset[_s32](svbool_t pg, int8_t *base, svint32_t offsets, svint32_t data)  ST1B Zdata.S, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[u32]offset[_s32](svbool_t pg, int8_t *base, svuint32_t offsets, svint32_t data)  ST1B Zdata.S, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[s64]offset[_s64](svbool_t pg, int8_t *base, svint64_t offsets, svint64_t data)  ST1B Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[u64]offset[_s64](svbool_t pg, int8_t *base, svuint64_t offsets, svint64_t data)  ST1B Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[s32]offset[_u32](svbool_t pg, uint8_t *base, svint32_t offsets, svuint32_t data)  ST1B Zdata.S, Pg, [Xbase, Zoffsets.S, SXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[u32]offset[_u32](svbool_t pg, uint8_t *base, svuint32_t offsets, svuint32_t data)  ST1B Zdata.S, Pg, [Xbase, Zoffsets.S, UXTW]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[s64]offset[_u64](svbool_t pg, uint8_t *base, svint64_t offsets, svuint64_t data)  ST1B Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def scatter_8_bit_with_byte_offsets_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, offsets: System.Numerics.Vector[int], data: System.Numerics.Vector[int]) -> None:
        """void svst1b_scatter_[u64]offset[_u64](svbool_t pg, uint8_t *base, svuint64_t offsets, svuint64_t data)  ST1B Zdata.D, Pg, [Xbase, Zoffsets.D]"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def set_ffr(value: System.Numerics.Vector[int]) -> None:
        """void svwrffr(svbool_t op)  WRFFR Pop.B"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svlsl[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svlsl[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svlsl[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  LSL Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svlsl_wide[_u8]_m(svbool_t pg, svuint8_t op1, svuint64_t op2)svuint8_t svlsl_wide[_u8]_x(svbool_t pg, svuint8_t op1, svuint64_t op2)  LSL Ztied1.B, Pg/M, Ztied1.B, Zop2.D  LSL Zresult.B, Zop1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svlsl[_s16]_m(svbool_t pg, svint16_t op1, svuint16_t op2)svint16_t svlsl[_s16]_x(svbool_t pg, svint16_t op1, svuint16_t op2)svint16_t svlsl[_s16]_z(svbool_t pg, svint16_t op1, svuint16_t op2)  LSL Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svlsl_wide[_s16]_m(svbool_t pg, svint16_t op1, svuint64_t op2)svint16_t svlsl_wide[_s16]_x(svbool_t pg, svint16_t op1, svuint64_t op2)svint16_t svlsl_wide[_s16]_z(svbool_t pg, svint16_t op1, svuint64_t op2)  LSL Ztied1.H, Pg/M, Ztied1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svlsl[_s32]_m(svbool_t pg, svint32_t op1, svuint32_t op2)svint32_t svlsl[_s32]_x(svbool_t pg, svint32_t op1, svuint32_t op2)svint32_t svlsl[_s32]_z(svbool_t pg, svint32_t op1, svuint32_t op2)  LSL Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svlsl_wide[_s32]_m(svbool_t pg, svint32_t op1, svuint64_t op2)svint32_t svlsl_wide[_s32]_x(svbool_t pg, svint32_t op1, svuint64_t op2)svint32_t svlsl_wide[_s32]_z(svbool_t pg, svint32_t op1, svuint64_t op2)  LSL Ztied1.S, Pg/M, Ztied1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svlsl[_s64]_m(svbool_t pg, svint64_t op1, svuint64_t op2)svint64_t svlsl[_s64]_x(svbool_t pg, svint64_t op1, svuint64_t op2)svint64_t svlsl[_s64]_z(svbool_t pg, svint64_t op1, svuint64_t op2)  LSL Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svlsl[_s8]_m(svbool_t pg, svint8_t op1, svuint8_t op2)svint8_t svlsl[_s8]_x(svbool_t pg, svint8_t op1, svuint8_t op2)svint8_t svlsl[_s8]_z(svbool_t pg, svint8_t op1, svuint8_t op2)  LSL Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svlsl_wide[_s8]_m(svbool_t pg, svint8_t op1, svuint64_t op2)svint8_t svlsl_wide[_s8]_x(svbool_t pg, svint8_t op1, svuint64_t op2)svint8_t svlsl_wide[_s8]_z(svbool_t pg, svint8_t op1, svuint64_t op2)  LSL Ztied1.B, Pg/M, Ztied1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svlsl[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svlsl[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svlsl[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  LSL Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svlsl_wide[_u16]_m(svbool_t pg, svuint16_t op1, svuint64_t op2)svuint16_t svlsl_wide[_u16]_x(svbool_t pg, svuint16_t op1, svuint64_t op2)svuint16_t svlsl_wide[_u16]_z(svbool_t pg, svuint16_t op1, svuint64_t op2)  LSL Ztied1.H, Pg/M, Ztied1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svlsl[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svlsl[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svlsl[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  LSL Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svlsl_wide[_u32]_m(svbool_t pg, svuint32_t op1, svuint64_t op2)svuint32_t svlsl_wide[_u32]_x(svbool_t pg, svuint32_t op1, svuint64_t op2)svuint32_t svlsl_wide[_u32]_z(svbool_t pg, svuint32_t op1, svuint64_t op2)  LSL Ztied1.S, Pg/M, Ztied1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_left_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svlsl[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svlsl[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svlsl[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  LSL Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svasr[_s16]_m(svbool_t pg, svint16_t op1, svuint16_t op2)svint16_t svasr[_s16]_x(svbool_t pg, svint16_t op1, svuint16_t op2)svint16_t svasr[_s16]_z(svbool_t pg, svint16_t op1, svuint16_t op2)  ASR Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svasr_wide[_s16]_m(svbool_t pg, svint16_t op1, svuint64_t op2)svint16_t svasr_wide[_s16]_x(svbool_t pg, svint16_t op1, svuint64_t op2)svint16_t svasr_wide[_s16]_z(svbool_t pg, svint16_t op1, svuint64_t op2)  ASR Ztied1.H, Pg/M, Ztied1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svasr[_s32]_m(svbool_t pg, svint32_t op1, svuint32_t op2)svint32_t svasr[_s32]_x(svbool_t pg, svint32_t op1, svuint32_t op2)svint32_t svasr[_s32]_z(svbool_t pg, svint32_t op1, svuint32_t op2)  ASR Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svasr_wide[_s32]_m(svbool_t pg, svint32_t op1, svuint64_t op2)svint32_t svasr_wide[_s32]_x(svbool_t pg, svint32_t op1, svuint64_t op2)svint32_t svasr_wide[_s32]_z(svbool_t pg, svint32_t op1, svuint64_t op2)  ASR Ztied1.S, Pg/M, Ztied1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svasr[_s64]_m(svbool_t pg, svint64_t op1, svuint64_t op2)svint64_t svasr[_s64]_x(svbool_t pg, svint64_t op1, svuint64_t op2)svint64_t svasr[_s64]_z(svbool_t pg, svint64_t op1, svuint64_t op2)  ASR Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svasr[_s8]_m(svbool_t pg, svint8_t op1, svuint8_t op2)svint8_t svasr[_s8]_x(svbool_t pg, svint8_t op1, svuint8_t op2)svint8_t svasr[_s8]_z(svbool_t pg, svint8_t op1, svuint8_t op2)  ASR Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svasr_wide[_s8]_m(svbool_t pg, svint8_t op1, svuint64_t op2)svint8_t svasr_wide[_s8]_x(svbool_t pg, svint8_t op1, svuint64_t op2)svint8_t svasr_wide[_s8]_z(svbool_t pg, svint8_t op1, svuint64_t op2)  ASR Ztied1.B, Pg/M, Ztied1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_for_divide(value: System.Numerics.Vector[int], control: int) -> System.Numerics.Vector[int]:
        """svint16_t svasrd[_n_s16]_m(svbool_t pg, svint16_t op1, uint64_t imm2)svint16_t svasrd[_n_s16]_x(svbool_t pg, svint16_t op1, uint64_t imm2)svint16_t svasrd[_n_s16]_z(svbool_t pg, svint16_t op1, uint64_t imm2)  ASRD Ztied1.H, Pg/M, Ztied1.H, #imm2"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_for_divide(value: System.Numerics.Vector[int], control: int) -> System.Numerics.Vector[int]:
        """svint32_t svasrd[_n_s32]_m(svbool_t pg, svint32_t op1, uint64_t imm2)svint32_t svasrd[_n_s32]_x(svbool_t pg, svint32_t op1, uint64_t imm2)svint32_t svasrd[_n_s32]_z(svbool_t pg, svint32_t op1, uint64_t imm2)  ASRD Ztied1.S, Pg/M, Ztied1.S, #imm2"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_for_divide(value: System.Numerics.Vector[int], control: int) -> System.Numerics.Vector[int]:
        """svint64_t svasrd[_n_s64]_m(svbool_t pg, svint64_t op1, uint64_t imm2)svint64_t svasrd[_n_s64]_x(svbool_t pg, svint64_t op1, uint64_t imm2)svint64_t svasrd[_n_s64]_z(svbool_t pg, svint64_t op1, uint64_t imm2)  ASRD Ztied1.D, Pg/M, Ztied1.D, #imm2"""
        ...

    @staticmethod
    @overload
    def shift_right_arithmetic_for_divide(value: System.Numerics.Vector[int], control: int) -> System.Numerics.Vector[int]:
        """svint8_t svasrd[_n_s8]_m(svbool_t pg, svint8_t op1, uint64_t imm2)svint8_t svasrd[_n_s8]_x(svbool_t pg, svint8_t op1, uint64_t imm2)svint8_t svasrd[_n_s8]_z(svbool_t pg, svint8_t op1, uint64_t imm2)  ASRD Ztied1.B, Pg/M, Ztied1.B, #imm2"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svlsr[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svlsr[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svlsr[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  LSR Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svlsr_wide[_u8]_m(svbool_t pg, svuint8_t op1, svuint64_t op2)svuint8_t svlsr_wide[_u8]_x(svbool_t pg, svuint8_t op1, svuint64_t op2)svuint8_t svlsr_wide[_u8]_z(svbool_t pg, svuint8_t op1, svuint64_t op2)  LSR Ztied1.B, Pg/M, Ztied1.B, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svlsr[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svlsr[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svlsr[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  LSR Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svlsr_wide[_u16]_m(svbool_t pg, svuint16_t op1, svuint64_t op2)svuint16_t svlsr_wide[_u16]_x(svbool_t pg, svuint16_t op1, svuint64_t op2)svuint16_t svlsr_wide[_u16]_z(svbool_t pg, svuint16_t op1, svuint64_t op2)  LSR Ztied1.H, Pg/M, Ztied1.H, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svlsr[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svlsr[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svlsr[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  LSR Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svlsr_wide[_u32]_m(svbool_t pg, svuint32_t op1, svuint64_t op2)svuint32_t svlsr_wide[_u32]_x(svbool_t pg, svuint32_t op1, svuint64_t op2)svuint32_t svlsr_wide[_u32]_z(svbool_t pg, svuint32_t op1, svuint64_t op2)  LSR Ztied1.S, Pg/M, Ztied1.S, Zop2.D"""
        ...

    @staticmethod
    @overload
    def shift_right_logical(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svlsr[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svlsr[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svlsr[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  LSR Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def sign_extend_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svexth[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svexth[_s32]_x(svbool_t pg, svint32_t op)svint32_t svexth[_s32]_z(svbool_t pg, svint32_t op)  SXTH Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def sign_extend_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svexth[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svexth[_s64]_x(svbool_t pg, svint64_t op)svint64_t svexth[_s64]_z(svbool_t pg, svint64_t op)  SXTH Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    def sign_extend_32(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svextw[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svextw[_s64]_x(svbool_t pg, svint64_t op)svint64_t svextw[_s64]_z(svbool_t pg, svint64_t op)  SXTW Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def sign_extend_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svextb[_s16]_m(svint16_t inactive, svbool_t pg, svint16_t op)svint16_t svextb[_s16]_x(svbool_t pg, svint16_t op)svint16_t svextb[_s16]_z(svbool_t pg, svint16_t op)  SXTB Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def sign_extend_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svextb[_s32]_m(svint32_t inactive, svbool_t pg, svint32_t op)svint32_t svextb[_s32]_x(svbool_t pg, svint32_t op)svint32_t svextb[_s32]_z(svbool_t pg, svint32_t op)  SXTB Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def sign_extend_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svextb[_s64]_m(svint64_t inactive, svbool_t pg, svint64_t op)svint64_t svextb[_s64]_x(svbool_t pg, svint64_t op)svint64_t svextb[_s64]_z(svbool_t pg, svint64_t op)  SXTB Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_lower(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svunpklo[_s16](svint8_t op)  SUNPKLO Zresult.H, Zop.B"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_lower(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svunpklo[_s32](svint16_t op)  SUNPKLO Zresult.S, Zop.H"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_lower(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svunpklo[_s64](svint32_t op)  SUNPKLO Zresult.D, Zop.S"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_upper(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svunpkhi[_s16](svint8_t op)  SUNPKHI Zresult.H, Zop.B"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_upper(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svunpkhi[_s32](svint16_t op)  SUNPKHI Zresult.S, Zop.H"""
        ...

    @staticmethod
    @overload
    def sign_extend_widening_upper(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svunpkhi[_s64](svint32_t op)  SUNPKHI Zresult.D, Zop.S"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svsplice[_u8](svbool_t pg, svuint8_t op1, svuint8_t op2)  SPLICE Ztied1.B, Pg, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svsplice[_f64](svbool_t pg, svfloat64_t op1, svfloat64_t op2)  SPLICE Ztied1.D, Pg, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svsplice[_s16](svbool_t pg, svint16_t op1, svint16_t op2)  SPLICE Ztied1.H, Pg, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svsplice[_s32](svbool_t pg, svint32_t op1, svint32_t op2)  SPLICE Ztied1.S, Pg, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svsplice[_s64](svbool_t pg, svint64_t op1, svint64_t op2)  SPLICE Ztied1.D, Pg, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svsplice[_s8](svbool_t pg, svint8_t op1, svint8_t op2)  SPLICE Ztied1.B, Pg, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[float], left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svsplice[_f32](svbool_t pg, svfloat32_t op1, svfloat32_t op2)  SPLICE Ztied1.S, Pg, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svsplice[_u16](svbool_t pg, svuint16_t op1, svuint16_t op2)  SPLICE Ztied1.H, Pg, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svsplice[_u32](svbool_t pg, svuint32_t op1, svuint32_t op2)  SPLICE Ztied1.S, Pg, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def splice(mask: System.Numerics.Vector[int], left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svsplice[_u64](svbool_t pg, svuint64_t op1, svuint64_t op2)  SPLICE Ztied1.D, Pg, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def sqrt(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svsqrt[_f64]_m(svfloat64_t inactive, svbool_t pg, svfloat64_t op)svfloat64_t svsqrt[_f64]_x(svbool_t pg, svfloat64_t op)svfloat64_t svsqrt[_f64]_z(svbool_t pg, svfloat64_t op)  FSQRT Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def sqrt(value: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svsqrt[_f32]_m(svfloat32_t inactive, svbool_t pg, svfloat32_t op)svfloat32_t svsqrt[_f32]_x(svbool_t pg, svfloat32_t op)svfloat32_t svsqrt[_f32]_z(svbool_t pg, svfloat32_t op)  FSQRT Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_u8](svbool_t pg, uint8_t *base, svuint8_t data)  ST1B Zdata.B, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_u8](svbool_t pg, uint8_t *base, svuint8x2_t data)  ST2B {Zdata0.B, Zdata1.B}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_u8](svbool_t pg, uint8_t *base, svuint8x3_t data)  ST3B {Zdata0.B - Zdata2.B}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_u8](svbool_t pg, uint8_t *base, svuint8x4_t data)  ST4B {Zdata0.B - Zdata3.B}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.Numerics.Vector[float]) -> None:
        """void svst1[_f64](svbool_t pg, float64_t *base, svfloat64_t data)  ST1D Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float]]) -> None:
        """void svst2[_f64](svbool_t pg, float64_t *base, svfloat64x2_t data)  ST2D {Zdata0.D, Zdata1.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]) -> None:
        """void svst3[_f64](svbool_t pg, float64_t *base, svfloat64x3_t data)  ST3D {Zdata0.D - Zdata2.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]) -> None:
        """void svst4[_f64](svbool_t pg, float64_t *base, svfloat64x4_t data)  ST4D {Zdata0.D - Zdata3.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_s16](svbool_t pg, int16_t *base, svint16_t data)  ST1H Zdata.H, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_s16](svbool_t pg, int16_t *base, svint16x2_t data)  ST2H {Zdata0.H, Zdata1.H}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_s16](svbool_t pg, int16_t *base, svint16x3_t data)  ST3H {Zdata0.H - Zdata2.H}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_s16](svbool_t pg, int16_t *base, svint16x4_t data)  ST4H {Zdata0.H - Zdata3.H}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_s32](svbool_t pg, int32_t *base, svint32_t data)  ST1W Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_s32](svbool_t pg, int32_t *base, svint32x2_t data)  ST2W {Zdata0.S, Zdata1.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_s32](svbool_t pg, int32_t *base, svint32x3_t data)  ST3W {Zdata0.S - Zdata2.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_s32](svbool_t pg, int32_t *base, svint32x4_t data)  ST4W {Zdata0.S - Zdata3.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_s64](svbool_t pg, int64_t *base, svint64_t data)  ST1D Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_s64](svbool_t pg, int64_t *base, svint64x2_t data)  ST2D {Zdata0.D, Zdata1.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_s64](svbool_t pg, int64_t *base, svint64x3_t data)  ST3D {Zdata0.D - Zdata2.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_s64](svbool_t pg, int64_t *base, svint64x4_t data)  ST4D {Zdata0.D - Zdata3.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_s8](svbool_t pg, int8_t *base, svint8_t data)  ST1B Zdata.B, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_s8](svbool_t pg, int8_t *base, svint8x2_t data)  ST2B {Zdata0.B, Zdata1.B}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_s8](svbool_t pg, int8_t *base, svint8x3_t data)  ST3B {Zdata0.B - Zdata2.B}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_s8](svbool_t pg, int8_t *base, svint8x4_t data)  ST4B {Zdata0.B - Zdata3.B}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.Numerics.Vector[float]) -> None:
        """void svst1[_f32](svbool_t pg, float32_t *base, svfloat32_t data)  ST1W Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float]]) -> None:
        """void svst2[_f32](svbool_t pg, float32_t *base, svfloat32x2_t data)  ST2W {Zdata0.S, Zdata1.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]) -> None:
        """void svst3[_f32](svbool_t pg, float32_t *base, svfloat32x3_t data)  ST3W {Zdata0.S - Zdata2.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[float], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float], System.Numerics.Vector[float]]) -> None:
        """void svst4[_f32](svbool_t pg, float32_t *base, svfloat32x4_t data)  ST4W {Zdata0.S - Zdata3.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_u16](svbool_t pg, uint16_t *base, svuint16_t data)  ST1H Zdata.H, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_u16](svbool_t pg, uint16_t *base, svuint16x2_t data)  ST2H {Zdata0.H, Zdata1.H}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_u16](svbool_t pg, uint16_t *base, svuint16x3_t data)  ST3H {Zdata0.H - Zdata2.H}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_u16](svbool_t pg, uint16_t *base, svuint16x4_t data)  ST4H {Zdata0.H - Zdata3.H}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_u32](svbool_t pg, uint32_t *base, svuint32_t data)  ST1W Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_u32](svbool_t pg, uint32_t *base, svuint32x2_t data)  ST2W {Zdata0.S, Zdata1.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_u32](svbool_t pg, uint32_t *base, svuint32x3_t data)  ST3W {Zdata0.S - Zdata2.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_u32](svbool_t pg, uint32_t *base, svuint32x4_t data)  ST4W {Zdata0.S - Zdata3.S}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1[_u64](svbool_t pg, uint64_t *base, svuint64_t data)  ST1D Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst2[_u64](svbool_t pg, uint64_t *base, svuint64x2_t data)  ST2D {Zdata0.D, Zdata1.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst3[_u64](svbool_t pg, uint64_t *base, svuint64x3_t data)  ST3D {Zdata0.D - Zdata2.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_and_zip(mask: System.Numerics.Vector[int], address: typing.Any, data: System.ValueTuple[System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int], System.Numerics.Vector[int]]) -> None:
        """void svst4[_u64](svbool_t pg, uint64_t *base, svuint64x4_t data)  ST4D {Zdata0.D - Zdata3.D}, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1b[_s16](svbool_t pg, int8_t *base, svint16_t data)  ST1B Zdata.H, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1b[_s32](svbool_t pg, int8_t *base, svint32_t data)  ST1B Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1h[_s32](svbool_t pg, int16_t *base, svint32_t data)  ST1H Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1b[_s64](svbool_t pg, int8_t *base, svint64_t data)  ST1B Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1h[_s64](svbool_t pg, int16_t *base, svint64_t data)  ST1H Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1w[_s64](svbool_t pg, int32_t *base, svint64_t data)  ST1W Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1b[_u16](svbool_t pg, uint8_t *base, svuint16_t data)  ST1B Zdata.H, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1b[_u32](svbool_t pg, uint8_t *base, svuint32_t data)  ST1B Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1h[_u32](svbool_t pg, uint16_t *base, svuint32_t data)  ST1H Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1b[_u64](svbool_t pg, uint8_t *base, svuint64_t data)  ST1B Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1h[_u64](svbool_t pg, uint16_t *base, svuint64_t data)  ST1H Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_narrowing(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svst1w[_u64](svbool_t pg, uint32_t *base, svuint64_t data)  ST1W Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_u8](svbool_t pg, uint8_t *base, svuint8_t data)  STNT1B Zdata.B, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[float], address: typing.Any, data: System.Numerics.Vector[float]) -> None:
        """void svstnt1[_f64](svbool_t pg, float64_t *base, svfloat64_t data)  STNT1D Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_s16](svbool_t pg, int16_t *base, svint16_t data)  STNT1H Zdata.H, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_s32](svbool_t pg, int32_t *base, svint32_t data)  STNT1W Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_s64](svbool_t pg, int64_t *base, svint64_t data)  STNT1D Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_s8](svbool_t pg, int8_t *base, svint8_t data)  STNT1B Zdata.B, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[float], address: typing.Any, data: System.Numerics.Vector[float]) -> None:
        """void svstnt1[_f32](svbool_t pg, float32_t *base, svfloat32_t data)  STNT1W Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_u16](svbool_t pg, uint16_t *base, svuint16_t data)  STNT1H Zdata.H, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_u32](svbool_t pg, uint32_t *base, svuint32_t data)  STNT1W Zdata.S, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def store_non_temporal(mask: System.Numerics.Vector[int], address: typing.Any, data: System.Numerics.Vector[int]) -> None:
        """void svstnt1[_u64](svbool_t pg, uint64_t *base, svuint64_t data)  STNT1D Zdata.D, Pg, [Xbase, #0, MUL VL]"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svsub[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svsub[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t svsub[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  SUB Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svsub[_f64]_m(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svsub[_f64]_x(svbool_t pg, svfloat64_t op1, svfloat64_t op2)svfloat64_t svsub[_f64]_z(svbool_t pg, svfloat64_t op1, svfloat64_t op2)  FSUB Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svsub[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svsub[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t svsub[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  SUB Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svsub[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svsub[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t svsub[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  SUB Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svsub[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svsub[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t svsub[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  SUB Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svsub[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svsub[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t svsub[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  SUB Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svsub[_f32]_m(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svsub[_f32]_x(svbool_t pg, svfloat32_t op1, svfloat32_t op2)svfloat32_t svsub[_f32]_z(svbool_t pg, svfloat32_t op1, svfloat32_t op2)  FSUB Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svsub[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svsub[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t svsub[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  SUB Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svsub[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svsub[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t svsub[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  SUB Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def subtract(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svsub[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svsub[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t svsub[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  SUB Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svqsub[_u8](svuint8_t op1, svuint8_t op2)  UQSUB Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svqsub[_s16](svint16_t op1, svint16_t op2)  SQSUB Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svqsub[_s32](svint32_t op1, svint32_t op2)  SQSUB Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svqsub[_s64](svint64_t op1, svint64_t op2)  SQSUB Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svqsub[_s8](svint8_t op1, svint8_t op2)  SQSUB Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svqsub[_u16](svuint16_t op1, svuint16_t op2)  UQSUB Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svqsub[_u32](svuint32_t op1, svuint32_t op2)  UQSUB Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def subtract_saturate(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svqsub[_u64](svuint64_t op1, svuint64_t op2)  UQSUB Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_any_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_any(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_first_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_first(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def test_last_true(mask: System.Numerics.Vector[int], src_mask: System.Numerics.Vector[int]) -> bool:
        """bool svptest_last(svbool_t pg, svbool_t op)  PTEST"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svtrn1[_u8](svuint8_t op1, svuint8_t op2)  TRN1 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svtrn1[_f64](svfloat64_t op1, svfloat64_t op2)  TRN1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svtrn1[_s16](svint16_t op1, svint16_t op2)  TRN1 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svtrn1[_s32](svint32_t op1, svint32_t op2)  TRN1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svtrn1[_s64](svint64_t op1, svint64_t op2)  TRN1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svtrn1[_s8](svint8_t op1, svint8_t op2)  TRN1 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svtrn1[_f32](svfloat32_t op1, svfloat32_t op2)  TRN1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svtrn1[_u16](svuint16_t op1, svuint16_t op2)  TRN1 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svtrn1[_u32](svuint32_t op1, svuint32_t op2)  TRN1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def transpose_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svtrn1[_u64](svuint64_t op1, svuint64_t op2)  TRN1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svtrn2[_u8](svuint8_t op1, svuint8_t op2)  TRN2 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svtrn2[_f64](svfloat64_t op1, svfloat64_t op2)  TRN2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svtrn2[_s16](svint16_t op1, svint16_t op2)  TRN2 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svtrn2[_s32](svint32_t op1, svint32_t op2)  TRN2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svtrn2[_s64](svint64_t op1, svint64_t op2)  TRN2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svtrn2[_s8](svint8_t op1, svint8_t op2)  TRN2 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svtrn2[_f32](svfloat32_t op1, svfloat32_t op2)  TRN2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svtrn2[_u16](svuint16_t op1, svuint16_t op2)  TRN2 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svtrn2[_u32](svuint32_t op1, svuint32_t op2)  TRN2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def transpose_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svtrn2[_u64](svuint64_t op1, svuint64_t op2)  TRN2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def trigonometric_multiply_add_coefficient(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], control: int) -> System.Numerics.Vector[float]:
        """svfloat64_t svtmad[_f64](svfloat64_t op1, svfloat64_t op2, uint64_t imm3)  FTMAD Ztied1.D, Ztied1.D, Zop2.D, #imm3"""
        ...

    @staticmethod
    @overload
    def trigonometric_multiply_add_coefficient(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float], control: int) -> System.Numerics.Vector[float]:
        """svfloat32_t svtmad[_f32](svfloat32_t op1, svfloat32_t op2, uint64_t imm3)  FTMAD Ztied1.S, Ztied1.S, Zop2.S, #imm3"""
        ...

    @staticmethod
    @overload
    def trigonometric_select_coefficient(value: System.Numerics.Vector[float], selector: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svtssel[_f64](svfloat64_t op1, svuint64_t op2)  FTSSEL Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def trigonometric_select_coefficient(value: System.Numerics.Vector[float], selector: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svtssel[_f32](svfloat32_t op1, svuint32_t op2)  FTSSEL Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def trigonometric_starting_value(value: System.Numerics.Vector[float], sign: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svtsmul[_f64](svfloat64_t op1, svuint64_t op2)  FTSMUL Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def trigonometric_starting_value(value: System.Numerics.Vector[float], sign: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svtsmul[_f32](svfloat32_t op1, svuint32_t op2)  FTSMUL Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svuzp1[_u8](svuint8_t op1, svuint8_t op2)  UZP1 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svuzp1[_f64](svfloat64_t op1, svfloat64_t op2)  UZP1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svuzp1[_s16](svint16_t op1, svint16_t op2)  UZP1 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svuzp1[_s32](svint32_t op1, svint32_t op2)  UZP1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svuzp1[_s64](svint64_t op1, svint64_t op2)  UZP1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svuzp1[_s8](svint8_t op1, svint8_t op2)  UZP1 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svuzp1[_f32](svfloat32_t op1, svfloat32_t op2)  UZP1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svuzp1[_u16](svuint16_t op1, svuint16_t op2)  UZP1 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svuzp1[_u32](svuint32_t op1, svuint32_t op2)  UZP1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def unzip_even(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svuzp1[_u64](svuint64_t op1, svuint64_t op2)  UZP1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svuzp2[_u8](svuint8_t op1, svuint8_t op2)  UZP2 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svuzp2[_f64](svfloat64_t op1, svfloat64_t op2)  UZP2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svuzp2[_s16](svint16_t op1, svint16_t op2)  UZP2 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svuzp2[_s32](svint32_t op1, svint32_t op2)  UZP2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svuzp2[_s64](svint64_t op1, svint64_t op2)  UZP2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svuzp2[_s8](svint8_t op1, svint8_t op2)  UZP2 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svuzp2[_f32](svfloat32_t op1, svfloat32_t op2)  UZP2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svuzp2[_u16](svuint16_t op1, svuint16_t op2)  UZP2 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svuzp2[_u32](svuint32_t op1, svuint32_t op2)  UZP2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def unzip_odd(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svuzp2[_u64](svuint64_t op1, svuint64_t op2)  UZP2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svtbl[_u8](svuint8_t data, svuint8_t indices)  TBL Zresult.B, {Zdata.B}, Zindices.B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[float], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat64_t svtbl[_f64](svfloat64_t data, svuint64_t indices)  TBL Zresult.D, {Zdata.D}, Zindices.D"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svtbl[_s16](svint16_t data, svuint16_t indices)  TBL Zresult.H, {Zdata.H}, Zindices.H"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svtbl[_s32](svint32_t data, svuint32_t indices)  TBL Zresult.S, {Zdata.S}, Zindices.S"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svtbl[_s64](svint64_t data, svuint64_t indices)  TBL Zresult.D, {Zdata.D}, Zindices.D"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svtbl[_s8](svint8_t data, svuint8_t indices)  TBL Zresult.B, {Zdata.B}, Zindices.B"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[float], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[float]:
        """svfloat32_t svtbl[_f32](svfloat32_t data, svuint32_t indices)  TBL Zresult.S, {Zdata.S}, Zindices.S"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svtbl[_u16](svuint16_t data, svuint16_t indices)  TBL Zresult.H, {Zdata.H}, Zindices.H"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svtbl[_u32](svuint32_t data, svuint32_t indices)  TBL Zresult.S, {Zdata.S}, Zindices.S"""
        ...

    @staticmethod
    @overload
    def vector_table_lookup(data: System.Numerics.Vector[int], indices: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svtbl[_u64](svuint64_t data, svuint64_t indices)  TBL Zresult.D, {Zdata.D}, Zindices.D"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t sveor[_u8]_m(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t sveor[_u8]_x(svbool_t pg, svuint8_t op1, svuint8_t op2)svuint8_t sveor[_u8]_z(svbool_t pg, svuint8_t op1, svuint8_t op2)  EOR Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t sveor[_s16]_m(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t sveor[_s16]_x(svbool_t pg, svint16_t op1, svint16_t op2)svint16_t sveor[_s16]_z(svbool_t pg, svint16_t op1, svint16_t op2)  EOR Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t sveor[_s32]_m(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t sveor[_s32]_x(svbool_t pg, svint32_t op1, svint32_t op2)svint32_t sveor[_s32]_z(svbool_t pg, svint32_t op1, svint32_t op2)  EOR Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t sveor[_s64]_m(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t sveor[_s64]_x(svbool_t pg, svint64_t op1, svint64_t op2)svint64_t sveor[_s64]_z(svbool_t pg, svint64_t op1, svint64_t op2)  EOR Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t sveor[_s8]_m(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t sveor[_s8]_x(svbool_t pg, svint8_t op1, svint8_t op2)svint8_t sveor[_s8]_z(svbool_t pg, svint8_t op1, svint8_t op2)  EOR Ztied1.B, Pg/M, Ztied1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t sveor[_u16]_m(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t sveor[_u16]_x(svbool_t pg, svuint16_t op1, svuint16_t op2)svuint16_t sveor[_u16]_z(svbool_t pg, svuint16_t op1, svuint16_t op2)  EOR Ztied1.H, Pg/M, Ztied1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t sveor[_u32]_m(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t sveor[_u32]_x(svbool_t pg, svuint32_t op1, svuint32_t op2)svuint32_t sveor[_u32]_z(svbool_t pg, svuint32_t op1, svuint32_t op2)  EOR Ztied1.S, Pg/M, Ztied1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def xor(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t sveor[_u64]_m(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t sveor[_u64]_x(svbool_t pg, svuint64_t op1, svuint64_t op2)svuint64_t sveor[_u64]_z(svbool_t pg, svuint64_t op1, svuint64_t op2)  EOR Ztied1.D, Pg/M, Ztied1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint8_t sveorv[_u8](svbool_t pg, svuint8_t op)  EORV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int16_t sveorv[_s16](svbool_t pg, svint16_t op)  EORV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int32_t sveorv[_s32](svbool_t pg, svint32_t op)  EORV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int64_t sveorv[_s64](svbool_t pg, svint64_t op)  EORV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """int8_t sveorv[_s8](svbool_t pg, svint8_t op)  EORV Bresult, Pg, Zop.B"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint16_t sveorv[_u16](svbool_t pg, svuint16_t op)  EORV Hresult, Pg, Zop.H"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint32_t sveorv[_u32](svbool_t pg, svuint32_t op)  EORV Sresult, Pg, Zop.S"""
        ...

    @staticmethod
    @overload
    def xor_across(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """uint64_t sveorv[_u64](svbool_t pg, svuint64_t op)  EORV Dresult, Pg, Zop.D"""
        ...

    @staticmethod
    @overload
    def zero_extend_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svexth[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svexth[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svexth[_u32]_z(svbool_t pg, svuint32_t op)  UXTH Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def zero_extend_16(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svexth[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svexth[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svexth[_u64]_z(svbool_t pg, svuint64_t op)  UXTH Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    def zero_extend_32(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svextw[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svextw[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svextw[_u64]_z(svbool_t pg, svuint64_t op)  UXTW Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def zero_extend_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svextb[_u16]_m(svuint16_t inactive, svbool_t pg, svuint16_t op)svuint16_t svextb[_u16]_x(svbool_t pg, svuint16_t op)svuint16_t svextb[_u16]_z(svbool_t pg, svuint16_t op)  UXTB Zresult.H, Pg/M, Zop.H"""
        ...

    @staticmethod
    @overload
    def zero_extend_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svextb[_u32]_m(svuint32_t inactive, svbool_t pg, svuint32_t op)svuint32_t svextb[_u32]_x(svbool_t pg, svuint32_t op)svuint32_t svextb[_u32]_z(svbool_t pg, svuint32_t op)  UXTB Zresult.S, Pg/M, Zop.S"""
        ...

    @staticmethod
    @overload
    def zero_extend_8(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svextb[_u64]_m(svuint64_t inactive, svbool_t pg, svuint64_t op)svuint64_t svextb[_u64]_x(svbool_t pg, svuint64_t op)svuint64_t svextb[_u64]_z(svbool_t pg, svuint64_t op)  UXTB Zresult.D, Pg/M, Zop.D"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svunpklo[_u16](svuint8_t op)  UUNPKLO Zresult.H, Zop.B"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svunpklo[_u32](svuint16_t op)  UUNPKLO Zresult.S, Zop.H"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_lower(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svunpklo[_u64](svuint32_t op)  UUNPKLO Zresult.D, Zop.S"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svunpkhi[_u16](svuint8_t op)  UUNPKHI Zresult.H, Zop.B"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svunpkhi[_u32](svuint16_t op)  UUNPKHI Zresult.S, Zop.H"""
        ...

    @staticmethod
    @overload
    def zero_extend_widening_upper(value: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svunpkhi[_u64](svuint32_t op)  UUNPKHI Zresult.D, Zop.S"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svzip2[_u8](svuint8_t op1, svuint8_t op2)  ZIP2 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svzip2[_f64](svfloat64_t op1, svfloat64_t op2)  ZIP2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svzip2[_s16](svint16_t op1, svint16_t op2)  ZIP2 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svzip2[_s32](svint32_t op1, svint32_t op2)  ZIP2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svzip2[_s64](svint64_t op1, svint64_t op2)  ZIP2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svzip2[_s8](svint8_t op1, svint8_t op2)  ZIP2 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svzip2[_f32](svfloat32_t op1, svfloat32_t op2)  ZIP2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svzip2[_u16](svuint16_t op1, svuint16_t op2)  ZIP2 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svzip2[_u32](svuint32_t op1, svuint32_t op2)  ZIP2 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def zip_high(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svzip2[_u64](svuint64_t op1, svuint64_t op2)  ZIP2 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint8_t svzip1[_u8](svuint8_t op1, svuint8_t op2)  ZIP1 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat64_t svzip1[_f64](svfloat64_t op1, svfloat64_t op2)  ZIP1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint16_t svzip1[_s16](svint16_t op1, svint16_t op2)  ZIP1 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint32_t svzip1[_s32](svint32_t op1, svint32_t op2)  ZIP1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint64_t svzip1[_s64](svint64_t op1, svint64_t op2)  ZIP1 Zresult.D, Zop1.D, Zop2.D"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svint8_t svzip1[_s8](svint8_t op1, svint8_t op2)  ZIP1 Zresult.B, Zop1.B, Zop2.B"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[float], right: System.Numerics.Vector[float]) -> System.Numerics.Vector[float]:
        """svfloat32_t svzip1[_f32](svfloat32_t op1, svfloat32_t op2)  ZIP1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint16_t svzip1[_u16](svuint16_t op1, svuint16_t op2)  ZIP1 Zresult.H, Zop1.H, Zop2.H"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint32_t svzip1[_u32](svuint32_t op1, svuint32_t op2)  ZIP1 Zresult.S, Zop1.S, Zop2.S"""
        ...

    @staticmethod
    @overload
    def zip_low(left: System.Numerics.Vector[int], right: System.Numerics.Vector[int]) -> System.Numerics.Vector[int]:
        """svuint64_t svzip1[_u64](svuint64_t op1, svuint64_t op2)  ZIP1 Zresult.D, Zop1.D, Zop2.D"""
        ...


